@@ -4,7 +4,13 @@ set -ex
 
 echo "Running clang-format against branch $TRAVIS_BRANCH, with hash $BASE_COMMIT"
 COMMIT_FILES=$(git diff --name-status $BASE_COMMIT | grep -i -v LinkDef | grep -v -E '^D +' | sed -E 's,^.[[:space:]]+,,')
-RESULT_OUTPUT="$(git-clang-format --commit $BASE_COMMIT --diff --binary `which clang-format` $COMMIT_FILES)"
+
+RESULT_OUTPUT="no modified files to format"
+if [ ! -z "$COMMIT_FILES" ]; then
+  # If COMMIT_FILES is empty, git-clang-format will take all the modified files
+  # Therefore, we only actually run git-clang-format if there is anything to check
+  RESULT_OUTPUT="$(git-clang-format --commit $BASE_COMMIT --diff --binary `which clang-format` $COMMIT_FILES)"
+fi
 
 if [ "$RESULT_OUTPUT" == "no modified files to format" ] \
   || [ "$RESULT_OUTPUT" == "clang-format did not modify any files" ] ; then
