@@ -16,12 +16,12 @@
 import argparse
 import datetime
 import os
+import platform
 import re
 import shutil
 import subprocess
 import sys
 import tarfile
-from hashlib import sha1
 
 import openstack
 
@@ -29,6 +29,7 @@ from build_utils import (
     die,
     github_log_group,
     load_config,
+    calc_options_hash,
     subprocess_with_log,
     subprocess_with_capture,
     upload_file
@@ -57,7 +58,7 @@ def main():
 
     args = parse_args()
 
-    build_utils.log = build_utils.Tracer(args.image, args.dockeropts)
+    build_utils.log = build_utils.Tracer(args.platform, args.dockeropts)
 
     pull_request = args.head_ref and args.head_ref != args.base_ref
 
@@ -83,10 +84,20 @@ def main():
         if args.architecture == 'x86':
             options = "-AWin32 " + options
 
-    # The sha1 of the build option string is used to find existing artifacts
+    # The hash of the build option string is used to find existing artifacts
     # with matching build options on s3 storage.
-    option_hash = sha1(options.encode('utf-8')).hexdigest()
-    obj_prefix = f'{args.platform}/{args.base_ref}/{args.buildtype}/{option_hash}'
+    options_hash = calc_options_hash(options)
+
+    # Differentiate between macos versions: it's possible to have the same label
+    # for different macos versions, especially different minor versions.
+    macos_version_prefix = ''
+    if 'Darwin' == platform.system():
+        macos_version_tuple = platform.mac_ver()
+        macos_version = macos_version_tuple[0]
+        macos_version_prefix = f'{macos_version}/'
+    platform_machine = platform.machine()
+
+    obj_prefix = f'{args.platform}/{macos_version_prefix}{args.base_ref}/{args.buildtype}_{platform_machine}/{options_hash}'
 
     # Make testing of CI in forks not impact artifacts
     if 'root-project/root' not in args.repository:
@@ -181,7 +192,6 @@ def parse_args():
     # true/false for boolean arguments instead.
     parser = argparse.ArgumentParser()
     parser.add_argument("--platform",                           help="Platform to build on")
-    parser.add_argument("--image",           default=None,      help="Container image, if any")
     parser.add_argument("--dockeropts",      default=None,      help="Extra docker options, if any")
     parser.add_argument("--incremental",     default="false",   help="Do incremental build")
     parser.add_argument("--buildtype",       default="Release", help="Release|Debug|RelWithDebInfo")
@@ -231,7 +241,8 @@ def cleanup_previous_build():
     else:
         # mac/linux/POSIX
         result = subprocess_with_log(f"""
-            rm -rf {WORKDIR}/*
+            rm -rf {WORKDIR}
+            mkdir -p {WORKDIR}
         """)
 
     if result != 0:
@@ -285,8 +296,6 @@ def download_artifacts(obj_prefix: str):
 @github_log_group("Node state")
 def show_node_state() -> None:
     result = subprocess_with_log("""
-        echo $SHELL
-        echo $PATH
         which cmake
         cmake --version
         which c++ || true
