@@ -4,7 +4,9 @@
 // Bindings
 #include "Executors.h"
 #include "CallContext.h"
+#include "Dimensions.h"
 
+// Standard
 #if __cplusplus > 201402L
 #include <cstddef>
 #endif
@@ -44,31 +46,43 @@ CPPYY_DECL_EXEC(Double);
 CPPYY_DECL_EXEC(LongDouble);
 CPPYY_DECL_EXEC(Void);
 CPPYY_DECL_EXEC(CString);
+CPPYY_DECL_EXEC(CStringRef);
 CPPYY_DECL_EXEC(WCString);
 CPPYY_DECL_EXEC(CString16);
 CPPYY_DECL_EXEC(CString32);
 
 // pointer/array executors
-CPPYY_DECL_EXEC(VoidArray);
-CPPYY_DECL_EXEC(BoolArray);
-CPPYY_DECL_EXEC(UCharArray);
+#define CPPYY_ARRAY_DECL_EXEC(name)                                          \
+class name##ArrayExecutor : public Executor {                                \
+    dims_t fShape;                                                           \
+public:                                                                      \
+    name##ArrayExecutor(dims_t dims) : fShape(dims) {}                       \
+    virtual PyObject* Execute(                                               \
+        Cppyy::TCppMethod_t, Cppyy::TCppObject_t, CallContext*);             \
+    virtual bool HasState() { return true; }                                 \
+}
+CPPYY_ARRAY_DECL_EXEC(Void);
+CPPYY_ARRAY_DECL_EXEC(Bool);
+CPPYY_ARRAY_DECL_EXEC(UChar);
 #if __cplusplus > 201402L
-CPPYY_DECL_EXEC(ByteArray);
+CPPYY_ARRAY_DECL_EXEC(Byte);
 #endif
-CPPYY_DECL_EXEC(ShortArray);
-CPPYY_DECL_EXEC(UShortArray);
-CPPYY_DECL_EXEC(IntArray);
-CPPYY_DECL_EXEC(UIntArray);
-CPPYY_DECL_EXEC(LongArray);
-CPPYY_DECL_EXEC(ULongArray);
-CPPYY_DECL_EXEC(LLongArray);
-CPPYY_DECL_EXEC(ULLongArray);
-CPPYY_DECL_EXEC(FloatArray);
-CPPYY_DECL_EXEC(DoubleArray);
-CPPYY_DECL_EXEC(ComplexFArray);
-CPPYY_DECL_EXEC(ComplexDArray);
-CPPYY_DECL_EXEC(ComplexIArray);
-CPPYY_DECL_EXEC(ComplexLArray);
+CPPYY_ARRAY_DECL_EXEC(Int8);
+CPPYY_ARRAY_DECL_EXEC(UInt8);
+CPPYY_ARRAY_DECL_EXEC(Short);
+CPPYY_ARRAY_DECL_EXEC(UShort);
+CPPYY_ARRAY_DECL_EXEC(Int);
+CPPYY_ARRAY_DECL_EXEC(UInt);
+CPPYY_ARRAY_DECL_EXEC(Long);
+CPPYY_ARRAY_DECL_EXEC(ULong);
+CPPYY_ARRAY_DECL_EXEC(LLong);
+CPPYY_ARRAY_DECL_EXEC(ULLong);
+CPPYY_ARRAY_DECL_EXEC(Float);
+CPPYY_ARRAY_DECL_EXEC(Double);
+CPPYY_ARRAY_DECL_EXEC(ComplexF);
+CPPYY_ARRAY_DECL_EXEC(ComplexD);
+CPPYY_ARRAY_DECL_EXEC(ComplexI);
+CPPYY_ARRAY_DECL_EXEC(ComplexL);
 
 // special cases
 CPPYY_DECL_EXEC(ComplexD);
@@ -95,12 +109,14 @@ public:
 
 protected:
     Cppyy::TCppType_t fClass;
-    unsigned int      fFlags;
+    uint32_t          fFlags;
 };
 
 class IteratorExecutor : public InstanceExecutor {
 public:
     IteratorExecutor(Cppyy::TCppType_t klass);
+    virtual PyObject* Execute(
+        Cppyy::TCppMethod_t, Cppyy::TCppObject_t, CallContext*);
 };
 
 CPPYY_DECL_EXEC(Constructor);
@@ -159,13 +175,13 @@ public:
 
 class InstanceArrayExecutor : public InstancePtrExecutor {
 public:
-    InstanceArrayExecutor(Cppyy::TCppType_t klass, Py_ssize_t array_size)
-        : InstancePtrExecutor(klass), fArraySize(array_size) {}
+    InstanceArrayExecutor(Cppyy::TCppType_t klass, dim_t array_size)
+        : InstancePtrExecutor(klass), fSize(array_size) {}
     virtual PyObject* Execute(
         Cppyy::TCppMethod_t, Cppyy::TCppObject_t, CallContext*);
 
 protected:
-    Py_ssize_t fArraySize;
+    dim_t fSize;
 };
 
 class FunctionPointerExecutor : public Executor {
