@@ -39,22 +39,27 @@ clean_environment()
       fi
       if [ -n "${LD_LIBRARY_PATH-}" ]; then
          drop_from_path "$LD_LIBRARY_PATH" "${old_rootsys}/lib"
+         drop_from_path "$LD_LIBRARY_PATH" "${old_rootsys}/lib/root"
          LD_LIBRARY_PATH=$newpath
       fi
       if [ -n "${DYLD_LIBRARY_PATH-}" ]; then
          drop_from_path "$DYLD_LIBRARY_PATH" "${old_rootsys}/lib"
+         drop_from_path "$DYLD_LIBRARY_PATH" "${old_rootsys}/lib/root"
          DYLD_LIBRARY_PATH=$newpath
       fi
       if [ -n "${SHLIB_PATH-}" ]; then
          drop_from_path "$SHLIB_PATH" "${old_rootsys}/lib"
+         drop_from_path "$SHLIB_PATH" "${old_rootsys}/lib/root"
          SHLIB_PATH=$newpath
       fi
       if [ -n "${LIBPATH-}" ]; then
          drop_from_path "$LIBPATH" "${old_rootsys}/lib"
+         drop_from_path "$LIBPATH" "${old_rootsys}/lib/root"
          LIBPATH=$newpath
       fi
       if [ -n "${PYTHONPATH-}" ]; then
          drop_from_path "$PYTHONPATH" "${old_rootsys}/lib"
+         drop_from_path "$PYTHONPATH" "${old_rootsys}/lib/root"
          PYTHONPATH=$newpath
       fi
       if [ -n "${MANPATH-}" ]; then
@@ -73,11 +78,6 @@ clean_environment()
          drop_from_path "$JUPYTER_CONFIG_DIR" "${old_rootsys}/etc/notebook"
          JUPYTER_CONFIG_DIR=$newpath
       fi
-      # Potential leftovers from future ROOT > v6.34
-      if [ -n "${JUPYTER_CONFIG_PATH-}" ]; then
-         drop_from_path "$JUPYTER_CONFIG_PATH" "${old_rootsys}/etc/notebook"
-         JUPYTER_CONFIG_PATH=$newpath
-      fi
    fi
    if [ -z "${MANPATH-}" ]; then
       # Grab the default man path before setting the path to avoid duplicates
