@@ -541,8 +541,12 @@ void TApplication::GetOptions(Int_t *argc, char **argv)
                   fFiles->Add(new TNamed("NOT FOUND!", argv[i]));
                   // only warn if we're plain root,
                   // other progs might have their own params
-                  if (!strcmp(gROOT->GetName(), "Rint"))
-                     Warning("GetOptions", "macro %s not found", fname.Data());
+                  if (!strcmp(gROOT->GetName(), "Rint")) {
+                     Error("GetOptions", "macro %s not found", fname.Data());
+                     // Return 2 as the Python interpreter does in case the macro
+                     // is not found.
+                     Terminate(2);
+                  }
                }
             }
          }
@@ -1032,10 +1036,8 @@ Steps to reproduce the behavior:
 -->
 
 ### Setup
-```
 )"+GetSetup()+
-R"(```
-
+R"(
 <!--
 Please specify also how you obtained ROOT, such as `dnf install` / binary download / you built it yourself.
 -->
@@ -1257,6 +1259,7 @@ void TApplication::Help(const char *line)
              "                         Specifying '::Member' is optional.");
       Printf("   .help edit          : show line editing shortcuts (or .?)");
       Printf("   .license            : show license");
+      Printf("   .libraries          : show loaded libraries");
       Printf("   .ls                 : list contents of current TDirectory");
       Printf("   .pwd                : show current TDirectory, pad and style");
       Printf("   .quit (or .exit)    : quit ROOT (long form of .q)");
@@ -1728,6 +1731,12 @@ Longptr_t TApplication::ProcessLine(const char *line, Bool_t sync, Int_t *err)
 #endif
    }
 
+   if (!strcmp(line, ".libraries")) {
+      // List the loaded libraries
+      gSystem->ListLibraries();
+      return 0;
+   }
+
    if (sync)
       return gInterpreter->ProcessLineSynch(line, (TInterpreter::EErrorCode*)err);
    else
