@@ -59,8 +59,8 @@ End_Macro
 
 `SetMarkerColorAlpha()`, allows to set a transparent color.
 In the following example the marker color of the histogram `histo`
-is set to blue with a transparency of 35%. The color `kBlue`
-itself remains fully opaque.
+is set to blue with an opacity of 35% (i.e. a transparency of 65%).
+(The color `kBlue` itself is internally stored as fully opaque.)
 
 ~~~ {.cpp}
 histo->SetMarkerColorAlpha(kBlue, 0.35);
@@ -70,6 +70,9 @@ The transparency is available on all platforms when the flag `OpenGL.CanvasPrefe
 in `$ROOTSYS/etc/system.rootrc`, or on Mac with the Cocoa backend. On the file output
 it is visible with PDF, PNG, Gif, JPEG, SVG, TeX ... but not PostScript.
 
+Alternatively, you can call at the top of your script `gSytle->SetCanvasPreferGL();`.
+Or if you prefer to activate GL for a single canvas `c`, then use `c->SetSupportGL(true);`.
+
 \anchor ATTMARKER2
 ## Marker style
 
@@ -178,7 +181,7 @@ method `GetMarkerSize`.
 
 Begin_Macro
 {
-   c = new TCanvas("c","Marker sizes",0,0,500,200);
+   auto c = new TCanvas("c","Marker sizes",0,0,500,200);
    TMarker marker;
    marker.SetMarkerStyle(3);
    Double_t x = 0;
@@ -367,8 +370,10 @@ void TAttMarker::SetMarkerAttributes()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set a transparent marker color. malpha defines the percentage of
-/// the color opacity from 0. (fully transparent) to 1. (fully opaque).
+/// Set a transparent marker color. 
+/// \param mcolor defines the marker color
+/// \param malpha defines the percentage of opacity from 0. (fully transparent) to 1. (fully opaque).
+/// \note malpha is ignored (treated as 1) if the TCanvas has no GL support activated.
 
 void TAttMarker::SetMarkerColorAlpha(Color_t mcolor, Float_t malpha)
 {
