@@ -940,6 +940,37 @@ bool XMLReader::Parse(const std::string &fileName, SelectionRules& out)
                     }
                   }
 
+                  // request RNTuple serialization mode
+                  if (tagKind == kClass && csr && "rntupleStreamerMode" == iAttrName) {
+                     if (iAttrValue == "false") {
+                        if (csr->RequestedRNTupleSerializationMode() == -1) {
+                           ROOT::TMetaUtils::Error(
+                              nullptr,
+                              "XML at line %s: class attribute 'rntupleStreamerMode' must be either 'true' or 'false', "
+                              "not both\n",
+                              lineNumCharp, iAttrValue.c_str());
+                        } else {
+                           csr->SetRequestedRNTupleSerializationMode(1);
+                        }
+                     } else if (iAttrValue == "true") {
+                        if (csr->RequestedRNTupleSerializationMode() == 1) {
+                           ROOT::TMetaUtils::Error(
+                              nullptr,
+                              "XML at line %s: class attribute 'rntupleStreamerMode' must be either 'true' or 'false', "
+                              "not both\n",
+                              lineNumCharp, iAttrValue.c_str());
+                        } else {
+                           csr->SetRequestedRNTupleSerializationMode(-1);
+                        }
+                     } else {
+                        ROOT::TMetaUtils::Error(
+                           nullptr,
+                           "XML at line %s: class attribute 'rntupleStreamerMode' must be 'true' or 'false' "
+                           "(it was %s)\n",
+                           lineNumCharp, iAttrValue.c_str());
+                     }
+                  }
+
                   // request no input operator
                   if (tagKind == kClass && csr && "noInputOperator" == iAttrName){
                     if (iAttrValue == "true") {
