@@ -113,7 +113,9 @@ Bool_t TBrowser::InitGraphics()
    // make sure that the Gpad and GUI libs are loaded
    TApplication::NeedGraphicsLibs();
 
-   Bool_t isweb = gROOT->IsWebDisplay();
+   TString hname = gEnv->GetValue("Browser.Name", "TRootBrowserLite");
+
+   Bool_t isweb = gROOT->IsWebDisplay() || (hname == "ROOT::RWebBrowserImp");
 
    if (gApplication)
       gApplication->InitializeGraphics(isweb);
