@@ -185,14 +185,14 @@ TObject *TListOfDataMembers::FindObject(const char *name) const
    if (!result) {
       if (IsLoaded() && fClass && fClass->Property() & (kIsClass|kIsStruct|kIsUnion)) {
          // We already have all the information, no need to search more
-         return 0;
+         return nullptr;
       }
 
       R__LOCKGUARD(gInterpreterMutex);
 
       TInterpreter::DeclId_t decl;
       if (fClass) decl = gInterpreter->GetDataMember(fClass->GetClassInfo(),name);
-      else        decl = gInterpreter->GetDataMember(0,name);
+      else        decl = gInterpreter->GetDataMember(nullptr,name);
       if (decl) result = const_cast<TListOfDataMembers*>(this)->Get(decl);
    }
    return result;
@@ -204,9 +204,9 @@ TObject *TListOfDataMembers::FindObject(const char *name) const
 
 TDictionary *TListOfDataMembers::Find(DeclId_t id) const
 {
-   if (!id) return 0;
+   if (!id) return nullptr;
 
-   return fIds ? (TDataMember*)fIds->GetValue((Long64_t)id) : 0;
+   return fIds ? (TDataMember*)fIds->GetValue((Long64_t)id) : nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -215,7 +215,7 @@ TDictionary *TListOfDataMembers::Find(DeclId_t id) const
 
 TDictionary *TListOfDataMembers::Get(DeclId_t id)
 {
-   if (!id) return 0;
+   if (!id) return nullptr;
 
    R__LOCKGUARD(gInterpreterMutex);
    //need the Find and possible Add to be one atomic operation
@@ -229,18 +229,18 @@ TDictionary *TListOfDataMembers::Get(DeclId_t id)
          // So this decl can not possibly be part of this class.
          // [In addition calling GetClassInfo would trigger a late parsing
          //  of the header which we want to avoid].
-         return 0;
+         return nullptr;
       }
-      if (!gInterpreter->ClassInfo_Contains(fClass->GetClassInfo(),id)) return 0;
+      if (!gInterpreter->ClassInfo_Contains(fClass->GetClassInfo(),id)) return nullptr;
    } else {
-      if (!gInterpreter->ClassInfo_Contains(0,id)) return 0;
+      if (!gInterpreter->ClassInfo_Contains(nullptr,id)) return nullptr;
    }
 
-   DataMemberInfo_t *info = gInterpreter->DataMemberInfo_Factory(id,fClass ? fClass->GetClassInfo() : 0);
+   DataMemberInfo_t *info = gInterpreter->DataMemberInfo_Factory(id,fClass ? fClass->GetClassInfo() : nullptr);
 
    // Let's see if this is a reload ...
    const char *name = gInterpreter->DataMemberInfo_Name(info);
-   TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(name) : 0;
+   TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(name) : nullptr;
    if (update) {
       if (fClass) {
          ((TDataMember*)update)->Update(info);
@@ -272,11 +272,11 @@ TDictionary *TListOfDataMembers::Get(DeclId_t id)
 
 TDictionary *TListOfDataMembers::Get(DataMemberInfo_t *info, bool skipChecks)
 {
-   if (!info) return 0;
+   if (!info) return nullptr;
 
    TDictionary::DeclId_t id = gInterpreter->GetDeclId(info);
-   R__ASSERT( id != 0 && "DeclId should not be null");
-   TDictionary *dm = fIds ? (TDataMember*)fIds->GetValue((Long64_t)id) : 0;
+   R__ASSERT( id != nullptr && "DeclId should not be null");
+   TDictionary *dm = fIds ? (TDataMember*)fIds->GetValue((Long64_t)id) : nullptr;
    if (!dm) {
       if (fClass) {
          if (!fClass->HasInterpreterInfoInMemory()) {
@@ -285,11 +285,11 @@ TDictionary *TListOfDataMembers::Get(DataMemberInfo_t *info, bool skipChecks)
             // So this decl can not possibly be part of this class.
             // [In addition calling GetClassInfo would trigger a late parsing
             //  of the header which we want to avoid].
-            return 0;
+            return nullptr;
          }
-         if (!skipChecks && !gInterpreter->ClassInfo_Contains(fClass->GetClassInfo(),id)) return 0;
+         if (!skipChecks && !gInterpreter->ClassInfo_Contains(fClass->GetClassInfo(),id)) return nullptr;
       } else {
-         if (!skipChecks && !gInterpreter->ClassInfo_Contains(0,id)) return 0;
+         if (!skipChecks && !gInterpreter->ClassInfo_Contains(nullptr,id)) return nullptr;
       }
 
       R__LOCKGUARD(gInterpreterMutex);
@@ -298,7 +298,7 @@ TDictionary *TListOfDataMembers::Get(DataMemberInfo_t *info, bool skipChecks)
 
       // Let's see if this is a reload ...
       const char *name = gInterpreter->DataMemberInfo_Name(info);
-      TDataMember *update = fUnloaded ? (TDataMember *)fUnloaded->FindObject(name) : 0;
+      TDataMember *update = fUnloaded ? (TDataMember *)fUnloaded->FindObject(name) : nullptr;
       if (update) {
          update->Update(dm_info);
          dm = update;
@@ -328,7 +328,7 @@ void TListOfDataMembers::UnmapObject(TObject* obj)
          if (d->GetDeclId()) {
             fIds->Remove((Long64_t)d->GetDeclId());
          }
-         d->Update(0);
+         d->Update(nullptr);
       }
    } else {
       TGlobal *g = dynamic_cast<TGlobal*>(obj);
@@ -336,7 +336,7 @@ void TListOfDataMembers::UnmapObject(TObject* obj)
          if (g->GetDeclId()) {
             fIds->Remove((Long64_t)g->GetDeclId());
          }
-         g->Update(0);
+         g->Update(nullptr);
       }
    }
 }
@@ -373,7 +373,7 @@ TObject* TListOfDataMembers::Remove(TObject *obj)
    }
    UnmapObject(obj);
    if (found) return obj;
-   else return 0;
+   else return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -381,7 +381,7 @@ TObject* TListOfDataMembers::Remove(TObject *obj)
 
 TObject* TListOfDataMembers::Remove(TObjLink *lnk)
 {
-   if (!lnk) return 0;
+   if (!lnk) return nullptr;
 
    TObject *obj = lnk->GetObject();
 
@@ -405,7 +405,7 @@ void TListOfDataMembers::Load()
    }
 
    // This will provoke the parsing of the headers if need be.
-   if (fClass && fClass->GetClassInfo() == 0) return;
+   if (fClass && fClass->GetClassInfo() == nullptr) return;
 
    R__LOCKGUARD(gInterpreterMutex);
 
@@ -502,7 +502,7 @@ void TListOfDataMembers::Update(TDictionary *member) {
             if (!fIds) fIds = new TExMap(idsSize);
             fIds->Add((Long64_t)d->GetDeclId(),(Long64_t)d);
          }
-         TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(d->GetName()) : 0;
+         TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(d->GetName()) : nullptr;
          if (update) fUnloaded->Remove(update);
 
          if (! THashList::FindObject(d) ) {
@@ -518,7 +518,7 @@ void TListOfDataMembers::Update(TDictionary *member) {
             if (!fIds) fIds = new TExMap(idsSize);
             fIds->Add((Long64_t)g->GetDeclId(),(Long64_t)g);
 
-            TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(g->GetName()) : 0;
+            TDictionary *update = fUnloaded ? (TDictionary *)fUnloaded->FindObject(g->GetName()) : nullptr;
             if (update) fUnloaded->Remove(update);
 
             if (! THashList::FindObject(g) ) {
