@@ -40,7 +40,7 @@
 
 #include <string>
 
-using namespace std;
+using std::string;
 
 const Int_t kMaxLen = 1024;
 
@@ -63,7 +63,7 @@ static TString ExtractClassName(const TString &type_name)
 
 static TStreamerBasicType *InitCounter(const char *countClass, const char *countName, TVirtualStreamerInfo *directive)
 {
-   TStreamerBasicType *counter = 0;
+   TStreamerBasicType *counter = nullptr;
 
    TClass *cl = TClass::GetClass(countClass);
 
@@ -73,28 +73,28 @@ static TStreamerBasicType *InitCounter(const char *countClass, const char *count
          // The info we have been passed is indeed describing the counter holder, just look there.
 
          TStreamerElement *element = (TStreamerElement *)directive->GetElements()->FindObject(countName);
-         if (!element) return 0;
-         if (element->IsA() != TStreamerBasicType::Class()) return 0;
+         if (!element) return nullptr;
+         if (element->IsA() != TStreamerBasicType::Class()) return nullptr;
          counter = (TStreamerBasicType*)element;
 
       } else {
          if (directive->GetClass()->GetListOfRealData()) {
             TRealData* rdCounter = (TRealData*) directive->GetClass()->GetListOfRealData()->FindObject(countName);
-            if (!rdCounter) return 0;
+            if (!rdCounter) return nullptr;
             TDataMember *dmCounter = rdCounter->GetDataMember();
             cl = dmCounter->GetClass();
          } else {
             TStreamerElement *element = (TStreamerElement *)directive->GetElements()->FindObject(countName);
-            if (!element) return 0;
-            if (element->IsA() != TStreamerBasicType::Class()) return 0;
+            if (!element) return nullptr;
+            if (element->IsA() != TStreamerBasicType::Class()) return nullptr;
             cl = directive->GetClass();
          }
-         if (cl==0) return 0;
+         if (cl==nullptr) return nullptr;
          counter = TVirtualStreamerInfo::GetElementCounter(countName,cl);
       }
    } else {
 
-      if (cl==0) return 0;
+      if (cl==nullptr) return nullptr;
       counter = TVirtualStreamerInfo::GetElementCounter(countName,cl);
    }
 
@@ -139,9 +139,9 @@ static void GetRange(const char *comments, Double_t &xmin, Double_t &xmax, Doubl
       if (!comma || comma >right) return;
    }
    //search if nbits is specified
-   const char *comma2 = 0;
+   const char *comma2 = nullptr;
    if (comma) comma2 = strstr(comma+1,",");
-   if (comma2 > right) comma2 = 0;
+   if (comma2 > right) comma2 = nullptr;
    Int_t nbits = 32;
    if (comma2) {
       TString sbits(comma2+1,right-comma2-1);
@@ -195,19 +195,21 @@ ClassImp(TStreamerElement);
 
 TStreamerElement::TStreamerElement()
 {
-   fType        = 0;
+   // clang-format off
+   fType        = TVirtualStreamerInfo::kUnset;
    fSize        = 0;
-   fNewType     = 0;
+   fNewType     = TVirtualStreamerInfo::kUnset;
    fArrayDim    = 0;
    fArrayLength = 0;
-   fStreamer    = 0;
+   fStreamer    = nullptr;
    fOffset      = 0;
    fClassObject = (TClass*)(-1);
-   fNewClass    = 0;
+   fNewClass    = nullptr;
    fTObjectOffset = 0;
    fFactor      = 0;
    fXmin        = 0;
    fXmax        = 0;
+   // clang-format on
    for (Int_t i=0;i<5;i++) fMaxIndex[i] = 0;
 }
 
@@ -231,9 +233,9 @@ TStreamerElement::TStreamerElement(const char *name, const char *title, Int_t of
       R__LOCKGUARD(gInterpreterMutex);
       fTypeName    = TClassEdit::ResolveTypedef(typeName);
    }
-   fStreamer    = 0;
+   fStreamer    = nullptr;
    fClassObject = (TClass*)(-1);
-   fNewClass    = 0;
+   fNewClass    = nullptr;
    fTObjectOffset = 0;
    fFactor      = 0;
    fXmin        = 0;
@@ -354,31 +356,18 @@ const char *TStreamerElement::GetFullName() const
 void TStreamerElement::GetSequenceType(TString &sequenceType) const
 {
    sequenceType.Clear();
-   Bool_t first = kTRUE;
-   if (TestBit(TStreamerElement::kWholeObject)) {
-      if (!first) sequenceType += ",";
-      first = kFALSE;
-      sequenceType += "wholeObject";
-   }
-   if (TestBit(TStreamerElement::kCache)) {
-      first = kFALSE;
-      sequenceType += "cached";
-   }
-   if (TestBit(TStreamerElement::kRepeat)) {
-      if (!first) sequenceType += ",";
-      first = kFALSE;
-      sequenceType += "repeat";
-   }
-   if (TestBit(TStreamerElement::kDoNotDelete)) {
-      if (!first) sequenceType += ",";
-      first = kFALSE;
-      sequenceType += "nodelete";
-   }
-   if (TestBit(TStreamerElement::kWrite)) {
-      if (!first) sequenceType += ",";
-      first = kFALSE;
-      sequenceType += "write";
-   }
+   auto test_bit = [this, &sequenceType](unsigned bit, const char *name) {
+      if (TestBit(bit)) {
+         if (!sequenceType.IsNull()) sequenceType += ",";
+         sequenceType += name;
+      }
+   };
+
+   test_bit(TStreamerElement::kWholeObject, "wholeObject");
+   test_bit(TStreamerElement::kCache, "cached");
+   test_bit(TStreamerElement::kRepeat, "repeat");
+   test_bit(TStreamerElement::kDoNotDelete, "nodelete");
+   test_bit(TStreamerElement::kWrite, "write");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -477,9 +466,12 @@ void TStreamerElement::ls(Option_t *) const
       sequenceType.Prepend(" (");
       sequenceType += ") ";
    }
-   printf("  %-14s %-15s offset=%3d type=%2d %s%-20s\n",
-          temp.Data(),GetFullName(),fOffset,fType,sequenceType.Data(),
-          GetTitle());
+   printf("  %-14s %-15s offset=%3d type=%2d",
+          temp.Data(),GetFullName(),fOffset,fType);
+   if (fType != fNewType && fNewType != TVirtualStreamerInfo::kUnset)
+      printf(" newtype=%2d", fNewType);
+   printf(" %s%-20s\n",
+          sequenceType.Data(), GetTitle());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -621,13 +613,13 @@ ClassImp(TStreamerBase);
 TStreamerBase::TStreamerBase() :
    // Abuse TStreamerElement data member that is not used by TStreamerBase
    fBaseCheckSum( *( (UInt_t*)&(fMaxIndex[1]) ) ),
-   fStreamerFunc(0), fConvStreamerFunc(0), fStreamerInfo(0)
+   fStreamerFunc(nullptr), fConvStreamerFunc(nullptr), fStreamerInfo(nullptr)
 {
    // Default ctor.
 
    fBaseClass = (TClass*)(-1);
    fBaseVersion = 0;
-   fNewBaseClass = 0;
+   fNewBaseClass = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -636,7 +628,7 @@ TStreamerBase::TStreamerBase(const char *name, const char *title, Int_t offset,
    : TStreamerElement(name,title,offset,TVirtualStreamerInfo::kBase,"BASE"),
      // Abuse TStreamerElement data member that is not used by TStreamerBase
      fBaseCheckSum( *( (UInt_t*)&(fMaxIndex[1]) ) ),
-     fStreamerFunc(0), fConvStreamerFunc(0), fStreamerInfo(0)
+     fStreamerFunc(nullptr), fConvStreamerFunc(nullptr), fStreamerInfo(nullptr)
 
 {
    // Create a TStreamerBase object.
@@ -655,7 +647,7 @@ TStreamerBase::TStreamerBase(const char *name, const char *title, Int_t offset,
    } else {
       fBaseVersion = 0;
    }
-   fNewBaseClass = 0;
+   fNewBaseClass = nullptr;
    Init(isTransient);
 }
 
@@ -724,9 +716,9 @@ void TStreamerBase::InitStreaming(Bool_t isTransient)
          fStreamerInfo = fBaseClass->FindStreamerInfo(fBaseCheckSum, isTransient);
       }
    } else {
-      fStreamerFunc = 0;
-      fConvStreamerFunc = 0;
-      fStreamerInfo = 0;
+      fStreamerFunc = nullptr;
+      fConvStreamerFunc = nullptr;
+      fStreamerInfo = nullptr;
    }
 }
 
@@ -822,7 +814,7 @@ void TStreamerBase::Streamer(TBuffer &R__b)
       // order (derived class,base class) and hence the base class is not
       // yet emulated.
       fBaseClass = (TClass*)-1;
-      fNewBaseClass = 0;
+      fNewBaseClass = nullptr;
       // Eventually we need a v3 that stores directly fBaseCheckSum (and
       // a version of TStreamerElement should not stored fMaxIndex)
       if (R__v > 2) {
@@ -914,9 +906,9 @@ ClassImp(TStreamerBasicPointer);
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor.
 
-TStreamerBasicPointer::TStreamerBasicPointer() : fCountVersion(0),fCountName(),fCountClass(),fCounter(0)
+TStreamerBasicPointer::TStreamerBasicPointer() : fCountVersion(0),fCountName(),fCountClass(),fCounter(nullptr)
 {
-   fCounter = 0;
+   fCounter = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -944,7 +936,7 @@ TStreamerBasicPointer::~TStreamerBasicPointer()
 ////////////////////////////////////////////////////////////////////////////////
 /// return offset of counter
 
-ULong_t TStreamerBasicPointer::GetMethod() const
+ULongptr_t TStreamerBasicPointer::GetMethod() const
 {
    if (!fCounter) ((TStreamerBasicPointer*)this)->Init();
    if (!fCounter) return 0;
@@ -953,7 +945,7 @@ ULong_t TStreamerBasicPointer::GetMethod() const
    // the left most (non virtual) base classes.  For the other we would
    // really need to use the object coming from the list of real data.
    // (and even that need analysis for virtual base class).
-   return (ULong_t)fCounter->GetOffset();
+   return (ULongptr_t)fCounter->GetOffset();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1027,7 +1019,7 @@ ClassImp(TStreamerLoop);
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor.
 
-TStreamerLoop::TStreamerLoop() : fCountVersion(0),fCountName(),fCountClass(),fCounter(0)
+TStreamerLoop::TStreamerLoop() : fCountVersion(0),fCountName(),fCountClass(),fCounter(nullptr)
 {
 }
 
@@ -1053,14 +1045,14 @@ TStreamerLoop::~TStreamerLoop()
 ////////////////////////////////////////////////////////////////////////////////
 /// return address of counter
 
-ULong_t TStreamerLoop::GetMethod() const
+ULongptr_t TStreamerLoop::GetMethod() const
 {
    //if (!fCounter) {
    //   Init();
    //   if (!fCounter) return 0;
    //}
    if (!fCounter) return 0;
-   return (ULong_t)fCounter->GetOffset();
+   return (ULongptr_t)fCounter->GetOffset();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1153,10 +1145,10 @@ TStreamerBasicType::~TStreamerBasicType()
 ////////////////////////////////////////////////////////////////////////////////
 /// return address of counter
 
-ULong_t TStreamerBasicType::GetMethod() const
+ULongptr_t TStreamerBasicType::GetMethod() const
 {
    if (fType ==  TVirtualStreamerInfo::kCounter ||
-       fType == (TVirtualStreamerInfo::kCounter+TVirtualStreamerInfo::kSkip)) return (ULong_t)&fCounter;
+       fType == (TVirtualStreamerInfo::kCounter+TVirtualStreamerInfo::kSkip)) return (ULongptr_t)&fCounter;
    return 0;
 }
 
@@ -1736,7 +1728,10 @@ TStreamerSTL::TStreamerSTL(const char *name, const char *title, Int_t offset,
    const char *t = trueType;
    if (!t || !*t) t = typeName;
 
-   fTypeName = TClassEdit::ShortType(fTypeName,TClassEdit::kDropStlDefault).c_str();
+   std::string answer;
+   TClassEdit::TSplitType arglist(t, TClassEdit::kDropStlDefault);
+   arglist.ShortType(answer, TClassEdit::kDropStlDefault);
+   fTypeName = answer;
 
    if (name==typeName /* intentional pointer comparison */
        || strcmp(name,typeName)==0) {
@@ -1744,56 +1739,31 @@ TStreamerSTL::TStreamerSTL(const char *name, const char *title, Int_t offset,
       fName = fTypeName;
    }
 
-   Int_t nch = strlen(t);
-   char *s = new char[nch+1];
-   strlcpy(s,t,nch+1);
-   char *sopen  = strchr(s,'<');
-   if (sopen == 0) {
-      Fatal("TStreamerSTL","For %s, the type name (%s) is seemingly not a template (template argument not found)", name, s);
+   if (arglist.fElements.size() < 2) {
+      Fatal("TStreamerSTL","For %s, the type name (%s) is seemingly not a template (template argument not found)", name, t);
       return;
    }
-   *sopen  = 0; sopen++;
-   // We are looking for the first arguments of the STL container, because
-   // this arguments can be a templates we need to count the < and >
-   char* current=sopen;
-   for(int count = 0; *current!='\0'; current++) {
-      if (*current=='<') count++;
-      if (*current=='>') {
-         if (count==0) break;
-         count--;
-      }
-      if (*current==',' && count==0) break;
+
+   const std::string& inside_type = arglist.fElements[1];
+   std::string inside = (inside_type.find("const ")==0) ? inside_type.substr(6) : inside_type;
+
+   // Let's treat the unique_ptr case
+   std::string intype = TClassEdit::GetNameForIO(inside, TClassEdit::EModType::kNone);
+
+   bool isPointer = false;
+   // The incoming name is normalized (it comes from splitting the name of a TClass),
+   // so all we need to do is drop the last trailing star (if any) and record that information.
+   while (intype.back() == '*') {
+      isPointer = true;
+      intype.pop_back();
    }
-   char *sclose = current; *sclose = 0; sclose--;
-   char *sconst = strstr(sopen,"const ");
-   char *sbracket = strstr(sopen,"<");
-   if (sconst && (sbracket==0 || sconst < sbracket)) {
-      // the string "const" may be part of the classname!
-      char *pconst = sconst-1;
-      if (*pconst == ' ' || *pconst == '<' || *pconst == '*' || *pconst == '\0') sopen = sconst + 5;
-   }
-   fSTLtype = TClassEdit::STLKind(s);
+
+   fSTLtype = TClassEdit::STLKind(arglist.fElements[0]);
    fCtype   = 0;
-   if (fSTLtype == ROOT::kNotSTL) { delete [] s; return;}
+   if (fSTLtype == ROOT::kNotSTL) { return; }
    if (dmPointer) fSTLtype += TVirtualStreamerInfo::kOffsetP;
 
-   // find STL contained type
-   while (*sopen==' ') sopen++;
-   Bool_t isPointer = kFALSE;
-   // Find stars outside of any template definitions in the
-   // first template argument.
-   char *star = strrchr(sopen,'>');
-   if (star) star = strchr(star,'*');
-   else star = strchr(sopen,'*');
-   if (star) {
-      isPointer = kTRUE;
-      *star = 0;
-      sclose = star - 1;
-   }
-   while (*sclose == ' ') {*sclose = 0; sclose--;}
-
-
-   TDataType *dt = (TDataType*)gROOT->GetListOfTypes()->FindObject(sopen);
+   TDataType *dt = (TDataType*)gROOT->GetListOfTypes()->FindObject(intype.c_str());
    if (fSTLtype == ROOT::kSTLbitset) {
       // Nothing to check
    } else if (dt) {
@@ -1801,28 +1771,28 @@ TStreamerSTL::TStreamerSTL(const char *name, const char *title, Int_t offset,
       if (isPointer) fCtype += TVirtualStreamerInfo::kOffsetP;
    } else {
      // this could also be a nested enums ... which should work ... be let's see.
-      TClass *cl = TClass::GetClass(sopen);
+      TClass *cl = TClass::GetClass(intype.c_str());
       if (cl) {
          if (isPointer) fCtype = TVirtualStreamerInfo::kObjectp;
          else           fCtype = TVirtualStreamerInfo::kObject;
       } else {
-         if (gCling->ClassInfo_IsEnum(sopen)) {
+         if (gCling->ClassInfo_IsEnum(intype.c_str())) {
             if (isPointer) fCtype += TVirtualStreamerInfo::kOffsetP;
          } else {
-            if(strcmp(sopen,"string")) {
+            if (intype != "string") {
                // This case can happens when 'this' is a TStreamerElement for
                // a STL container containing something for which we do not have
                // a TVirtualStreamerInfo (This happens in particular is the collection
                // objects themselves are always empty) and we do not have the
                // dictionary/shared library for the container.
                if (GetClassPointer() && GetClassPointer()->IsLoaded()) {
-                  Warning("TStreamerSTL","For %s we could not find any information about the type %s %d %s",fTypeName.Data(),sopen,fSTLtype,s);
+                  Warning("TStreamerSTL", "For %s we could not find any information about the type %s %d %s",
+                          fTypeName.Data(), arglist.fElements[1].c_str(), fSTLtype, arglist.fElements[0].c_str());
                }
             }
          }
       }
    }
-   delete [] s;
 
    if (TStreamerSTL::IsaPointer()) fType = TVirtualStreamerInfo::kSTLp;
 }
@@ -1884,7 +1854,7 @@ Int_t TStreamerSTL::GetSize() const
    // current size!
    TClass *cl = GetClassPointer();
    UInt_t size = 0;
-   if (cl==0) {
+   if (cl==nullptr) {
       if (!TestBit(kWarned)) {
          Error("GetSize","Could not find the TClass for %s.\n"
                "This is likely to have been a typedef, if possible please declare it in CINT to work around the issue\n",fTypeName.Data());
@@ -1916,9 +1886,12 @@ void TStreamerSTL::ls(Option_t *) const
       sequenceType.Prepend(" (");
       sequenceType += ") ";
    }
-   printf("  %-14s %-15s offset=%3d type=%2d %s,stl=%d, ctype=%d, %-20s\n",
-          GetTypeName(),name.Data(),fOffset,fType,sequenceType.Data(),
-          fSTLtype,fCtype,GetTitle());
+   printf("  %-14s %-15s offset=%3d type=%2d",
+          GetTypeName(), name.Data(), fOffset, fType);
+   if (fType != fNewType && fNewType != TVirtualStreamerInfo::kUnset)
+      printf(" newtype=%2d", fNewType);
+   printf(" stl=%d ctype=%d %s%-20s\n",
+          fSTLtype, fCtype, sequenceType.Data(), GetTitle());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
