@@ -33,6 +33,7 @@
 #include "clang/Sema/Scope.h"
 #include "clang/Serialization/ASTReader.h"
 #include "clang/Serialization/GlobalModuleIndex.h"
+#include "clang/Basic/DiagnosticSema.h"
 
 #include "llvm/ExecutionEngine/Orc/Core.h"
 
@@ -44,6 +45,8 @@
 #include "TClingUtils.h"
 #include "ClingRAII.h"
 
+#include <optional>
+
 using namespace clang;
 using namespace cling;
 using namespace ROOT::Internal;
@@ -87,7 +90,7 @@ class AutoloadLibraryMU : public llvm::orc::MaterializationUnit {
    llvm::orc::SymbolNameVector fSymbols;
 public:
    AutoloadLibraryMU(const TClingCallbacks &cb, const std::string &Library, const llvm::orc::SymbolNameVector &Symbols)
-      : MaterializationUnit(getSymbolFlagsMap(Symbols), nullptr), fCallbacks(cb), fLibrary(Library), fSymbols(Symbols)
+      : MaterializationUnit({getSymbolFlagsMap(Symbols), nullptr}), fCallbacks(cb), fLibrary(Library), fSymbols(Symbols)
    {
    }
 
@@ -125,8 +128,7 @@ public:
          }
 
          if (addr) {
-            loadedSymbols[symbol] =
-               llvm::JITEvaluatedSymbol(llvm::pointerToJITTargetAddress(addr), llvm::JITSymbolFlags::Exported);
+            loadedSymbols[symbol] = {llvm::orc::ExecutorAddr::fromPtr(addr), llvm::JITSymbolFlags::Exported};
          } else {
             // Collect all failing symbols, delegate their responsibility and then
             // fail their materialization. R->defineNonExistent() sounds like it
@@ -179,7 +181,6 @@ public:
       // provides the symbol and create one MaterializationUnit per library to
       // actually load it if needed.
       std::unordered_map<std::string, llvm::orc::SymbolNameVector> found;
-      llvm::orc::SymbolNameSet missing;
 
       // TODO: Do we need to take gInterpreterMutex?
       // R__LOCKGUARD(gInterpreterMutex);
@@ -201,9 +202,7 @@ public:
          // is made available as argument to `CreateInterpreter`.
          assert(libName.find("/libCling.") == std::string::npos && "Must not autoload libCling!");
 
-         if (libName.empty())
-            missing.insert(name);
-         else
+         if (!libName.empty())
             found[libName].push_back(name);
       }
 
@@ -213,9 +212,6 @@ public:
             return Err;
       }
 
-      if (!missing.empty())
-         return llvm::make_error<llvm::orc::SymbolsNotFound>(std::move(missing));
-
       return llvm::Error::success();
    }
 };
@@ -223,7 +219,7 @@ public:
 TClingCallbacks::TClingCallbacks(cling::Interpreter *interp, bool hasCodeGen) : InterpreterCallbacks(interp)
 {
    if (hasCodeGen) {
-      Transaction* T = 0;
+      Transaction* T = nullptr;
       m_Interpreter->declare("namespace __ROOT_SpecialObjects{}", &T);
       fROOTSpecialNamespace = dyn_cast<NamespaceDecl>(T->getFirstDecl().getSingleDecl());
 
@@ -239,7 +235,7 @@ void TClingCallbacks::InclusionDirective(clang::SourceLocation sLoc/*HashLoc*/,
                                          llvm::StringRef FileName,
                                          bool /*IsAngled*/,
                                          clang::CharSourceRange /*FilenameRange*/,
-                                         const clang::FileEntry *FE,
+                                         clang::OptionalFileEntryRef FE,
                                          llvm::StringRef /*SearchPath*/,
                                          llvm::StringRef /*RelativePath*/,
                                          const clang::Module * Imported,
@@ -287,9 +283,8 @@ bool TClingCallbacks::LibraryLoadingFailed(const std::string& errmessage, const
 // Preprocessor callbacks used to handle special cases like for example:
 // #include "myMacro.C+"
 //
-bool TClingCallbacks::FileNotFound(llvm::StringRef FileName,
-                                   llvm::SmallVectorImpl<char> &RecoveryPath) {
-   // Method called via Callbacks->FileNotFound(Filename, RecoveryPath)
+bool TClingCallbacks::FileNotFound(llvm::StringRef FileName) {
+   // Method called via Callbacks->FileNotFound(Filename)
    // in Preprocessor::HandleIncludeDirective(), initially allowing to
    // change the include path, and allowing us to compile code via ACLiC
    // when specifying #include "myfile.C+", and suppressing the preprocessor
@@ -338,22 +333,12 @@ bool TClingCallbacks::FileNotFound(llvm::StringRef FileName,
                                                 SemaR.TUScope);
          int retcode = TCling__CompileMacro(fname.c_str(), options.c_str());
          if (retcode) {
-            // compilation was successful, let's remember the original
-            // preprocessor "include not found" error suppression flag
-            if (!fPPChanged)
-               fPPOldFlag = PP.GetSuppressIncludeNotFoundError();
-            PP.SetSuppressIncludeNotFoundError(true);
-            fPPChanged = true;
+            // compilation was successful, tell the preprocess to silently
+            // skip the file
+            return true;
          }
-         return false;
       }
    }
-   if (fPPChanged) {
-      // restore the original preprocessor "include not found" error
-      // suppression flag
-      PP.SetSuppressIncludeNotFoundError(fPPOldFlag);
-      fPPChanged = false;
-   }
    return false;
 }
 
@@ -432,8 +417,8 @@ bool TClingCallbacks::LookupObject(LookupResult &R, Scope *S) {
 
 bool TClingCallbacks::findInGlobalModuleIndex(DeclarationName Name, bool loadFirstMatchOnly /*=true*/)
 {
-   llvm::Optional<std::string> envUseGMI = llvm::sys::Process::GetEnv("ROOT_USE_GMI");
-   if (envUseGMI.hasValue())
+   std::optional<std::string> envUseGMI = llvm::sys::Process::GetEnv("ROOT_USE_GMI");
+   if (envUseGMI.has_value())
       if (!envUseGMI->empty() && !ROOT::FoundationUtils::ConvertEnvValueToBool(*envUseGMI))
          return false;
 
@@ -556,8 +541,7 @@ bool TClingCallbacks::LookupObject(const DeclContext* DC, DeclarationName Name)
       llvm::SmallVector<NamedDecl*, 4> lookupResults;
       for(LookupResult::iterator I = R.begin(), E = R.end(); I < E; ++I)
          lookupResults.push_back(*I);
-      UpdateWithNewDecls(DC, Name, llvm::makeArrayRef(lookupResults.data(),
-                                                      lookupResults.size()));
+      UpdateWithNewDecls(DC, Name, llvm::ArrayRef(lookupResults.data(), lookupResults.size()));
       return true;
    }
    return false;
@@ -602,7 +586,7 @@ bool TClingCallbacks::LookupObject(clang::TagDecl* Tag) {
 
       // Use the Normalized name for the autoload
       std::string Name;
-      const ROOT::TMetaUtils::TNormalizedCtxt* tNormCtxt = NULL;
+      const ROOT::TMetaUtils::TNormalizedCtxt* tNormCtxt = nullptr;
       TCling__GetNormalizedContext(tNormCtxt);
       ROOT::TMetaUtils::GetNormalizedName(Name,
                                           C.getTypeDeclType(RD),
@@ -627,7 +611,7 @@ bool TClingCallbacks::LookupObject(clang::TagDecl* Tag) {
 // filename.
 //
 bool TClingCallbacks::tryAutoParseInternal(llvm::StringRef Name, LookupResult &R,
-                                           Scope *S, const FileEntry* FE /*=0*/) {
+                                           Scope *S, clang::OptionalFileEntryRef FE) {
    if (!fROOTSpecialNamespace) {
       // init error or rootcling
       return false;
@@ -791,7 +775,7 @@ bool TClingCallbacks::tryFindROOTSpecialInternal(LookupResult &R, Scope *S) {
 
          VD = VarDecl::Create(C, fROOTSpecialNamespace, SourceLocation(),
                               SourceLocation(), Name.getAsIdentifierInfo(), QT,
-                              /*TypeSourceInfo*/0, SC_None);
+                              /*TypeSourceInfo*/nullptr, SC_None);
          // Build an initializer
          Expr* Init
            = utils::Synthesize::CStyleCastPtrExpr(&SemaR, QT, (uint64_t)obj);
@@ -861,8 +845,14 @@ bool TClingCallbacks::tryResolveAtRuntimeInternal(LookupResult &R, Scope *S) {
       return false;
    }
 
+   // Prevent redundant declarations for control statements (e.g., for, if, while)
+   // that have already been annotated.
+   if (auto annot = Wrapper->getAttr<AnnotateAttr>())
+      if (annot->getAnnotation().equals("__ResolveAtRuntime") && S->isControlScope())
+         return false;
+
    VarDecl* Result = VarDecl::Create(C, TU, Loc, Loc, II, C.DependentTy,
-                                     /*TypeSourceInfo*/0, SC_None);
+                                     /*TypeSourceInfo*/nullptr, SC_None);
 
    if (!Result) {
       // We cannot handle the situation. Give up
@@ -873,7 +863,7 @@ bool TClingCallbacks::tryResolveAtRuntimeInternal(LookupResult &R, Scope *S) {
    // is a gross hack, because TClingCallbacks shouldn't know about
    // EvaluateTSynthesizer at all!
 
-   Wrapper->addAttr(AnnotateAttr::CreateImplicit(C, "__ResolveAtRuntime"));
+   Wrapper->addAttr(AnnotateAttr::CreateImplicit(C, "__ResolveAtRuntime", nullptr, 0));
 
    // Here we have the scope but we cannot do Sema::PushDeclContext, because
    // on pop it will try to go one level up, which we don't want.
@@ -999,7 +989,7 @@ bool TClingCallbacks::tryInjectImplicitAutoKeyword(LookupResult &R, Scope *S) {
                                      C.getAutoType(QualType(),
                                                    clang::AutoTypeKeyword::Auto,
                                                    /*IsDependent*/false),
-                                     /*TypeSourceInfo*/0, SC_None);
+                                     /*TypeSourceInfo*/nullptr, SC_None);
 
    if (!Result) {
       ROOT::TMetaUtils::Error("TClingCallbacks::tryInjectImplicitAutoKeyword",
@@ -1010,9 +1000,15 @@ bool TClingCallbacks::tryInjectImplicitAutoKeyword(LookupResult &R, Scope *S) {
    // Annotate the decl to give a hint in cling.
    // FIXME: We should move this in cling, when we implement turning it on
    // and off.
-   Result->addAttr(AnnotateAttr::CreateImplicit(C, "__Auto"));
+   Result->addAttr(AnnotateAttr::CreateImplicit(C, "__Auto", nullptr, 0));
 
    R.addDecl(Result);
+
+   // Raise a warning when trying to use implicit auto injection feature.
+   SemaRef.getDiagnostics().setSeverity(diag::warn_deprecated_message, diag::Severity::Warning, SourceLocation());
+   SemaRef.Diag(Loc, diag::warn_deprecated_message)
+      << "declaration without the 'auto' keyword" << DC << Loc << FixItHint::CreateInsertion(Loc, "auto ");
+
    // Say that we can handle the situation. Clang should try to recover
    return true;
 }
@@ -1061,19 +1057,6 @@ void TClingCallbacks::DefinitionShadowed(const clang::NamedDecl *D) {
    TCling__InvalidateGlobal(D);
 }
 
-void TClingCallbacks::DeclDeserialized(const clang::Decl* D) {
-   if (const RecordDecl* RD = dyn_cast<RecordDecl>(D)) {
-      // FIXME: Our AutoLoading doesn't work (load the library) when the looked
-      // up decl is found in the PCH/PCM. We have to do that extra step, which
-      // loads the corresponding library when a decl was deserialized.
-      //
-      // Unfortunately we cannot do that with the current implementation,
-      // because the library load will pull in the header files of the library
-      // as well, even though they are in the PCH/PCM and available.
-      (void)RD;//TCling__AutoLoadCallback(RD->getNameAsString().c_str());
-   }
-}
-
 void TClingCallbacks::LibraryLoaded(const void* dyLibHandle,
                                     llvm::StringRef canonicalName) {
    TCling__LibraryLoadedRTTI(dyLibHandle, canonicalName);
