@@ -81,7 +81,7 @@
 #include <stdarg.h>
 #include <memory>
 
-using namespace std;
+using std::string, std::ios_base, std::unique_ptr;
 
 // Auxiliary functions
 void   FilterClass();
@@ -155,7 +155,7 @@ int main(int argc, char *argv[])
    ReplaceAll(gSourceDir,"\"","");
 
    // Retrieve the python executable
-   gPythonExec = getenv("PYTHON_EXECUTABLE");
+   gPythonExec = getenv("Python3_EXECUTABLE");
    ReplaceAll(gPythonExec,"\"","");
 
    // Open the input file name.
@@ -327,6 +327,15 @@ void FilterTutorial()
    gMacroName  = gFileName.substr(i1,i2-i1+1);
    gImageName  = StringFormat("%s.%s", gMacroName.c_str(), gImageType.c_str()); // Image name
    gOutputName = StringFormat("%s.out", gMacroName.c_str()); // output name
+   if (gPython) {
+      FILE *cn = fopen("CleanNamespaces.sh", "a");
+      string name = gMacroName;
+      ReplaceAll(name,".py","");
+      ReplaceAll(name,"_","__");
+      if (cn)
+         fprintf(cn,"./modifyNamespacesWebpage.sh %s\n",name.c_str());
+      fclose(cn);
+   }
 
    // Parse the source and generate the image if needed
    while (fgets(gLine,255,f)) {
@@ -336,10 +345,25 @@ void FilterTutorial()
       if (gLineString.find("\\macro_image") != string::npos) {
          bool nobatch = (gLineString.find("(nobatch)") != string::npos);
          ReplaceAll(gLineString,"(nobatch)","");
-         bool tcanvas_js = (gLineString.find("(tcanvas_js)") != string::npos);
-         ReplaceAll(gLineString,"(tcanvas_js)","");
-         bool rcanvas_js = (gLineString.find("(rcanvas_js)") != string::npos);
-         ReplaceAll(gLineString,"(rcanvas_js)","");
+         bool tcanvas_js = false, rcanvas_js = false;
+         int tcanvas_aclic = 0;
+
+         if (gLineString.find("(tcanvas_js)") != string::npos) {
+            tcanvas_js = true;
+            tcanvas_aclic = 0;
+            ReplaceAll(gLineString,"(tcanvas_js)", "");
+         }
+
+         if (gLineString.find("(tcanvas_jsp)") != string::npos) {
+            tcanvas_js = true;
+            tcanvas_aclic = 1;
+            ReplaceAll(gLineString,"(tcanvas_jsp)", "");
+         }
+
+         if (gLineString.find("(rcanvas_js)") != string::npos) {
+            rcanvas_js = true;
+            ReplaceAll(gLineString,"(rcanvas_js)", "");
+         }
 
          bool image_created_by_macro = (gLineString.find(".png)") != string::npos) ||
                                        (gLineString.find(".svg)") != string::npos) ||
@@ -362,8 +386,8 @@ void FilterTutorial()
             IN = gImageName;
             int i = IN.find(".");
             IN.erase(i,IN.length());
-            ExecuteCommand(StringFormat("root -l -b -q \"MakeTCanvasJS.C(\\\"%s\\\",\\\"%s\\\",\\\"%s\\\",false,%d)\"",
-                                         gFileName.c_str(), IN.c_str(), gOutDir.c_str(), gPython));
+            ExecuteCommand(StringFormat("root -l -b -q \"MakeTCanvasJS.C(\\\"%s\\\",\\\"%s\\\",\\\"%s\\\",false,%d,%d)\"",
+                                         gFileName.c_str(), IN.c_str(), gOutDir.c_str(), gPython, tcanvas_aclic));
             ReplaceAll(gLineString, "macro_image", StringFormat("htmlinclude %s.html",IN.c_str()));
          } else if (rcanvas_js) {
             string IN;
