@@ -158,7 +158,7 @@ are interpreted and which is their result inside specific shape classes.
 
 ClassImp(TGeoShape);
 
-TGeoMatrix *TGeoShape::fgTransform = NULL;
+TGeoMatrix *TGeoShape::fgTransform = nullptr;
 Double_t TGeoShape::fgEpsMch = 2.220446049250313e-16;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -686,7 +686,7 @@ void TGeoShape::FillBuffer3D(TBuffer3D &buffer, Int_t reqSections, Bool_t localF
 
       // Set up local -> master translation matrix
       if (localFrame) {
-         TGeoMatrix *localMasterMat = 0;
+         TGeoMatrix *localMasterMat = nullptr;
          if (TGeoShape::GetTransform()) {
             localMasterMat = TGeoShape::GetTransform();
          } else {
