@@ -525,8 +525,8 @@ TGeoTrd2::Divide(TGeoVolume *voldiv, const char *divname, Int_t iaxis, Int_t ndi
    Int_t id;
    Double_t end = start + ndiv * step;
    switch (iaxis) {
-   case 1: Warning("Divide", "dividing a Trd2 on X not implemented"); return 0;
-   case 2: Warning("Divide", "dividing a Trd2 on Y not implemented"); return 0;
+   case 1: Warning("Divide", "dividing a Trd2 on X not implemented"); return nullptr;
+   case 2: Warning("Divide", "dividing a Trd2 on Y not implemented"); return nullptr;
    case 3:
       finder = new TGeoPatternZ(voldiv, ndiv, start, end);
       vmulti = gGeoManager->MakeVolumeMulti(divname, voldiv->GetMedium());
@@ -547,7 +547,7 @@ TGeoTrd2::Divide(TGeoVolume *voldiv, const char *divname, Int_t iaxis, Int_t ndi
          ((TGeoNodeOffset *)voldiv->GetNodes()->At(voldiv->GetNdaughters() - 1))->SetFinder(finder);
       }
       return vmulti;
-   default: Error("Divide", "Wrong axis type for division"); return 0;
+   default: Error("Divide", "Wrong axis type for division"); return nullptr;
    }
 }
 
@@ -624,10 +624,10 @@ Int_t TGeoTrd2::GetFittingBox(const TGeoBBox *parambox, TGeoMatrix *mat, Double_
 TGeoShape *TGeoTrd2::GetMakeRuntimeShape(TGeoShape *mother, TGeoMatrix * /*mat*/) const
 {
    if (!TestShapeBit(kGeoRunTimeShape))
-      return 0;
+      return nullptr;
    if (!mother->TestShapeBit(kGeoTrd2)) {
       Error("GetMakeRuntimeShape", "invalid mother");
-      return 0;
+      return nullptr;
    }
    Double_t dx1, dx2, dy1, dy2, dz;
    if (fDx1 < 0)
