@@ -42,7 +42,7 @@ enum ETGeoEltuWid { kELTU_NAME, kELTU_A, kELTU_B, kELTU_DZ, kELTU_APPLY, kELTU_U
 TGeoEltuEditor::TGeoEltuEditor(const TGWindow *p, Int_t width, Int_t height, UInt_t options, Pixel_t back)
    : TGeoGedFrame(p, width, height, options | kVerticalFrame, back)
 {
-   fShape = 0;
+   fShape = nullptr;
    fAi = fBi = fDzi = 0.0;
    fNamei = "";
    fIsModified = kFALSE;
@@ -148,7 +148,7 @@ void TGeoEltuEditor::ConnectSignals2Slots()
 
 void TGeoEltuEditor::SetModel(TObject *obj)
 {
-   if (obj == 0 || (obj->IsA() != TGeoEltu::Class())) {
+   if (obj == nullptr || (obj->IsA() != TGeoEltu::Class())) {
       SetActive(kFALSE);
       return;
    }
