@@ -64,8 +64,11 @@ const Size_t kDefaultCanvasSize   = 20;
 ClassImpQ(TCanvas)
 
 
-auto GetNewCanvasName()
+TString GetNewCanvasName(const char *arg = nullptr)
 {
+   if (arg && *arg)
+      return arg;
+
    const char *defcanvas = gROOT->GetDefCanvasName();
    TString cdef = defcanvas;
 
@@ -176,7 +179,7 @@ TCanvas::TCanvas(Bool_t build) : TPad(), fDoubleBuffer(0)
    if (!build || TClass::IsCallingNew() != TClass::kRealNew) {
       Constructor();
    } else {
-      TString cdef = GetNewCanvasName();
+      auto cdef = GetNewCanvasName();
 
       Constructor(cdef.Data(), cdef.Data(), 1);
    }
@@ -247,7 +250,7 @@ TCanvas::TCanvas(const char *name, Int_t ww, Int_t wh, Int_t winid) : TPad(), fD
    if (!fCanvasImp) return;
 
    CreatePainter();
-   SetName(name);
+   fName = GetNewCanvasName(name); // avoid Modified() signal from SetName
    Build();
 }
 
@@ -292,10 +295,10 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t form)
    }
 
    Init();
-   SetBit(kMenuBar,1);
+   SetBit(kMenuBar,true);
    if (form < 0) {
       form     = -form;
-      SetBit(kMenuBar,0);
+      SetBit(kMenuBar,false);
    }
 
    fCanvas = this;
@@ -347,7 +350,7 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t form)
 
    CreatePainter();
 
-   SetName(name);
+   fName = GetNewCanvasName(name); // avoid Modified() signal from SetName
    SetTitle(title); // requires fCanvasImp set
    Build();
 
@@ -392,10 +395,10 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t ww, Int_t w
    }
 
    Init();
-   SetBit(kMenuBar,1);
+   SetBit(kMenuBar,true);
    if (ww < 0) {
       ww       = -ww;
-      SetBit(kMenuBar,0);
+      SetBit(kMenuBar,false);
    }
    if (wh <= 0) {
       Error("Constructor", "Invalid canvas height: %d",wh);
@@ -433,7 +436,7 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t ww, Int_t w
 
    CreatePainter();
 
-   SetName(name);
+   fName = GetNewCanvasName(name); // avoid Modified() signal from SetName
    SetTitle(title); // requires fCanvasImp set
    Build();
 
@@ -483,10 +486,10 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t wtopx,
    }
 
    Init();
-   SetBit(kMenuBar,1);
+   SetBit(kMenuBar,true);
    if (wtopx < 0) {
       wtopx    = -wtopx;
-      SetBit(kMenuBar,0);
+      SetBit(kMenuBar,false);
    }
    fCw       = ww;
    fCh       = wh;
@@ -520,7 +523,7 @@ void TCanvas::Constructor(const char *name, const char *title, Int_t wtopx,
 
    CreatePainter();
 
-   SetName(name);
+   fName = GetNewCanvasName(name); // avoid Modified() signal from SetName
    SetTitle(title); // requires fCanvasImp set
    Build();
 
@@ -957,8 +960,7 @@ TObject *TCanvas::DrawClonePad()
    TIter next(GetListOfPrimitives());
    while (auto obj = next()) {
       pad->cd();
-      auto clone = obj->Clone();
-      pad->GetListOfPrimitives()->Add(clone, next.GetOption());
+      pad->Add(obj->Clone(), next.GetOption(), kFALSE); // do not issue modified for each object
    }
    pad->ResizePad();
    pad->Modified();
@@ -993,14 +995,14 @@ void TCanvas::DrawEventStatus(Int_t event, Int_t px, Int_t py, TObject *selected
    fCanvasImp->SetStatusText(atext,2);
 
    // Show date/time if TimeDisplay is selected
-   TAxis *xaxis = NULL;
+   TAxis *xaxis = nullptr;
    if ( selected->InheritsFrom("TH1") )
       xaxis = ((TH1*)selected)->GetXaxis();
    else if ( selected->InheritsFrom("TGraph") )
       xaxis = ((TGraph*)selected)->GetXaxis();
    else if ( selected->InheritsFrom("TAxis") )
       xaxis = (TAxis*)selected;
-   if ( xaxis != NULL && xaxis->GetTimeDisplay()) {
+   if ( xaxis != nullptr && xaxis->GetTimeDisplay()) {
       TString objinfo = selected->GetObjectInfo(px,py);
       // check if user has overwritten GetObjectInfo and altered
       // the default text from TObject::GetObjectInfo "x=.. y=.."
@@ -1513,9 +1515,9 @@ void TCanvas::ls(Option_t *option) const
 
 TCanvas *TCanvas::MakeDefCanvas()
 {
-   TString cdef = GetNewCanvasName();
+   auto cdef = GetNewCanvasName();
 
-   TCanvas *c = new TCanvas(cdef.Data(), cdef.Data(), 1);
+   auto c = new TCanvas(cdef.Data(), cdef.Data(), 1);
 
    ::Info("TCanvas::MakeDefCanvas"," created default TCanvas with name %s", cdef.Data());
    return c;
@@ -1823,23 +1825,25 @@ void TCanvas::SaveSource(const char *filename, Option_t * /*option*/)
    gROOT->ResetClassSaved();
 
    char quote = '"';
-   std::ofstream out;
-   TString fname;
-   const char *cname = GetName();
+   TString cname0 = GetName();
    Bool_t invalid = kFALSE;
-   //    if filename is given, open this file, otherwise create a file
-   //    with a name equal to the canvasname.C
-   if (filename && (strlen(filename) > 0)) {
+
+   TString cname = cname0.Strip(TString::kBoth);
+   if (cname.IsNull()) {
+      invalid = kTRUE;
+      cname = "c1";
+   }
+
+   //  if filename is given, open this file, otherwise create a file
+   //  with a name equal to the canvasname.C
+   TString fname;
+   if (filename && *filename) {
       fname = filename;
    } else {
-      fname = cname;
-      fname = fname.Strip(TString::kBoth);
-      if (fname.IsNull()) {
-         invalid = kTRUE;
-         fname = "c1";
-      }
-      fname.Append(".C");
+      fname = cname + ".C";
    }
+
+   std::ofstream out;
    out.open(fname.Data(), std::ios::out);
    if (!out.good()) {
       Error("SaveSource", "Cannot open file: %s", fname.Data());
@@ -1931,7 +1935,7 @@ void TCanvas::SaveSource(const char *filename, Option_t * /*option*/)
 
    //   Now recursively scan all pads of this canvas
    cd();
-   if (invalid) SetName("c1");
+   if (invalid) fName = cname;
    TPad::SavePrimitive(out,"toplevel");
 
    //   Write canvas options related to pad editor
@@ -1939,7 +1943,7 @@ void TCanvas::SaveSource(const char *filename, Option_t * /*option*/)
    if (GetShowToolBar()) {
       out<<"   "<<GetName()<<"->ToggleToolBar();"<<std::endl;
    }
-   if (invalid) SetName(" ");
+   if (invalid) fName = cname0;
 
    out <<"}"<<std::endl;
    out.close();
@@ -2041,16 +2045,13 @@ void TCanvas::SetFolder(Bool_t isfolder)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Set canvas name. In case `name` is an empty string, a default name is set.
+/// Canvas automatically marked as modified when SetName method called
 
 void TCanvas::SetName(const char *name)
 {
-   if (name && *name)
-      fName = name;
-   else
-      fName = GetNewCanvasName();
+   fName = GetNewCanvasName(name);
 
-   if (gPad && TestBit(kMustCleanup))
-      gPad->Modified();
+   Modified();
 }
 
 
@@ -2232,38 +2233,51 @@ void TCanvas::Streamer(TBuffer &b)
       TPad::Streamer(b);
       gPad    = this;
       //restore the colors
-      TObjArray *colors = (TObjArray*)fPrimitives->FindObject("ListOfColors");
+      auto colors = dynamic_cast<TObjArray *>(fPrimitives->FindObject("ListOfColors"));
       if (colors) {
+         auto root_colors = dynamic_cast<TObjArray *>(gROOT->GetListOfColors());
+
          TIter next(colors);
-         TColor *colold;
-         while ((colold = (TColor*)next())) {
-            if (colold) {
-               Int_t cn = 0;
-               if (colold) cn = colold->GetNumber();
-               TColor *colcur = gROOT->GetColor(cn);
+         while (auto colold = static_cast<TColor *>(next())) {
+            Int_t cn = colold->GetNumber();
+            TColor *colcur = gROOT->GetColor(cn);
+            if (colcur && (colcur->IsA() == TColor::Class()) && (colold->IsA() == TColor::Class())) {
+               colcur->SetName(colold->GetName());
+               colcur->SetRGB(colold->GetRed(), colold->GetGreen(), colold->GetBlue());
+               colcur->SetAlpha(colold->GetAlpha());
+            } else {
                if (colcur) {
-                  colcur->SetRGB(colold->GetRed(),colold->GetGreen(),colold->GetBlue());
-               } else {
-                  colcur = new TColor(cn,colold->GetRed(),
-                                        colold->GetGreen(),
-                                        colold->GetBlue(),
-                                        colold->GetName());
-                  if (!colcur) return;
+                  if (root_colors) root_colors->Remove(colcur);
+                  delete colcur;
+               }
+               colors->Remove(colold);
+               if (root_colors) {
+                  if (colcur) {
+                     root_colors->AddAtAndExpand(colold, cn);
+                  }
+                  else {
+                     // Copy to current session
+                     // do not use copy constructor which does not update highest color index
+                     [[maybe_unused]] TColor* const colnew = new TColor(cn, colold->GetRed(), colold->GetGreen(), colold->GetBlue(), colold->GetName(), colold->GetAlpha());
+                     delete colold;
+                     // No need to delete colnew, as the constructor adds it to global list of colors
+                     assert(root_colors->At(cn) == colnew);
+                  }
                }
             }
          }
          //restore the palette if needed
-         TObjArray *currentpalette = (TObjArray*)fPrimitives->FindObject("CurrentColorPalette");
-         if (currentpalette) {
-           TIter nextpal(currentpalette);
-           Int_t n = currentpalette->GetEntries();
-           TArrayI palcolors(n);
-           TColor *col = nullptr;
-           Int_t i = 0;
-           while ((col = (TColor*)nextpal())) palcolors[i++] = col->GetNumber();
-           gStyle->SetPalette(n,palcolors.GetArray());
-           fPrimitives->Remove(currentpalette);
-           delete currentpalette;
+         auto palette = dynamic_cast<TObjArray *>(fPrimitives->FindObject("CurrentColorPalette"));
+         if (palette) {
+            TIter nextcol(palette);
+            Int_t number = palette->GetEntries();
+            TArrayI palcolors(number);
+            Int_t i = 0;
+            while (auto col = static_cast<TColor *>(nextcol()))
+               palcolors[i++] = col->GetNumber();
+            gStyle->SetPalette(number, palcolors.GetArray());
+            fPrimitives->Remove(palette);
+            delete palette;
          }
          fPrimitives->Remove(colors);
          colors->Delete();
@@ -2642,13 +2656,13 @@ void TCanvas::DeleteCanvasPainter()
 
 Bool_t TCanvas::SaveAll(const std::vector<TPad *> &pads, const char *filename, Option_t *option)
 {
-   if (pads.size() == 0) {
+   if (pads.empty()) {
       std::vector<TPad *> canvases;
       TIter iter(gROOT->GetListOfCanvases());
       while (auto c = dynamic_cast<TCanvas *>(iter()))
          canvases.emplace_back(c);
 
-      if (canvases.size() == 0) {
+      if (canvases.empty()) {
          ::Warning("TCanvas::SaveAll", "No pads are provided");
          return kFALSE;
       }
