@@ -9,7 +9,7 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
-#include <stdlib.h>
+#include <cstdlib>
 
 #include <iostream>
 #include "TROOT.h"
@@ -21,7 +21,7 @@
 #include "TVirtualX.h"
 
 
-const Double_t kPI = 3.14159265358979323846;
+constexpr Double_t kPI = TMath::Pi();
 
 ClassImp(TEllipse);
 
@@ -101,7 +101,7 @@ TEllipse::TEllipse(const TEllipse &ellipse) : TObject(ellipse), TAttLine(ellipse
    fPhimax = 360;
    fTheta  = 0;
 
-   ((TEllipse&)ellipse).Copy(*this);
+   ellipse.TEllipse::Copy(*this);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -132,6 +132,7 @@ void TEllipse::Copy(TObject &obj) const
 
 Int_t TEllipse::DistancetoPrimitive(Int_t px, Int_t py)
 {
+   if (!gPad) return 9999;
    Double_t x = gPad->PadtoX(gPad->AbsPixeltoX(px));
    Double_t y = gPad->PadtoY(gPad->AbsPixeltoY(py));
 
@@ -173,7 +174,7 @@ void TEllipse::Draw(Option_t *option)
 ////////////////////////////////////////////////////////////////////////////////
 /// Draw this ellipse with new coordinates.
 
-void TEllipse::DrawEllipse(Double_t x1, Double_t y1,Double_t r1,Double_t r2,Double_t phimin,Double_t phimax,Double_t theta,Option_t *option)
+TEllipse *TEllipse::DrawEllipse(Double_t x1, Double_t y1,Double_t r1,Double_t r2,Double_t phimin,Double_t phimax,Double_t theta,Option_t *option)
 {
    TEllipse *newellipse = new TEllipse(x1, y1, r1, r2, phimin, phimax,theta);
    TAttLine::Copy(*newellipse);
@@ -181,6 +182,7 @@ void TEllipse::DrawEllipse(Double_t x1, Double_t y1,Double_t r1,Double_t r2,Doub
    newellipse->SetBit(kCanDelete);
    newellipse->AppendPad(option);
    if (TestBit(kNoEdges)) newellipse->SetBit(kNoEdges);
+   return newellipse;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -505,6 +507,38 @@ void TEllipse::ExecuteEvent(Int_t event, Int_t px, Int_t py)
    }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Return 1 if the point (x,y) is inside the polygon defined by
+/// the ellipse 0 otherwise.
+/// Author: Ole Hansen (ole@jlab.org)
+Int_t TEllipse::IsInside(Double_t x, Double_t y) const
+{
+   x -= fX1;
+   y -= fY1;
+   Double_t th = fTheta * TMath::DegToRad();
+   Double_t st = TMath::Sin(th);
+   Double_t ct = TMath::Cos(th);
+   Double_t xx =  ct * x + st * y;
+   Double_t yy = -st * x + ct * y;
+
+   if (TMath::Abs(xx) > fR1 || TMath::Abs(yy) > fR2)
+      return 0;
+   Double_t xn = xx / fR1;
+   Double_t yn = yy / fR2;
+   if (xn * xn + yn * yn > 1.)
+      return 0;
+   if (fPhimax - fPhimin >= 360.)
+      return 1;
+   Double_t phimin = std::fmod(fPhimin, 360.);
+   Double_t phimax = std::fmod(fPhimax, 360.);
+   Double_t phi = TMath::RadToDeg()*(TMath::Pi() + TMath::ATan2(-yy * fR1 / fR2, -xx));
+   if (phi < phimin || phi > phimax)
+      return 0;
+
+   return 1;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 /// List this ellipse with its attributes.
 
@@ -529,6 +563,7 @@ void TEllipse::PaintEllipse(Double_t x1, Double_t y1, Double_t r1, Double_t r2,
                             Double_t phimin, Double_t phimax, Double_t theta,
                             Option_t *option)
 {
+   if (!gPad) return;
    const Int_t np = 200;
    static Double_t x[np+3], y[np+3];
    TAttLine::Modify();  //Change line attributes only if necessary
@@ -662,7 +697,8 @@ void TEllipse::Streamer(TBuffer &R__b)
 
 Rectangle_t TEllipse::GetBBox()
 {
-   Rectangle_t BBox;
+   Rectangle_t BBox{0,0,0,0};
+   if (!gPad) return BBox;
    if (!gPad) return (BBox);
    BBox.fX = gPad->XtoPixel(fX1-fR1);
    BBox.fY = gPad->YtoPixel(fY1+fR2);
@@ -676,7 +712,8 @@ Rectangle_t TEllipse::GetBBox()
 
 TPoint TEllipse::GetBBoxCenter()
 {
-   TPoint p;
+   TPoint p(0,0);
+   if (!gPad) return (p);
    if (!gPad) return (p);
    p.SetX(gPad->XtoPixel(fX1));
    p.SetY(gPad->YtoPixel(fY1));
@@ -688,7 +725,7 @@ TPoint TEllipse::GetBBoxCenter()
 
 void TEllipse::SetBBoxCenter(const TPoint &p)
 {
-   if (!gPad) return; 
+   if (!gPad) return;
    fX1 = gPad->PixeltoX(p.GetX());
    fY1 = gPad->PixeltoY(p.GetY()-gPad->VtoPixel(0));
 }
@@ -717,6 +754,7 @@ void TEllipse::SetBBoxCenterY(const Int_t y)
 
 void TEllipse::SetBBoxX1(const Int_t x)
 {
+   if (!gPad) return;
    Double_t x1 = gPad->PixeltoX(x);
    if (x1>fX1+fR1) return;
 
