@@ -16,6 +16,7 @@
 #include <ROOT/REveSceneInfo.hxx>
 #include <ROOT/REveManager.hxx>
 #include <ROOT/REveSelection.hxx>
+#include <ROOT/REveText.hxx>
 
 #include <nlohmann/json.hpp>
 
@@ -106,6 +107,10 @@ List of Viewers providing common operations on REveViewer collections.
 void REveViewer::SetAxesType(int at)
 {
    fAxesType = (EAxesType)at;
+   if (fAxesType != kAxesNone) {
+      std::string rf_dir = gSystem->ExpandPathName("${ROOTSYS}/fonts/");
+      REX::REveText::AssertSdfFont("LiberationSerif-Regular", rf_dir + "LiberationSerif-Regular.ttf");
+   }
    StampObjProps();
 }
 
@@ -357,126 +362,6 @@ void REveViewerList::HandleTooltip()
    }
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::MouseOver() signal.
-///
-/// The attempt is made to determine the REveElement being
-/// represented by the physical shape and global highlight is updated
-/// accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// highlighted.
-///
-/// Highlight is always in single-selection mode.
-
-void REveViewerList::OnMouseOver(TObject *, UInt_t /*state*/)
-{
-   // REveElement *el = dynamic_cast<REveElement*>(obj);
-   // if (el && !el->IsPickable())
-   //   el = nullptr;
-
-   // void *qsender = gTQSender;
-   // REX::gEve->GetHighlight()->UserPickedElement(el, kFALSE);
-   // gTQSender = qsender;
-
-   HandleTooltip();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::ReMouseOver().
-///
-/// The obj is dyn-casted to the REveElement and global selection is
-/// updated accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// selected.
-
-void REveViewerList::OnReMouseOver(TObject *obj, UInt_t /*state*/)
-{
-   REveElement* el = dynamic_cast<REveElement*>(obj);
-   if (el && !el->IsPickable())
-      el = nullptr;
-
-   // void *qsender = gTQSender;
-   // REX::gEve->GetHighlight()->UserRePickedElement(el);
-   // gTQSender = qsender;
-
-   HandleTooltip();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::UnMouseOver().
-///
-/// The obj is dyn-casted to the REveElement and global selection is
-/// updated accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// selected.
-
-void REveViewerList::OnUnMouseOver(TObject *obj, UInt_t /*state*/)
-{
-   REveElement* el = dynamic_cast<REveElement*>(obj);
-   if (el && !el->IsPickable())
-      el = nullptr;
-
-   // void *qsender = gTQSender;
-   // REX::gEve->GetHighlight()->UserUnPickedElement(el);
-   // gTQSender = qsender;
-
-   HandleTooltip();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::Clicked().
-///
-/// The obj is dyn-casted to the REveElement and global selection is
-/// updated accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// selected.
-
-void REveViewerList::OnClicked(TObject *obj, UInt_t /*button*/, UInt_t state)
-{
-   REveElement* el = dynamic_cast<REveElement*>(obj);
-   if (el && !el->IsPickable())
-      el = nullptr;
-   REX::gEve->GetSelection()->UserPickedElement(el, state & kKeyControlMask);
-}
-
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::ReClicked().
-///
-/// The obj is dyn-casted to the REveElement and global selection is
-/// updated accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// selected.
-
-void REveViewerList::OnReClicked(TObject *obj, UInt_t /*button*/, UInt_t /*state*/)
-{
-   REveElement* el = dynamic_cast<REveElement*>(obj);
-   if (el && !el->IsPickable())
-      el = nullptr;
-   REX::gEve->GetSelection()->UserRePickedElement(el);
-}
-
-////////////////////////////////////////////////////////////////////////////////
-/// Slot for global TGLViewer::UnClicked().
-///
-/// The obj is dyn-casted to the REveElement and global selection is
-/// updated accordingly.
-///
-/// If REveElement::IsPickable() returns false, the element is not
-/// selected.
-
-void REveViewerList::OnUnClicked(TObject *obj, UInt_t /*button*/, UInt_t /*state*/)
-{
-   REveElement* el = dynamic_cast<REveElement*>(obj);
-   if (el && !el->IsPickable())
-      el = nullptr;
-   REX::gEve->GetSelection()->UserUnPickedElement(el);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Set color brightness.
 
@@ -491,13 +376,14 @@ void REveViewerList::SetColorBrightness(Float_t b)
 void REveViewerList::SwitchColorSet()
 {
    fUseLightColorSet = ! fUseLightColorSet;
+   // To implement something along the lines of:
+   // BeginChanges on EveWorld; // Here or in the calling function
    // for (auto &c: fChildren) {
-      // TGLViewer* glv = ((REveViewer *)c)->GetGLViewer();
+      // REveViewer* v = (REveViewer *)c;
       // if ( fUseLightColorSet)
-      //    glv->UseLightColorSet();
+      //    c->UseLightColorSet();
       // else
-      //    glv->UseDarkColorSet();
-
-      // glv->RequestDraw(TGLRnrCtx::kLODHigh);
+      //    c->UseDarkColorSet();
    // }
+   // EndChanges on EveWorld;
 }
