@@ -125,7 +125,7 @@ THelix::THelix()
    fW        = 1.5E7;   // roughly the cyclon frequency of proton in AMS
    fRange[0] = 0.0;
    fRange[1] = 1.0;
-   fRotMat   = 0;
+   fRotMat   = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -143,8 +143,8 @@ THelix::THelix(Double_t x,  Double_t y,  Double_t z,
    v[0] = vx;
    v[1] = vy;
    v[2] = vz;
-   Double_t *range = 0;
-   fRotMat   = 0;
+   Double_t *range = nullptr;
+   fRotMat   = nullptr;
 
    SetHelix(p, v, w, range, kHelixZ);
    fOption = "";
@@ -164,7 +164,7 @@ THelix::THelix(Double_t const* xyz, Double_t const* v, Double_t w,
       r[0] = 0.0;        r[1] = 1.0;
    }
 
-   fRotMat   = 0;
+   fRotMat   = nullptr;
    if ( axis ) {                        // specify axis
       SetHelix(xyz, v, w, r, rType, axis);
    } else {                             // default axis
@@ -213,7 +213,8 @@ THelix& THelix::operator=(const THelix& hx)
       fW=hx.fW;
       for(Int_t i=0; i<3; i++)
          fAxis[i]=hx.fAxis[i];
-      fRotMat=hx.fRotMat;
+      delete fRotMat;
+      fRotMat = hx.fRotMat ? new TRotMatrix(*hx.fRotMat) : nullptr;
       for(Int_t i=0; i<2; i++)
          fRange[i]=hx.fRange[i];
    }
@@ -234,7 +235,7 @@ THelix::~THelix()
 
 THelix::THelix(const THelix &helix) : TPolyLine3D(helix)
 {
-   fRotMat=0;
+   fRotMat = nullptr;
    ((THelix&)helix).THelix::Copy(*this);
 }
 
@@ -259,7 +260,8 @@ void THelix::Copy(TObject &obj) const
 
    if (((THelix&)obj).fRotMat)
       delete ((THelix&)obj).fRotMat;
-   ((THelix&)obj).fRotMat    = new TRotMatrix(*fRotMat);
+
+   ((THelix&)obj).fRotMat    = fRotMat ? new TRotMatrix(*fRotMat) : nullptr;
 
    ((THelix&)obj).fRange[0]  = fRange[0];
    ((THelix&)obj).fRange[1]  = fRange[1];
