@@ -9,6 +9,7 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
+#include <memory>
 #include <stdexcept>
 #include <cstring>
 
@@ -200,10 +201,10 @@ The following types of plots are provided:
 
 TGLHistPainter::TGLHistPainter(TH1 *hist)
                    : fDefaultPainter(TVirtualHistPainter::HistPainter(hist)),
-                     fEq(0),
+                     fEq(nullptr),
                      fHist(hist),
-                     fF3(0),
-                     fStack(0),
+                     fF3(nullptr),
+                     fStack(nullptr),
                      fPlotType(kGLDefaultPlot)//THistPainter
 {
 }
@@ -213,38 +214,38 @@ TGLHistPainter::TGLHistPainter(TH1 *hist)
 
 TGLHistPainter::TGLHistPainter(TGLParametricEquation *equation)
                    : fEq(equation),
-                     fHist(0),
-                     fF3(0),
-                     fStack(0),
+                     fHist(nullptr),
+                     fF3(nullptr),
+                     fStack(nullptr),
                      fPlotType(kGLParametricPlot)//THistPainter
 {
-   fGLPainter.reset(new TGLParametricPlot(equation, &fCamera));
+   fGLPainter = std::make_unique<TGLParametricPlot>(equation, &fCamera);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 ///This ctor creates plot painter for TGL5DDataSet.
 
 TGLHistPainter::TGLHistPainter(TGL5DDataSet *data)
-                   : fEq(0),
-                     fHist(0),
-                     fF3(0),
-                     fStack(0),
+                   : fEq(nullptr),
+                     fHist(nullptr),
+                     fF3(nullptr),
+                     fStack(nullptr),
                      fPlotType(kGL5D)//THistPainter
 {
-   fGLPainter.reset(new TGL5DPainter(data, &fCamera, &fCoord));
+   fGLPainter = std::make_unique<TGL5DPainter>(data, &fCamera, &fCoord);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 ///This ctor creates plot painter for TGL5DDataSet.
 
 TGLHistPainter::TGLHistPainter(TGLTH3Composition *data)
-                   : fEq(0),
+                   : fEq(nullptr),
                      fHist(data),
-                     fF3(0),
-                     fStack(0),
+                     fF3(nullptr),
+                     fStack(nullptr),
                      fPlotType(kGLTH3Composition)
 {
-   fGLPainter.reset(new TGLTH3CompositionPainter(data, &fCamera, &fCoord));
+   fGLPainter = std::make_unique<TGLTH3CompositionPainter>(data, &fCamera, &fCoord);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -437,7 +438,7 @@ void TGLHistPainter::ExecuteEvent(Int_t event, Int_t px, Int_t py)
 
 TList *TGLHistPainter::GetContourList(Double_t contour)const
 {
-   return fDefaultPainter.get() ? fDefaultPainter->GetContourList(contour) : 0;
+   return fDefaultPainter.get() ? fDefaultPainter->GetContourList(contour) : nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -700,31 +701,31 @@ void TGLHistPainter::CreatePainter(const PlotOption_t &option, const TString &ad
 {
    if (option.fPlotType != fPlotType) {
       fCoord.ResetModified();
-      fGLPainter.reset(0);
+      fGLPainter.reset(nullptr);
    }
 
    if (option.fPlotType == kGLLegoPlot) {
       if (!fGLPainter.get()) {
          if (dynamic_cast<TH2Poly*>(fHist))
-            fGLPainter.reset(new TGLH2PolyPainter(fHist, &fCamera, &fCoord));
+            fGLPainter = std::make_unique<TGLH2PolyPainter>(fHist, &fCamera, &fCoord);
          else
-            fGLPainter.reset(new TGLLegoPainter(fHist, &fCamera, &fCoord));
+            fGLPainter = std::make_unique<TGLLegoPainter>(fHist, &fCamera, &fCoord);
       }
    } else if (option.fPlotType == kGLSurfacePlot) {
       if (!fGLPainter.get())
-         fGLPainter.reset(new TGLSurfacePainter(fHist, &fCamera, &fCoord));
+         fGLPainter = std::make_unique<TGLSurfacePainter>(fHist, &fCamera, &fCoord);
    } else if (option.fPlotType == kGLBoxPlot) {
       if (!fGLPainter.get())
-         fGLPainter.reset(new TGLBoxPainter(fHist, &fCamera, &fCoord));
+         fGLPainter = std::make_unique<TGLBoxPainter>(fHist, &fCamera, &fCoord);
    } else if (option.fPlotType == kGLTF3Plot) {
       if (!fGLPainter.get())
-         fGLPainter.reset(new TGLTF3Painter(fF3, fHist, &fCamera, &fCoord));
+         fGLPainter = std::make_unique<TGLTF3Painter>(fF3, fHist, &fCamera, &fCoord);
    } else if (option.fPlotType == kGLIsoPlot) {
       if (!fGLPainter.get())
-         fGLPainter.reset(new TGLIsoPainter(fHist, &fCamera, &fCoord));
+         fGLPainter = std::make_unique<TGLIsoPainter>(fHist, &fCamera, &fCoord);
    } else if (option.fPlotType == kGLVoxel) {
       if (!fGLPainter.get())
-         fGLPainter.reset(new TGLVoxelPainter(fHist, &fCamera, &fCoord));
+         fGLPainter = std::make_unique<TGLVoxelPainter>(fHist, &fCamera, &fCoord);
    }
 
    if (fGLPainter.get()) {
@@ -750,6 +751,14 @@ void TGLHistPainter::SetShowProjection(const char *option, Int_t nbins)
    if (fDefaultPainter.get()) fDefaultPainter->SetShowProjection(option, nbins);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Set show projectionXY.
+
+void TGLHistPainter::SetShowProjectionXY(const char *option, Int_t nbinsY, Int_t nbinsX)
+{
+   if (fDefaultPainter.get()) fDefaultPainter->SetShowProjectionXY(option, nbinsY, nbinsX);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 void TGLHistPainter::PadToViewport(Bool_t /*selectionPass*/)
