@@ -47,7 +47,7 @@ ClassImp(TStructViewerGUI);
 //
 //////////////////////////////////////////////////////////////////////////
 
-TGeoMedium* TStructViewerGUI::fgMedium = NULL;
+TGeoMedium* TStructViewerGUI::fgMedium = nullptr;
 UInt_t      TStructViewerGUI::fgCounter = 0;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -63,7 +63,7 @@ TStructViewerGUI::TStructViewerGUI(TStructViewer* parent, TStructNode* nodePtr,
    fMaxSlices = 10;
    fMouseX = 0;
    fMouseY = 0;
-   fSelectedObject = NULL;
+   fSelectedObject = nullptr;
    fMaxRatio = 0;
    fColors = colors;
 
@@ -509,7 +509,7 @@ void TStructViewerGUI::DrawNode(TStructNode* node)
       for (Float_t i = -(slices-1)/2; i < slices/2; i++) {
          TGeoVolume* sub = gGeoManager->MakeBox(Form("%s_%d", node->GetName(), fgCounter++), fgMedium,0.45*node->GetWidth() * 0.7 / slices, 0.45*node->GetHeight(), fBoxHeightEntry->GetNumber());
          sub->SetLineColor(GetColor(node));
-         fVolumes.Add((Long_t)sub, (Long_t)node);
+         fVolumes.Add((Longptr_t)sub, (Longptr_t)node);
          TGeoTranslation* subtrans = new TGeoTranslation("subtranslation", i * node->GetWidth() / slices, 0, 0);
          vol->AddNodeOverlap(sub, 1, subtrans);
       }
@@ -521,7 +521,7 @@ void TStructViewerGUI::DrawNode(TStructNode* node)
    vol->SetLineWidth(1);
 
    TGeoTranslation* trans = new TGeoTranslation("translation", node->GetCenter(), node->GetMiddle(), -(node->GetLevel() * fLevelDistanceEntry->GetNumber()));
-   fVolumes.Add((Long_t)vol, (Long_t)node);
+   fVolumes.Add((Longptr_t)vol, (Longptr_t)node);
 
    fTopVolume->AddNode(vol,1, trans);
 }
@@ -665,16 +665,16 @@ void TStructViewerGUI::LevelDistValueSetSlot(Long_t /* dist */)
 void TStructViewerGUI::MouseOverSlot(TGLPhysicalShape* shape)
 {
    fToolTip->Hide();
-   fSelectedObject = NULL;
+   fSelectedObject = nullptr;
    if (shape && shape->GetLogical()) {
       fSelectedObject =  (TStructNode*)(shape->GetLogical()->ID());
       if (fSelectedObject) {
          if (fSelectedObject->IsA()->InheritsFrom(TPolyLine3D::Class())) {
-            fSelectedObject = NULL;
+            fSelectedObject = nullptr;
             return;
          }
-         Long_t shapeID  = (Long_t)(shape->GetLogical()->ID());
-         Long_t volValue = (Long_t)fVolumes.GetValue(shapeID);
+         Longptr_t shapeID  = (Longptr_t)(shape->GetLogical()->ID());
+         Longptr_t volValue = (Longptr_t)fVolumes.GetValue(shapeID);
          fSelectedObject = (TStructNode*)volValue;
 
          fToolTip->SetText(TString(fSelectedObject->GetName()) + "\n" + fSelectedObject->GetTypeName());
@@ -813,18 +813,24 @@ void TStructViewerGUI::UnCheckMaxObjects()
 
 void TStructViewerGUI::Update(Bool_t resetCamera)
 {
-   if (!fNodePtr) {
+   if (!fNodePtr)
       return;
-   }
 
-   fCanvas->GetListOfPrimitives()->Clear();
-   fTopVolume->ClearNodes();
+   if (fCanvas && fCanvas->GetListOfPrimitives())
+      fCanvas->GetListOfPrimitives()->Clear();
+
+   if (fTopVolume)
+      fTopVolume->ClearNodes();
+
    Draw();
-   fCanvas->GetListOfPrimitives()->Add(fTopVolume);
-   fGLViewer->UpdateScene();
 
-   if(resetCamera) {
-      fGLViewer->ResetCurrentCamera();
+   if (fCanvas && fCanvas->GetListOfPrimitives())
+      fCanvas->GetListOfPrimitives()->Add(fTopVolume);
+
+   if (fGLViewer) {
+      fGLViewer->UpdateScene();
+      if(resetCamera)
+         fGLViewer->ResetCurrentCamera();
    }
 }
 
