@@ -28,17 +28,16 @@ public:
 
    RNTupleDraw6Provider()
    {
-      RegisterDraw6(TClass::GetClass<ROOT::Experimental::RNTuple>(), [this](TVirtualPad *pad, std::unique_ptr<RHolder> &obj, const std::string &opt) -> bool {
-
-         auto h1 = DrawField(dynamic_cast<RFieldHolder*> (obj.get()));
-         if (!h1) return false;
-
-         pad->GetListOfPrimitives()->Add(h1, opt.c_str());
-
-         return true;
-      });
+      RegisterDraw6(TClass::GetClass<ROOT::RNTuple>(),
+                    [this](TVirtualPad *pad, std::unique_ptr<RHolder> &obj, const std::string &opt) -> bool {
+                       auto h1 = DrawField(dynamic_cast<RFieldHolder *>(obj.get()));
+                       if (!h1)
+                          return false;
 
+                       pad->Add(h1, opt.c_str());
 
+                       return true;
+                    });
    }
 
 } newRNTupleDraw6Provider;
