@@ -20,23 +20,24 @@
 
 **************************************************************************/
 
-//////////////////////////////////////////////////////////////////////////
-//                                                                      //
-// TGListView, TGLVContainer and TGLVEntry                              //
-//                                                                      //
-// A list view is a widget that can contain a number of items           //
-// arranged in a grid or list. The items can be represented either      //
-// by a string or by an icon.                                           //
-//                                                                      //
-// The TGListView is user callable. The other classes are service       //
-// classes of the list view.                                            //
-//                                                                      //
-// A list view can generate the following events:                       //
-// kC_CONTAINER, kCT_SELCHANGED, total items, selected items.           //
-// kC_CONTAINER, kCT_ITEMCLICK, which button, location (y<<16|x).       //
-// kC_CONTAINER, kCT_ITEMDBLCLICK, which button, location (y<<16|x).    //
-//                                                                      //
-//////////////////////////////////////////////////////////////////////////
+
+/** \class TGListView
+    \ingroup guiwidgets
+
+A list view is a widget that can contain a number of items
+arranged in a grid or list. The items can be represented either
+by a string or by an icon.
+
+The TGListView is user callable. The other classes are service
+classes of the list view.
+
+A list view can generate the following events:
+  - kC_CONTAINER, kCT_SELCHANGED, total items, selected items.
+  - kC_CONTAINER, kCT_ITEMCLICK, which button, location (y<<16|x).
+  - kC_CONTAINER, kCT_ITEMDBLCLICK, which button, location (y<<16|x).
+
+*/
+
 
 #include "TGListView.h"
 #include "TGPicture.h"
@@ -68,7 +69,7 @@ ClassImp(TGListView);
 TGLVEntry::TGLVEntry(const TGWindow *p, const TGPicture *bigpic,
                      const TGPicture *smallpic, TGString *name,
                      TGString **subnames, EListViewMode viewMode,
-                     UInt_t options, ULong_t back) :
+                     UInt_t options, Pixel_t back) :
    TGFrame(p, 10, 10, options, back)
 {
    fSelPic = 0;
@@ -84,9 +85,8 @@ TGLVEntry::TGLVEntry(const TGWindow *p, const TGPicture *bigpic,
    fSubnames = subnames;
    fUserData = 0;
 
-   fCpos  =
-   fJmode = 0;
-
+   fCpos  = fJmode = 0;
+   fCtw = nullptr;
    fActive = kFALSE;
 
    fFontStruct = GetDefaultFontStruct();
@@ -109,8 +109,6 @@ TGLVEntry::TGLVEntry(const TGWindow *p, const TGPicture *bigpic,
          fCtw[i] = gVirtualX->TextWidth(fFontStruct, fSubnames[i]->GetString(),
                                         fSubnames[i]->GetLength());
       }
-   } else {
-      fCtw = 0;
    }
 
    fViewMode = (EListViewMode)-1;
@@ -148,8 +146,9 @@ TGLVEntry::TGLVEntry(const TGLVContainer *p, const TString& name,
    fSubnames = subnames;
    fUserData = 0;
 
-   fCpos  =
-   fJmode = 0;
+   fCpos = fJmode = 0;
+
+   fCtw = nullptr;
 
    fActive = kFALSE;
 
@@ -171,8 +170,6 @@ TGLVEntry::TGLVEntry(const TGLVContainer *p, const TString& name,
          fCtw[i] = gVirtualX->TextWidth(fFontStruct, fSubnames[i]->GetString(),
                                         fSubnames[i]->GetLength());
       }
-   } else {
-      fCtw = 0;
    }
 
    fViewMode = (EListViewMode)-1;
@@ -189,7 +186,11 @@ TGLVEntry::~TGLVEntry()
    if (fSubnames) {
       for (Int_t i = 0; fSubnames[i] != 0; ++i) delete fSubnames[i];
       delete [] fSubnames;
+      fSubnames = nullptr;
+   }
+   if (fCtw) {
       delete [] fCtw;
+      fCtw = nullptr;
    }
 }
 
@@ -204,11 +205,14 @@ void TGLVEntry::SetSubnames(const char* n1,const char* n2,const char* n3,
    if (fSubnames) {
       for (Int_t i = 0; fSubnames[i] != 0; ++i) delete fSubnames[i];
       delete [] fSubnames;
+   }
+   if (fCtw) {
       delete [] fCtw;
    }
 
    Int_t ncol = 0;
-   fSubnames = 0;
+   fSubnames = nullptr;
+   fCtw = nullptr;
 
    if (n12 && strlen(n12)) ncol=12;
    else if (n11 && strlen(n11)) ncol=11;
@@ -262,7 +266,7 @@ void TGLVEntry::Activate(Bool_t a)
       fSelPic = new TGSelectedPicture(gClient, fCurrent);
    } else {
       if (fSelPic) delete fSelPic;
-      fSelPic = 0;
+      fSelPic = nullptr;
    }
    DoRedraw();
 }
@@ -310,7 +314,7 @@ void TGLVEntry::SetPictures(const TGPicture *bigpic, const TGPicture *smallpic)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Redraw list view item.
-/// List view item is placed and layouted in the container frame,
+/// List view item is placed and laid out in the container frame,
 /// but is drawn in viewport.
 
 void TGLVEntry::DoRedraw()
@@ -358,8 +362,8 @@ void TGLVEntry::DrawCopy(Handle_t id, Int_t x, Int_t y)
          gVirtualX->SetForeground(fNormGC, fgBlackPixel);
       }
    }
-   // This if tries to print the elements with ... appened at the end if
-   // the widht of the string is longer than that of the column
+   // This if tries to print the elements with ... appended at the end if
+   // the width of the string is longer than that of the column
    if (fViewMode == kLVDetails && fSubnames && fCpos && fJmode && fCtw) {
       TString tmpString = *fItemName;
       Int_t ftmpWidth = gVirtualX->TextWidth(fFontStruct, tmpString,
@@ -515,7 +519,7 @@ const TGGC &TGLVEntry::GetDefaultGC()
 /// created by the TGCanvas derived TGListView).
 
 TGLVContainer::TGLVContainer(const TGWindow *p, UInt_t w, UInt_t h,
-                             UInt_t options, ULong_t back) :
+                             UInt_t options, Pixel_t back) :
    TGContainer(p, w, h, options, back)
 {
    fListView = 0;
@@ -534,7 +538,7 @@ TGLVContainer::TGLVContainer(const TGWindow *p, UInt_t w, UInt_t h,
 /// all the list items. It will be shown through a TGViewPort (which is
 /// created by the TGCanvas derived TGListView).
 
-TGLVContainer::TGLVContainer(TGCanvas *p,UInt_t options, ULong_t back) :
+TGLVContainer::TGLVContainer(TGCanvas *p,UInt_t options, Pixel_t back) :
    TGContainer(p,options, back)
 {
    fListView = 0;
@@ -847,7 +851,6 @@ Bool_t TGLVContainer::HandleButton(Event_t* event)
          total = selected = 0;
       }
 
-      select_frame = kFALSE;
       while ((el = (TGFrameElement *) next())) {
          select_frame = kFALSE;
 
@@ -1169,7 +1172,7 @@ void TGLVContainer::SelectEntry(TGLVEntry *item)
 /// Create a list view widget.
 
 TGListView::TGListView(const TGWindow *p, UInt_t w, UInt_t h,
-                       UInt_t options, ULong_t back) :
+                       UInt_t options, Pixel_t back) :
    TGCanvas(p, w, h, options, back)
 {
    fViewMode    = kLVLargeIcons;
@@ -1602,7 +1605,7 @@ void TGListView::LayoutHeader(TGFrame *head)
 ////////////////////////////////////////////////////////////////////////////////
 /// Handle messages generated by the list view components.
 
-Bool_t TGListView::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGListView::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    TGLVContainer *cnt = (TGLVContainer*)GetContainer();
 
@@ -1642,9 +1645,9 @@ Bool_t TGListView::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
 
 void TGListView::Clicked(TGLVEntry *entry, Int_t btn)
 {
-   Long_t args[2];
+   Longptr_t args[2];
 
-   args[0] = (Long_t)entry;
+   args[0] = (Longptr_t)entry;
    args[1] = btn;
 
    Emit("Clicked(TGLVEntry*,Int_t)", args);
@@ -1655,9 +1658,9 @@ void TGListView::Clicked(TGLVEntry *entry, Int_t btn)
 
 void TGListView::Clicked(TGLVEntry *entry, Int_t btn, Int_t x, Int_t y)
 {
-   Long_t args[4];
+   Longptr_t args[4];
 
-   args[0] = (Long_t)entry;
+   args[0] = (Longptr_t)entry;
    args[1] = btn;
    args[2] = x;
    args[3] = y;
@@ -1670,9 +1673,9 @@ void TGListView::Clicked(TGLVEntry *entry, Int_t btn, Int_t x, Int_t y)
 
 void TGListView::DoubleClicked(TGLVEntry *entry, Int_t btn)
 {
-   Long_t args[2];
+   Longptr_t args[2];
 
-   args[0] = (Long_t)entry;
+   args[0] = (Longptr_t)entry;
    args[1] = btn;
 
    Emit("DoubleClicked(TGLVEntry*,Int_t)", args);
@@ -1683,9 +1686,9 @@ void TGListView::DoubleClicked(TGLVEntry *entry, Int_t btn)
 
 void TGListView::DoubleClicked(TGLVEntry *entry, Int_t btn, Int_t x, Int_t y)
 {
-   Long_t args[4];
+   Longptr_t args[4];
 
-   args[0] = (Long_t)entry;
+   args[0] = (Longptr_t)entry;
    args[1] = btn;
    args[2] = x;
    args[3] = y;
