@@ -25,15 +25,16 @@
 const UInt_t kLogElemFilled = BIT(17); // If the log element has been retrieved at least once
 const UInt_t kDefaultActive = BIT(18); // If the log element is active by default
 
-///////////////////////////////////////////////////////////////////////////
-//                                                                       //
-// TProofProgressLog                                                     //
-//                                                                       //
-// Dialog used to display Proof session logs from the Proof progress     //
-// dialog.                                                               //
-// It uses TProofMgr::GetSessionLogs() mechanism internally              //
-//                                                                       //
-///////////////////////////////////////////////////////////////////////////
+
+/** \class  TProofProgressLog
+    \ingroup sessionviewer
+
+Dialog used to display Proof session logs from the Proof progress
+dialog.
+It uses TProofMgr::GetSessionLogs() mechanism internally
+
+*/
+
 
 ClassImp(TProofProgressLog);
 
@@ -124,7 +125,7 @@ void TProofProgressLog::Init(Int_t w, Int_t h)
    clearall->Connect("Clicked()", "TProofProgressLog", this, "Select(=1)");
    hfselbox->AddFrame(clearall, new TGLayoutHints(kLHintsCenterY | kLHintsRight, 10, 0, 0, 0));
 
-   //select the defaut actives to start with
+   //select the default actives to start with
    Select(0, kFALSE);
 
    //Display button
@@ -336,7 +337,7 @@ void TProofProgressLog::BuildLogList(Bool_t create)
 
    if (fSessionUrl.IsNull()) {
       if (gDebug > 0)
-         Info("BuildLogList", "sesssion URL undefined - do nothing");
+         Info("BuildLogList", "session URL undefined - do nothing");
       return;
    }
    TProofMgr *mgr = TProof::Mgr(fSessionUrl.Data());
@@ -648,13 +649,16 @@ void TProofProgressLog::Rebuild()
    if (sameurl) {
       idx = fSessNum->GetIntNumber();
       if (idx == fSessionIdx) {
-         Info("Rebuild", "same paremeters {%s, %s}, {%d, %d}: no need to rebuild TProofLog",
+         Info("Rebuild", "same parameters {%s, %s}, {%d, %d}: no need to rebuild TProofLog",
                          url.GetUrl(), urlref.GetUrl(), idx, fSessionIdx);
          return;
       }
    }
    // Cleanup current TProofLog
-   if (fProofLog) delete fProofLog;
+   if (fProofLog) {
+      delete fProofLog;
+      fProofLog = nullptr;
+   }
 
    // Set new parameters
    fSessionUrl = fUrlText->GetText();
@@ -663,7 +667,7 @@ void TProofProgressLog::Rebuild()
    // Rebuild the list now
    BuildLogList(kFALSE);
 
-   // Select the defaut actives to start with
+   // Select the default actives to start with
    Select(0, kFALSE);
    // Redraw
    fLogList->Layout();
