@@ -69,13 +69,7 @@ End_Macro
 ////////////////////////////////////////////////////////////////////////////////
 /// TGraphAsymmErrors default constructor.
 
-TGraphAsymmErrors::TGraphAsymmErrors(): TGraph()
-{
-   fEXlow       = 0;
-   fEYlow       = 0;
-   fEXhigh      = 0;
-   fEYhigh      = 0;
-}
+TGraphAsymmErrors::TGraphAsymmErrors() {}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -164,14 +158,22 @@ TGraphAsymmErrors::TGraphAsymmErrors(Int_t n, const Double_t *x, const Double_t
    if (!CtorAllocate()) return;
 
    n = fNpoints*sizeof(Double_t);
-   if(exl) { memcpy(fEXlow, exl, n);
-   } else { memset(fEXlow, 0, n); }
-   if(exh) { memcpy(fEXhigh, exh, n);
-   } else { memset(fEXhigh, 0, n); }
-   if(eyl) { memcpy(fEYlow, eyl, n);
-   } else { memset(fEYlow, 0, n); }
-   if(eyh) { memcpy(fEYhigh, eyh, n);
-   } else { memset(fEYhigh, 0, n); }
+   if (exl)
+      memcpy(fEXlow, exl, n);
+   else
+      memset(fEXlow, 0, n);
+   if (exh)
+      memcpy(fEXhigh, exh, n);
+   else
+      memset(fEXhigh, 0, n);
+   if (eyl)
+      memcpy(fEYlow, eyl, n);
+   else
+      memset(fEYlow, 0, n);
+   if (eyh)
+      memcpy(fEYhigh, eyh, n);
+   else
+      memset(fEYhigh, 0, n);
 }
 
 
@@ -183,7 +185,6 @@ TGraphAsymmErrors::TGraphAsymmErrors(Int_t n, const Double_t *x, const Double_t
 /// in vx and vy.
 
 TGraphAsymmErrors::TGraphAsymmErrors(const TVectorF  &vx, const TVectorF  &vy, const TVectorF  &vexl, const TVectorF  &vexh, const TVectorF  &veyl, const TVectorF  &veyh)
-                  :TGraph()
 {
    fNpoints = TMath::Min(vx.GetNrows(), vy.GetNrows());
    if (!TGraph::CtorAllocate()) return;
@@ -213,7 +214,6 @@ TGraphAsymmErrors::TGraphAsymmErrors(const TVectorF  &vx, const TVectorF  &vy, c
 /// in vx and vy.
 
 TGraphAsymmErrors::TGraphAsymmErrors(const TVectorD &vx, const TVectorD &vy, const TVectorD &vexl, const TVectorD &vexh, const TVectorD &veyl, const TVectorD &veyh)
-                  :TGraph()
 {
    fNpoints = TMath::Min(vx.GetNrows(), vy.GetNrows());
    if (!TGraph::CtorAllocate()) return;
@@ -244,11 +244,11 @@ TGraphAsymmErrors::TGraphAsymmErrors(const TH1 *h)
 {
    if (!CtorAllocate()) return;
 
-   for (Int_t i=0;i<fNpoints;i++) {
+   for (Int_t i = 0; i < fNpoints; i++) {
       fEXlow[i]  = h->GetBinWidth(i+1)*gStyle->GetErrorX();
       fEXhigh[i] = fEXlow[i];
       fEYlow[i]  = h->GetBinErrorLow(i+1);
-      fEYhigh[i] = h->GetBinErrorUp(i+1);;
+      fEYhigh[i] = h->GetBinErrorUp(i+1);
    }
 }
 
@@ -289,9 +289,9 @@ TGraphAsymmErrors::TGraphAsymmErrors(const TH1* pass, const TH1* total, Option_t
 ///  - format = `"%lg %lg %lg %lg"`     read only 4 first columns into X, Y,  ELY, EHY
 ///  - format = `"%lg %lg %lg %lg %lg %lg"` read only 6 first columns into X, Y, EXL, EYH, EYL, EHY
 ///
-/// For files separated by a specific delimiter different from `' '` and `'\t'` (e.g. `';'` in csv files)
+/// For files separated by a specific delimiter different from `' '` and `'\\t'` (e.g. `';'` in csv files)
 /// you can avoid using `%*s` to bypass this delimiter by explicitly specify the `"option" argument,
-/// e.g. `option=" \t,;"` for columns of figures separated by any of these characters `(' ', '\t', ',', ';')`
+/// e.g. `option=" \\t,;"` for columns of figures separated by any of these characters `(' ', '\\t', ',', ';')`
 /// used once `(e.g. "1;1")` or in a combined way `(" 1;,;;  1")`.
 /// Note in that case, the instantiation is about 2 times slower.
 /// In case a delimiter is specified, the format `"%lg %lg %lg"` will read X,Y,EX.
@@ -318,7 +318,7 @@ TGraphAsymmErrors::TGraphAsymmErrors(const char *filename, const char *format, O
       Int_t ncol = TGraphErrors::CalculateScanfFields(format);  //count number of columns in format
       Int_t res;
       while (std::getline(infile, line, '\n')) {
-         exl = exh = eyl = eyh = 0;
+         exl = exh = eyl = eyh = 0.;
          if (ncol < 3) {
             res = sscanf(line.c_str(), format, &x, &y);
          } else if (ncol < 5) {
@@ -354,40 +354,39 @@ TGraphAsymmErrors::TGraphAsymmErrors(const char *filename, const char *format, O
          Error("TGraphAsymmErrors", "Incorrect input format! Only %d tag(s) in format whereas at least 2 \"%%lg\" tags are expected!", ntokens);
          return ;
       }
-      Int_t ntokensToBeSaved = 0 ;
-      Bool_t * isTokenToBeSaved = new Bool_t [ntokens] ;
+      Int_t ntokensToBeSaved = 0;
+      Bool_t * isTokenToBeSaved = new Bool_t[ntokens];
       for (Int_t idx = 0; idx < ntokens; idx++) {
-         isTokenToBeSaved[idx] = TString::Format("%c", format_[idx]).Atoi() ; //atoi(&format_[idx]) does not work for some reason...
+         isTokenToBeSaved[idx] = TString::Format("%c", format_[idx]).Atoi(); //atoi(&format_[idx]) does not work for some reason...
          if (isTokenToBeSaved[idx] == 1) {
             ntokensToBeSaved++ ;
          }
       }
       if (ntokens >= 2 && (ntokensToBeSaved < 2 || ntokensToBeSaved > 4)) { //first condition not to repeat the previous error message
          Error("TGraphAsymmErrors", "Incorrect input format! There are %d \"%%lg\" tag(s) in format whereas 2,3 or 4 are expected!", ntokensToBeSaved);
-         delete [] isTokenToBeSaved ;
+         delete [] isTokenToBeSaved;
          return ;
       }
 
       // Initializing loop variables
-      Bool_t isLineToBeSkipped = kFALSE ; //empty and ill-formed lines
-      char * token = NULL ;
-      TString token_str = "" ;
-      Int_t token_idx = 0 ;
-      Double_t * value = new Double_t [6] ; //x,y,exl, exh, eyl, eyh buffers
-      for (Int_t k = 0; k < 6; k++) {
-         value[k] = 0. ;
-      }
-      Int_t value_idx = 0 ;
+      Bool_t isLineToBeSkipped = kFALSE; //empty and ill-formed lines
+      char *token = nullptr;
+      TString token_str = "";
+      Int_t token_idx = 0;
+      Double_t value[6]; //x,y,exl, exh, eyl, eyh buffers
+      for (Int_t k = 0; k < 6; k++)
+         value[k] = 0.;
+      Int_t value_idx = 0;
 
       // Looping
       char *rest;
       while (std::getline(infile, line, '\n')) {
-         if (line != "") {
+         if (!line.empty()) {
             if (line[line.size() - 1] == char(13)) {  // removing DOS CR character
                line.erase(line.end() - 1, line.end()) ;
             }
             token = R__STRTOK_R(const_cast<char*>(line.c_str()), option, &rest) ;
-            while (token != NULL && value_idx < ntokensToBeSaved) {
+            while (token != nullptr && value_idx < ntokensToBeSaved) {
                if (isTokenToBeSaved[token_idx]) {
                   token_str = TString(token) ;
                   token_str.ReplaceAll("\t", "") ;
@@ -399,32 +398,31 @@ TGraphAsymmErrors::TGraphAsymmErrors(const char *filename, const char *format, O
                      value_idx++ ;
                   }
                }
-               token = R__STRTOK_R(NULL, option, &rest); // next token
+               token = R__STRTOK_R(nullptr, option, &rest); // next token
                token_idx++ ;
             }
             if (!isLineToBeSkipped && value_idx > 1) { //i.e. 2,3 or 4
-               x = value[0] ;
-               y = value[1] ;
-               exl = value[2] ;
-               exh = value[3] ;
-               eyl = value[4] ;
-               eyh = value[5] ;
-               SetPoint(np, x, y) ;
+               x = value[0];
+               y = value[1];
+               exl = value[2];
+               exh = value[3];
+               eyl = value[4];
+               eyh = value[5];
+               SetPoint(np, x, y);
                SetPointError(np, exl, exh, eyl, eyh);
                np++ ;
             }
          }
-         isLineToBeSkipped = kFALSE ;
-         token = NULL ;
-         token_idx = 0 ;
-         value_idx = 0 ;
+         isLineToBeSkipped = kFALSE;
+         token = nullptr;
+         token_idx = 0;
+         value_idx = 0;
       }
       Set(np) ;
 
       // Cleaning
-      delete [] isTokenToBeSaved ;
-      delete [] value ;
-      delete token ;
+      delete [] isTokenToBeSaved;
+      delete token;
    }
    infile.close();
 }
@@ -447,6 +445,15 @@ Double_t** TGraphAsymmErrors::Allocate(Int_t size) {
    return AllocateArrays(6, size);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Add a point with asymmetric errorbars to the graph.
+
+void TGraphAsymmErrors::AddPointError(Double_t x, Double_t y, Double_t exl, Double_t exh, Double_t eyl, Double_t eyh)
+{
+   AddPoint(x, y);
+   SetPointError(fNpoints - 1, exl, exh, eyl, eyh);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Apply a function to all data points \f$ y = f(x,y) \f$
 ///
@@ -464,14 +471,14 @@ void TGraphAsymmErrors::Apply(TF1 *f)
 
    if (fHistogram) {
       delete fHistogram;
-      fHistogram = 0;
+      fHistogram = nullptr;
    }
    for (Int_t i=0;i<GetN();i++) {
       GetPoint(i,x,y);
-      exl=GetErrorXlow(i);
-      exh=GetErrorXhigh(i);
-      eyl=GetErrorYlow(i);
-      eyh=GetErrorYhigh(i);
+      exl = GetErrorXlow(i);
+      exh = GetErrorXhigh(i);
+      eyl = GetErrorYlow(i);
+      eyh = GetErrorYhigh(i);
 
       fxy = f->Eval(x,y);
       SetPoint(i,x,fxy);
@@ -481,8 +488,7 @@ void TGraphAsymmErrors::Apply(TF1 *f)
       if (f->Eval(x,y-eyl)<f->Eval(x,y+eyh)) {
          eyl_new = TMath::Abs(fxy - f->Eval(x,y-eyl));
          eyh_new = TMath::Abs(f->Eval(x,y+eyh) - fxy);
-      }
-      else {
+      } else {
          eyh_new = TMath::Abs(fxy - f->Eval(x,y-eyl));
          eyl_new = TMath::Abs(f->Eval(x,y+eyh) - fxy);
       }
@@ -1027,7 +1033,7 @@ void TGraphAsymmErrors::CopyAndRelease(Double_t **newarrays,
 Bool_t TGraphAsymmErrors::CopyPoints(Double_t **arrays,
                                      Int_t ibegin, Int_t iend, Int_t obegin)
 {
-   if (TGraph::CopyPoints(arrays ? arrays+4 : 0, ibegin, iend, obegin)) {
+   if (TGraph::CopyPoints(arrays ? arrays+4 : nullptr, ibegin, iend, obegin)) {
       Int_t n = (iend - ibegin)*sizeof(Double_t);
       if (arrays) {
          memmove(&arrays[0][obegin], &fEXlow[ibegin], n);
@@ -1052,10 +1058,10 @@ Bool_t TGraphAsymmErrors::CopyPoints(Double_t **arrays,
 /// Note: This function should be called only from the constructor
 /// since it does not delete previously existing arrays
 
-Bool_t TGraphAsymmErrors::CtorAllocate(void)
+Bool_t TGraphAsymmErrors::CtorAllocate()
 {
    if (!fNpoints) {
-      fEXlow = fEYlow = fEXhigh = fEYhigh = 0;
+      fEXlow = fEYlow = fEXhigh = fEYhigh = nullptr;
       return kFALSE;
    }
    fEXlow = new Double_t[fMaxSize];
@@ -1076,7 +1082,7 @@ Bool_t TGraphAsymmErrors::DoMerge(const TGraph *g)
    Double_t * exh = g->GetEXhigh();
    Double_t * eyl = g->GetEYlow();
    Double_t * eyh = g->GetEYhigh();
-   if (exl == 0 || exh == 0 || eyl == 0 || eyh == 0) {
+   if (exl == nullptr || exh == nullptr || eyl == nullptr || eyh == nullptr) {
       if (g->IsA() != TGraph::Class() )
          Warning("DoMerge","Merging a %s is not compatible with a TGraphAsymmErrors - errors will be ignored",g->IsA()->GetName());
       return TGraph::DoMerge(g);
@@ -1110,7 +1116,8 @@ void TGraphAsymmErrors::FillZero(Int_t begin, Int_t end,
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// It returns the error along X at point i.
+/// Returns the combined error along X at point i by computing the average
+/// of the lower and upper variance.
 
 Double_t TGraphAsymmErrors::GetErrorX(Int_t i) const
 {
@@ -1124,7 +1131,8 @@ Double_t TGraphAsymmErrors::GetErrorX(Int_t i) const
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// It returns the error along Y at point i.
+/// Returns the combined error along Y at point i by computing the average
+/// of the lower and upper variance.
 
 Double_t TGraphAsymmErrors::GetErrorY(Int_t i) const
 {
@@ -1232,86 +1240,28 @@ void TGraphAsymmErrors::Print(Option_t *) const
 
 void TGraphAsymmErrors::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
-   char quote = '"';
    out << "   " << std::endl;
    static Int_t frameNumber = 3000;
    frameNumber++;
 
-   Int_t i;
-   TString fXName   = TString(GetName()) + Form("_fx%d",frameNumber);
-   TString fYName   = TString(GetName()) + Form("_fy%d",frameNumber);
-   TString fElXName = TString(GetName()) + Form("_felx%d",frameNumber);
-   TString fElYName = TString(GetName()) + Form("_fely%d",frameNumber);
-   TString fEhXName = TString(GetName()) + Form("_fehx%d",frameNumber);
-   TString fEhYName = TString(GetName()) + Form("_fehy%d",frameNumber);
-   out << "   Double_t " << fXName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fX[i] << "," << std::endl;
-   out << "   " << fX[fNpoints-1] << "};" << std::endl;
-   out << "   Double_t " << fYName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fY[i] << "," << std::endl;
-   out << "   " << fY[fNpoints-1] << "};" << std::endl;
-   out << "   Double_t " << fElXName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fEXlow[i] << "," << std::endl;
-   out << "   " << fEXlow[fNpoints-1] << "};" << std::endl;
-   out << "   Double_t " << fElYName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fEYlow[i] << "," << std::endl;
-   out << "   " << fEYlow[fNpoints-1] << "};" << std::endl;
-   out << "   Double_t " << fEhXName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fEXhigh[i] << "," << std::endl;
-   out << "   " << fEXhigh[fNpoints-1] << "};" << std::endl;
-   out << "   Double_t " << fEhYName << "[" << fNpoints << "] = {" << std::endl;
-   for (i = 0; i < fNpoints-1; i++) out << "   " << fEYhigh[i] << "," << std::endl;
-   out << "   " << fEYhigh[fNpoints-1] << "};" << std::endl;
+   auto fXName   = SaveArray(out, "fx", frameNumber, fX);
+   auto fYName   = SaveArray(out, "fy", frameNumber, fY);
+   auto fElXName = SaveArray(out, "felx", frameNumber, fEXlow);
+   auto fElYName = SaveArray(out, "fely", frameNumber, fEYlow);
+   auto fEhXName = SaveArray(out, "fehx", frameNumber, fEXhigh);
+   auto fEhYName = SaveArray(out, "fehy", frameNumber, fEYhigh);
 
-   if (gROOT->ClassSaved(TGraphAsymmErrors::Class())) out<<"   ";
-   else out << "   TGraphAsymmErrors *";
+   if (gROOT->ClassSaved(TGraphAsymmErrors::Class()))
+      out<<"   ";
+   else
+      out << "   TGraphAsymmErrors *";
    out << "grae = new TGraphAsymmErrors("<< fNpoints << ","
                                     << fXName   << ","  << fYName  << ","
                                     << fElXName  << ","  << fEhXName << ","
                                     << fElYName  << ","  << fEhYName << ");"
                                     << std::endl;
 
-   out << "   grae->SetName(" << quote << GetName() << quote << ");" << std::endl;
-   out << "   grae->SetTitle(" << quote << GetTitle() << quote << ");" << std::endl;
-
-   SaveFillAttributes(out, "grae", 0, 1001);
-   SaveLineAttributes(out, "grae", 1, 1, 1);
-   SaveMarkerAttributes(out, "grae", 1, 1, 1);
-
-   if (fHistogram) {
-      TString hname = fHistogram->GetName();
-      hname += frameNumber;
-      fHistogram->SetName(Form("Graph_%s",hname.Data()));
-      fHistogram->SavePrimitive(out,"nodraw");
-      out<<"   grae->SetHistogram("<<fHistogram->GetName()<<");"<<std::endl;
-      out<<"   "<<std::endl;
-   }
-
-   // save list of functions
-   TIter next(fFunctions);
-   TObject *obj;
-   while ((obj = next())) {
-      obj->SavePrimitive(out, Form("nodraw #%d\n",++frameNumber));
-      if (obj->InheritsFrom("TPaveStats")) {
-         out << "   grae->GetListOfFunctions()->Add(ptstats);" << std::endl;
-         out << "   ptstats->SetParent(grae->GetListOfFunctions());" << std::endl;
-      } else {
-         TString objname;
-         objname.Form("%s%d",obj->GetName(),frameNumber);
-         if (obj->InheritsFrom("TF1")) {
-            out << "   " << objname << "->SetParent(grae);\n";
-         }
-         out << "   grae->GetListOfFunctions()->Add("
-             << objname << ");" << std::endl;
-      }
-   }
-
-   const char *l = strstr(option,"multigraph");
-   if (l) {
-      out<<"   multigraph->Add(grae,"<<quote<<l+10<<quote<<");"<<std::endl;
-   } else {
-      out<<"   grae->Draw("<<quote<<option<<quote<<");"<<std::endl;
-   }
+   SaveHistogramAndFunctions(out, "grae", frameNumber, option);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1348,6 +1298,11 @@ void TGraphAsymmErrors::Scale(Double_t c1, Option_t *option)
 
 void TGraphAsymmErrors::SetPointError(Double_t exl, Double_t exh, Double_t eyl, Double_t eyh)
 {
+   if (!gPad) {
+      Error("SetPointError", "Cannot be used without gPad, requires last mouse position");
+      return;
+   }
+
    Int_t px = gPad->GetEventX();
    Int_t py = gPad->GetEventY();
 
@@ -1377,7 +1332,7 @@ void TGraphAsymmErrors::SetPointError(Int_t i, Double_t exl, Double_t exh, Doubl
 {
    if (i < 0) return;
    if (i >= fNpoints) {
-   // re-allocate the object
+      // re-allocate the object
       TGraphAsymmErrors::SetPoint(i,0,0);
    }
    fEXlow[i]  = exl;
@@ -1394,7 +1349,7 @@ void TGraphAsymmErrors::SetPointEXlow(Int_t i, Double_t exl)
 {
    if (i < 0) return;
    if (i >= fNpoints) {
-   // re-allocate the object
+      // re-allocate the object
       TGraphAsymmErrors::SetPoint(i,0,0);
    }
    fEXlow[i]  = exl;
@@ -1408,7 +1363,7 @@ void TGraphAsymmErrors::SetPointEXhigh(Int_t i, Double_t exh)
 {
    if (i < 0) return;
    if (i >= fNpoints) {
-   // re-allocate the object
+      // re-allocate the object
       TGraphAsymmErrors::SetPoint(i,0,0);
    }
    fEXhigh[i]  = exh;
@@ -1506,3 +1461,32 @@ void TGraphAsymmErrors::SwapPoints(Int_t pos1, Int_t pos2)
    SwapValues(fEYhigh, pos1, pos2);
    TGraph::SwapPoints(pos1, pos2);
 }
+
+////////////////////////////////////////////////////////////////////////////////
+/// Update the fX, fY, fEXlow, fEXhigh, fEYlow and fEYhigh arrays with the sorted values.
+
+void TGraphAsymmErrors::UpdateArrays(const std::vector<Int_t> &sorting_indices, Int_t numSortedPoints, Int_t low)
+{
+   std::vector<Double_t> fEXlowSorted(numSortedPoints);
+   std::vector<Double_t> fEXhighSorted(numSortedPoints);
+   std::vector<Double_t> fEYlowSorted(numSortedPoints);
+   std::vector<Double_t> fEYhighSorted(numSortedPoints);
+
+   // Fill the sorted X and Y error values based on the sorted indices
+   std::generate(fEXlowSorted.begin(), fEXlowSorted.end(),
+                 [begin = low, &sorting_indices, this]() mutable { return fEXlow[sorting_indices[begin++]]; });
+   std::generate(fEXhighSorted.begin(), fEXhighSorted.end(),
+                 [begin = low, &sorting_indices, this]() mutable { return fEXhigh[sorting_indices[begin++]]; });
+   std::generate(fEYlowSorted.begin(), fEYlowSorted.end(),
+                 [begin = low, &sorting_indices, this]() mutable { return fEYlow[sorting_indices[begin++]]; });
+   std::generate(fEYhighSorted.begin(), fEYhighSorted.end(),
+                 [begin = low, &sorting_indices, this]() mutable { return fEYhigh[sorting_indices[begin++]]; });
+
+   // Copy the sorted X and Y error values back to the original arrays
+   std::copy(fEXlowSorted.begin(), fEXlowSorted.end(), fEXlow + low);
+   std::copy(fEXhighSorted.begin(), fEXhighSorted.end(), fEXhigh + low);
+   std::copy(fEYlowSorted.begin(), fEYlowSorted.end(), fEYlow + low);
+   std::copy(fEYhighSorted.begin(), fEYhighSorted.end(), fEYhigh + low);
+
+   TGraph::UpdateArrays(sorting_indices, numSortedPoints, low);
+}
