@@ -13,6 +13,7 @@
 #include "TH2Poly.h"
 #include "TMultiGraph.h"
 #include "TGraph.h"
+#include "TInterpreter.h"
 #include "Riostream.h"
 #include "TList.h"
 #include "TMath.h"
@@ -546,19 +547,6 @@ void TH2Poly::ChangePartition(Int_t n, Int_t m)
    }
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Make a complete copy of the underlying object.  If 'newname' is set,
-/// the copy's name will be set to that name.
-
-TObject* TH2Poly::Clone(const char* newname) const
-{
-   // TH1::Clone relies on ::Copy to implemented by the derived class.
-   // Until this is implemented, revert to the much slower default version
-   // (and possibly non-thread safe).
-
-   return TNamed::Clone(newname);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Clears the contents of all bins in the histogram.
 
@@ -1344,7 +1332,8 @@ void TH2Poly::SavePrimitive(std::ostream &out, Option_t *option)
       histName += "__";
       histName += hcounter;
    }
-   const char *hname = histName.Data();
+
+   TString hname = gInterpreter->MapCppName(histName.Data());
 
    //Construct the class initialization
    out << hname << " = new " << ClassName() << "(\"" << hname << "\", \""
@@ -1360,8 +1349,7 @@ void TH2Poly::SavePrimitive(std::ostream &out, Option_t *option)
 
    while((obj = next())){
       th2pBin = (TH2PolyBin*) obj;
-      th2pBin->GetPolygon()->SavePrimitive(out,
-                                           TString::Format("th2poly%s",histName.Data()));
+      th2pBin->GetPolygon()->SavePrimitive(out, TString::Format("th2poly%s",hname.Data()));
    }
 
    // save bin contents
