@@ -525,6 +525,29 @@ TFitResultPtr THnBase::Fit(TF1 *f ,Option_t *option ,Option_t *goption)
    return ROOT::Fit::FitObject(this, f , fitOption , minOption, goption, range);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// \brief THnBase::GetBinCenter
+/// \param idx an array of bin index in each dimension.
+/// \return vector of bin centers in each dimension; empty in case of error.
+/// \note Throws error if size is different from nDimensions.
+/// \sa GetAxis(dim)::GetBinCenter(idx) as an alternative
+std::vector<Double_t> THnBase::GetBinCenter(const std::vector<Int_t> &idx) const
+{
+   if (idx.size() != static_cast<decltype(idx.size())>(fNdimensions)) {
+      Error("THnBase::GetBinCenter",
+            "Mismatched number of dimensions %d with bin index vector size %zu, returning an empty vector.",
+            fNdimensions, idx.size());
+      return {};
+   }
+   std::vector<Double_t> centers(fNdimensions);
+   std::generate(centers.begin(), centers.end(), [i = 0, &idx, this]() mutable {
+      auto bincenter = GetAxis(i)->GetBinCenter(idx[i]);
+      i++;
+      return bincenter;
+   });
+   return centers;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Generate an n-dimensional random tuple based on the histogrammed
 /// distribution. If subBinRandom, the returned tuple will be additionally
@@ -797,14 +820,10 @@ void THnBase::AddInternal(const THnBase* h, Double_t c, Bool_t rebinned)
    Long64_t i = 0;
    std::unique_ptr<ROOT::Internal::THnBaseBinIter> iter{h->CreateIter(false)};
    // Add to this whatever is found inside the other histogram
-   const bool sparse = InheritsFrom(THnSparse::Class());
    while ((i = iter->Next(coord)) >= 0) {
       // Get the content of the bin from the second histogram
       Double_t v = h->GetBinContent(i);
-      Double_t err2 = haveErrors ? h->GetBinError2(i) * c * c : 0;
-      if (sparse && v == 0 && err2 == 0) {
-         continue;
-      }
+
       Long64_t mybinidx = -1;
       if (rebinned) {
          // Get the bin center given a coord
@@ -817,6 +836,7 @@ void THnBase::AddInternal(const THnBase* h, Double_t c, Bool_t rebinned)
       }
 
       if (haveErrors) {
+         Double_t err2 = h->GetBinError2(i) * c * c;
          AddBinError2(mybinidx, err2);
       }
       // only _after_ error calculation, or sqrt(v) is taken into account!
@@ -1315,6 +1335,24 @@ void THnBase::ResetBase(Option_t * /*option = ""*/)
    }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// \brief Compute integral (sum of counts) of histogram in all dimensions
+/// \param respectAxisRange if false, count all bins including under/overflows,
+///                         if true, restrict sum to the user-set axis range
+/// \sa Projection(0)::Integral() as alternative
+/// \note this function is different from ComputeIntegral, that is a normalized
+/// cumulative sum
+Double_t THnBase::Integral(Bool_t respectAxisRange) const
+{
+   Long64_t myLinBin = 0;
+   std::unique_ptr<ROOT::Internal::THnBaseBinIter> iter{CreateIter(respectAxisRange)};
+   Double_t sum = 0.;
+   while ((myLinBin = iter->Next()) >= 0) {
+      sum += GetBinContent(myLinBin);
+   }
+   return sum;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 ///  Compute integral (normalized cumulative sum of bins) w/o under/overflows
 ///  The result is stored in fIntegral and used by the GetRandom functions.
