@@ -91,15 +91,6 @@ namespace cling {
     //  Tell the parser to not attempt spelling correction.
     //
     const_cast<LangOptions&>(PP.getLangOpts()).SpellChecking = 0;
-    //
-    //  Turn on ignoring of the main file eof token.
-    //
-    //  Note: We need this because token readahead in the following
-    //        routine calls ends up parsing it multiple times.
-    //
-    if (!PP.isIncrementalProcessingEnabled()) {
-      PP.enableIncrementalProcessing();
-    }
     assert(!code.empty() &&
            "prepareForParsing should only be called when need");
 
@@ -512,7 +503,7 @@ namespace cling {
                                    clang::AS_none, 0, &Attrs));
     if (Res.isUsable()) {
       // Accept it only if the whole name was parsed.
-      if (P.NextToken().getKind() == clang::tok::eof) {
+      if (P.NextToken().getKind() == clang::tok::annot_repl_input_end) {
         TypeSourceInfo* TSI = 0;
         TheQT = clang::Sema::GetTypeFromParser(Res.get(), &TSI);
       }
@@ -651,7 +642,7 @@ namespace cling {
         NestedNameSpecifier* NNS = SS.getScopeRep();
         NestedNameSpecifier::SpecifierKind Kind = NNS->getKind();
         // Only accept the parse if we consumed all of the name.
-        if (P.NextToken().getKind() == clang::tok::eof) {
+        if (P.NextToken().getKind() == clang::tok::annot_repl_input_end) {
           //
           //  Be careful, not all nested name specifiers refer to classes
           //  and namespaces, and those are the only things we want.
@@ -686,10 +677,11 @@ namespace cling {
                 // Note: Do we need to check for a dependent type here?
                 NestedNameSpecifier *prefix = NNS->getPrefix();
                 if (prefix) {
-                   QualType temp
-                     = Context.getElaboratedType(ETK_None,prefix,
-                                                 QualType(NNS->getAsType(),0));
-                   *setResultType = temp.getTypePtr();
+                  QualType temp =
+                      Context.getElaboratedType(ElaboratedTypeKeyword::None,
+                                                prefix,
+                                                QualType(NNS->getAsType(), 0));
+                  *setResultType = temp.getTypePtr();
                 } else {
                    *setResultType = NNS->getAsType();
                 }
@@ -758,7 +750,7 @@ namespace cling {
     //
     //  Cleanup after failed parse as a nested-name-specifier.
     //
-    P.SkipUntil(clang::tok::eof);
+    P.SkipUntil(clang::tok::annot_repl_input_end);
     // Doesn't reset the diagnostic mappings
     S.getDiagnostics().Reset(/*soft=*/true);
     //
@@ -786,7 +778,7 @@ namespace cling {
     if (P.getCurToken().getKind() == tok::annot_typename) {
       TypeResult T = P.getTypeAnnotation(const_cast<Token&>(P.getCurToken()));
       // Only accept the parse if we consumed all of the name.
-      if (P.NextToken().getKind() == clang::tok::eof)
+      if (P.NextToken().getKind() == clang::tok::annot_repl_input_end)
         if (!T.get().get().isNull()) {
           TypeSourceInfo *TSI = 0;
           clang::QualType QT =
@@ -1689,7 +1681,7 @@ namespace cling {
       // ParseTypeName might trigger deserialization.
       Interpreter::PushTransactionRAII TforDeser(Interp);
       unsigned int nargs = 0;
-      while (P.getCurToken().isNot(tok::eof)) {
+      while (P.getCurToken().isNot(tok::annot_repl_input_end)) {
         TypeResult Res(P.ParseTypeName());
         if (!Res.isUsable()) {
           // Bad parse, done.
@@ -1723,14 +1715,14 @@ namespace cling {
         Expr* val = (OpaqueValueExpr*)( &ExprMemory[slot] );
         GivenArgs.push_back(val);
       }
-      if (P.getCurToken().isNot(tok::eof)) {
+      if (P.getCurToken().isNot(tok::annot_repl_input_end)) {
         // We did not consume all of the prototype, bad parse.
         return false;
       }
       //
       //  Cleanup after prototype parse.
       //
-      P.SkipUntil(clang::tok::eof);
+      P.SkipUntil(clang::tok::annot_repl_input_end);
       // Doesn't reset the diagnostic mappings
       Sema& S = P.getActions();
       S.getDiagnostics().Reset(/*soft=*/true);
@@ -1967,7 +1959,7 @@ namespace cling {
       std::string proto;
       {
         bool first_time = true;
-        while (P.getCurToken().isNot(tok::eof)) {
+        while (P.getCurToken().isNot(tok::annot_repl_input_end)) {
           ExprResult Res = P.ParseAssignmentExpression();
           if (Res.isUsable()) {
             Expr* expr = Res.get();
@@ -1991,14 +1983,14 @@ namespace cling {
       }
       // For backward compatibility with CINT accept (for now?) a trailing close
       // parenthesis.
-      if (P.getCurToken().isNot(tok::eof) && P.getCurToken().isNot(tok::r_paren) ) {
+      if (P.getCurToken().isNot(tok::annot_repl_input_end) && P.getCurToken().isNot(tok::r_paren) ) {
         // We did not consume all of the arg list, bad parse.
         return false;
       }
       //
       //  Cleanup after the arg list parse.
       //
-      P.SkipUntil(clang::tok::eof);
+      P.SkipUntil(clang::tok::annot_repl_input_end);
       // Doesn't reset the diagnostic mappings
       S.getDiagnostics().Reset(/*soft=*/true);
       return true;
@@ -2042,7 +2034,7 @@ namespace cling {
     //
     {
       bool hasUnusableResult = false;
-      while (P.getCurToken().isNot(tok::eof)) {
+      while (P.getCurToken().isNot(tok::annot_repl_input_end)) {
         ExprResult Res = P.ParseAssignmentExpression();
         if (Res.isUsable()) {
           argExprs.push_back(Res.get());
