@@ -250,23 +250,22 @@ namespace cling {
       m_Interpreter.enableRawInput(mode);
   }
 
-  void MetaSema::actOndebugCommand(llvm::Optional<int> mode) const {
-    constexpr clang::codegenoptions::DebugInfoKind DebugInfo[] = {
-      clang::codegenoptions::NoDebugInfo,
-      clang::codegenoptions::LocTrackingOnly,
-      clang::codegenoptions::DebugLineTablesOnly,
-      clang::codegenoptions::LimitedDebugInfo,
-      clang::codegenoptions::FullDebugInfo
-    };
+  void MetaSema::actOndebugCommand(std::optional<int> mode) const {
+    constexpr llvm::codegenoptions::DebugInfoKind DebugInfo[] = {
+        llvm::codegenoptions::NoDebugInfo,
+        llvm::codegenoptions::LocTrackingOnly,
+        llvm::codegenoptions::DebugLineTablesOnly,
+        llvm::codegenoptions::LimitedDebugInfo,
+        llvm::codegenoptions::FullDebugInfo};
     constexpr int N = (int)std::extent<decltype(DebugInfo)>::value;
 
     clang::CodeGenOptions& CGO = m_Interpreter.getCI()->getCodeGenOpts();
     if (!mode) {
-      bool flag = (CGO.getDebugInfo() == clang::codegenoptions::NoDebugInfo);
+      bool flag = (CGO.getDebugInfo() == llvm::codegenoptions::NoDebugInfo);
       if (flag)
-        CGO.setDebugInfo(clang::codegenoptions::LimitedDebugInfo);
+        CGO.setDebugInfo(llvm::codegenoptions::LimitedDebugInfo);
       else
-        CGO.setDebugInfo(clang::codegenoptions::NoDebugInfo);
+        CGO.setDebugInfo(llvm::codegenoptions::NoDebugInfo);
       m_MetaProcessor.getOuts() << (flag ? "G" : "Not g") << "enerating debug symbols\n";
     } else {
       mode = (*mode < 0) ? 0 : ((*mode >= N) ? N - 1 : *mode);
@@ -406,7 +405,7 @@ namespace cling {
 
     for (clang::SourceManager::fileinfo_iterator I = SM.fileinfo_begin(),
            E = SM.fileinfo_end(); I != E; ++I) {
-      m_MetaProcessor.getOuts() << (*I).first->getName();
+      m_MetaProcessor.getOuts() << (*I).first.getName();
       m_MetaProcessor.getOuts() << "\n";
     }
 #if 0
