@@ -15,5 +15,5 @@
 
 class MyHeaderFileProtectedClass {};
 #undef P
-error_here; // expected-error {{C++ requires a type specifier for all declarations}}
+error_here; // expected-error {{use of undeclared identifier}}
 #endif // HEADER_FILE_PROTECTOR
