@@ -1179,17 +1179,20 @@ void TDirectoryFile::ls(Option_t *option) const
    TString opt  = opta.Strip(TString::kBoth);
    Bool_t memobj  = kTRUE;
    Bool_t diskobj = kTRUE;
-   TString reg = "*";
+   TString reg;
    if (opt.BeginsWith("-m")) {
       diskobj = kFALSE;
-      if (opt.Length() > 2)
+      if (opt.Length() > 2) {
          reg = opt(2,opt.Length());
+      }
    } else if (opt.BeginsWith("-d")) {
       memobj  = kFALSE;
-      if (opt.Length() > 2)
+      if (opt.Length() > 2) {
          reg = opt(2,opt.Length());
-   } else if (!opt.IsNull())
+      }
+   } else if (!opt.IsNull()) {
       reg = opt;
+   }
 
    TRegexp re(reg, kTRUE);
 
@@ -1198,7 +1201,8 @@ void TDirectoryFile::ls(Option_t *option) const
       TIter nextobj(fList);
       while ((obj = (TObject *) nextobj())) {
          TString s = obj->GetName();
-         if (s.Index(re) == kNPOS) continue;
+         if (!reg.IsNull() && s.Index(re) == kNPOS)
+            continue;
          obj->ls(option);            //*-* Loop on all the objects in memory
       }
    }
@@ -1208,7 +1212,8 @@ void TDirectoryFile::ls(Option_t *option) const
       for (TObjLink *lnk = fKeys->FirstLink(); lnk != nullptr; lnk = lnk->Next()) {
          TKey *key = (TKey*)lnk->GetObject();
          TString s = key->GetName();
-         if (s.Index(re) == kNPOS) continue;
+         if (!reg.IsNull() && s.Index(re) == kNPOS)
+            continue;
          bool first = (lnk->Prev() == nullptr) || (s != lnk->Prev()->GetObject()->GetName());
          bool hasbackup = (lnk->Next() != nullptr) && (s == lnk->Next()->GetObject()->GetName());
          if (first)
@@ -1577,7 +1582,7 @@ void TDirectoryFile::Save()
 /// By default a message is printed. Use option "q" to not print the message.
 /// If filename contains ".json" extension, JSON representation of the object
 /// will be created and saved in the text file. Such file can be used in
-/// JavaScript ROOT (https://root.cern.ch/js/) to display object in web browser
+/// JavaScript ROOT (https://root.cern/js/) to display object in web browser
 /// When creating JSON file, option string may contain compression level from 0 to 3 (default 0)
 
 Int_t TDirectoryFile::SaveObjectAs(const TObject *obj, const char *filename, Option_t *option) const
@@ -1941,6 +1946,8 @@ Int_t TDirectoryFile::WriteTObject(const TObject *obj, const char *name, Option_
    Int_t nch = strlen(oname);
    char *newName = nullptr;
    if (nch && oname[nch-1] == ' ') {
+      Warning("WriteTObject", "The key name '%s' will be stored in file without the trailing blanks.", obj->GetName());
+      // See https://github.com/root-project/root/issues/17003
       newName = new char[nch+1];
       strlcpy(newName,oname,nch+1);
       for (Int_t i=0;i<nch;i++) {
