@@ -43,12 +43,12 @@ public:
    {
    }
    // Standard Destructor
-   virtual ~TGenVectorProxy()
-{
+   ~TGenVectorProxy() override
+   {
    }
    // Return the address of the value at index 'idx'
-   virtual void* At(UInt_t idx)
-{
+   void* At(UInt_t idx)  override
+   {
       if ( fEnv && fEnv->fObject ) {
          fEnv->fIdx = idx;
          switch( idx ) {
@@ -60,10 +60,10 @@ public:
          }
       }
       Fatal("TGenVectorProxy","At> Logic error - no proxy object set.");
-      return 0;
+      return nullptr;
    }
    // Call to delete/destruct individual item
-   virtual void DeleteItem(Bool_t force, void* ptr) const
+   void DeleteItem(Bool_t force, void* ptr) const override
    {
       if ( force && ptr ) {
          if ( fVal->fProperties&kNeedDelete) {
@@ -93,11 +93,11 @@ public:
    {
       // Standard Constructor.
    }
-   virtual ~TGenVectorBoolProxy()
+   ~TGenVectorBoolProxy() override
    {
       // Standard Destructor.
    }
-   virtual void* At(UInt_t idx)
+   void* At(UInt_t idx) override
    {
       // Return the address of the value at index 'idx'
 
@@ -109,10 +109,10 @@ public:
          return &fLastValue;
       }
       Fatal("TGenVectorProxy","At> Logic error - no proxy object set.");
-      return 0;
+      return nullptr;
    }
 
-   virtual void DeleteItem(Bool_t force, void* ptr) const
+   void DeleteItem(Bool_t force, void* ptr) const override
    {
       // Call to delete/destruct individual item
       if ( force && ptr ) {
@@ -139,11 +139,11 @@ public:
    {
       // Standard Constructor.
    }
-   virtual ~TGenBitsetProxy()
+   ~TGenBitsetProxy() override
    {
       // Standard Destructor.
    }
-   virtual void* At(UInt_t idx)
+   void* At(UInt_t idx) override
    {
       // Return the address of the value at index 'idx'
 
@@ -168,7 +168,7 @@ public:
       return 0;
    }
 
-   virtual void DeleteItem(Bool_t force, void* ptr) const
+   void DeleteItem(Bool_t force, void* ptr) const override
    {
       // Call to delete/destruct individual item
       if ( force && ptr ) {
@@ -191,15 +191,15 @@ class TGenListProxy : public TGenVectorProxy {
 public:
    // Standard Destructor
    TGenListProxy(const TGenCollectionProxy& c) : TGenVectorProxy(c)
-{
+   {
    }
    // Standard Destructor
-   virtual ~TGenListProxy()
-{
+   ~TGenListProxy() override
+   {
    }
    // Return the address of the value at index 'idx'
-   void* At(UInt_t idx)
-{
+   void* At(UInt_t idx) override
+   {
       if ( fEnv && fEnv->fObject ) {
          switch( idx ) {
          case 0:
@@ -215,7 +215,7 @@ public:
          }
       }
       Fatal("TGenListProxy","At> Logic error - no proxy object set.");
-      return 0;
+      return nullptr;
    }
 };
 
@@ -233,15 +233,15 @@ class TGenSetProxy : public TGenVectorProxy {
 public:
    // Standard Destructor
    TGenSetProxy(const TGenCollectionProxy& c) : TGenVectorProxy(c)
-{
+   {
    }
    // Standard Destructor
-   virtual ~TGenSetProxy()
-{
+   ~TGenSetProxy() override
+   {
    }
    // Return the address of the value at index 'idx'
-   void* At(UInt_t idx)
-{
+   void* At(UInt_t idx) override
+   {
       if ( fEnv && fEnv->fObject ) {
          if ( fEnv->fUseTemp ) {
             return (((char*)fEnv->fTemp)+idx*fValDiff);
@@ -260,7 +260,7 @@ public:
          }
       }
       Fatal("TGenSetProxy","At> Logic error - no proxy object set.");
-      return 0;
+      return nullptr;
    }
 };
 
@@ -278,14 +278,14 @@ class TGenMapProxy : public TGenSetProxy {
 public:
    // Standard Destructor
    TGenMapProxy(const TGenCollectionProxy& c) : TGenSetProxy(c)
-{
+   {
    }
    // Standard Destructor
-   virtual ~TGenMapProxy()
-{
+   ~TGenMapProxy() override
+   {
    }
    // Call to delete/destruct individual item
-   virtual void DeleteItem(Bool_t force, void* ptr) const
+   void DeleteItem(Bool_t force, void* ptr) const override
    {
       if (force) {
          if ( fKey->fProperties&kNeedDelete) {
@@ -325,16 +325,15 @@ TGenCollectionProxy::Value::Value(const std::string& inside_type, Bool_t silent,
    fKind = kNoType_t;
 
    // Let's treat the unique_ptr case
-   bool nameChanged = false;
-   std::string intype = TClassEdit::GetNameForIO(inside.c_str(), TClassEdit::EModType::kNone, &nameChanged);
+   std::string intype = TClassEdit::GetNameForIO(inside, TClassEdit::EModType::kNone);
 
-   bool isPointer = nameChanged; // unique_ptr is considered a pointer
+   bool isPointer = false;
    // The incoming name is normalized (it comes from splitting the name of a TClass),
    // so all we need to do is drop the last trailing star (if any) and record that information.
-   if (!nameChanged && intype[intype.length()-1] == '*') {
+   if (intype.back() == '*') {
       isPointer = true;
       intype.pop_back();
-      if (intype[intype.length()-1] == '*') {
+      if (intype.back() == '*') {
          // The value is a pointer to a pointer
          if (!silent)
             Warning("TGenCollectionProxy::Value::Value", "I/O not supported for collection of pointer to pointer: %s", inside_type.c_str());
@@ -637,7 +636,7 @@ TGenCollectionProxy::TGenCollectionProxy(Info_t info, size_t iter_size)
 ////////////////////////////////////////////////////////////////////////////////
 /// Build a proxy for a collection whose type is described by 'collectionClass'.
 
-TGenCollectionProxy::TGenCollectionProxy(const ROOT::TCollectionProxyInfo &info, TClass *cl)
+TGenCollectionProxy::TGenCollectionProxy(const ROOT::Detail::TCollectionProxyInfo &info, TClass *cl)
    : TVirtualCollectionProxy(cl),
      fTypeinfo(info.fInfo), fOnFileClass(0)
 {
@@ -919,8 +918,7 @@ TGenCollectionProxy *TGenCollectionProxy::InitializeEx(Bool_t silent)
                               Fatal("InitializeEx",
                                     "The TClass creation for %s did not get the right size: %d instead of%d\n",
                                     nam.c_str(), (int)paircl->GetClassSize(), (int)fValDiff);
-                           paircl->ReplaceWith(newpaircl);
-                           delete paircl;
+                           newpaircl->ForceReload(paircl);
                         }
                      }
                   }
@@ -959,6 +957,12 @@ TGenCollectionProxy *TGenCollectionProxy::InitializeEx(Bool_t silent)
                }
                break;
          }
+         if (!(fProperties & kIsEmulated) && newfValue->fType && !newfValue->fType->IsSyntheticPair()) {
+            if (!newfValue->fType->IsLoaded() && !newfValue->fType->HasInterpreterInfo())
+               Error("TGenCollectionProxy::InitializeEx",
+                     "The TClass for %s used as the value type of the compiled collection proxy %s is not loaded.",
+                     newfValue->fType->GetName(), cl->GetName());
+         }
 
          fPointers = fPointers || (0 != (fVal->fCase&kIsPointer));
          if (fPointers || (0 != (fVal->fProperties&kNeedDelete))) {
