@@ -82,7 +82,9 @@ static bool IsUnsupportedUniquePointer(const char *normName, TDataMember *dm)
          return true;
       }
 
-      clm->BuildRealData();
+      // TODO: Is it not clear what situation we are checking for by checking if
+      // the unique_ptr class has any data members.
+      clm->BuildRealData(nullptr, /* istransient = */ true);
       auto upDms = clm->GetListOfRealData();
       if (!upDms) {
          Error("CloseStreamerInfoROOTFile", "Cannot determine unique pointer %s data members.", dmTypeName);
@@ -241,9 +243,9 @@ bool CloseStreamerInfoROOTFile(bool writeEmptyRootPCM)
    TObjArray enums(gEnumsToStore.size());
    for (const auto & enumname : gEnumsToStore) {
       TEnum *en = nullptr;
-      const size_t lastSepPos = enumname.find_last_of("::");
+      const size_t lastSepPos = enumname.rfind("::");
       if (lastSepPos != std::string::npos) {
-         const std::string nsName = enumname.substr(0, lastSepPos - 1);
+         const std::string nsName = enumname.substr(0, lastSepPos);
          TClass *tclassInstance = TClass::GetClass(nsName.c_str());
          if (!tclassInstance) {
             Error("CloseStreamerInfoROOTFile", "Cannot find TClass instance for namespace %s.", nsName.c_str());
@@ -256,7 +258,7 @@ bool CloseStreamerInfoROOTFile(bool writeEmptyRootPCM)
             Error("CloseStreamerInfoROOTFile", "TClass instance for namespace %s does not have any enum associated. This is an inconsistency.", nsName.c_str());
             return false;
          }
-         const std::string unqualifiedEnumName = enumname.substr(lastSepPos + 1);
+         const std::string unqualifiedEnumName = enumname.substr(lastSepPos + 2);
          en = (TEnum *)enumListPtr->FindObject(unqualifiedEnumName.c_str());
          if (en) en->SetTitle(nsName.c_str());
       } else {
