@@ -116,7 +116,7 @@ public:
      Note:  (0,0) refers to the XX component; (3,3) refers to the TT component.
   */
   template<class ForeignMatrix>
-  explicit LorentzRotation(const ForeignMatrix & m) { SetComponents(m); }
+  explicit constexpr LorentzRotation(const ForeignMatrix & m) { SetComponents(m); }
 
   /**
      Construct from four orthosymplectic vectors (which must have methods
