@@ -99,7 +99,7 @@ public:
    /**
       Default constructor
    */
-   Fitter ();
+   Fitter () {}
 
    /**
       Constructor from a result
@@ -108,21 +108,23 @@ public:
 
 
    /**
-      Destructor
+      Destructor.
+      Make it virtual in case users derive from Fitter class to extend it by adding new methods.
+      This is needed to avoid a warning seen when doing from Python
+      (see ROOT issue [#12391](https://github.com/root-project/root/issues/12391) ).
+      Note that the Fitter class does not provide virtual functions to be re-implemented by derived classes.
    */
-   ~Fitter ();
-
-private:
+   virtual ~Fitter () {}
 
    /**
       Copy constructor (disabled, class is not copyable)
    */
-   Fitter(const Fitter &);
+   Fitter(const Fitter &) = delete;
 
    /**
       Assignment operator (disabled, class is not copyable)
    */
-   Fitter & operator = (const Fitter & rhs);
+   Fitter & operator = (const Fitter &) = delete;
 
 
 public:
@@ -132,7 +134,8 @@ public:
        If data set is binned a least square fit is performed
        If data set is unbinned a maximum likelihood fit (not extended) is done
        Pre-requisite on the function:
-       it must implement the 1D or multidimensional parametric function interface
+       it must implement the 1D or multidimensional parametric function interface.
+       Note that both the input data and the function object are copied by the Fitter.
    */
    template <class Data, class Function,
              class cond = typename std::enable_if<!(std::is_same<Function, ROOT::EExecutionPolicy>::value ||
@@ -146,62 +149,91 @@ public:
    }
 
    /**
-       Fit a binned data set using a least square fit (default method)
+      Fit a binned data set using a least square fit.
+      Note that the provided input data are copied in the Fitter class.
+      Use the next function (passing a `shared_ptr` to the BinData class if you want to avoid
+      copying.
    */
    bool Fit(const BinData & data, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
-      SetData(data);
-      return DoLeastSquareFit(executionPolicy);
-   }
-   bool Fit(const std::shared_ptr<BinData> & data, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
-      SetData(data);
-      return DoLeastSquareFit(executionPolicy);
+      return LeastSquareFit(data, executionPolicy);
    }
 
    /**
-       Fit a binned data set using a least square fit
+      Fit a binned data set using a least square fit.
+      Pass the input data using a `shared_ptr` for NOT copying the input data.
    */
-   bool LeastSquareFit(const BinData & data) {
-      return Fit(data);
+   bool Fit(const std::shared_ptr<BinData> & data, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
+      return LeastSquareFit(data, executionPolicy);
    }
 
    /**
-       fit an unbinned data set using loglikelihood method
+       Fit a binned data set using a least square fit copying the input data.
+   */
+   bool LeastSquareFit(const BinData & data, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
+      SetData(data);
+      return DoLeastSquareFit(executionPolicy);
+   }
+   /**
+       Fit a binned data set using a least square fit NOT copying the input data.
+   */
+   bool LeastSquareFit(const std::shared_ptr<BinData> & data, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
+      SetData(data);
+      return DoLeastSquareFit(executionPolicy);
+   }
+
+   /**
+       Fit an un-binned data set using the negative log-likelihood method.
+       This function copies the input data.
    */
    bool Fit(const UnBinData & data, bool extended = false, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
-      SetData(data);
-      return DoUnbinnedLikelihoodFit(extended, executionPolicy);
+      return LikelihoodFit(data, extended, executionPolicy);
+   }
+   /**
+       Fit an un-binned data set using the negative log-likelihood method.
+       This function uses a `shared_ptr` to avoid copying the input data.
+   */
+   bool Fit(const std::shared_ptr<UnBinData> & data, bool extended = false, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
+      return LikelihoodFit(data, extended, executionPolicy);
    }
 
    /**
-      Binned Likelihood fit. Default is extended
+      Binned Likelihood fit copying the input data.
+      Default is extended.
     */
    bool LikelihoodFit(const BinData &data, bool extended = true,
                       const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
       SetData(data);
       return DoBinnedLikelihoodFit(extended, executionPolicy);
    }
-
+   /**
+      Binned Likelihood fit using a `shared_ptr` for NOT copying the input data.
+      Default is extended.
+    */
    bool LikelihoodFit(const std::shared_ptr<BinData> &data, bool extended = true,
                       const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
       SetData(data);
       return DoBinnedLikelihoodFit(extended, executionPolicy);
    }
    /**
-      Unbinned Likelihood fit. Default is not extended
+      Un-binned Likelihood fit copying the input data
+      Default is NOT extended
     */
    bool LikelihoodFit(const UnBinData & data, bool extended = false, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
       SetData(data);
       return DoUnbinnedLikelihoodFit(extended, executionPolicy);
    }
+   /**
+      Un-binned Likelihood fit using a `shared_ptr` for NOT copying the input data.
+      Default is NOT extended
+    */
    bool LikelihoodFit(const std::shared_ptr<UnBinData> & data, bool extended = false, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential) {
       SetData(data);
       return DoUnbinnedLikelihoodFit(extended, executionPolicy);
    }
 
-
    /**
-       fit a data set using any  generic model  function
-       Pre-requisite on the function:
+      Likelihood fit given a data set (Binned or Un-binned) using any  generic model  function.
+      This interface copies the input data and the model function object
    */
    template < class Data , class Function>
    bool LikelihoodFit( const Data & data, const Function & func, bool extended) {
@@ -210,18 +242,20 @@ public:
    }
 
    /**
-      do a linear fit on a set of bin-data
+      Do a linear fit copying the input data
     */
    bool LinearFit(const BinData & data) {
       SetData(data);
       return DoLinearFit();
    }
+   /**
+      Do a linear fit using a `shared_ptr` for NOT copying the input data
+    */
    bool LinearFit(const std::shared_ptr<BinData> & data) {
       SetData(data);
       return DoLinearFit();
    }
 
-
    /**
       Fit using the a generic FCN function as a C++ callable object implementing
       double () (const double *)
@@ -229,7 +263,7 @@ public:
       For the options see documentation for following methods FitFCN(IMultiGenFunction & fcn,..)
     */
    template <class Function>
-   bool FitFCN(unsigned int npar, Function  & fcn, const double * params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool FitFCN(unsigned int npar, Function  & fcn, const double * params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
       Set a generic FCN function as a C++ callable object implementing
@@ -238,13 +272,14 @@ public:
       For the options see documentation for following methods FitFCN(IMultiGenFunction & fcn,..)
     */
    template <class Function>
-   bool SetFCN(unsigned int npar, Function  & fcn, const double * params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool SetFCN(unsigned int npar, Function  & fcn, const double * params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
       Fit using the given FCN function represented by a multi-dimensional function interface
       (ROOT::Math::IMultiGenFunction).
-      Give optionally the initial arameter values, data size to have the fit Ndf correctly
-      set in the FitResult and flag specifying if it is a chi2 fit.
+      Give optionally the initial parameter values, data size to have the fit Ndf correctly
+      set in the FitResult and flag specifying the type of fit. The fitType can be:
+      0 undefined, 1 least square fit, 2 unbinned likelihood fit, 3 binned likelihood fit
       Note that if the parameters values are not given (params=0) the
       current parameter settings are used. The parameter settings can be created before
       by using the FitConfig::SetParamsSetting. If they have not been created they are created
@@ -252,20 +287,20 @@ public:
       Note that passing a params != 0 will set the parameter settings to the new value AND also the
       step sizes to some pre-defined value (stepsize = 0.3 * abs(parameter_value) )
     */
-   bool FitFCN(const ROOT::Math::IMultiGenFunction &fcn, const double *params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool FitFCN(const ROOT::Math::IMultiGenFunction &fcn, const double *params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
        Fit using a FitMethodFunction interface. Same as method above, but now extra information
        can be taken from the function class
    */
-   bool FitFCN(const ROOT::Math::FitMethodFunction & fcn, const double * params = 0);
+   bool FitFCN(const ROOT::Math::FitMethodFunction & fcn, const double *params = nullptr);
 
    /**
       Set the FCN function represented by a multi-dimensional function interface
       (ROOT::Math::IMultiGenFunction) and optionally the initial parameters
       See also note above for the initial parameters for FitFCN
     */
-   bool SetFCN(const ROOT::Math::IMultiGenFunction &fcn, const double *params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool SetFCN(const ROOT::Math::IMultiGenFunction &fcn, const double *params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
       Set the FCN function represented by a multi-dimensional function interface
@@ -274,51 +309,26 @@ public:
       With this interface we pass in addition a ModelFunction that will be attached to the FitResult and
       used to compute confidence interval of the fit
    */
-   bool SetFCN(const ROOT::Math::IMultiGenFunction &fcn, const IModelFunction & func, const double *params = 0,
-               unsigned int dataSize = 0, bool chi2fit = false);
+   bool SetFCN(const ROOT::Math::IMultiGenFunction &fcn, const IModelFunction & func, const double *params = nullptr,
+               unsigned int dataSize = 0, int fitType = 0);
 
    /**
        Set the objective function (FCN)  using a FitMethodFunction interface.
        Same as method above, but now extra information can be taken from the function class
    */
-   bool SetFCN(const ROOT::Math::FitMethodFunction & fcn, const double * params = 0);
-
-   /**
-      Fit using the given FCN function representing a multi-dimensional gradient function
-      interface (ROOT::Math::IMultiGradFunction). In this case the minimizer will use the
-      gradient information provided by the function.
-      For the options same consideration as in the previous method
-    */
-   bool FitFCN(const ROOT::Math::IMultiGradFunction &fcn, const double *params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool SetFCN(const ROOT::Math::FitMethodFunction & fcn, const double *params = nullptr);
 
    /**
        Fit using a FitMethodGradFunction interface. Same as method above, but now extra information
        can be taken from the function class
    */
-   bool FitFCN(const ROOT::Math::FitMethodGradFunction & fcn, const double * params = 0);
-
-   /**
-      Set the FCN function represented by a multi-dimensional gradient function interface
-      (ROOT::Math::IMultiGradFunction) and optionally the initial parameters
-      See also note above for the initial parameters for FitFCN
-    */
-   bool SetFCN(const ROOT::Math::IMultiGradFunction &fcn, const double *params = 0, unsigned int dataSize = 0, bool chi2fit = false);
-
-   /**
-      Set the FCN function represented by a multi-dimensional gradient function interface
-     (ROOT::Math::IMultiGradFunction) and optionally the initial parameters
-      See also note above for the initial parameters for FitFCN
-      With this interface we pass in addition a ModelFunction that will be attached to the FitResult and
-      used to compute confidence interval of the fit
-   */
-   bool SetFCN(const ROOT::Math::IMultiGradFunction &fcn, const IModelFunction &func, const double *params = 0,
-               unsigned int dataSize = 0, bool chi2fit = false);
+   bool FitFCN(const ROOT::Math::FitMethodGradFunction & fcn, const double *params = nullptr);
 
    /**
        Set the objective function (FCN)  using a FitMethodGradFunction interface.
        Same as method above, but now extra information can be taken from the function class
    */
-   bool SetFCN(const ROOT::Math::FitMethodGradFunction & fcn, const double * params = 0);
+   bool SetFCN(const ROOT::Math::FitMethodGradFunction & fcn, const double *params = nullptr);
 
 
    /**
@@ -327,14 +337,14 @@ public:
       For the options same consideration as in the previous method
     */
    typedef  void (* MinuitFCN_t )(int &npar, double *gin, double &f, double *u, int flag);
-   bool FitFCN( MinuitFCN_t fcn, int npar = 0, const double * params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool FitFCN( MinuitFCN_t fcn, int npar = 0, const double *params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
       set objective function using user provided FCN with Minuit-like interface
       If npar = 0 it is assumed that the parameters are specified in the parameter settings created before
       For the options same consideration as in the previous method
     */
-   bool SetFCN( MinuitFCN_t fcn, int npar = 0, const double * params = 0, unsigned int dataSize = 0, bool chi2fit = false);
+   bool SetFCN( MinuitFCN_t fcn, int npar = 0, const double *params = nullptr, unsigned int dataSize = 0, int fitType = 0);
 
    /**
       Perform a fit with the previously set FCN function. Require SetFCN before
@@ -389,7 +399,7 @@ public:
 
    /**
       perform an error analysis on the result using the Hessian
-      Errors are obtaied from the inverse of the Hessian matrix
+      Errors are obtained from the inverse of the Hessian matrix
       To be called only after fitting and when a minimizer supporting the Hessian calculations is used
       otherwise an error (false) is returned.
       A new  FitResult with the Hessian result will be produced
@@ -401,7 +411,7 @@ public:
       To be called only after fitting and when a minimizer supporting MINOS is used
       otherwise an error (false) is returned.
       The result will be appended in the fit result class
-      Optionally a vector of parameter indeces can be passed for selecting
+      Optionally a vector of parameter indices can be passed for selecting
       the parameters to analyse using FitConfig::SetMinosErrors
     */
    bool CalculateMinosErrors();
@@ -417,15 +427,15 @@ public:
    FitConfig & Config() { return fConfig; }
 
    /**
-      query if fit is binned. In cse of false teh fit can be unbinned
-      or is not defined (like in case of fitting through a ::FitFCN)
+      query if fit is binned. In cse of false the fit can be unbinned
+      or is not defined (like in case of fitting through a ROOT::Fit::Fitter::FitFCN)
     */
    bool IsBinFit() const { return fBinFit; }
 
    /**
       return pointer to last used minimizer
       (is NULL in case fit is not yet done)
-      This pointer is guranteed to be valid as far as the fitter class is valid and a new fit is not redone.
+      This pointer is guaranteed to be valid as far as the fitter class is valid and a new fit is not redone.
       To be used only after fitting.
       The pointer should not be stored and will be invalided after performing a new fitting.
       In this case a new instance of ROOT::Math::Minimizer will be re-created and can be
@@ -442,19 +452,36 @@ public:
       In this case a new instance of the function pointer will be re-created and can be
       obtained calling again GetFCN()
     */
-   ROOT::Math::IMultiGenFunction * GetFCN() const { return fObjFunction.get(); }
+   ROOT::Math::IMultiGenFunction * GetFCN() const {
+      return fObjFunction.get();
+    }
 
 
    /**
       apply correction in the error matrix for the weights for likelihood fits
       This method can be called only after a fit. The
-      passed function (loglw2) is a log-likelihood function impelemented using the
+      passed function (loglw2) is a log-likelihood function implemented using the
       sum of weight squared
       When using FitConfig.SetWeightCorrection() this correction is applied
       automatically when doing a likelihood fit (binned or unbinned)
    */
    bool ApplyWeightCorrection(const ROOT::Math::IMultiGenFunction & loglw2, bool minimizeW2L=false);
 
+   /// Set number of fit points when using an external FCN function
+   /// This function can be called after Fit to set the correct number of Ndf in FitResult
+   void SetNumberOfFitPoints(unsigned int npoints) {
+      if (fExtObjFunction) fDataSize = npoints;
+      if (!fResult->IsEmpty()) fResult->SetChi2AndNdf(-1,npoints);
+   }
+
+   /// Set the type of fit when using an external FCN
+   /// possible types are : 1 (least-square), 2 (unbinned-likelihood), 3 (binned-likelihood)
+   /// Note that in case of binned likelihood fit the chi2 will be computed as 2 * MinFCN()
+   /// Note this function should be called before fitting to have effect on th FitResult
+   void SetFitType(int type) {
+      if (fExtObjFunction) fFitType = type;
+   }
+
 
 protected:
 
@@ -467,13 +494,20 @@ protected:
    bool DoUnbinnedLikelihoodFit( bool extended = false, const ROOT::EExecutionPolicy &executionPolicy = ROOT::EExecutionPolicy::kSequential);
    /// linear least square fit
    bool DoLinearFit();
+   /// Set Objective function
+   bool DoSetFCN(bool useExtFCN, const ROOT::Math::IMultiGenFunction &fcn, const double *params, unsigned int dataSize,
+                 int fitType);
 
    // initialize the minimizer
    bool DoInitMinimizer();
    /// do minimization
-   bool DoMinimization(const BaseFunc & f, const ROOT::Math::IMultiGenFunction * chifunc = 0);
-   // do minimization after having set obj function
-   bool DoMinimization(const ROOT::Math::IMultiGenFunction * chifunc = 0);
+   template<class ObjFunc_t>
+   bool DoMinimization(std::unique_ptr<ObjFunc_t> f, const ROOT::Math::IMultiGenFunction * chifunc = nullptr);
+   // do minimization for weighted likelihood fits
+   template<class ObjFunc_t>
+   bool DoWeightMinimization(std::unique_ptr<ObjFunc_t> f, const ROOT::Math::IMultiGenFunction * chifunc = nullptr);
+   // do minimization after having set the objective function
+   bool DoMinimization(const ROOT::Math::IMultiGenFunction * chifunc = nullptr);
    // update config after fit
    void DoUpdateFitConfig();
    // update minimizer options for re-fitting
@@ -481,23 +515,19 @@ protected:
    // get function calls from the FCN
    int GetNCallsFromFCN();
 
-   //set data for the fit
-   void SetData(const FitData & data) {
-      fData = std::shared_ptr<FitData>(const_cast<FitData*>(&data),DummyDeleter<FitData>());
-   }
-   // set data and function without cloning them
-   template <class T>
-   void SetFunctionAndData(const IModelFunctionTempl<T> & func, const FitData & data) {
-      SetData(data);
-      fFunc = std::shared_ptr<IModelFunctionTempl<T>>(const_cast<IModelFunctionTempl<T>*>(&func),DummyDeleter<IModelFunctionTempl<T>>());
-   }
-
-   //set data for the fit using a shared ptr
+   /// Set the input data for the fit using a shared ptr (No Copying)
    template <class Data>
    void SetData(const std::shared_ptr<Data> & data) {
       fData = std::static_pointer_cast<Data>(data);
    }
 
+   /// Set the input data for the fit (Copying the given data object)
+   template <class Data>
+   void SetData(const Data & data) {
+      auto dataClone = std::make_shared<Data>(data);
+      SetData(dataClone);
+   }
+
    /// look at the user provided FCN and get data and model function is
    /// they derive from ROOT::Fit FCN classes
    void ExamineFCN();
@@ -508,32 +538,42 @@ protected:
    template <class ObjFuncType>
    bool GetDataFromFCN();
 
+   /// Return pointer to the used objective function for fitting.
+   /// If using an external function (e.g. given in SetFCN), return the cached pointer,
+   /// otherwise use the one stored as shared ptr and managed by the Fitter class
+   const ROOT::Math::IBaseFunctionMultiDimTempl<double> * ObjFunction() const {
+      // need to specify here full return type since when using the typedef (IMultiGenFunction)
+      // there is an error when using the class in Python (see issue #12391)
+      return (fExtObjFunction) ? fExtObjFunction : fObjFunction.get();
+   }
 
 private:
 
-   bool fUseGradient;       // flag to indicate if using gradient or not
+   bool fUseGradient = false;  ///< flag to indicate if using gradient or not
 
-   bool fBinFit;            // flag to indicate if fit is binned
-                            // in case of false the fit is unbinned or undefined)
-                            // flag it is used to compute chi2 for binned likelihood fit
+   bool fBinFit = false;    ///< flag to indicate if fit is binned
+                            ///< in case of false the fit is unbinned or undefined)
+                            ///< flag it is used to compute chi2 for binned likelihood fit
 
-   int fFitType;   // type of fit   (0 undefined, 1 least square, 2 likelihood)
+   int fFitType = 0;   ///< type of fit   (0 undefined, 1 least square, 2 likelihood, 3 binned likelihood)
 
-   int fDataSize;  // size of data sets (need for Fumili or LM fitters)
+   int fDataSize = 0;  ///< size of data sets (need for Fumili or LM fitters)
 
-   FitConfig fConfig;       // fitter configuration (options and parameter settings)
+   FitConfig fConfig;       ///< fitter configuration (options and parameter settings)
 
-   std::shared_ptr<IModelFunction_v> fFunc_v;  //! copy of the fitted  function containing on output the fit result
+   std::shared_ptr<IModelFunction_v> fFunc_v;  ///<! copy of the fitted  function containing on output the fit result
 
-   std::shared_ptr<IModelFunction> fFunc;  //! copy of the fitted  function containing on output the fit result
+   std::shared_ptr<IModelFunction> fFunc;  ///<! copy of the fitted  function containing on output the fit result
 
-   std::shared_ptr<ROOT::Fit::FitResult>  fResult;  //! pointer to the object containing the result of the fit
+   std::shared_ptr<ROOT::Fit::FitResult>  fResult;  ///<! pointer to the object containing the result of the fit
 
-   std::shared_ptr<ROOT::Math::Minimizer>  fMinimizer;  //! pointer to used minimizer
+   std::shared_ptr<ROOT::Math::Minimizer>  fMinimizer;  ///<! pointer to used minimizer
 
-   std::shared_ptr<ROOT::Fit::FitData>  fData;  //! pointer to the fit data (binned or unbinned data)
+   std::shared_ptr<ROOT::Fit::FitData>  fData;  ///<! pointer to the fit data (binned or unbinned data)
 
-   std::shared_ptr<ROOT::Math::IMultiGenFunction>  fObjFunction;  //! pointer to used objective function
+   std::shared_ptr<ROOT::Math::IMultiGenFunction>  fObjFunction;  ///<! pointer to used objective function
+
+   const ROOT::Math::IMultiGenFunction * fExtObjFunction = nullptr;     ///<! pointer to an external FCN
 
 };
 
@@ -542,7 +582,7 @@ private:
 // useful for fits done with customized FCN classes
 template <class ObjFuncType>
 bool Fitter::GetDataFromFCN()  {
-   ObjFuncType * objfunc = dynamic_cast<ObjFuncType*>(fObjFunction.get() );
+   const ObjFuncType * objfunc = dynamic_cast<const ObjFuncType*>(ObjFunction());
    if (objfunc) {
       fFunc = objfunc->ModelFunctionPtr();
       fData = objfunc->DataPtr();
@@ -602,24 +642,23 @@ void Fitter::SetFunction(const IGradModelFunction_v &func, bool useGradient)
 // implementation of inline methods
 
 
-#ifndef __CINT__
 
 #include "Math/WrappedFunction.h"
 
 template<class Function>
-bool ROOT::Fit::Fitter::FitFCN(unsigned int npar, Function & f, const double * par, unsigned int datasize,bool chi2fit) {
+bool ROOT::Fit::Fitter::FitFCN(unsigned int npar, Function & f, const double * par, unsigned int datasize,int fitType) {
    ROOT::Math::WrappedMultiFunction<Function &> wf(f,npar);
-   return FitFCN(wf,par,datasize,chi2fit);
+   if (!DoSetFCN(false, wf, par, datasize, fitType))
+      return false;
+   return FitFCN();
 }
 template<class Function>
-bool ROOT::Fit::Fitter::SetFCN(unsigned int npar, Function & f, const double * par, unsigned int datasize,bool chi2fit) {
+bool ROOT::Fit::Fitter::SetFCN(unsigned int npar, Function & f, const double * par, unsigned int datasize,int fitType) {
    ROOT::Math::WrappedMultiFunction<Function &> wf(f,npar);
-   return SetFCN(wf,par,datasize,chi2fit);
+   return DoSetFCN(false, wf, par, datasize, fitType);
 }
 
 
 
 
-#endif  // endif __CINT__
-
 #endif /* ROOT_Fit_Fitter */
