@@ -18,12 +18,14 @@
 #include "Minuit2/MnMachinePrecision.h"
 #include "Minuit2/MnPrint.h"
 
+#include <array>
+
 namespace ROOT {
 
 namespace Minuit2 {
 
-MnCross MnFunctionCross::operator()(const std::vector<unsigned int> &par, const std::vector<double> &pmid,
-                                    const std::vector<double> &pdir, double tlr, unsigned int maxcalls) const
+MnCross MnFunctionCross::operator()(std::span<const unsigned int> par, std::span<const double> pmid,
+                                    std::span<const double> pdir, double tlr, unsigned int maxcalls) const
 {
    // evaluate crossing point where function is equal to MIN + UP,
    // with direction pdir from values pmid
@@ -44,21 +46,22 @@ MnCross MnFunctionCross::operator()(const std::vector<unsigned int> &par, const
    // for finding the point :
    double tlf = tlr * up;
    double tla = tlr;
-   unsigned int maxitr = 15;
+   unsigned int maxitr = 30;
    unsigned int ipt = 0;
    double aminsv = fFval;
    double aim = aminsv + up;
    // std::cout<<"aim= "<<aim<<std::endl;
    double aopt = 0.;
    bool limset = false;
-   std::vector<double> alsb(3, 0.), flsb(3, 0.);
+   std::array<double, 3> alsb{0., 0., 0.};
+   std::array<double, 3> flsb{0., 0., 0.};
 
    MnPrint print("MnFunctionCross");
 
    print.Debug([&](std::ostream &os) {
       for (unsigned int i = 0; i < par.size(); ++i)
          os << "Parameter " << par[i] << " value " << pmid[i] << " dir " << pdir[i] << " function min = " << aminsv
-            << " contur value aim = (fmin + up) = " << aim;
+            << " contour value aim = (fmin + up) = " << aim;
    });
 
    // find the largest allowed aulim
@@ -473,9 +476,9 @@ L500:
 
       // evaluate at new point aopt
       print.Info([&](std::ostream &os) {
-         os << "Run Migrad again at new point (#iter = " << ipt << " ):";
+         os << "Run Migrad again at new point (#iter = " << ipt+1 << " ):";
          for (unsigned i = 0; i < npar; ++i)
-            os << "\n\t - parameter " << i << " fixed to " << pmid[i] + (aopt)*pdir[i];
+            os << "\n\t - parameter " << par[i] << " fixed to " << pmid[i] + (aopt)*pdir[i];
       });
 
       for (unsigned int i = 0; i < npar; i++)
