@@ -41,54 +41,60 @@ void loadWithErrorChecking(const std::string &libName)
 namespace RooBatchCompute {
 
 /// Inspect hardware capabilities, and load the optimal library for RooFit computations.
-void init()
+int initCPU()
 {
    // Check if the library was not initialised already
    static bool isInitialised = false;
    if (isInitialised)
-      return;
+      return 0;
    isInitialised = true;
 
    const std::string userChoice = gEnv->GetValue("RooFit.BatchCompute", "auto");
 #ifdef R__RF_ARCHITECTURE_SPECIFIC_LIBS
-#ifdef R__HAS_CUDA
-   if(gSystem->Load("libRooBatchCompute_CUDA") != 0) {
-      RooBatchCompute::dispatchCUDA = nullptr;
-   }
-#endif // R__HAS_CUDA
 
    __builtin_cpu_init();
-#if __GNUC__ > 5 || defined(__clang__)
    bool supported_avx512 = __builtin_cpu_supports("avx512cd") && __builtin_cpu_supports("avx512vl") &&
                            __builtin_cpu_supports("avx512bw") && __builtin_cpu_supports("avx512dq");
-#else
-   bool supported_avx512 = false;
-#endif
 
    if (userChoice == "auto") {
-      if (supported_avx512)
+      if (supported_avx512) {
          loadWithErrorChecking("libRooBatchCompute_AVX512");
-      else if (__builtin_cpu_supports("avx2"))
+      } else if (__builtin_cpu_supports("avx2")) {
          loadWithErrorChecking("libRooBatchCompute_AVX2");
-      else if (__builtin_cpu_supports("avx"))
+      } else if (__builtin_cpu_supports("avx")) {
          loadWithErrorChecking("libRooBatchCompute_AVX");
-      else if (__builtin_cpu_supports("sse4.1"))
+      } else if (__builtin_cpu_supports("sse4.1")) {
          loadWithErrorChecking("libRooBatchCompute_SSE4.1");
-   } else if (userChoice == "avx512")
+      }
+   } else if (userChoice == "avx512") {
       loadWithErrorChecking("libRooBatchCompute_AVX512");
-   else if (userChoice == "avx2")
+   } else if (userChoice == "avx2") {
       loadWithErrorChecking("libRooBatchCompute_AVX2");
-   else if (userChoice == "avx")
+   } else if (userChoice == "avx") {
       loadWithErrorChecking("libRooBatchCompute_AVX");
-   else if (userChoice == "sse")
+   } else if (userChoice == "sse") {
       loadWithErrorChecking("libRooBatchCompute_SSE4.1");
-   else if (userChoice != "generic")
+   } else if (userChoice != "generic") {
       throw std::invalid_argument(
          "Supported options for `RooFit.BatchCompute` are `auto`, `avx512`, `avx2`, `avx`, `sse`, `generic`.");
+   }
 #endif // R__RF_ARCHITECTURE_SPECIFIC_LIBS
 
    if (RooBatchCompute::dispatchCPU == nullptr)
       loadWithErrorChecking("libRooBatchCompute_GENERIC");
+
+   return 0;
+}
+
+int initCUDA()
+{
+   // Check if the library was not initialised already
+   static bool isInitialised = false;
+   if (isInitialised)
+      return 0;
+   isInitialised = true;
+
+   return gSystem->Load("libRooBatchCompute_CUDA");
 }
 
 } // namespace RooBatchCompute
