@@ -1,22 +1,40 @@
 /** \class PiecewiseInterpolation
- * \ingroup HistFactory
- * The PiecewiseInterpolation is a class that can morph distributions into each other, which
- * is useful to estimate systematic uncertainties. Given a nominal distribution and one or
- * more altered or distorted ones, it computes a new shape depending on the value of the nuisance
- * parameters \f$ \alpha_i \f$:
- * \f[
- *   A = \sum_i \mathrm{Interpolate}(\mathrm{low}_i, \mathrm{nominal}, \mathrm{high}_i, \alpha_i).
- * \f]
- * If an \f$ \alpha_i \f$ is zero, the distribution is identical to the nominal distribution, at
- * \f$ \pm 1 \f$ it is identical to the up/down distribution for that specific \f$ i \f$.
- *
- * The class supports several interpolation methods, which can be selected for each parameter separately
- * using setInterpCode(). The default interpolation code is 4. This performs
- * - \f$ |\alpha | > 1 \f$: Linear extrapolation.
- * - \f$ |\alpha | < 1 \f$: Polynomial interpolation. A sixth-order polynomial is used. Its coefficients
- * are chosen such that function, first, and second derivative at \f$ \alpha \pm 1 \f$ match the values
- * that the extrapolation procedure uses.
- */
+* \ingroup HistFactory
+* The PiecewiseInterpolation is a class that can morph distributions into each other, which
+* is useful to estimate systematic uncertainties. Given a nominal distribution and one or
+* more altered or distorted ones, it computes a new shape depending on the value of the nuisance
+* parameters \f$ \theta_i \f$:
+* \f[
+*   A = \mathrm{nominal} + \sum_i I_i(\theta_i;\mathrm{low}_i, \mathrm{nominal}, \mathrm{high}_i).
+* \f]
+* for additive interpolation modes (interpCodes 0, 2, 3, and 4), or:
+* \f[
+*   A = \mathrm{nominal}\prod_i I_i(\theta_i;\mathrm{low}_i/\mathrm{nominal}, 1, \mathrm{high}_i/\mathrm{nominal}).
+* \f]
+* for multiplicative interpolation modes (interpCodes 1, 5, and 6). The interpCodes determine the function \f$ I_i \f$ (see table below).
+*
+* Note that a PiecewiseInterpolation with \f$ \mathrm{nominal}=1 \f$, N variations, and a multiplicative interpolation mode is equivalent to N
+* PiecewiseInterpolations each with a single variation and the same interpolation code, all inside a RooProduct.
+*
+* If an \f$ \theta_i \f$ is zero, the distribution is identical to the nominal distribution, at
+* \f$ \pm 1 \f$ it is identical to the up/down distribution for that specific \f$ i \f$.
+*
+* PiecewiseInterpolation will behave identically (except for differences in the interpCode assignments) to a FlexibleInterpVar if both its nominal, and high and low variation sets
+* are all RooRealVar.
+*
+* The class supports several interpolation methods, which can be selected for each parameter separately
+* using setInterpCode(). The default interpolation code is 0. The table below provides details of the interpCodes:
+
+| **interpCode** | **Name** | **Description** |
+|----------------|----------|-----------------|
+| 0   (default)  | Additive Piecewise Linear | \f$ I_0(\theta;x_{-},x_0,x_{+}) = \theta(x_{+} - x_0) \f$ for \f$ \theta>=0 \f$, otherwise \f$ \theta(x_0 - x_{-}) \f$. Not recommended except if using a symmetric variation, because of discontinuities in derivatives. |
+| 1              | Multiplicative Piecewise Exponential | \f$ I_1(\theta;x_{-},x_0,x_{+}) = (x_{+}/x_0)^{\theta} \f$ for \f$ \theta>=0 \f$, otherwise \f$ (x_{-}/x_0)^{-\theta} \f$. |
+| 2              | Additive Quadratic Interp. + Linear Extrap. | Deprecated by interpCode 4. |
+| 4              | Additive Poly Interp. + Linear Extrap. | \f$ I_4(\theta;x_{-},x_0,x_{+}) = I_0(\theta;x_{-},x_0,x_{+}) \f$ if \f$ |\theta|>=1 \f$, otherwise \f$ \theta(\frac{x_{+}-x_{-}}{2}+\theta\frac{x_{+}+x_{-}-2x_{0}}{16}(15+\theta^2(3\alpha^2-10))) \f$  (6th-order polynomial through origin for with matching 0th,1st,2nd derivatives at boundary). |
+| 5              | Multiplicative Poly Interp. + Exponential Extrap. | \f$ I_5(\theta;x_{-},x_0,x_{+}) = I_1(\theta;x_{-},x_0,x_{+}) \f$ if \f$ |\theta|>=1 \f$, otherwise 6th-order polynomial for \f$ |\theta_i|<1 \f$ with matching 0th,1st,2nd derivatives at boundary. Recommended for normalization factors. In FlexibleInterpVar this is interpCode=4. |
+| 6              | Multiplicative Poly Interp. + Linear Extrap. | \f$ I_6(\theta;x_{-},x_0,x_{+}) = 1+I_4(\theta;x_{-},x_0,x_{+}). \f$ Recommended for normalization factors that must not have roots (i.e. be equal to 0) outside of \f$ |\theta_i|<1 \f$. |
+
+*/
 
 #include "RooStats/HistFactory/PiecewiseInterpolation.h"
 
@@ -224,7 +242,7 @@ void PiecewiseInterpolation::translate(RooFit::Detail::CodeSquashContext &ctx) c
    std::string lowName = ctx.getTmpVarName();
    std::string highName = ctx.getTmpVarName();
    std::string nominalName = ctx.getTmpVarName();
-   code += "unsigned int " + idxName + " = " + nomHist.calculateTreeIndexForCodeSquash(this, ctx, dynamic_cast<RooHistFunc const &>(*_nominal).variables()) + ";\n";
+   code += "unsigned int " + idxName + " = " + nomHist.calculateTreeIndexForCodeSquash(ctx, dynamic_cast<RooHistFunc const &>(*_nominal).variables()) + ";\n";
    code += "double const* " + lowName + " = " + valsLowStr + " + " + nStr + " * " + idxName + ";\n";
    code += "double const* " + highName + " = " + valsHighStr + " + " + nStr + " * " + idxName + ";\n";
    code += "double " + nominalName + " = *(" + valsNominalStr + " + " + idxName + ");\n";
@@ -576,7 +594,7 @@ void PiecewiseInterpolation::setAllInterpCodes(int code)
 void PiecewiseInterpolation::setInterpCodeForParam(int iParam, int code)
 {
    RooAbsArg const &param = _paramSet[iParam];
-   if (code < 0 || code > 5) {
+   if (code < 0 || code > 6) {
       coutE(InputArguments) << "PiecewiseInterpolation::setInterpCode ERROR: " << param.GetName()
                             << " with unknown interpolation code " << code << ", keeping current code "
                             << _interpCode[iParam] << std::endl;
