@@ -10,7 +10,6 @@
  * listed in LICENSE (http://roofit.sourceforge.net/license.txt)
  */
 
-#include <RooFitHS3/HistFactoryJSONTool.h>
 #include <RooFitHS3/RooJSONFactoryWSTool.h>
 #include <RooFitHS3/JSONIO.h>
 #include <RooFit/Detail/JSONInterface.h>
@@ -29,7 +28,6 @@
 #include <RooPoisson.h>
 #include <RooLognormal.h>
 #include <RooGaussian.h>
-#include <RooBinning.h>
 #include <RooProduct.h>
 #include <RooWorkspace.h>
 
@@ -38,6 +36,10 @@
 
 using RooFit::Detail::JSONNode;
 
+using namespace RooStats::HistFactory;
+using namespace RooStats::HistFactory::Detail;
+using namespace RooStats::HistFactory::Detail::MagicConstants;
+
 namespace {
 
 inline void writeAxis(JSONNode &axis, RooRealVar const &obs)
@@ -48,13 +50,13 @@ inline void writeAxis(JSONNode &axis, RooRealVar const &obs)
       axis["min"] << obs.getMin();
       axis["max"] << obs.getMax();
    } else {
-      auto &bounds = axis["bounds"];
-      bounds.set_seq();
+      auto &edges = axis["edges"];
+      edges.set_seq();
       double val = binning.binLow(0);
-      bounds.append_child() << val;
+      edges.append_child() << val;
       for (int i = 0; i < binning.numBins(); ++i) {
          val = binning.binHigh(i);
-         bounds.append_child() << val;
+         edges.append_child() << val;
       }
    }
 }
@@ -142,80 +144,70 @@ std::string toString(TClass *c)
    return "unknown";
 }
 
-using namespace RooStats::HistFactory::Detail;
+inline std::string defaultGammaName(std::string const &sysname, std::size_t i)
+{
+   return "gamma_" + sysname + "_bin_" + std::to_string(i);
+}
+
+/// Export the names of the gamma parameters to the modifier struct if the
+/// names don't match the default gamma parameter names, which is gamma_<sysname>_bin_<i>
+void optionallyExportGammaParameters(JSONNode &mod, std::string const &sysname,
+                                     std::vector<std::string> const &paramNames)
+{
+   for (std::size_t i = 0; i < paramNames.size(); ++i) {
+      if (paramNames[i] != defaultGammaName(sysname, i)) {
+         mod["parameters"].fill_seq(paramNames);
+         return;
+      }
+   }
+}
 
 RooRealVar &createNominal(RooWorkspace &ws, std::string const &parname, double val, double min, double max)
 {
    RooRealVar &nom = getOrCreate<RooRealVar>(ws, "nom_" + parname, val, min, max);
    nom.setConstant(true);
-   nom.setAttribute("globs");
    return nom;
 }
 
 /// Get the conventional name of the constraint pdf for a constrained
 /// parameter.
-std::string constraintName(std::string const &sysname)
+std::string constraintName(std::string const &paramName)
 {
-   return sysname + "_constraint";
+   return paramName + "Constraint";
 }
 
-RooAbsPdf &getConstraint(RooWorkspace &ws, const std::string &sysname, const std::string &pname)
+RooAbsPdf &getConstraint(RooWorkspace &ws, const std::string &pname)
 {
-   return getOrCreate<RooGaussian>(ws, constraintName(sysname), *ws.var(pname), *ws.var("nom_" + pname),
-                                   RooFit::RooConst(1.));
+   RooRealVar *constrParam = ws.var(pname);
+   constrParam->setError(1.0);
+   return getOrCreate<RooGaussian>(ws, constraintName(pname), *constrParam, *ws.var("nom_" + pname), 1.);
 }
 
-void setMCStatGammaRanges(RooArgList const &gammas, std::vector<double> const &errs, double statErrThresh)
-{
-   // Set a reasonable range for gamma and set constant NPs which are below the
-   // MC stat threshold, and remove them from the np list.
-   for (size_t i = 0; i < errs.size(); ++i) {
-      auto g = static_cast<RooRealVar *>(gammas.at(i));
-      g->setMax(1 + 5 * errs[i]);
-      g->setError(errs[i]);
-      if (errs[i] < statErrThresh) {
-         g->setConstant(true); // all negative errs are set constant
-      }
-   }
-}
-
-ParamHistFunc &createPHF(const std::string &sysname, const std::string &phfname, const std::vector<double> &vals,
+ParamHistFunc &createPHF(const std::string &phfname, std::string const &sysname,
+                         const std::vector<std::string> &parnames, const std::vector<double> &vals,
                          RooJSONFactoryWSTool &tool, RooArgList &constraints, const RooArgSet &observables,
-                         const std::string &constraintType, RooArgList &gammas, double gamma_min, double gamma_max)
+                         const std::string &constraintType, double gammaMin, double gammaMax, double minSigma)
 {
    RooWorkspace &ws = *tool.workspace();
 
-   gammas.add(ParamHistFunc::createParamSet(ws, ("gamma_" + sysname).c_str(), observables, gamma_min, gamma_max));
-   auto &phf = tool.wsEmplace<ParamHistFunc>(phfname, observables, gammas);
-   for (size_t i = 0; i < gammas.size(); ++i) {
-      RooRealVar *v = dynamic_cast<RooRealVar *>(&gammas[i]);
-      if (!v)
-         continue;
-      std::string basename = v->GetName();
-      v->setConstant(false);
-      if (constraintType == "Const" || vals[i] == 0.) {
-         v->setConstant(true);
-      } else if (constraintType == "Gauss") {
-         auto &nom = createNominal(ws, basename, 1.0, 0, std::max(10., gamma_max));
-         auto &sigma = tool.wsEmplace<RooConstVar>(basename + "_sigma", vals[i]);
-         constraints.add(tool.wsEmplace<RooGaussian>(constraintName(basename), nom, *v, sigma), true);
-      } else if (constraintType == "Poisson") {
-         double tau_float = vals[i];
-         auto &tau = tool.wsEmplace<RooConstVar>(basename + "_tau", tau_float);
-         auto &nom = createNominal(ws, basename, tau_float, 0, RooNumber::infinity());
-         auto &prod = tool.wsEmplace<RooProduct>(basename + "_poisMean", *v, tau);
-         auto &pois = tool.wsEmplace<RooPoisson>(constraintName(basename), nom, prod);
-         pois.setNoRounding(true);
-         constraints.add(pois, true);
-      } else {
-         RooJSONFactoryWSTool::error("unknown constraint type " + constraintType);
-      }
+   RooArgList gammas;
+   for (std::size_t i = 0; i < vals.size(); ++i) {
+      const std::string name = parnames.empty() ? defaultGammaName(sysname, i) : parnames[i];
+      gammas.add(getOrCreate<RooRealVar>(ws, name, 1., gammaMin, gammaMax));
    }
-   for (auto &g : gammas) {
-      for (auto client : g->clients()) {
-         if (dynamic_cast<RooAbsPdf *>(client) && !constraints.find(*client)) {
-            constraints.add(*client);
-         }
+
+   auto &phf = tool.wsEmplace<ParamHistFunc>(phfname, observables, gammas);
+
+   if (constraintType != "Const") {
+      auto constraintsInfo = createGammaConstraints(
+         gammas, vals, minSigma, constraintType == "Poisson" ? Constraint::Poisson : Constraint::Gaussian);
+      for (auto const &term : constraintsInfo.constraints) {
+         ws.import(*term, RooFit::RecycleConflictNodes());
+         constraints.add(*ws.pdf(term->GetName()));
+      }
+   } else {
+      for (auto *gamma : static_range_cast<RooRealVar *>(gammas)) {
+         gamma->setConstant(true);
       }
    }
 
@@ -233,17 +225,32 @@ bool hasStaterror(const JSONNode &comp)
    return false;
 }
 
+const JSONNode &findStaterror(const JSONNode &comp)
+{
+   if (comp.has_child("modifiers")) {
+      for (const auto &mod : comp["modifiers"].children()) {
+         if (mod["type"].val() == ::Literals::staterror)
+            return mod;
+      }
+   }
+   RooJSONFactoryWSTool::error("sample '" + RooJSONFactoryWSTool::name(comp) + "' does not have a " +
+                               ::Literals::staterror + " modifier!");
+}
+
 bool importHistSample(RooJSONFactoryWSTool &tool, RooDataHist &dh, RooArgSet const &varlist,
                       RooAbsArg const *mcStatObject, const std::string &fprefix, const JSONNode &p,
                       RooArgList &constraints)
 {
    RooWorkspace &ws = *tool.workspace();
 
-   std::string name = RooJSONFactoryWSTool::name(p);
-   std::string prefixedName = fprefix + "_" + name;
+   std::string sampleName = RooJSONFactoryWSTool::name(p);
+   std::string prefixedName = fprefix + "_" + sampleName;
+
+   std::string channelName = fprefix;
+   erasePrefix(channelName, "model_");
 
    if (!p.has_child("data")) {
-      RooJSONFactoryWSTool::error("sample '" + name + "' does not define a 'data' key");
+      RooJSONFactoryWSTool::error("sample '" + sampleName + "' does not define a 'data' key");
    }
 
    auto &hf = tool.wsEmplace<RooHistFunc>("hist_" + prefixedName, varlist, dh);
@@ -267,15 +274,25 @@ bool importHistSample(RooJSONFactoryWSTool &tool, RooDataHist &dh, RooArgSet con
       RooArgList histoLo;
       RooArgList histoHi;
 
+      int idx = 0;
       for (const auto &mod : p["modifiers"].children()) {
          std::string const &modtype = mod["type"].val();
-         std::string const &sysname = RooJSONFactoryWSTool::name(mod);
+         std::string const &sysname =
+            mod.has_child("name")
+               ? mod["name"].val()
+               : (mod.has_child("parameter") ? mod["parameter"].val() : "syst_" + std::to_string(idx));
+         ++idx;
          if (modtype == "staterror") {
             // this is dealt with at a different place, ignore it for now
          } else if (modtype == "normfactor") {
-            normElems.add(getOrCreate<RooRealVar>(ws, sysname, 1., -3, 5));
+            RooRealVar &constrParam = getOrCreate<RooRealVar>(ws, sysname, 1., -3, 5);
+            normElems.add(constrParam);
             if (auto constrInfo = mod.find("constraint_name")) {
-               constraints.add(*tool.request<RooAbsReal>(constrInfo->val(), name));
+               auto constraint = tool.request<RooAbsReal>(constrInfo->val(), sampleName);
+               if (auto gauss = dynamic_cast<RooGaussian const *>(constraint)) {
+                  constrParam.setError(gauss->getSigma().getVal());
+               }
+               constraints.add(*constraint);
             }
          } else if (modtype == "normsys") {
             auto *parameter = mod.find("parameter");
@@ -291,7 +308,7 @@ bool importHistSample(RooJSONFactoryWSTool &tool, RooDataHist &dh, RooArgSet con
                                                               : std::numeric_limits<double>::epsilon());
             overall_high.push_back(data["hi"].val_double() > 0 ? data["hi"].val_double()
                                                                : std::numeric_limits<double>::epsilon());
-            constraints.add(getConstraint(ws, sysname, parname));
+            constraints.add(getConstraint(ws, parname));
          } else if (modtype == "histosys") {
             auto *parameter = mod.find("parameter");
             std::string parname(parameter ? parameter->val() : "alpha_" + sysname);
@@ -304,16 +321,24 @@ bool importHistSample(RooJSONFactoryWSTool &tool, RooDataHist &dh, RooArgSet con
             histoHi.add(tool.wsEmplace<RooHistFunc>(
                sysname + "High_" + prefixedName, varlist,
                RooJSONFactoryWSTool::readBinnedData(data["hi"], sysname + "High_" + prefixedName, varlist)));
-            constraints.add(getConstraint(ws, sysname, parname));
+            constraints.add(getConstraint(ws, parname));
          } else if (modtype == "shapesys") {
-            std::string funcName = prefixedName + "_" + sysname + "_" + prefixedName + "_ShapeSys";
+            std::string funcName = channelName + "_" + sysname + "_ShapeSys";
+            // funcName should be "<channel_name>_<sysname>_ShapeSys"
             std::vector<double> vals;
             for (const auto &v : mod["data"]["vals"].children()) {
                vals.push_back(v.val_double());
             }
-            RooArgList gammas;
+            std::vector<std::string> parnames;
+            for (const auto &v : mod["parameters"].children()) {
+               parnames.push_back(v.val());
+            }
+            if (vals.empty()) {
+               RooJSONFactoryWSTool::error("unable to instantiate shapesys '" + sysname + "' with 0 values!");
+            }
             std::string constraint(mod["constraint"].val());
-            shapeElems.add(createPHF(sysname, funcName, vals, tool, constraints, varlist, constraint, gammas, 0, 1000));
+            shapeElems.add(createPHF(funcName, sysname, parnames, vals, tool, constraints, varlist, constraint,
+                                     defaultGammaMin, defaultShapeSysGammaMax, minShapeUncertainty));
          } else if (modtype == "custom") {
             RooAbsReal *obj = ws.function(sysname);
             if (!obj) {
@@ -329,9 +354,10 @@ bool importHistSample(RooJSONFactoryWSTool &tool, RooDataHist &dh, RooArgSet con
          }
       }
 
+      std::string interpName = sampleName + "_" + channelName + "_epsilon";
       if (!overall_nps.empty()) {
-         auto &v = tool.wsEmplace<RooStats::HistFactory::FlexibleInterpVar>("overallSys_" + prefixedName, overall_nps,
-                                                                            1., overall_low, overall_high);
+         auto &v = tool.wsEmplace<RooStats::HistFactory::FlexibleInterpVar>(interpName, overall_nps, 1., overall_low,
+                                                                            overall_high);
          v.setAllInterpCodes(4); // default HistFactory interpCode
          normElems.add(v);
       }
@@ -359,8 +385,6 @@ class HistFactoryImporter : public RooFit::JSONIO::Importer {
 public:
    bool importArg(RooJSONFactoryWSTool *tool, const JSONNode &p) const override
    {
-      RooWorkspace &ws = *tool->workspace();
-
       std::string name = RooJSONFactoryWSTool::name(p);
       if (!p.has_child("samples")) {
          RooJSONFactoryWSTool::error("no samples in '" + name + "', skipping.");
@@ -376,28 +400,8 @@ public:
       }
       std::vector<double> sumW;
       std::vector<double> sumW2;
-      RooArgSet observables;
-      for (auto const &obsNode : p["axes"].children()) {
-         if (obsNode.has_child("bounds")) {
-            std::vector<double> bounds;
-            for (auto const &bound : obsNode["bounds"].children()) {
-               bounds.push_back(bound.val_double());
-            }
-            RooRealVar &obs = getOrCreate<RooRealVar>(ws, obsNode["name"].val(), bounds[0], bounds[bounds.size() - 1]);
-            RooBinning bins(obs.getMin(), obs.getMax());
-            ;
-            for (auto b : bounds) {
-               bins.addBoundary(b);
-            }
-            obs.setBinning(bins);
-            observables.add(obs);
-         } else {
-            RooRealVar &obs = getOrCreate<RooRealVar>(ws, obsNode["name"].val(), obsNode["min"].val_double(),
-                                                      obsNode["max"].val_double());
-            obs.setBins(obsNode["nbins"].val_int());
-            observables.add(obs);
-         }
-      }
+      std::vector<std::string> gammaParnames;
+      RooArgSet observables = RooJSONFactoryWSTool::readAxes(p);
 
       std::string fprefix = name;
 
@@ -416,6 +420,13 @@ public:
                sumW[i] += dh->weight(i);
                sumW2[i] += dh->weightSquared(i);
             }
+            if (gammaParnames.empty()) {
+               if (auto staterrorParams = findStaterror(comp).find("parameters")) {
+                  for (const auto &v : staterrorParams->children()) {
+                     gammaParnames.push_back(v.val());
+                  }
+               }
+            }
          }
          data.emplace_back(std::move(dh));
       }
@@ -423,25 +434,19 @@ public:
       RooAbsArg *mcStatObject = nullptr;
       RooArgList constraints;
       if (!sumW.empty()) {
-         std::string phfName = name;
-         erasePrefix(phfName, "model_");
+         std::string channelName = name;
+         erasePrefix(channelName, "model_");
 
-         std::vector<double> vals(sumW.size());
          std::vector<double> errs(sumW.size());
-
          for (size_t i = 0; i < sumW.size(); ++i) {
             errs[i] = std::sqrt(sumW2[i]) / sumW[i];
-            if (statErrType == "Gauss") {
-               vals[i] = std::max(errs[i], 0.); // avoid negative sigma. This NP will be set constant anyway later
-            } else if (statErrType == "Poisson") {
-               vals[i] = sumW[i] * sumW[i] / sumW2[i];
-            }
+            // avoid negative sigma. This NP will be set constant anyway later
+            errs[i] = std::max(errs[i], 0.);
          }
 
-         RooArgList gammas;
-         mcStatObject = &createPHF("stat_" + phfName, "mc_stat_" + phfName, vals, *tool, constraints, observables,
-                                   statErrType, gammas, 0, 10);
-         setMCStatGammaRanges(gammas, errs, statErrThresh);
+         mcStatObject =
+            &createPHF("mc_stat_" + channelName, "stat_" + channelName, gammaParnames, errs, *tool, constraints,
+                       observables, statErrType, defaultGammaMin, defaultStatErrorGammaMax, statErrThresh);
       }
 
       int idx = 0;
@@ -457,14 +462,12 @@ public:
       }
 
       if (constraints.empty()) {
-         auto &sum = tool->wsEmplace<RooRealSumPdf>(name, funcs, coefs, true);
-         sum.setAttribute("BinnedLikelihood");
+         tool->wsEmplace<RooRealSumPdf>(name, funcs, coefs, true);
       } else {
          std::string sumName = name + "_model";
          erasePrefix(sumName, "model_");
          auto &sum = tool->wsEmplace<RooRealSumPdf>(sumName, funcs, coefs, true);
          sum.SetTitle(name.c_str());
-         sum.setAttribute("BinnedLikelihood");
          tool->wsEmplace<RooProdPdf>(name, constraints, RooFit::Conditional(sum, observables));
       }
       return true;
@@ -482,11 +485,11 @@ public:
    {
       auto fip = static_cast<const RooStats::HistFactory::FlexibleInterpVar *>(func);
       elem["type"] << key();
-      RooJSONFactoryWSTool::fillSeq(elem["vars"], fip->variables());
       elem["interpolationCodes"].fill_seq(fip->interpolationCodes());
+      RooJSONFactoryWSTool::fillSeq(elem["vars"], fip->variables());
       elem["nom"] << fip->nominal();
-      elem["high"].fill_seq(fip->high());
-      elem["low"].fill_seq(fip->low());
+      elem["high"].fill_seq(fip->high(), fip->variables().size());
+      elem["low"].fill_seq(fip->low(), fip->variables().size());
       return true;
    }
 };
@@ -506,8 +509,8 @@ public:
       elem["positiveDefinite"] << pip->positiveDefinite();
       RooJSONFactoryWSTool::fillSeq(elem["vars"], pip->paramList());
       elem["nom"] << pip->nominalHist()->GetName();
-      RooJSONFactoryWSTool::fillSeq(elem["high"], pip->highList());
-      RooJSONFactoryWSTool::fillSeq(elem["low"], pip->lowList());
+      RooJSONFactoryWSTool::fillSeq(elem["high"], pip->highList(), pip->paramList().size());
+      RooJSONFactoryWSTool::fillSeq(elem["low"], pip->lowList(), pip->paramList().size());
       return true;
    }
 };
@@ -630,6 +633,7 @@ struct HistoSys {
 struct ShapeSys {
    std::string name;
    std::vector<double> constraints;
+   std::vector<std::string> parameters;
    TClass *constraint = nullptr;
    ShapeSys(const std::string &n) : name{n} {}
 };
@@ -643,6 +647,7 @@ struct Sample {
    std::vector<ShapeSys> shapesys;
    std::vector<RooAbsReal *> otherElements;
    bool useBarlowBeestonLight = false;
+   std::vector<std::string> staterrorParameters;
    TClass *barlowBeestonLightConstraint = RooPoisson::Class();
    Sample(const std::string &n) : name{n} {}
 };
@@ -663,21 +668,34 @@ void addNormFactor(RooRealVar const *par, Sample &sample, RooWorkspace *ws)
       sample.normfactors.emplace_back(*par);
 }
 
+namespace {
+
+bool verbose = false;
+
+}
+
 bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname, const RooRealSumPdf *sumpdf,
                           JSONNode &elem)
 {
    RooWorkspace *ws = tool->workspace();
    RooArgSet customModifiers;
 
-   if (!sumpdf)
+   if (!sumpdf) {
+      if (verbose) {
+         std::cout << pdfname << " is not a sumpdf" << std::endl;
+      }
       return false;
+   }
 
-   std::string chname = pdfname;
-   erasePrefix(chname, "model_");
-   eraseSuffix(chname, "_model");
+   std::string channelName = pdfname;
+   erasePrefix(channelName, "model_");
+   eraseSuffix(channelName, "_model");
 
    for (RooAbsArg *sample : sumpdf->funcList()) {
       if (!dynamic_cast<RooProduct *>(sample) && !dynamic_cast<RooRealSumPdf *>(sample)) {
+         if (verbose)
+            std::cout << "sample " << sample->GetName() << " is no RooProduct or RooRealSumPdf in " << pdfname
+                      << std::endl;
          return false;
       }
    }
@@ -692,14 +710,14 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
 
    for (size_t sampleidx = 0; sampleidx < sumpdf->funcList().size(); ++sampleidx) {
       PiecewiseInterpolation *pip = nullptr;
-      RooStats::HistFactory::FlexibleInterpVar *fip = nullptr;
+      std::vector<RooStats::HistFactory::FlexibleInterpVar *> fips;
       std::vector<ParamHistFunc *> phfs;
 
       const auto func = sumpdf->funcList().at(sampleidx);
       Sample sample(func->GetName());
       erasePrefix(sample.name, "L_x_");
       eraseSuffix(sample.name, "_shapes");
-      eraseSuffix(sample.name, "_" + chname);
+      eraseSuffix(sample.name, "_" + channelName);
       erasePrefix(sample.name, pdfname + "_");
       RooArgSet elems;
       collectElements(elems, func);
@@ -717,7 +735,15 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       };
 
       for (RooAbsArg *e : elems) {
-         if (auto constVar = dynamic_cast<RooConstVar *>(e)) {
+         if (TString(e->GetName()).Contains("binWidth")) {
+            // The bin width modifiers are handled separately. We can't just
+            // check for the RooBinWidthFunction type here, because prior to
+            // ROOT 6.26, the multiplication with the inverse bin width was
+            // done in a different way (like a normfactor with a RooRealVar,
+            // but it was stored in the dataset).
+            // Fortunately, the name was similar, so we can match the modifier
+            // name.
+         } else if (auto constVar = dynamic_cast<RooConstVar *>(e)) {
             if (constVar->getVal() != 1.) {
                sample.normfactors.emplace_back(*e);
             }
@@ -727,12 +753,13 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
             updateObservables(hf->dataHist());
          } else if (auto phf = dynamic_cast<ParamHistFunc *>(e)) {
             phfs.push_back(phf);
-         } else if (!fip && (fip = dynamic_cast<RooStats::HistFactory::FlexibleInterpVar *>(e))) {
+         } else if (auto fip = dynamic_cast<RooStats::HistFactory::FlexibleInterpVar *>(e)) {
+            // some (modified) histfactory models have several instances of FlexibleInterpVar
+            // we collect and merge them here
+            fips.push_back(fip);
          } else if (!pip && (pip = dynamic_cast<PiecewiseInterpolation *>(e))) {
          } else if (auto real = dynamic_cast<RooAbsReal *>(e)) {
-            if (!dynamic_cast<RooBinWidthFunction *>(real)) {
-               sample.otherElements.push_back(real);
-            }
+            sample.otherElements.push_back(real);
          }
       }
 
@@ -747,15 +774,19 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       sortByName(sample.normfactors);
 
       // sort and configure the normsys
-      if (fip) {
+      for (auto *fip : fips) {
          for (size_t i = 0; i < fip->variables().size(); ++i) {
             RooAbsArg *var = fip->variables().at(i);
             std::string sysname(var->GetName());
             erasePrefix(sysname, "alpha_");
-            sample.normsys.emplace_back(sysname, var, fip->high()[i], fip->low()[i], findConstraint(var)->IsA());
+            const auto *constraint = findConstraint(var);
+            if (!constraint)
+               RooJSONFactoryWSTool::error("cannot find constraint for " + std::string(var->GetName()));
+            sample.normsys.emplace_back(sysname, var, fip->high()[i], fip->low()[i],
+                                        constraint ? constraint->IsA() : nullptr);
          }
-         sortByName(sample.normsys);
       }
+      sortByName(sample.normsys);
 
       // sort and configure the histosys
       if (pip) {
@@ -765,7 +796,10 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
             erasePrefix(sysname, "alpha_");
             if (auto lo = dynamic_cast<RooHistFunc *>(pip->lowList().at(i))) {
                if (auto hi = dynamic_cast<RooHistFunc *>(pip->highList().at(i))) {
-                  sample.histosys.emplace_back(sysname, var, lo, hi, findConstraint(var)->IsA());
+                  const auto *constraint = findConstraint(var);
+                  if (!constraint)
+                     RooJSONFactoryWSTool::error("cannot find constraint for " + std::string(var->GetName()));
+                  sample.histosys.emplace_back(sysname, var, lo, hi, constraint ? constraint->IsA() : nullptr);
                }
             }
          }
@@ -776,6 +810,7 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
          if (startsWith(std::string(phf->GetName()), "mc_stat_")) { // MC stat uncertainty
             int idx = 0;
             for (const auto &g : phf->paramList()) {
+               sample.staterrorParameters.push_back(g->GetName());
                ++idx;
                RooAbsPdf *constraint = findConstraint(g);
                if (tot_yield.find(idx) == tot_yield.end()) {
@@ -801,25 +836,20 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
             sample.useBarlowBeestonLight = true;
          } else { // other ShapeSys
             ShapeSys sys(phf->GetName());
-            erasePrefix(sys.name, "model_" + chname + "_");
-            erasePrefix(sys.name, chname + "_");
-            erasePrefix(sys.name, sample.name + "_");
+            erasePrefix(sys.name, channelName + "_");
             eraseSuffix(sys.name, "_ShapeSys");
-            eraseSuffix(sys.name, "_" + sample.name);
-            eraseSuffix(sys.name, "_model_" + chname);
-            eraseSuffix(sys.name, "_" + chname);
-            eraseSuffix(sys.name, "_" + sample.name);
 
             for (const auto &g : phf->paramList()) {
+               sys.parameters.push_back(g->GetName());
                RooAbsPdf *constraint = findConstraint(g);
                if (!constraint)
                   constraint = ws->pdf(constraintName(g->GetName()));
-               if (!constraint && !g->isConstant())
+               if (!constraint && !g->isConstant()) {
                   RooJSONFactoryWSTool::error("cannot find constraint for " + std::string(g->GetName()));
-               else if (!constraint) {
+               } else if (!constraint) {
                   sys.constraints.push_back(0.0);
                } else if (auto constraint_p = dynamic_cast<RooPoisson *>(constraint)) {
-                  sys.constraints.push_back(constraint_p->getX().getVal());
+                  sys.constraints.push_back(1. / std::sqrt(constraint_p->getX().getVal()));
                   if (!sys.constraint) {
                      sys.constraint = RooPoisson::Class();
                   }
@@ -853,7 +883,7 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
             const int i = bin.first;
             const double relerr_tot = bin.second;
             const double count = sample.hist[i - 1];
-            // this reconstruction is inherently unprecise, so we truncate it at some decimal places to make sure that
+            // this reconstruction is inherently imprecise, so we truncate it at some decimal places to make sure that
             // we don't carry around too many useless digits
             sample.histError[i - 1] = round_prec(relerr_tot * tot_yield[i] / std::sqrt(tot_yield2[i]) * count, 7);
          }
@@ -868,9 +898,12 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       auto &s = RooJSONFactoryWSTool::appendNamedChild(elem["samples"], sample.name);
 
       auto &modifiers = s["modifiers"];
+      modifiers.set_seq();
 
       for (const auto &nf : sample.normfactors) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, nf.name);
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << nf.name;
          mod["parameter"] << nf.param->GetName();
          mod["type"] << "normfactor";
          if (nf.constraint) {
@@ -880,7 +913,9 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       }
 
       for (const auto &sys : sample.normsys) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, sys.name);
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << sys.name;
          mod["type"] << "normsys";
          mod["parameter"] << sys.param->GetName();
          mod["constraint"] << toString(sys.constraint);
@@ -890,7 +925,9 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       }
 
       for (const auto &sys : sample.histosys) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, sys.name);
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << sys.name;
          mod["type"] << "histosys";
          mod["parameter"] << sys.param->GetName();
          mod["constraint"] << toString(sys.constraint);
@@ -906,25 +943,38 @@ bool tryExportHistFactory(RooJSONFactoryWSTool *tool, const std::string &pdfname
       }
 
       for (const auto &sys : sample.shapesys) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, sys.name);
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << sys.name;
          mod["type"] << "shapesys";
+         optionallyExportGammaParameters(mod, sys.name, sys.parameters);
          mod["constraint"] << toString(sys.constraint);
          if (sys.constraint) {
-            auto &data = mod["data"].set_map();
-            auto &vals = data["vals"];
+            auto &vals = mod["data"].set_map()["vals"];
             vals.fill_seq(sys.constraints);
+         } else {
+            auto &vals = mod["data"].set_map()["vals"];
+            vals.set_seq();
+            for (std::size_t i = 0; i < sys.parameters.size(); ++i) {
+               vals.append_child() << 0;
+            }
          }
       }
 
       for (const auto &other : sample.otherElements) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, other->GetName());
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << other->GetName();
          customModifiers.add(*other);
          mod["type"] << "custom";
       }
 
       if (sample.useBarlowBeestonLight) {
-         auto &mod = RooJSONFactoryWSTool::appendNamedChild(modifiers, ::Literals::staterror);
+         auto &mod = modifiers.append_child();
+         mod.set_map();
+         mod["name"] << ::Literals::staterror;
          mod["type"] << ::Literals::staterror;
+         optionallyExportGammaParameters(mod, "stat_" + channelName, sample.staterrorParameters);
          mod["constraint"] << toString(sample.barlowBeestonLightConstraint);
       }
 
@@ -981,8 +1031,17 @@ public:
    {
       RooRealSumPdf *sumpdf = nullptr;
       for (RooAbsArg *v : prodpdf->pdfList()) {
-         sumpdf = dynamic_cast<RooRealSumPdf *>(v);
+         auto thispdf = dynamic_cast<RooRealSumPdf *>(v);
+         if (thispdf) {
+            if (!sumpdf)
+               sumpdf = thispdf;
+            else
+               return false;
+         }
       }
+      if (!sumpdf)
+         return false;
+
       return tryExportHistFactory(tool, prodpdf->GetName(), sumpdf, elem);
    }
    std::string const &key() const override
