@@ -81,7 +81,6 @@ in calculation speed.
 #include "Riostream.h"
 
 #include "RooIntegralMorph.h"
-#include "RooAbsReal.h"
 #include "RooAbsCategory.h"
 #include "RooBrentRootFinder.h"
 #include "RooAbsFunc.h"
@@ -89,7 +88,7 @@ in calculation speed.
 #include "RooDataHist.h"
 #include "TH1.h"
 
-using namespace std;
+ using std::flush, std::endl;
 
 ClassImp(RooIntegralMorph);
 
@@ -104,14 +103,13 @@ RooIntegralMorph::RooIntegralMorph(const char *name, const char *title,
                 RooAbsReal& _pdf2,
                 RooAbsReal& _x,
                 RooAbsReal& _alpha,
-                Bool_t doCacheAlpha) :
+                bool doCacheAlpha) :
   RooAbsCachedPdf(name,title,2),
   pdf1("pdf1","pdf1",this,_pdf1),
   pdf2("pdf2","pdf2",this,_pdf2),
   x("x","x",this,_x),
   alpha("alpha","alpha",this,_alpha),
-  _cacheAlpha(doCacheAlpha),
-  _cache(0)
+  _cacheAlpha(doCacheAlpha)
 {
 }
 
@@ -124,8 +122,7 @@ RooIntegralMorph::RooIntegralMorph(const RooIntegralMorph& other, const char* na
   pdf2("pdf2",this,other.pdf2),
   x("x",this,other.x),
   alpha("alpha",this,other.alpha),
-  _cacheAlpha(other._cacheAlpha),
-  _cache(0)
+  _cacheAlpha(other._cacheAlpha)
 {
 }
 
@@ -134,31 +131,31 @@ RooIntegralMorph::RooIntegralMorph(const RooIntegralMorph& other, const char* na
 /// Returns the 'x' observable unless doCacheAlpha is set in which
 /// case a set with both x and alpha
 
-RooArgSet* RooIntegralMorph::actualObservables(const RooArgSet& /*nset*/) const
+RooFit::OwningPtr<RooArgSet> RooIntegralMorph::actualObservables(const RooArgSet& /*nset*/) const
 {
   RooArgSet* obs = new RooArgSet ;
   if (_cacheAlpha) {
     obs->add(alpha.arg()) ;
   }
   obs->add(x.arg()) ;
-  return obs ;
+  return RooFit::OwningPtr<RooArgSet>{obs};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Parameters of the cache. Returns parameters of both pdf1 and pdf2
 /// and parameter cache, in case doCacheAlpha is not set.
 
-RooArgSet* RooIntegralMorph::actualParameters(const RooArgSet& /*nset*/) const
+RooFit::OwningPtr<RooArgSet> RooIntegralMorph::actualParameters(const RooArgSet& /*nset*/) const
 {
-  RooArgSet* par1 = pdf1.arg().getParameters(RooArgSet()) ;
-  RooArgSet* par2 = pdf2.arg().getParameters(RooArgSet()) ;
-  par1->add(*par2,kTRUE) ;
-  par1->remove(x.arg(),kTRUE,kTRUE) ;
+  std::unique_ptr<RooArgSet> par1{pdf1->getParameters(static_cast<RooArgSet*>(nullptr))};
+  RooArgSet par2;
+  pdf2->getParameters(nullptr, par2);
+  par1->add(par2,true) ;
+  par1->remove(x.arg(),true,true) ;
   if (!_cacheAlpha) {
     par1->add(alpha.arg()) ;
   }
-  delete par2 ;
-  return par1 ;
+  return RooFit::makeOwningPtr(std::move(par1));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -186,26 +183,23 @@ void RooIntegralMorph::fillCacheObject(PdfCacheElem& cache) const
 
   if (!_cacheAlpha) {
 
-    TIterator* dIter = cache.hist()->sliceIterator((RooAbsArg&)x.arg(),RooArgSet()) ;
-    mcache.calculate(dIter) ;
-    delete dIter ;
+    std::unique_ptr<TIterator> dIter{cache.hist()->sliceIterator(const_cast<RooAbsReal&>(x.arg()),RooArgSet())};
+    mcache.calculate(dIter.get());
 
   } else {
-    TIterator* slIter = cache.hist()->sliceIterator((RooAbsArg&)alpha.arg(),RooArgSet()) ;
+    std::unique_ptr<TIterator> slIter{cache.hist()->sliceIterator(const_cast<RooAbsReal&>(alpha.arg()),RooArgSet())};
 
-    Double_t alphaSave = alpha ;
+    double alphaSave = alpha ;
     RooArgSet alphaSet(alpha.arg()) ;
     coutP(Eval) << "RooIntegralMorph::fillCacheObject(" << GetName() << ") filling multi-dimensional cache" ;
     while(slIter->Next()) {
       alphaSet.assign(*cache.hist()->get()) ;
-      TIterator* dIter = cache.hist()->sliceIterator((RooAbsArg&)x.arg(),RooArgSet(alpha.arg())) ;
-      mcache.calculate(dIter) ;
+      std::unique_ptr<TIterator> dIter{cache.hist()->sliceIterator(const_cast<RooAbsReal&>(x.arg()),RooArgSet(alpha.arg()))};
+      mcache.calculate(dIter.get());
       ccoutP(Eval) << "." << flush;
-      delete dIter ;
     }
-    ccoutP(Eval) << endl ;
+    ccoutP(Eval) << std::endl;
 
-    delete slIter ;
     const_cast<RooIntegralMorph*>(this)->alpha = alphaSave ;
   }
 }
@@ -241,37 +235,38 @@ RooArgList RooIntegralMorph::MorphCacheElem::containedArgs(Action action)
 /// create the cdfs from the input p.d.fs and instantiate the root finders
 /// on the cdfs to perform the inversion.
 
-RooIntegralMorph::MorphCacheElem::MorphCacheElem(RooIntegralMorph& self, const RooArgSet* nsetIn) : PdfCacheElem(self,nsetIn)
+RooIntegralMorph::MorphCacheElem::MorphCacheElem(RooIntegralMorph &self, const RooArgSet *nsetIn)
+   : PdfCacheElem(self, nsetIn),
+     _self(&self),
+     _pdf1(static_cast<RooAbsPdf *>(self.pdf1.absArg())),
+     _pdf2(static_cast<RooAbsPdf *>(self.pdf2.absArg())),
+     _x(static_cast<RooRealVar *>(self.x.absArg())),
+     _alpha(static_cast<RooAbsReal *>(self.alpha.absArg())),
+     _yatXmin(0),
+     _yatXmax(0),
+     _ccounter(0),
+     _ycutoff(1e-7)
 {
   // Mark in base class that normalization of cached pdf is invariant under pdf parameters
-  _x = (RooRealVar*)self.x.absArg() ;
-  _nset = new RooArgSet(*_x) ;
 
-  _alpha = (RooAbsReal*)self.alpha.absArg() ;
-  _pdf1 = (RooAbsPdf*)(self.pdf1.absArg()) ;
-  _pdf2 = (RooAbsPdf*)(self.pdf2.absArg()) ;
-  _c1 = _pdf1->createCdf(*_x);
-  _c2 = _pdf2->createCdf(*_x) ;
-  _cb1 = _c1->bindVars(*_x,_nset) ;
-  _cb2 = _c2->bindVars(*_x,_nset) ;
-  _self = &self ;
+  _nset = std::make_unique<RooArgSet>(*_x);
 
-  _rf1 = new RooBrentRootFinder(*_cb1) ;
-  _rf2 = new RooBrentRootFinder(*_cb2) ;
-  _ccounter = 0 ;
+  _c1 = std::unique_ptr<RooAbsReal>{_pdf1->createCdf(*_x)};
+  _c2 = std::unique_ptr<RooAbsReal>{_pdf2->createCdf(*_x)};
+  _cb1 = std::unique_ptr<RooAbsFunc>{_c1->bindVars(*_x,_nset.get())};
+  _cb2 = std::unique_ptr<RooAbsFunc>{_c2->bindVars(*_x,_nset.get())};
+
+  _rf1 = std::make_unique<RooBrentRootFinder>(*_cb1);
+  _rf2 = std::make_unique<RooBrentRootFinder>(*_cb2);
 
   _rf1->setTol(1e-12) ;
   _rf2->setTol(1e-12) ;
-  _ycutoff = 1e-7 ;
 
   // _yatX = 0 ;
   // _calcX = 0 ;
 
   // Must do this here too: fillCache() may not be called if cache contents is retrieved from EOcache
-  pdf()->setUnitNorm(kTRUE) ;
-
-  _yatXmax = 0 ;
-  _yatXmin = 0 ;
+  pdf()->setUnitNorm(true) ;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -279,27 +274,24 @@ RooIntegralMorph::MorphCacheElem::MorphCacheElem(RooIntegralMorph& self, const R
 
 RooIntegralMorph::MorphCacheElem::~MorphCacheElem()
 {
-  delete _rf1 ;
-  delete _rf2 ;
-  // delete[] _yatX ;
-  // delete[] _calcX ;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Calculate the x value of the output p.d.f at the given cdf value y.
 /// The ok boolean is filled with the success status of the operation.
 
-Double_t RooIntegralMorph::MorphCacheElem::calcX(Double_t y, Bool_t& ok)
+double RooIntegralMorph::MorphCacheElem::calcX(double y, bool& ok)
 {
   if (y<0 || y>1) {
     oocoutW(_self,Eval) << "RooIntegralMorph::MorphCacheElem::calcX() WARNING: requested root finding for unphysical CDF value " << y << endl ;
   }
-  Double_t x1,x2 ;
+  double x1;
+  double x2;
 
-  Double_t xmax = _x->getMax("cache") ;
-  Double_t xmin = _x->getMin("cache") ;
+  double xmax = _x->getMax("cache") ;
+  double xmin = _x->getMin("cache") ;
 
-  ok=kTRUE ;
+  ok=true ;
   ok &= _rf1->findRoot(x1,xmin,xmax,y) ;
   ok &= _rf2->findRoot(x2,xmin,xmax,y) ;
   if (!ok) return 0 ;
@@ -311,10 +303,10 @@ Double_t RooIntegralMorph::MorphCacheElem::calcX(Double_t y, Bool_t& ok)
 ////////////////////////////////////////////////////////////////////////////////
 /// Return the bin number enclosing the given x value
 
-Int_t RooIntegralMorph::MorphCacheElem::binX(Double_t X)
+Int_t RooIntegralMorph::MorphCacheElem::binX(double X)
 {
-  Double_t xmax = _x->getMax("cache") ;
-  Double_t xmin = _x->getMin("cache") ;
+  double xmax = _x->getMax("cache") ;
+  double xmin = _x->getMin("cache") ;
   return (Int_t)(_x->numBins("cache")*(X-xmin)/(xmax-xmin)) ;
 }
 
@@ -324,17 +316,16 @@ Int_t RooIntegralMorph::MorphCacheElem::binX(Double_t X)
 
 void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
 {
-  Double_t xsave = _self->x ;
+  double xsave = _self->x ;
 
   // if (!_yatX) {
-  //   _yatX = new Double_t[_x->numBins("cache")+1] ;
-  //   _calcX = new Double_t[_x->numBins("cache")+1] ;
+  //   _yatX = new double[_x->numBins("cache")+1] ;
+  //   _calcX = new double[_x->numBins("cache")+1] ;
   // }
 
  _yatX.resize(_x->numBins("cache")+1);
  _calcX.resize(_x->numBins("cache")+1);
 
-  RooArgSet nsetTmp(*_x) ;
   _ccounter = 0 ;
 
   // Get number of bins from PdfCacheElem histogram
@@ -353,13 +344,13 @@ void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
   for (int i=0 ; i<10 ; i++) {
 
     // Take a point in y
-    Double_t offset = _yatX[_yatXmin] ;
-    Double_t delta = (_yatX[_yatXmax] - _yatX[_yatXmin])/10 ;
-    Double_t y = offset + i*delta ;
+    double offset = _yatX[_yatXmin] ;
+    double delta = (_yatX[_yatXmax] - _yatX[_yatXmin])/10 ;
+    double y = offset + i*delta ;
 
     // Calculate corresponding X
-    Bool_t ok ;
-    Double_t X = calcX(y,ok) ;
+    bool ok ;
+    double X = calcX(y,ok) ;
     if (ok) {
       Int_t iX = binX(X) ;
       _yatX[iX] = y ;
@@ -383,17 +374,17 @@ void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
   }
 
   // Make one more iteration to recalculate Y value at bin centers
-  Double_t xmax = _x->getMax("cache") ;
-  Double_t xmin = _x->getMin("cache") ;
-  Double_t binw = (xmax-xmin)/_x->numBins("cache") ;
+  double xmax = _x->getMax("cache") ;
+  double xmin = _x->getMin("cache") ;
+  double binw = (xmax-xmin)/_x->numBins("cache") ;
   for (int i=_yatXmin+1 ; i<_yatXmax-1 ; i++) {
 
     // Calculate additional offset to apply if bin ixlo does not have X value calculated at bin center
-    Double_t xBinC = xmin + (i+0.5)*binw ;
-    Double_t xOffset = xBinC-_calcX[i] ;
-    if (fabs(xOffset/binw)>1e-3) {
-      Double_t slope = (_yatX[i+1]-_yatX[i-1])/(_calcX[i+1]-_calcX[i-1]) ;
-      Double_t newY = _yatX[i] + slope*xOffset ;
+    double xBinC = xmin + (i+0.5)*binw ;
+    double xOffset = xBinC-_calcX[i] ;
+    if (std::abs(xOffset/binw)>1e-3) {
+      double slope = (_yatX[i+1]-_yatX[i-1])/(_calcX[i+1]-_calcX[i-1]) ;
+      double newY = _yatX[i] + slope*xOffset ;
       //cout << "bin " << i << " needs to be re-centered " << xOffset/binw << " slope = " << slope << " origY = " << _yatX[i] << " newY = " << newY << endl ;
       _yatX[i] = newY ;
     }
@@ -405,21 +396,29 @@ void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
     //_hist->get(i) ;
     hist()->set(0) ;
   }
+
+  double x1 = _x->getMin("cache");
+  double x2 = _x->getMin("cache");
+
+  double xMax = _x->getMax("cache");
+
   // Transfer calculated values to histogram
   for (int i=_yatXmin ; i<_yatXmax ; i++) {
 
-    Double_t y = _yatX[i] ;
+    double y = _yatX[i] ;
 
-    Double_t x1,x2 ;
+    // Little optimization here exploiting the fact that th cumulative
+    // distribution functions increase monotonically, so we already know that
+    // the next x-value must be higher than the last one as y is increasing. So
+    // we can use the previous x values as lower bounds.
+    _rf1->findRoot(x1,x1,xMax,y) ;
+    _rf2->findRoot(x2,x2,xMax,y) ;
 
-    Double_t xMin = _x->getMin("cache") ;
-    Double_t xMax = _x->getMax("cache") ;
-    _rf1->findRoot(x1,xMin,xMax,y) ;
-    _rf2->findRoot(x2,xMin,xMax,y) ;
-
-    _x->setVal(x1) ; Double_t f1x1 = _pdf1->getVal(&nsetTmp) ;
-    _x->setVal(x2) ; Double_t f2x2 = _pdf2->getVal(&nsetTmp) ;
-    Double_t fbarX = f1x1*f2x2 / ( _alpha->getVal()*f2x2 + (1-_alpha->getVal())*f1x1 ) ;
+    _x->setVal(x1);
+    double f1x1 = _pdf1->getVal(_nset.get());
+    _x->setVal(x2);
+    double f2x2 = _pdf2->getVal(_nset.get());
+    double fbarX = f1x1*f2x2 / ( _alpha->getVal()*f2x2 + (1-_alpha->getVal())*f1x1 ) ;
 
     dIter->Next() ;
     //_hist->get(i) ;
@@ -432,7 +431,7 @@ void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
     hist()->set(0) ;
   }
 
-  pdf()->setUnitNorm(kTRUE) ;
+  pdf()->setUnitNorm(true) ;
   _self->x = xsave ;
 
   oocxcoutD(_self,Eval) << "RooIntegralMorph::MorphCacheElem::calculate(" << _self->GetName() << ") calculation required " << _ccounter << " samplings of cdfs" << endl ;
@@ -444,7 +443,7 @@ void RooIntegralMorph::MorphCacheElem::calculate(TIterator* dIter)
 /// If the midpoint value of y is very close to the midpoint in x, use interpolation
 /// to fill the gaps, otherwise the intervals again.
 
-void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t splitPoint)
+void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, double splitPoint)
 {
   // CONVENTION: _yatX[ixlo] is filled, _yatX[ixhi] is filled, elements in between are empty
   //   cout << "fillGap: gap from _yatX[" << ixlo << "]=" << _yatX[ixlo] << " to _yatX[" << ixhi << "]=" << _yatX[ixhi] << ", size = " << ixhi-ixlo << endl ;
@@ -459,9 +458,9 @@ void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t
   }
 
   // Determine where half-way Y value lands
-  Double_t ymid = _yatX[ixlo]*splitPoint + _yatX[ixhi]*(1-splitPoint) ;
-  Bool_t ok ;
-  Double_t Xmid = calcX(ymid,ok) ;
+  double ymid = _yatX[ixlo]*splitPoint + _yatX[ixhi]*(1-splitPoint) ;
+  bool ok ;
+  double Xmid = calcX(ymid,ok) ;
   if (!ok) {
     oocoutW(_self,Eval) << "RooIntegralMorph::MorphCacheElem::fillGap(" << _self->GetName() << ") unable to calculate midpoint in gap ["
          << ixlo << "," << ixhi << "], resorting to interpolation" << endl ;
@@ -469,7 +468,7 @@ void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t
   }
 
   Int_t iX = binX(Xmid) ;
-  Double_t cq = (Xmid-_calcX[ixlo])/(_calcX[ixhi]-_calcX[ixlo])-0.5 ;
+  double cq = (Xmid-_calcX[ixlo])/(_calcX[ixhi]-_calcX[ixlo])-0.5 ;
 
   // Store midway point
   _yatX[iX] = ymid ;
@@ -477,7 +476,7 @@ void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t
 
 
   // Policy: If centration quality is better than 1% OR better than 1/10 of a bin, fill interval with linear interpolation
-  if (fabs(cq)<0.01 || fabs(cq*(ixhi-ixlo))<0.1 || ymid<_ycutoff ) {
+  if (std::abs(cq)<0.01 || std::abs(cq*(ixhi-ixlo))<0.1 || ymid<_ycutoff ) {
 
     // Fill remaining gaps on either side with linear interpolation
     if (iX-ixlo>1) {
@@ -493,7 +492,7 @@ void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t
 
       if (splitPoint<0.95) {
    // Midway value lands on lowest bin, retry split with higher split point
-   Double_t newSplit = splitPoint + 0.5*(1-splitPoint) ;
+   double newSplit = splitPoint + 0.5*(1-splitPoint) ;
    fillGap(ixlo,ixhi,newSplit) ;
       } else {
    // Give up and resort to interpolation
@@ -504,7 +503,7 @@ void RooIntegralMorph::MorphCacheElem::fillGap(Int_t ixlo, Int_t ixhi, Double_t
 
       // Midway value lands on highest bin, retry split with lower split point
       if (splitPoint>0.05) {
-   Double_t newSplit = splitPoint/2 ;
+   double newSplit = splitPoint/2 ;
    fillGap(ixlo,ixhi,newSplit) ;
       } else {
    // Give up and resort to interpolation
@@ -532,16 +531,16 @@ void RooIntegralMorph::MorphCacheElem::interpolateGap(Int_t ixlo, Int_t ixhi)
 {
   //cout << "filling gap with linear interpolation ixlo=" << ixlo << " ixhi=" << ixhi << endl ;
 
-  Double_t xmax = _x->getMax("cache") ;
-  Double_t xmin = _x->getMin("cache") ;
-  Double_t binw = (xmax-xmin)/_x->numBins("cache") ;
+  double xmax = _x->getMax("cache") ;
+  double xmin = _x->getMin("cache") ;
+  double binw = (xmax-xmin)/_x->numBins("cache") ;
 
   // Calculate deltaY in terms of actual X difference calculate, not based on nominal bin width
-  Double_t deltaY = (_yatX[ixhi]-_yatX[ixlo])/((_calcX[ixhi]-_calcX[ixlo])/binw) ;
+  double deltaY = (_yatX[ixhi]-_yatX[ixlo])/((_calcX[ixhi]-_calcX[ixlo])/binw) ;
 
   // Calculate additional offset to apply if bin ixlo does not have X value calculated at bin center
-  Double_t xBinC = xmin + (ixlo+0.5)*binw ;
-  Double_t xOffset = xBinC-_calcX[ixlo] ;
+  double xBinC = xmin + (ixlo+0.5)*binw ;
+  double xOffset = xBinC-_calcX[ixlo] ;
 
   for (int j=ixlo+1 ; j<ixhi ; j++) {
     _yatX[j] = _yatX[ixlo]+(xOffset+(j-ixlo))*deltaY ;
@@ -559,14 +558,17 @@ void RooIntegralMorph::MorphCacheElem::interpolateGap(Int_t ixlo, Int_t ixhi)
 
 void RooIntegralMorph::MorphCacheElem::findRange()
 {
-  Double_t xmin = _x->getMin("cache") ;
-  Double_t xmax = _x->getMax("cache") ;
+  double xmin = _x->getMin("cache") ;
+  double xmax = _x->getMax("cache") ;
   Int_t nbins = _x->numBins("cache") ;
 
-  Double_t x1,x2 ;
-  Bool_t ok = kTRUE ;
-  Double_t ymin=0.1,yminSave(-1) ;
-  Double_t Xsave(-1),Xlast=xmax ;
+  double x1;
+  double x2;
+  bool ok = true ;
+  double ymin = 0.1;
+  double yminSave(-1);
+  double Xsave(-1);
+  double Xlast = xmax;
 
   // Find lowest Y value that can be measured
   // Start at 0.1 and iteratively lower limit by sqrt(10)
@@ -579,8 +581,8 @@ void RooIntegralMorph::MorphCacheElem::findRange()
     if (!ok) break ;
 
     // Terminate if value of X no longer moves by >0.1 bin size
-    Double_t X = _alpha->getVal()*x1 + (1-_alpha->getVal())*x2 ;
-    if (fabs(X-Xlast)/(xmax-xmin)<0.0001) {
+    double X = _alpha->getVal()*x1 + (1-_alpha->getVal())*x2 ;
+    if (std::abs(X-Xlast)/(xmax-xmin)<0.0001) {
       break ;
     }
     Xlast=X ;
@@ -603,8 +605,9 @@ void RooIntegralMorph::MorphCacheElem::findRange()
 
   // Find highest Y value that can be measured
   // Start at 1 - 0.1 and iteratively lower delta by sqrt(10)
-  ok = kTRUE ;
-  Double_t deltaymax=0.1, deltaymaxSave(-1) ;
+  ok = true ;
+  double deltaymax = 0.1;
+  double deltaymaxSave(-1);
   Xlast=xmin ;
   while(true) {
     ok &= _rf1->findRoot(x1,xmin,xmax,1-deltaymax) ;
@@ -616,8 +619,8 @@ void RooIntegralMorph::MorphCacheElem::findRange()
     if (!ok) break ;
 
     // Terminate if value of X no longer moves by >0.1 bin size
-    Double_t X = _alpha->getVal()*x1 + (1-_alpha->getVal())*x2 ;
-    if (fabs(X-Xlast)/(xmax-xmin)<0.0001) {
+    double X = _alpha->getVal()*x1 + (1-_alpha->getVal())*x2 ;
+    if (std::abs(X-Xlast)/(xmax-xmin)<0.0001) {
       break ;
     }
     Xlast=X ;
@@ -650,7 +653,7 @@ void RooIntegralMorph::MorphCacheElem::findRange()
 ////////////////////////////////////////////////////////////////////////////////
 /// Dummy
 
-Double_t RooIntegralMorph::evaluate() const
+double RooIntegralMorph::evaluate() const
 {
   return 0 ;
 }
