@@ -44,8 +44,6 @@ Cranmer KS, Kernel Estimation in High-Energy Physics.
 
 #include "TError.h"
 
-using namespace std;
-
 ClassImp(RooKeysPdf);
 
 const double RooKeysPdf::_nSigma = std::sqrt(-2. *
@@ -54,84 +52,59 @@ const double RooKeysPdf::_nSigma = std::sqrt(-2. *
 ////////////////////////////////////////////////////////////////////////////////
 /// coverity[UNINIT_CTOR]
 
-  RooKeysPdf::RooKeysPdf() : _nEvents(0), _dataPts(0), _dataWgts(0), _weights(0), _sumWgt(0),
-              _mirrorLeft(false), _mirrorRight(false),
-              _asymLeft(false), _asymRight(false)
+RooKeysPdf::RooKeysPdf()
 {
-  TRACE_CREATE
+   TRACE_CREATE;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// cache stuff about x
 
-RooKeysPdf::RooKeysPdf(const char *name, const char *title,
-                       RooAbsReal& x, RooDataSet& data,
-                       Mirror mirror, double rho) :
-  RooAbsPdf(name,title),
-  _x("x","observable",this,x),
-  _nEvents(0),
-  _dataPts(0),
-  _dataWgts(0),
-  _weights(0),
-  _mirrorLeft(mirror==MirrorLeft || mirror==MirrorBoth || mirror==MirrorLeftAsymRight),
-  _mirrorRight(mirror==MirrorRight || mirror==MirrorBoth || mirror==MirrorAsymLeftRight),
-  _asymLeft(mirror==MirrorAsymLeft || mirror==MirrorAsymLeftRight || mirror==MirrorAsymBoth),
-  _asymRight(mirror==MirrorAsymRight || mirror==MirrorLeftAsymRight || mirror==MirrorAsymBoth),
-  _rho(rho)
+RooKeysPdf::RooKeysPdf(const char *name, const char *title, RooAbsReal &x, RooDataSet &data, Mirror mirror, double rho)
+   : RooKeysPdf(name, title, x, static_cast<RooRealVar &>(x), data, mirror, rho)
 {
-  snprintf(_varName, 128,"%s", x.GetName());
-  RooAbsRealLValue& real= (RooRealVar&)(_x.arg());
-  _lo = real.getMin();
-  _hi = real.getMax();
-  _binWidth = (_hi-_lo)/(_nPoints-1);
-
-  // form the lookup table
-  LoadDataSet(data);
-  TRACE_CREATE
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// cache stuff about x
 
-RooKeysPdf::RooKeysPdf(const char *name, const char *title,
-                       RooAbsReal& xpdf, RooRealVar& xdata, RooDataSet& data,
-                       Mirror mirror, double rho) :
-  RooAbsPdf(name,title),
-  _x("x","Observable",this,xpdf),
-  _nEvents(0),
-  _dataPts(0),
-  _dataWgts(0),
-  _weights(0),
-  _mirrorLeft(mirror==MirrorLeft || mirror==MirrorBoth || mirror==MirrorLeftAsymRight),
-  _mirrorRight(mirror==MirrorRight || mirror==MirrorBoth || mirror==MirrorAsymLeftRight),
-  _asymLeft(mirror==MirrorAsymLeft || mirror==MirrorAsymLeftRight || mirror==MirrorAsymBoth),
-  _asymRight(mirror==MirrorAsymRight || mirror==MirrorLeftAsymRight || mirror==MirrorAsymBoth),
-  _rho(rho)
+RooKeysPdf::RooKeysPdf(const char *name, const char *title, RooAbsReal &xpdf, RooRealVar &xdata, RooDataSet &data,
+                       Mirror mirror, double rho)
+   : RooAbsPdf(name, title),
+     _x("x", "Observable", this, xpdf),
+     _mirrorLeft(mirror == MirrorLeft || mirror == MirrorBoth || mirror == MirrorLeftAsymRight),
+     _mirrorRight(mirror == MirrorRight || mirror == MirrorBoth || mirror == MirrorAsymLeftRight),
+     _asymLeft(mirror == MirrorAsymLeft || mirror == MirrorAsymLeftRight || mirror == MirrorAsymBoth),
+     _asymRight(mirror == MirrorAsymRight || mirror == MirrorLeftAsymRight || mirror == MirrorAsymBoth),
+     _lo(xdata.getMin()),
+     _hi(xdata.getMax()),
+     _binWidth((_hi - _lo) / (_nPoints - 1)),
+     _rho(rho)
 {
   snprintf(_varName, 128,"%s", xdata.GetName());
-  RooAbsRealLValue& real= (RooRealVar&)(xdata);
-  _lo = real.getMin();
-  _hi = real.getMax();
-  _binWidth = (_hi-_lo)/(_nPoints-1);
 
   // form the lookup table
   LoadDataSet(data);
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-RooKeysPdf::RooKeysPdf(const RooKeysPdf& other, const char* name):
-  RooAbsPdf(other,name), _x("x",this,other._x), _nEvents(other._nEvents),
-  _dataPts(0), _dataWgts(0), _weights(0), _sumWgt(0),
-  _mirrorLeft( other._mirrorLeft ), _mirrorRight( other._mirrorRight ),
-  _asymLeft(other._asymLeft), _asymRight(other._asymRight),
-  _rho( other._rho ) {
+RooKeysPdf::RooKeysPdf(const RooKeysPdf &other, const char *name)
+   : RooAbsPdf(other, name),
+     _x("x", this, other._x),
+     _nEvents(other._nEvents),
+     _mirrorLeft(other._mirrorLeft),
+     _mirrorRight(other._mirrorRight),
+     _asymLeft(other._asymLeft),
+     _asymRight(other._asymRight),
+     _lo(other._lo),
+     _hi(other._hi),
+     _binWidth(other._binWidth),
+     _rho(other._rho)
+{
   // cache stuff about x
   snprintf(_varName, 128, "%s", other._varName );
-  _lo = other._lo;
-  _hi = other._hi;
-  _binWidth = other._binWidth;
 
   // copy over data and weights... not necessary, commented out for speed
 //    _dataPts = new double[_nEvents];
@@ -145,7 +118,7 @@ RooKeysPdf::RooKeysPdf(const RooKeysPdf& other, const char* name):
   for (Int_t i= 0; i<_nPoints+1; i++)
     _lookupTable[i]= other._lookupTable[i];
 
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -155,7 +128,7 @@ RooKeysPdf::~RooKeysPdf() {
   delete[] _dataWgts;
   delete[] _weights;
 
-  TRACE_DESTROY
+  TRACE_DESTROY;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -179,10 +152,12 @@ void RooKeysPdf::LoadDataSet( RooDataSet& data) {
 
   std::vector<Data> tmp;
   tmp.reserve((1 + _mirrorLeft + _mirrorRight) * data.numEntries());
-  double x0 = 0., x1 = 0., x2 = 0.;
+  double x0 = 0.;
+  double x1 = 0.;
+  double x2 = 0.;
   _sumWgt = 0.;
   // read the data set into tmp and accumulate some statistics
-  RooRealVar& real = (RooRealVar&)(data.get()->operator[](_varName));
+  RooRealVar& real = static_cast<RooRealVar&>(data.get()->operator[](_varName));
   for (Int_t i = 0; i < data.numEntries(); ++i) {
     data.get(i);
     const double x = real.getVal();
