@@ -52,7 +52,7 @@ The namespace RooFit contains mostly switches that change the behaviour of funct
 
 These switches are documented with the relevant functions, e.g. RooAbsPdf::fitTo().
 For an introduction to RooFit (not the namespace), check the [user's guides](https://root.cern/root-user-guides-and-manuals),
-[courses](https://root.cern/courses) or [the RooFit chapter of the Manual](https://root.cern/manual/roofit/).
+[courses](https://root.cern/learn/courses) or [the RooFit chapter of the Manual](https://root.cern/manual/roofit/).
 */
 namespace RooFit {
 
@@ -103,10 +103,14 @@ RooCmdArg Range(double lo, double hi, bool adjustNorm=true) ;
 RooCmdArg NormRange(const char* rangeNameList) ;
 RooCmdArg VLines() ;
 RooCmdArg LineColor(Color_t color) ;
+RooCmdArg LineColor(std::string const &color) ;
 RooCmdArg LineStyle(Style_t style) ;
+RooCmdArg LineStyle(std::string const &style) ;
 RooCmdArg LineWidth(Width_t width) ;
 RooCmdArg FillColor(Color_t color) ;
+RooCmdArg FillColor(std::string const &color) ;
 RooCmdArg FillStyle(Style_t style) ;
+RooCmdArg FillStyle(std::string const &style) ;
 RooCmdArg ProjectionRange(const char* rangeName) ;
 RooCmdArg Name(const char* name) ;
 RooCmdArg Invisible(bool inv=true) ;
@@ -135,8 +139,10 @@ RooCmdArg Binning(const RooAbsBinning& binning) ;
 RooCmdArg Binning(const char* binningName) ;
 RooCmdArg Binning(int nBins, double xlo=0.0, double xhi=0.0) ;
 RooCmdArg MarkerStyle(Style_t style) ;
+RooCmdArg MarkerStyle(std::string const &style) ;
 RooCmdArg MarkerSize(Size_t size) ;
 RooCmdArg MarkerColor(Color_t color) ;
+RooCmdArg MarkerColor(std::string const &color) ;
 RooCmdArg CutRange(const char* rangeName) ;
 RooCmdArg XErrorSize(double width) ;
 RooCmdArg RefreshNorm() ;
