@@ -27,8 +27,11 @@
 #include <list>
 #include <string>
 
-typedef RooArgList* pRooArgList ;
-typedef RooLinkedList* pRooLinkedList ;
+namespace RooFit {
+namespace Detail {
+class RooFixedProdPdf;
+}
+}
 
 class RooProdPdf : public RooAbsPdf {
 public:
@@ -75,8 +78,7 @@ public:
   bool isDirectGenSafe(const RooAbsArg& arg) const override ;
 
   // Constraint management
-  RooArgSet* getConstraints(const RooArgSet& observables, RooArgSet& constrainedParams,
-                            bool stripDisconnected, bool removeConstraintsFromPdf=false) const override ;
+  RooArgSet* getConstraints(const RooArgSet& observables, RooArgSet const& constrainedParams, RooArgSet &pdfParams) const override ;
 
   std::list<double>* plotSamplingHint(RooAbsRealLValue& obs, double xlo, double xhi) const override ;
   std::list<double>* binBoundaries(RooAbsRealLValue& /*obs*/, double /*xlo*/, double /*xhi*/) const override ;
@@ -99,6 +101,25 @@ public:
 
   std::unique_ptr<RooAbsArg> compileForNormSet(RooArgSet const &normSet, RooFit::Detail::CompileContext & ctx) const override;
 
+  // The cache object. Internal, do not use.
+  class CacheElem final : public RooAbsCacheElement {
+  public:
+    CacheElem() : _isRearranged(false) { }
+    // Payload
+    RooArgList _partList ;
+    RooArgList _numList ;
+    RooArgList _denList ;
+    RooArgList _ownedList ;
+    std::vector<std::unique_ptr<RooArgSet>> _normList;
+    bool _isRearranged ;
+    std::unique_ptr<RooAbsReal> _rearrangedNum{};
+    std::unique_ptr<RooAbsReal> _rearrangedDen{};
+    // Cache management functions
+    RooArgList containedArgs(Action) override ;
+    void printCompactTreeHook(std::ostream&, const char *, Int_t, Int_t) override ;
+    void writeToStream(std::ostream& os) const ;
+  } ;
+
 private:
 
   std::unique_ptr<RooArgSet> fillNormSetForServer(RooArgSet const& normSet, RooAbsArg const& server) const;
@@ -115,6 +136,7 @@ private:
                         RooLinkedList& termList,   RooLinkedList& normList,
                         RooLinkedList& impDepList, RooLinkedList& crossDepList,
                         RooLinkedList& intList) const;
+
   std::string makeRGPPName(const char* pfx, const RooArgSet& term, const RooArgSet& iset, const RooArgSet& nset, const char* isetRangeName) const ;
   void groupProductTerms(std::list<std::vector<RooArgSet*>>& groupedTerms, RooArgSet& outerIntDeps,
                          const RooLinkedList& terms, const RooLinkedList& norms,
@@ -128,30 +150,9 @@ private:
                      const RooArgSet* term,const RooArgSet& termNSet, const RooArgSet& termISet,
                      bool& isOwned, bool forceWrap=false) const ;
 
-
   CacheMode canNodeBeCached() const override { return RooAbsArg::NotAdvised ; } ;
   void setCacheAndTrackHints(RooArgSet&) override ;
 
-  // The cache object
-  class CacheElem final : public RooAbsCacheElement {
-  public:
-    CacheElem() : _isRearranged(false) { }
-    ~CacheElem() override = default;
-    // Payload
-    RooArgList _partList ;
-    RooArgList _numList ;
-    RooArgList _denList ;
-    RooArgList _ownedList ;
-    std::vector<std::unique_ptr<RooArgSet>> _normList;
-    bool _isRearranged ;
-    std::unique_ptr<RooAbsReal> _rearrangedNum{};
-    std::unique_ptr<RooAbsReal> _rearrangedDen{};
-    // Cache management functions
-    RooArgList containedArgs(Action) override ;
-    void printCompactTreeHook(std::ostream&, const char *, Int_t, Int_t) override ;
-    void writeToStream(std::ostream& os) const ;
-  } ;
-
   std::unique_ptr<CacheElem> createCacheElem(const RooArgSet* nset, const RooArgSet* iset, const char* isetRangeName=nullptr) const;
 
   mutable RooObjCacheManager _cacheMgr ; //! The cache manager
@@ -161,11 +162,11 @@ private:
   std::unique_ptr<RooAbsReal> specializeIntegral(RooAbsReal& orig, const char* targetRangeName) const ;
   std::unique_ptr<RooAbsReal> specializeRatio(RooFormulaVar& input, const char* targetRangeName) const ;
   double calculate(const RooProdPdf::CacheElem& cache, bool verbose=false) const ;
-  void calculateBatch(RooAbsArg const* caller, const RooProdPdf::CacheElem &cache, double* output, size_t nEvents, RooFit::Detail::DataMap const&) const;
+  void doEvalImpl(RooAbsArg const* caller, const RooProdPdf::CacheElem &cache, RooFit::EvalContext &) const;
 
 
   friend class RooProdGenContext ;
-  friend class RooFixedProdPdf ;
+  friend class RooFit::Detail::RooFixedProdPdf ;
   RooAbsGenContext* genContext(const RooArgSet &vars, const RooDataSet *prototype=nullptr,
                                   const RooArgSet *auxProto=nullptr, bool verbose= false) const override ;
 
@@ -192,5 +193,75 @@ private:
   ClassDefOverride(RooProdPdf,6) // PDF representing a product of PDFs
 };
 
+namespace RooFit {
+namespace Detail {
+
+/// A RooProdPdf with a fixed normalization set can be replaced by this class.
+/// Its purpose is to provide the right client-server interface for the
+/// evaluation of RooProdPdf cache elements that were created for a given
+/// normalization set.
+class RooFixedProdPdf : public RooAbsPdf {
+public:
+   RooFixedProdPdf(std::unique_ptr<RooProdPdf> &&prodPdf, RooArgSet const &normSet);
+   RooFixedProdPdf(const RooFixedProdPdf &other, const char *name = nullptr);
+
+   inline TObject *clone(const char *newname) const override { return new RooFixedProdPdf(*this, newname); }
+
+   inline bool selfNormalized() const override { return true; }
+
+   inline bool canComputeBatchWithCuda() const override { return true; }
+
+   inline void doEval(RooFit::EvalContext &ctx) const override { _prodPdf->doEvalImpl(this, *_cache, ctx); }
+
+   void translate(RooFit::Detail::CodeSquashContext &ctx) const override;
+
+   inline ExtendMode extendMode() const override { return _prodPdf->extendMode(); }
+   inline double expectedEvents(const RooArgSet * /*nset*/) const override
+   {
+      return _prodPdf->expectedEvents(&_normSet);
+   }
+   inline std::unique_ptr<RooAbsReal> createExpectedEventsFunc(const RooArgSet * /*nset*/) const override
+   {
+      return _prodPdf->createExpectedEventsFunc(&_normSet);
+   }
+
+   // Analytical Integration handling
+   inline bool forceAnalyticalInt(const RooAbsArg &dep) const override { return _prodPdf->forceAnalyticalInt(dep); }
+   inline Int_t getAnalyticalIntegralWN(RooArgSet &allVars, RooArgSet &analVars, const RooArgSet *normSet,
+                                        const char *rangeName = nullptr) const override
+   {
+      return _prodPdf->getAnalyticalIntegralWN(allVars, analVars, normSet, rangeName);
+   }
+   inline Int_t
+   getAnalyticalIntegral(RooArgSet &allVars, RooArgSet &numVars, const char *rangeName = nullptr) const override
+   {
+      return _prodPdf->getAnalyticalIntegral(allVars, numVars, rangeName);
+   }
+   inline double analyticalIntegralWN(Int_t code, const RooArgSet *normSet, const char *rangeName) const override
+   {
+      return _prodPdf->analyticalIntegralWN(code, normSet, rangeName);
+   }
+   inline double analyticalIntegral(Int_t code, const char *rangeName = nullptr) const override
+   {
+      return _prodPdf->analyticalIntegral(code, rangeName);
+   }
+
+   RooProdPdf::CacheElem const &cache() const { return *_cache; }
+
+private:
+   void initialize();
+
+   inline double evaluate() const override { return _prodPdf->calculate(*_cache); }
+
+   RooArgSet _normSet;
+   std::unique_ptr<RooProdPdf::CacheElem> _cache;
+   RooSetProxy _servers;
+   std::unique_ptr<RooProdPdf> _prodPdf;
+
+   ClassDefOverride(RooFit::Detail::RooFixedProdPdf, 0);
+};
+
+} // namespace Detail
+} // namespace RooFit
 
 #endif
