@@ -18,7 +18,7 @@
 /**  \class RooAbsArg
      \ingroup Roofitcore
 
-RooAbsArg is the common abstract base class for objects that
+Common abstract base class for objects that
 represent a value and a "shape" in RooFit. Values or shapes usually depend on values
 or shapes of other RooAbsArg instances. Connecting several RooAbsArg in
 a computation graph models an expression tree that can be evaluated.
@@ -27,7 +27,7 @@ a computation graph models an expression tree that can be evaluated.
 Therefore, RooAbsArg provides functionality to connect objects of type RooAbsArg into
 a computation graph to pass values between those objects.
 A value can e.g. be a real-valued number, (instances of RooAbsReal), or an integer, that is,
-catgory index (instances of RooAbsCategory). The third subclass of RooAbsArg is RooStringVar,
+category index (instances of RooAbsCategory). The third subclass of RooAbsArg is RooStringVar,
 but it is rarely used.
 
 The "shapes" that a RooAbsArg can possess can e.g. be the definition
@@ -66,43 +66,39 @@ for single nodes.
 
 */
 
-#include "TBuffer.h"
-#include "TClass.h"
-#include "TVirtualStreamerInfo.h"
-#include "strlcpy.h"
+#include <RooAbsArg.h>
 
-#include "RooSecondMoment.h"
-#include "RooWorkspace.h"
+#include <RooAbsCategoryLValue.h>
+#include <RooAbsData.h>
+#include <RooAbsDataStore.h>
+#include <RooArgProxy.h>
+#include <RooArgSet.h>
+#include <RooConstVar.h>
+#include <RooExpensiveObjectCache.h>
+#include <RooHelpers.h>
+#include "RooFitImplHelpers.h"
+#include <RooListProxy.h>
+#include <RooMsgService.h>
+#include <RooRealIntegral.h>
+#include <RooResolutionModel.h>
+#include <RooSetProxy.h>
+#include <RooTrace.h>
+#include <RooTreeDataStore.h>
+#include <RooVectorDataStore.h>
+#include <RooWorkspace.h>
 
-#include "RooMsgService.h"
-#include "RooAbsArg.h"
-#include "RooArgSet.h"
-#include "RooArgProxy.h"
-#include "RooSetProxy.h"
-#include "RooListProxy.h"
-#include "RooAbsData.h"
-#include "RooAbsCategoryLValue.h"
-#include "RooTrace.h"
-#include "RooRealIntegral.h"
-#include "RooConstVar.h"
-#include "RooExpensiveObjectCache.h"
-#include "RooAbsDataStore.h"
-#include "RooResolutionModel.h"
-#include "RooVectorDataStore.h"
-#include "RooTreeDataStore.h"
-#include "RooHelpers.h"
+#include <TBuffer.h>
+#include <TClass.h>
+#include <TVirtualStreamerInfo.h>
 
 #include <algorithm>
 #include <cstring>
 #include <fstream>
-#include <iostream>
-#include <memory>
 #include <sstream>
 
-using namespace std;
+using std::cout, std::endl, std::ostream, std::string, std::set, std::map, std::istream, std::pair, std::ofstream, std::make_pair;
 
 ClassImp(RooAbsArg);
-;
 
 bool RooAbsArg::_verboseDirty(false) ;
 bool RooAbsArg::_inhibitDirty(false) ;
@@ -115,14 +111,7 @@ std::stack<RooAbsArg*> RooAbsArg::_ioReadStack ;
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
 
-RooAbsArg::RooAbsArg()
-   : TNamed(), _deleteWatch(false), _valueDirty(true), _shapeDirty(true), _operMode(Auto), _fast(false), _ownedComponents(nullptr),
-     _prohibitServerRedirect(false), _namePtr(0), _isConstant(false), _localNoInhibitDirty(false),
-     _myws(0)
-{
-  _namePtr = RooNameReg::instance().constPtr(GetName()) ;
-
-}
+RooAbsArg::RooAbsArg() : _namePtr(RooNameReg::instance().constPtr(GetName())) {}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Create an object with the specified name and descriptive title.
@@ -130,9 +119,7 @@ RooAbsArg::RooAbsArg()
 /// dirty flags set.
 
 RooAbsArg::RooAbsArg(const char *name, const char *title)
-   : TNamed(name, title), _deleteWatch(false), _valueDirty(true), _shapeDirty(true), _operMode(Auto), _fast(false),
-     _ownedComponents(0), _prohibitServerRedirect(false), _namePtr(0), _isConstant(false),
-     _localNoInhibitDirty(false), _myws(0)
+   : TNamed(name, title)
 {
   if (name == nullptr || strlen(name) == 0) {
     throw std::logic_error("Each RooFit object needs a name. "
@@ -146,16 +133,19 @@ RooAbsArg::RooAbsArg(const char *name, const char *title)
 /// object. Transient properties and client-server links are not copied
 
 RooAbsArg::RooAbsArg(const RooAbsArg &other, const char *name)
-   : TNamed(name ? name : other.GetName(), other.GetTitle()), RooPrintable(other),
+   : TNamed(name ? name : other.GetName(), other.GetTitle()),
+     RooPrintable(other),
      _boolAttrib(other._boolAttrib),
-     _stringAttrib(other._stringAttrib), _deleteWatch(other._deleteWatch), _operMode(Auto), _fast(false),
-     _ownedComponents(0), _prohibitServerRedirect(false),
+     _stringAttrib(other._stringAttrib),
+     _deleteWatch(other._deleteWatch),
      _namePtr(name ? RooNameReg::instance().constPtr(name) : other._namePtr),
-     _isConstant(other._isConstant), _localNoInhibitDirty(other._localNoInhibitDirty), _myws(0)
+     _isConstant(other._isConstant),
+     _localNoInhibitDirty(other._localNoInhibitDirty)
 {
 
   // Copy server list by hand
-  bool valueProp, shapeProp ;
+  bool valueProp;
+  bool shapeProp;
   for (const auto server : other._serverList) {
     valueProp = server->_clientListValue.containsByNamePtr(&other);
     shapeProp = server->_clientListShape.containsByNamePtr(&other);
@@ -164,10 +154,6 @@ RooAbsArg::RooAbsArg(const RooAbsArg &other, const char *name)
 
   setValueDirty() ;
   setShapeDirty() ;
-
-  //setAttribute(Form("CloneOf(%08x)",&other)) ;
-  //cout << "RooAbsArg::cctor(" << this << ") #bools = " << _boolAttrib.size() << " #strings = " << _stringAttrib.size() << endl ;
-
 }
 
 
@@ -186,10 +172,9 @@ RooAbsArg::~RooAbsArg()
   bool first(true) ;
   for (auto client : clientListTmp) {
     client->setAttribute("ServerDied") ;
-    TString attr("ServerDied:");
-    attr.Append(GetName());
-    attr.Append(Form("(%zx)",(size_t)this)) ;
-    client->setAttribute(attr.Data());
+    std::stringstream attr;
+    attr << "ServerDied:" << GetName() << "(" << reinterpret_cast<std::size_t>(this) << ")";
+    client->setAttribute(attr.str().c_str());
     client->removeServer(*this,true);
 
     if (_verboseDirty) {
@@ -206,7 +191,7 @@ RooAbsArg::~RooAbsArg()
 
   if (_ownedComponents) {
     delete _ownedComponents ;
-    _ownedComponents = 0 ;
+    _ownedComponents = nullptr ;
   }
 
 }
@@ -230,15 +215,6 @@ void RooAbsArg::verboseDirty(bool flag)
   _verboseDirty = flag ;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Check if this object was created as a clone of 'other'
-
-bool RooAbsArg::isCloneOf(const RooAbsArg& other) const
-{
-  return (getAttribute(Form("CloneOf(%zx)",(size_t)&other)) ||
-     other.getAttribute(Form("CloneOf(%zx)",(size_t)this))) ;
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Set (default) or clear a named boolean attribute of this object.
@@ -246,7 +222,7 @@ bool RooAbsArg::isCloneOf(const RooAbsArg& other) const
 void RooAbsArg::setAttribute(const Text_t* name, bool value)
 {
   // Preserve backward compatibility - any strong
-  if(string("Constant")==name) {
+  if(std::string{"Constant"} == name) {
     _isConstant = value ;
   }
 
@@ -302,7 +278,7 @@ const Text_t* RooAbsArg::getStringAttribute(const Text_t* key) const
   if (iter!=_stringAttrib.end()) {
     return iter->second.c_str() ;
   } else {
-    return 0 ;
+    return nullptr ;
   }
 }
 
@@ -402,9 +378,11 @@ void RooAbsArg::addServerList(RooAbsCollection& serverList, bool valueProp, bool
 void RooAbsArg::removeServer(RooAbsArg& server, bool force)
 {
   if (_prohibitServerRedirect) {
-    cxcoutF(LinkStateMgmt) << "RooAbsArg::addServer(" << this << "," << GetName() << "): PROHIBITED SERVER REMOVAL REQUESTED: removing server "
-            << server.GetName() << "(" << &server << ")" << endl ;
-    assert(0) ;
+    std::stringstream ss;
+    ss << "RooAbsArg::addServer(" << this << "," << GetName() << "): PROHIBITED SERVER REMOVAL REQUESTED: removing server "
+       << server.GetName() << "(" << &server << ")";
+    cxcoutF(LinkStateMgmt) << ss.str() << std::endl;
+    throw std::runtime_error(ss.str());
   }
 
   if (_verboseDirty) {
@@ -562,11 +540,11 @@ void RooAbsArg::treeNodeServerList(RooAbsCollection* list, const RooAbsArg* arg,
 
 RooFit::OwningPtr<RooArgSet> RooAbsArg::getParameters(const RooAbsData* set, bool stripDisconnected) const
 {
-  return getParameters(set?set->get():0,stripDisconnected) ;
+  return getParameters(set?set->get():nullptr,stripDisconnected) ;
 }
 
 
-/// Return the parameters of this p.d.f when used in conjuction with dataset 'data'.
+/// Return the parameters of this p.d.f when used in conjunction with dataset 'data'.
 RooFit::OwningPtr<RooArgSet> RooAbsArg::getParameters(const RooAbsData& data, bool stripDisconnected) const
 {
   return getParameters(&data,stripDisconnected) ;
@@ -634,7 +612,7 @@ void RooAbsArg::addParameters(RooAbsCollection& params, const RooArgSet* nset, b
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Obtain an estimate of the number of parameters of the function and its daughters.
-/// Calling `addParamters` for large functions (NLL) can cause many reallocations of
+/// Calling `addParameters` for large functions (NLL) can cause many reallocations of
 /// `params` due to the recursive behaviour. This utility function aims to pre-compute
 /// the total number of parameters, so that enough memory is reserved.
 /// The estimate is not fully accurate (overestimate) as there is no equivalent to `getParametersHook`.
@@ -673,7 +651,7 @@ std::size_t RooAbsArg::getParametersSizeEstimate(const RooArgSet* nset) const
 /// ourself as top node that don't match any of the names the args in the
 /// supplied argset. Returns `true` only if something went wrong.
 /// The complement of this function is getObservables().
-/// \param[in] observables Set of leafs to ignore because they are observables and not parameters.
+/// \param[in] observables Set of leaves to ignore because they are observables and not parameters.
 /// \param[out] outputSet Output set.
 /// \param[in] stripDisconnected Allow pdf to strip parameters from list before adding it.
 
@@ -767,7 +745,7 @@ RooFit::OwningPtr<RooArgSet> RooAbsArg::getObservables(const RooArgSet* dataList
 /// The complement of this function is getParameters().
 /// \param[in] dataList Set of leaf nodes to match.
 /// \param[out] outputSet Output set.
-/// \param[in] valueOnly If this parameter is true, we only match leafs that
+/// \param[in] valueOnly If this parameter is true, we only match leaves that
 ///                      depend on the value of any arg in `dataList`.
 
 bool RooAbsArg::getObservables(const RooAbsCollection* dataList, RooArgSet& outputSet, bool valueOnly) const
@@ -779,7 +757,7 @@ bool RooAbsArg::getObservables(const RooAbsCollection* dataList, RooArgSet& outp
 
   // Make iterator over tree leaf node list
   RooArgSet leafList("leafNodeServerList") ;
-  treeNodeServerList(&leafList,0,false,true,valueOnly) ;
+  treeNodeServerList(&leafList,nullptr,false,true,valueOnly) ;
 
   if (valueOnly) {
     for (const auto arg : leafList) {
@@ -966,7 +944,7 @@ void RooAbsArg::setValueDirty(const RooAbsArg* source)
   }
 
   // Cyclical dependency interception
-  if (source==0) {
+  if (source==nullptr) {
     source=this ;
   } else if (source==this) {
     // Cyclical dependency, abort
@@ -1010,7 +988,7 @@ void RooAbsArg::setShapeDirty(const RooAbsArg* source)
   }
 
   // Set 'dirty' shape state for this object and propagate flag to all its clients
-  if (source==0) {
+  if (source==nullptr) {
     source=this ;
   } else if (source==this) {
     // Cyclical dependency, abort
@@ -1087,8 +1065,6 @@ bool RooAbsArg::redirectServers(const RooAbsCollection& newSetOrig, bool mustRep
     newSet = newSetOwned.get();
   }
 
-  bool ret(false) ;
-
   // Replace current servers with new servers with the same name from the given list
   for (auto oldServer : _serverList) {
 
@@ -1114,6 +1090,8 @@ bool RooAbsArg::redirectServers(const RooAbsCollection& newSetOrig, bool mustRep
   setValueDirty() ;
   setShapeDirty() ;
 
+  bool ret(false) ;
+
   // Process the proxies
   for (int i=0 ; i<numProxies() ; i++) {
     RooAbsProxy* p = getProxy(i) ;
@@ -1168,7 +1146,7 @@ bool RooAbsArg::callRedirectServersHook(RooAbsCollection const &newSet, bool mus
 ////////////////////////////////////////////////////////////////////////////////
 /// Replace some servers of this object. If there are proxies that correspond
 /// to the replaced servers, these proxies are adjusted as well.
-/// \param[in] replacements Map that specifiecs which args replace which servers.
+/// \param[in] replacements Map that specifies which args replace which servers.
 bool RooAbsArg::redirectServers(std::unordered_map<RooAbsArg *, RooAbsArg *> const &replacements)
 {
    bool ret(false);
@@ -1222,7 +1200,7 @@ bool RooAbsArg::redirectServers(std::unordered_map<RooAbsArg *, RooAbsArg *> con
 /// \return Pointer to the new server or `nullptr` if there's no unique match.
 RooAbsArg *RooAbsArg::findNewServer(const RooAbsCollection &newSet, bool nameChange) const
 {
-  RooAbsArg *newServer = 0;
+  RooAbsArg *newServer = nullptr;
   if (!nameChange) {
     newServer = newSet.find(*this) ;
   }
@@ -1232,26 +1210,25 @@ RooAbsArg *RooAbsArg::findNewServer(const RooAbsCollection &newSet, bool nameCha
     TString nameAttrib("ORIGNAME:") ;
     nameAttrib.Append(GetName()) ;
 
-    RooArgSet* tmp = (RooArgSet*) newSet.selectByAttrib(nameAttrib,true) ;
-    if(0 != tmp) {
+    if (auto tmp = std::unique_ptr<RooAbsCollection>{newSet.selectByAttrib(nameAttrib,true)}) {
 
       // Check if any match was found
       if (tmp->empty()) {
-        delete tmp ;
-        return 0 ;
+        return nullptr;
       }
 
       // Check if match is unique
-      if(tmp->getSize()>1) {
-        coutF(LinkStateMgmt) << "RooAbsArg::redirectServers(" << GetName() << "): FATAL Error, " << tmp->getSize() << " servers with "
-            << nameAttrib << " attribute" << endl ;
+      if(tmp->size()>1) {
+        std::stringstream ss;
+        ss << "RooAbsArg::redirectServers(" << GetName() << "): FATAL Error, " << tmp->size() << " servers with "
+            << nameAttrib << " attribute";
+        coutF(LinkStateMgmt) << ss.str() << std::endl;
         tmp->Print("v") ;
-        assert(0) ;
+        throw std::runtime_error(ss.str());
       }
 
       // use the unique element in the set
       newServer= tmp->first();
-      delete tmp ;
     }
   }
   return newServer;
@@ -1344,7 +1321,7 @@ bool RooAbsArg::redirectServersHook(const RooAbsCollection & /*newServerList*/,
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Register an RooArgProxy in the proxy list. This function is called by owned
-/// proxies upon creation. After registration, this arg wil forward pointer
+/// proxies upon creation. After registration, this arg will forward pointer
 /// changes from serverRedirects and updates in cached normalization sets
 /// to the proxies immediately after they occur. The proxied argument is
 /// also added as value and/or shape server
@@ -1389,7 +1366,7 @@ void RooAbsArg::unRegisterProxy(RooArgProxy& proxy)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Register an RooSetProxy in the proxy list. This function is called by owned
-/// proxies upon creation. After registration, this arg wil forward pointer
+/// proxies upon creation. After registration, this arg will forward pointer
 /// changes from serverRedirects and updates in cached normalization sets
 /// to the proxies immediately after they occur.
 
@@ -1424,7 +1401,7 @@ void RooAbsArg::unRegisterProxy(RooSetProxy& proxy)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Register an RooListProxy in the proxy list. This function is called by owned
-/// proxies upon creation. After registration, this arg wil forward pointer
+/// proxies upon creation. After registration, this arg will forward pointer
 /// changes from serverRedirects and updates in cached normalization sets
 /// to the proxies immediately after they occur.
 
@@ -1565,9 +1542,9 @@ void RooAbsArg::printClassName(ostream& os) const
 }
 
 
+/// Print address of this RooAbsArg.
 void RooAbsArg::printAddress(ostream& os) const
 {
-  // Print addrss of this RooAbsArg
   os << this ;
 }
 
@@ -1584,7 +1561,7 @@ void RooAbsArg::printArgs(ostream& os) const
   os << "[ " ;
   for (Int_t i=0 ; i<numProxies() ; i++) {
     RooAbsProxy* p = getProxy(i) ;
-    if (p==0) continue ;
+    if (p==nullptr) continue ;
     if (!TString(p->name()).BeginsWith("!")) {
       p->print(os) ;
       os << " " ;
@@ -1728,7 +1705,7 @@ void RooAbsArg::printAttribList(ostream& os) const
 void RooAbsArg::attachArgs(const RooAbsCollection &set)
 {
   RooArgSet branches;
-  branchNodeServerList(&branches,0,true);
+  branchNodeServerList(&branches,nullptr,true);
 
   for(auto const& branch : branches) {
     branch->redirectServers(set,false,false);
@@ -1842,10 +1819,11 @@ void RooAbsArg::optimizeCacheMode(const RooArgSet& observables, RooArgSet& optim
   // this node has not been processed (FindObject returns a null pointer)
   auto obj = processedNodes.findArg(this);
   assert(obj != this); // obj == this cannot happen
-  if (obj)
+  if (obj) {
      // here for nodes with duplicate names
-     cxcoutI(Optimization) << "RooAbsArg::optimizeCacheMode(" << GetName()
-                           << " node " << this << " exists already as " << obj << " but with the SAME name !" << endl;
+     cxcoutI(Optimization) << "RooAbsArg::optimizeCacheMode(" << GetName() << " node " << this << " exists already as "
+                           << obj << " but with the SAME name !" << endl;
+  }
 
   processedNodes.Add(this);
 
@@ -1988,7 +1966,7 @@ void RooAbsArg::setOperMode(OperMode mode, bool recurseADirty)
   if (mode==_operMode) return ;
 
   _operMode = mode ;
-  _fast = ((mode==AClean) || dynamic_cast<RooRealVar*>(this)!=0 || dynamic_cast<RooConstVar*>(this)!=0 ) ;
+  _fast = ((mode==AClean) || dynamic_cast<RooRealVar*>(this) || dynamic_cast<RooConstVar*>(this) ) ;
   for (Int_t i=0 ;i<numCaches() ; i++) {
     getCache(i)->operModeHook() ;
   }
@@ -2073,7 +2051,7 @@ void RooAbsArg::printComponentTree(const char* indent, const char* namePat, Int_
 {
   if (nLevel==0) return ;
   if (isFundamental()) return ;
-  RooResolutionModel* rmodel = dynamic_cast<RooResolutionModel*>(this) ;
+  auto rmodel = dynamic_cast<RooResolutionModel*>(this) ;
   if (rmodel && rmodel->isConvolved()) return ;
   if (InheritsFrom("RooConstVar")) return ;
 
@@ -2171,7 +2149,7 @@ RooAbsCache* RooAbsArg::getCache(Int_t index) const
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Return RooArgSet with all variables (tree leaf nodes of expresssion tree)
+/// Return RooArgSet with all variables (tree leaf nodes of expression tree)
 
 RooFit::OwningPtr<RooArgSet> RooAbsArg::getVariables(bool stripDisconnected) const
 {
@@ -2179,30 +2157,6 @@ RooFit::OwningPtr<RooArgSet> RooAbsArg::getVariables(bool stripDisconnected) con
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// Return ancestors in cloning chain of this RooAbsArg. NOTE: Returned pointers
-/// are not guaranteed to be 'live', so do not dereference without proper caution
-
-RooLinkedList RooAbsArg::getCloningAncestors() const
-{
-  RooLinkedList retVal ;
-
-  set<string>::const_iterator iter= _boolAttrib.begin() ;
-  while(iter != _boolAttrib.end()) {
-    if (TString(*iter).BeginsWith("CloneOf(")) {
-      char buf[128] ;
-      strlcpy(buf,iter->c_str(),128) ;
-      strtok(buf,"(") ;
-      char* ptrToken = strtok(0,")") ;
-      RooAbsArg* ptr = (RooAbsArg*) strtoll(ptrToken,0,16) ;
-      retVal.Add(ptr) ;
-    }
-  }
-
-  return retVal ;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a GraphViz .dot file visualizing the expression tree headed by
 /// this RooAbsArg object. Use the GraphViz tool suite to make e.g. a gif
@@ -2253,7 +2207,7 @@ void RooAbsArg::graphVizTree(ostream& os, const char* delimiter, bool useTitle,
     string nodeLabel = (useTitle && !nodeTitle.empty()) ? nodeTitle : nodeName;
 
     // if using latex, replace ROOT's # with normal latex backslash
-    string::size_type position = nodeLabel.find("#") ;
+    string::size_type position = nodeLabel.find('#') ;
     while(useLatex && position!=nodeLabel.npos){
       nodeLabel.replace(position, 1, "\\");
     }
@@ -2338,32 +2292,12 @@ bool RooAbsArg::addOwnedComponents(RooArgList&& comps) {
 
 RooAbsArg* RooAbsArg::cloneTree(const char* newname) const
 {
-  // Clone tree using snapshot
-  RooArgSet clonedNodes;
-  RooArgSet(*this).snapshot(clonedNodes, true);
-
-  // Find the head node in the cloneSet
-  RooAbsArg* head = clonedNodes.find(*this) ;
-  assert(head);
-
-  // We better to release the ownership before removing the "head". Otherwise,
-  // "head" might also be deleted as the clonedNodes collection owns it.
-  // (Actually this does not happen because even an owning collection doesn't
-  // delete the element when removed by pointer lookup, but it's better not to
-  // rely on this unexpected fact).
-  clonedNodes.releaseOwnership();
-
-  // Remove the head node from the cloneSet
-  // To release it from the set ownership
-  clonedNodes.remove(*head) ;
-
-  // Add the set as owned component of the head
-  head->addOwnedComponents(std::move(clonedNodes)) ;
+  // In the RooHelpers, there is a more general implementation that we will reuse here
+  RooAbsArg *head = RooHelpers::Detail::cloneTreeWithSameParametersImpl(*this, nullptr);
 
   // Adjust name of head node if requested
   if (newname) {
-    head->TNamed::SetName(newname) ;
-    head->_namePtr = RooNameReg::instance().constPtr(newname) ;
+    head->SetName(newname) ;
   }
 
   // Return the head
@@ -2377,9 +2311,9 @@ RooAbsArg* RooAbsArg::cloneTree(const char* newname) const
 void RooAbsArg::attachToStore(RooAbsDataStore& store)
 {
   if (dynamic_cast<RooTreeDataStore*>(&store)) {
-    attachToTree(((RooTreeDataStore&)store).tree()) ;
+    attachToTree((static_cast<RooTreeDataStore&>(store)).tree()) ;
   } else if (dynamic_cast<RooVectorDataStore*>(&store)) {
-    attachToVStore((RooVectorDataStore&)store) ;
+    attachToVStore(static_cast<RooVectorDataStore&>(store)) ;
   }
 }
 
@@ -2536,14 +2470,17 @@ RooAbsArg::makeLegacyIterator(const RooAbsArg::RefCountList_t& list) const {
 
 void RooRefArray::Streamer(TBuffer &R__b)
 {
-   UInt_t R__s, R__c;
+   UInt_t R__s;
+   UInt_t R__c;
    if (R__b.IsReading()) {
 
-      Version_t R__v = R__b.ReadVersion(&R__s, &R__c); if (R__v) { }
+      Version_t R__v = R__b.ReadVersion(&R__s, &R__c);
+      if (R__v) {
+      }
 
       // Make temporary refArray and read that from the streamer
       auto refArray = std::make_unique<TRefArray>();
-      refArray->Streamer(R__b) ;
+      refArray->Streamer(R__b);
       R__b.CheckByteCount(R__s, R__c, refArray->IsA());
 
       // Schedule deferred processing of TRefArray into proxy list
@@ -2551,17 +2488,16 @@ void RooRefArray::Streamer(TBuffer &R__b)
 
    } else {
 
-     R__c = R__b.WriteVersion(RooRefArray::IsA(), true);
+      R__c = R__b.WriteVersion(RooRefArray::IsA(), true);
 
-     // Make a temporary refArray and write that to the streamer
-     TRefArray refArray;
-     for(TObject * tmpObj : *this) {
-       refArray.Add(tmpObj) ;
-     }
-
-     refArray.Streamer(R__b) ;
-     R__b.SetByteCount(R__c, true) ;
+      // Make a temporary refArray and write that to the streamer
+      TRefArray refArray;
+      for (TObject *tmpObj : *this) {
+         refArray.Add(tmpObj);
+      }
 
+      refArray.Streamer(R__b);
+      R__b.SetByteCount(R__c, true);
    }
 }
 
@@ -2598,6 +2534,23 @@ std::unique_ptr<RooAbsArg> RooAbsArg::compileForNormSet(RooArgSet const & normSe
 }
 
 
+////////////////////////////////////////////////////////////////////////////////
+/// This function defines a translation for each RooAbsReal based object that can be used
+/// to express the class as simple C++ code. The function adds the code represented by
+/// each class as an std::string (that is later concatenated with code strings from translate calls)
+/// to form the C++ code that AD tools can understand. Any class that wants to support AD, has to
+/// implement this function.
+///
+/// \param[in] ctx An object to manage auxiliary information for code-squashing. Also takes the
+/// code string that this class outputs into the squashed code through the 'addToCodeBody' function.
+void RooAbsArg::translate(RooFit::Detail::CodeSquashContext & /*ctx*/) const
+{
+   std::stringstream errorMsg;
+   errorMsg << "Translate function for class \"" << ClassName() << "\" has not yet been implemented.";
+   coutE(Minimization) << errorMsg.str() << std::endl;
+   throw std::runtime_error(errorMsg.str().c_str());
+}
+
 /// Sets the token for retrieving results in the BatchMode. For internal use only.
 void RooAbsArg::setDataToken(std::size_t index)
 {
@@ -2607,7 +2560,7 @@ void RooAbsArg::setDataToken(std::size_t index)
    if (_dataToken != std::numeric_limits<std::size_t>::max()) {
       std::stringstream errMsg;
       errMsg << "The data token for \"" << GetName() << "\" is already set!"
-             << " Are you trying to evaluate the same object by multiple RooFitDriver instances?"
+             << " Are you trying to evaluate the same object by multiple RooFit::Evaluator instances?"
              << " This is not allowed.";
       throw std::runtime_error(errMsg.str());
    }
