@@ -19,32 +19,27 @@
 \class RooAbsCacheElement
 \ingroup Roofitcore
 
-RooAbsCacheElement is the abstract base class for objects to be stored
-in RooAbsCache cache manager objects. Each storage element has an 
+Abstract base class for objects to be stored
+in RooAbsCache cache manager objects. Each storage element has an
 interface to pass on calls for server redirection, operation mode
 change calls and constant term optimization management calls
 **/
 
-
-#include "RooFit.h"
 #include "RooAbsCacheElement.h"
 #include "RooAbsArg.h"
 #include "RooArgList.h"
 
-using namespace std;
-
-ClassImp(RooAbsCacheElement); 
-   ;
+ClassImp(RooAbsCacheElement);
 
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Interface for server redirect calls
 
-Bool_t RooAbsCacheElement::redirectServersHook(const RooAbsCollection& /*newServerList*/, Bool_t /*mustReplaceAll*/, 
-					       Bool_t /*nameChange*/, Bool_t /*isRecursive*/) 
-{ 
-  return kFALSE ; 
-} 
+bool RooAbsCacheElement::redirectServersHook(const RooAbsCollection& /*newServerList*/, bool /*mustReplaceAll*/,
+                      bool /*nameChange*/, bool /*isRecursive*/)
+{
+  return false ;
+}
 
 
 
@@ -61,15 +56,11 @@ void RooAbsCacheElement::printCompactTreeHook(std::ostream&, const char *, Int_t
 /// Interface for cache optimization calls. The default implementation is to forward all these
 /// calls to all contained RooAbsArg objects as publicized through containedArg()
 
-void RooAbsCacheElement::optimizeCacheMode(const RooArgSet& obs, RooArgSet& optNodes, RooLinkedList& processedNodes) 
+void RooAbsCacheElement::optimizeCacheMode(const RooArgSet& obs, RooArgSet& optNodes, RooLinkedList& processedNodes)
 {
-  RooArgList list = containedArgs(OptimizeCaching) ;
-  TIterator* iter = list.createIterator() ;
-  RooAbsArg* arg ;
-  while((arg=(RooAbsArg*)iter->Next())) {    
+  for (RooAbsArg * arg : containedArgs(OptimizeCaching)) {
     arg->optimizeCacheMode(obs, optNodes, processedNodes) ;
   }
-  delete iter ;
 }
 
 
@@ -78,7 +69,7 @@ void RooAbsCacheElement::optimizeCacheMode(const RooArgSet& obs, RooArgSet& optN
 /// Interface for constant term optimization calls. The default implementation is to forward all these
 /// calls to all contained RooAbsArg objects as publicized through containedArg()
 
-void RooAbsCacheElement::findConstantNodes(const RooArgSet& obs, RooArgSet& cacheList, RooLinkedList& processedNodes) 
+void RooAbsCacheElement::findConstantNodes(const RooArgSet& obs, RooArgSet& cacheList, RooLinkedList& processedNodes)
 {
   RooArgList list = containedArgs(FindConstantNodes) ;
   for (const auto arg : list) {
