@@ -178,6 +178,10 @@ RooAbsCollection::~RooAbsCollection()
   if(_ownCont){
     deleteList() ;
   }
+  if (_hashAssistedFind) {
+    delete _hashAssistedFind;
+  }
+  _hashAssistedFind = nullptr;
 }
 
 
@@ -189,6 +193,9 @@ RooAbsCollection::~RooAbsCollection()
 
 void RooAbsCollection::deleteList()
 {
+  if (_hashAssistedFind) {
+    delete _hashAssistedFind;
+  }
   _hashAssistedFind = nullptr;
 
   // Built-in delete remaining elements
@@ -748,6 +755,9 @@ bool RooAbsCollection::remove(const RooAbsCollection& list, bool /*silent*/, boo
 
 void RooAbsCollection::removeAll()
 {
+  if (_hashAssistedFind) {
+    delete _hashAssistedFind;
+  }
   _hashAssistedFind = nullptr;
 
   if(_ownCont) {
@@ -920,7 +930,7 @@ RooAbsArg * RooAbsCollection::find(const char *name) const
 
   if (_hashAssistedFind || _list.size() >= _sizeThresholdForMapSearch) {
     if (!_hashAssistedFind || !_hashAssistedFind->isValid()) {
-      _hashAssistedFind = std::make_unique<HashAssistedFind>(_list.begin(), _list.end());
+      _hashAssistedFind = new HashAssistedFind{_list.begin(), _list.end()};
     }
 
     return _hashAssistedFind->find(nptr);
@@ -940,7 +950,7 @@ RooAbsArg * RooAbsCollection::find(const RooAbsArg& arg) const
 
   if (_hashAssistedFind || _list.size() >= _sizeThresholdForMapSearch) {
     if (!_hashAssistedFind || !_hashAssistedFind->isValid()) {
-      _hashAssistedFind = std::make_unique<HashAssistedFind>(_list.begin(), _list.end());
+      _hashAssistedFind = new HashAssistedFind{_list.begin(), _list.end()};
     }
 
     return _hashAssistedFind->find(nptr);
@@ -1143,9 +1153,7 @@ std::string RooAbsCollection::contentsString() const
     retVal += ",";
   }
 
-  if (!retVal.empty()) {
-    retVal.erase(retVal.end()-1);
-  }
+  retVal.erase(retVal.end()-1);
 
   return retVal;
 }
@@ -1574,18 +1582,6 @@ void RooAbsCollection::sortTopologically() {
    }
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Factory for legacy iterators.
-
-std::unique_ptr<RooAbsCollection::LegacyIterator_t> RooAbsCollection::makeLegacyIterator (bool forward) const {
-   if (!forward) {
-      ccoutE(DataHandling) << "The legacy RooFit collection iterators don't support reverse iterations, any more. "
-                           << "Use begin() and end()" << std::endl;
-   }
-  return std::make_unique<LegacyIterator_t>(_list);
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Insert an element into the owned collections.
 void RooAbsCollection::insert(RooAbsArg* item) {
@@ -1619,10 +1615,13 @@ void RooAbsCollection::useHashMapForFind(bool flag) const
       throw std::runtime_error(msg.str());
    }
 #endif
-   if (flag && !_hashAssistedFind)
-      _hashAssistedFind = std::make_unique<HashAssistedFind>(_list.begin(), _list.end());
-   if (!flag)
+   if (flag && !_hashAssistedFind) {
+      _hashAssistedFind = new HashAssistedFind{_list.begin(), _list.end()};
+   }
+   if (!flag && _hashAssistedFind) {
+      delete _hashAssistedFind;
       _hashAssistedFind = nullptr;
+   }
 }
 
 
