@@ -19,7 +19,7 @@
 \class RooAddition
 \ingroup Roofitcore
 
-RooAddition calculates the sum of a set of RooAbsReal terms, or
+Calculates the sum of a set of RooAbsReal terms, or
 when constructed with two sets, it sums the product of the terms
 in the two sets.
 **/
@@ -33,9 +33,10 @@ in the two sets.
 #include "RooErrorHandler.h"
 #include "RooArgSet.h"
 #include "RooNameReg.h"
-#include "RooNLLVarNew.h"
+#include "RooFit/Detail/RooNLLVarNew.h"
 #include "RooMsgService.h"
 #include "RooBatchCompute.h"
+#include "RooFuncWrapper.h"
 
 #ifdef ROOFIT_LEGACY_EVAL_BACKEND
 #include "RooNLLVar.h"
@@ -53,29 +54,11 @@ ClassImp(RooAddition);
 /// \param[in] name Name of the PDF
 /// \param[in] title Title
 /// \param[in] sumSet The value of the function will be the sum of the values in this set
-/// \param[in] takeOwnership If true, the RooAddition object will take ownership of the arguments in `sumSet`
 
-RooAddition::RooAddition(const char* name, const char* title, const RooArgList& sumSet
-#ifndef ROOFIT_MEMORY_SAFE_INTERFACES
-                         , bool takeOwnership
-#endif
-  )
-  : RooAbsReal(name, title)
-  , _set("!set","set of components",this)
-  , _cacheMgr(this,10)
+RooAddition::RooAddition(const char *name, const char *title, const RooArgList &sumSet)
+   : RooAbsReal(name, title), _set("!set", "set of components", this), _cacheMgr(this, 10)
 {
-  for (RooAbsArg *comp : sumSet) {
-    if (!dynamic_cast<RooAbsReal*>(comp)) {
-      coutE(InputArguments) << "RooAddition::ctor(" << GetName() << ") ERROR: component " << comp->GetName()
-             << " is not of type RooAbsReal" << std::endl;
-      RooErrorHandler::softAbort() ;
-    }
-    _set.add(*comp) ;
-#ifndef ROOFIT_MEMORY_SAFE_INTERFACES
-    if (takeOwnership) _ownedList.addOwned(std::unique_ptr<RooAbsArg>{comp});
-#endif
-  }
-
+  _set.addTyped<RooAbsReal>(sumSet);
 }
 
 
@@ -92,18 +75,11 @@ RooAddition::RooAddition(const char* name, const char* title, const RooArgList&
 /// \param[in] title Title
 /// \param[in] sumSet1 Left-hand element of the pair-wise products
 /// \param[in] sumSet2 Right-hand element of the pair-wise products
-/// \param[in] takeOwnership If true, the RooAddition object will take ownership of the arguments in the `sumSets`
 ///
-RooAddition::RooAddition(const char* name, const char* title, const RooArgList& sumSet1, const RooArgList& sumSet2
-#ifndef ROOFIT_MEMORY_SAFE_INTERFACES
-                         , bool takeOwnership
-#endif
-  )
-    : RooAbsReal(name, title)
-    , _set("!set","set of components",this)
-    , _cacheMgr(this,10)
+RooAddition::RooAddition(const char *name, const char *title, const RooArgList &sumSet1, const RooArgList &sumSet2)
+   : RooAbsReal(name, title), _set("!set", "set of components", this), _cacheMgr(this, 10)
 {
-  if (sumSet1.getSize() != sumSet2.getSize()) {
+  if (sumSet1.size() != sumSet2.size()) {
     coutE(InputArguments) << "RooAddition::ctor(" << GetName() << ") ERROR: input lists should be of equal length" << std::endl;
     RooErrorHandler::softAbort() ;
   }
@@ -132,12 +108,6 @@ RooAddition::RooAddition(const char* name, const char* title, const RooArgList&
     auto prod = std::make_unique<RooProduct>( _name, _name , RooArgSet(*comp1, *comp2));
     _set.add(*prod);
     _ownedList.addOwned(std::move(prod));
-#ifndef ROOFIT_MEMORY_SAFE_INTERFACES
-    if (takeOwnership) {
-        _ownedList.addOwned(std::unique_ptr<RooAbsArg>{comp1});
-        _ownedList.addOwned(std::unique_ptr<RooAbsArg>{comp2});
-    }
-#endif
   }
 }
 
@@ -172,58 +142,47 @@ double RooAddition::evaluate() const
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute addition of PDFs in batches.
-void RooAddition::computeBatch(double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
+void RooAddition::doEval(RooFit::EvalContext &ctx) const
 {
-  RooBatchCompute::VarVector pdfs;
-  RooBatchCompute::ArgVector coefs;
-  pdfs.reserve(_set.size());
-  coefs.reserve(_set.size());
-  for (const auto arg : _set)
-  {
-    pdfs.push_back(dataMap.at(arg));
-    coefs.push_back(1.0);
-  }
-  RooBatchCompute::compute(dataMap.config(this), RooBatchCompute::AddPdf, output, nEvents, pdfs, coefs);
+   std::vector<std::span<const double>> pdfs;
+   std::vector<double> coefs;
+   pdfs.reserve(_set.size());
+   coefs.reserve(_set.size());
+   for (const auto arg : _set) {
+      pdfs.push_back(ctx.at(arg));
+      coefs.push_back(1.0);
+   }
+   RooBatchCompute::compute(ctx.config(this), RooBatchCompute::AddPdf, ctx.output(), pdfs, coefs);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void RooAddition::translate(RooFit::Detail::CodeSquashContext &ctx) const
 {
-   // If the number of elements to sum is less than 3, just build a sum expression.
-   // else build a loop to sum over the values.
-   unsigned int eleSize = _set.size();
+   if (_set.empty()) {
+      ctx.addResult(this, "0.0");
+   }
    std::string result;
-   if (eleSize > 3) {
-      std::string className = GetName();
-      std::string varName = "elements" + className;
-      std::string sumName = "sum" + className;
-      std::string code = "";
-      std::string decl = "double " + varName + "[" + std::to_string(eleSize) + "]{";
-      int idx = 0;
-      for (RooAbsArg *it : _set) {
-         decl += ctx.getResult(*it) + ",";
-         ctx.addResult(it, varName + "[" + std::to_string(idx) + "]");
-         idx++;
+   if (_set.size() > 1)
+      result += "(";
+
+   std::size_t i = 0;
+   for (auto *component : static_range_cast<RooAbsReal *>(_set)) {
+
+      if (!dynamic_cast<RooFit::Detail::RooNLLVarNew *>(component) || _set.size() == 1) {
+         result += ctx.getResult(*component);
+         ++i;
+         if (i < _set.size()) result += '+';
+         continue;
       }
-      decl.back() = '}';
-      code += decl + ";\n";
-
-      ctx.addToGlobalScope("double " + sumName + " = 0;\n");
-      std::string iterator = "i_" + className;
-      code += "for(int " + iterator + " = 0; " + iterator + " < " + std::to_string(eleSize) + "; " + iterator +
-              "++) {\n" + sumName + " += " + varName + "[" + iterator + "];\n}\n";
-      result = sumName;
-      ctx.addResult(this, result);
-
-      ctx.addToCodeBody(this, code);
+      auto &wrp = *ctx._wrapper;
+      auto funcName = wrp.declareFunction(wrp.buildCode(*component));
+      result += funcName + "(params, obs, xlArr)";
+      ++i;
+      if (i < _set.size()) result += '+';
    }
-
-   result = "(";
-   for (RooAbsArg *it : _set) {
-      result += ctx.getResult(*it) + '+';
-   }
-   result.back() = ')';
+   if (_set.size() > 1)
+      result += ')';
    ctx.addResult(this, result);
 }
 
@@ -243,15 +202,15 @@ double RooAddition::defaultErrorLevel() const
 
   std::unique_ptr<RooArgSet> comps{getComponents()};
   for(RooAbsArg * arg : *comps) {
-    if (dynamic_cast<RooNLLVarNew*>(arg)) {
-      nllArg = (RooAbsReal*)arg ;
+    if (dynamic_cast<RooFit::Detail::RooNLLVarNew*>(arg)) {
+      nllArg = static_cast<RooAbsReal*>(arg) ;
     }
 #ifdef ROOFIT_LEGACY_EVAL_BACKEND
     if (dynamic_cast<RooNLLVar*>(arg)) {
-      nllArg = (RooAbsReal*)arg ;
+      nllArg = static_cast<RooAbsReal*>(arg) ;
     }
     if (dynamic_cast<RooChi2Var*>(arg)) {
-      chi2Arg = (RooAbsReal*)arg ;
+      chi2Arg = static_cast<RooAbsReal*>(arg) ;
     }
 #endif
   }
@@ -307,7 +266,7 @@ Int_t RooAddition::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars
 
   // check if we already have integrals for this combination of factors
   Int_t sterileIndex(-1);
-  CacheElem* cache = (CacheElem*) _cacheMgr.getObj(&analVars,&analVars,&sterileIndex,RooNameReg::ptr(rangeName));
+  CacheElem* cache = static_cast<CacheElem*>(_cacheMgr.getObj(&analVars,&analVars,&sterileIndex,RooNameReg::ptr(rangeName)));
   if (cache!=nullptr) {
     Int_t code = _cacheMgr.lastIndex();
     return code+1;
@@ -329,7 +288,7 @@ Int_t RooAddition::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars
 double RooAddition::analyticalIntegral(Int_t code, const char* rangeName) const
 {
   // note: rangeName implicit encoded in code: see _cacheMgr.setObj in getPartIntList...
-  CacheElem *cache = (CacheElem*) _cacheMgr.getObjByIndex(code-1);
+  CacheElem *cache = static_cast<CacheElem*>(_cacheMgr.getObjByIndex(code-1));
   if (cache==nullptr) {
     // cache got sterilized, trigger repopulation of this slot, then try again...
     std::unique_ptr<RooArgSet> vars( getParameters(RooArgSet()) );
