@@ -19,7 +19,7 @@
 \class RooBinningCategory
 \ingroup Roofitcore
 
-Class RooBinningCategory provides a real-to-category mapping defined
+Provides a real-to-category mapping defined
 by a series of thresholds. It evaluates the value of `inputVar` passed in the
 constructor, and converts this into a bin number using a binning defined for
 the inputVar. The name of this binning is passed in the constructor.
@@ -31,12 +31,8 @@ the inputVar. The name of this binning is passed in the constructor.
 #include "Riostream.h"
 #include "RooStreamParser.h"
 
-using namespace std;
-
 ClassImp(RooBinningCategory);
 
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor with input function to be mapped and name and index of default
 /// output state of unmapped values
@@ -46,7 +42,6 @@ RooBinningCategory::RooBinningCategory(const char *name, const char *title, RooA
   RooAbsCategory(name, title), _inputVar("inputVar","Input category",this,inputVar), _bname(binningName)
 {
   initialize(catTypeName) ;
-
 }
 
 
@@ -59,18 +54,6 @@ RooBinningCategory::RooBinningCategory(const RooBinningCategory& other, const ch
 {
 }
 
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor
-
-RooBinningCategory::~RooBinningCategory()
-{
-}
-
-
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Iterator over all bins in input variable and define corresponding state labels
 
@@ -78,7 +61,7 @@ void RooBinningCategory::initialize(const char* catTypeName)
 {
   const int nbins = _inputVar->getBinning(_bname.Length() > 0 ? _bname.Data() : nullptr).numBins();
   for (Int_t i=0 ; i<nbins ; i++) {
-    string name = catTypeName!=nullptr ? Form("%s%d",catTypeName,i)
+    std::string name = catTypeName!=nullptr ? Form("%s%d",catTypeName,i)
             : (_bname.Length()>0 ? Form("%s_%s_bin%d",_inputVar.arg().GetName(),_bname.Data(),i)
             : Form("%s_bin%d",_inputVar.arg().GetName(),i)) ;
     defineState(name,i);
@@ -96,7 +79,7 @@ RooAbsCategory::value_type RooBinningCategory::evaluate() const
   Int_t ibin = _inputVar->getBin(_bname.Length() > 0 ? _bname.Data() : nullptr);
 
   if (!hasIndex(ibin)) {
-    string name = (_bname.Length()>0) ? Form("%s_%s_bin%d",_inputVar.arg().GetName(),_bname.Data(),ibin)
+    std::string name = (_bname.Length()>0) ? Form("%s_%s_bin%d",_inputVar.arg().GetName(),_bname.Data(),ibin)
                                  : Form("%s_bin%d",_inputVar.arg().GetName(),ibin) ;
     const_cast<RooBinningCategory*>(this)->defineState(name,ibin);
   }
@@ -115,15 +98,13 @@ RooAbsCategory::value_type RooBinningCategory::evaluate() const
 ///     Shape : default value
 ///   Verbose : list of thresholds
 
-void RooBinningCategory::printMultiline(ostream& os, Int_t content, bool verbose, TString indent) const
+void RooBinningCategory::printMultiline(std::ostream& os, Int_t content, bool verbose, TString indent) const
 {
    RooAbsCategory::printMultiline(os,content,verbose,indent);
 
    if (verbose) {
-     os << indent << "--- RooBinningCategory ---" << endl
+     os << indent << "--- RooBinningCategory ---" << std::endl
    << indent << "  Maps from " ;
      _inputVar.arg().printStream(os,kName|kValue,kSingleLine);
    }
 }
-
-
