@@ -19,7 +19,7 @@
 \class RooConvCoefVar
 \ingroup Roofitcore
 
-RooConvCoefVar is an auxiliary class that represents the coefficient
+Auxiliary class that represents the coefficient
 of a RooAbsAnaConvPdf implementation as a separate RooAbsReal object
 to be able to interface these coefficient terms with the generic
 RooRealIntegral integration mechanism.
@@ -28,8 +28,6 @@ RooRealIntegral integration mechanism.
 #include "RooAbsAnaConvPdf.h"
 #include "RooConvCoefVar.h"
 
-using namespace std;
-
 ClassImp(RooConvCoefVar);
 
 
@@ -77,7 +75,7 @@ double RooConvCoefVar::getValV(const RooArgSet*) const
 
 double RooConvCoefVar::evaluate() const
 {
-  return ((RooAbsAnaConvPdf&)_convPdf.arg()).coefficient(_coefIdx) ;
+  return (const_cast<RooAbsAnaConvPdf &>(static_cast<RooAbsAnaConvPdf const&>(_convPdf.arg()))).coefficient(_coefIdx) ;
 }
 
 
@@ -87,7 +85,7 @@ double RooConvCoefVar::evaluate() const
 
 Int_t RooConvCoefVar::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* rangeName) const
 {
-  Int_t code = ((RooAbsAnaConvPdf&)_convPdf.arg()).getCoefAnalyticalIntegral(_coefIdx,allVars,analVars,rangeName) ;
+  Int_t code = (static_cast<RooAbsAnaConvPdf const&>(_convPdf.arg())).getCoefAnalyticalIntegral(_coefIdx,allVars,analVars,rangeName) ;
   return code ;
 }
 
@@ -98,6 +96,5 @@ Int_t RooConvCoefVar::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analV
 
 double RooConvCoefVar::analyticalIntegral(Int_t code, const char* rangeName) const
 {
-  return ((RooAbsAnaConvPdf&)_convPdf.arg()).coefAnalyticalIntegral(_coefIdx,code,rangeName) ;
+  return (static_cast<RooAbsAnaConvPdf const&>(_convPdf.arg())).coefAnalyticalIntegral(_coefIdx,code,rangeName) ;
 }
-
