@@ -55,8 +55,8 @@
  *     runblock.defineType("run1") ;
  *     runblock.defineType("run2") ;
  *
- *     RooArgSet splitLeafs;
- *     RooCustomizer cust(pdf,runblock,splitLeafs);
+ *     RooArgSet splitLeaves;
+ *     RooCustomizer cust(pdf,runblock,splitLeaves);
  *     cust.splitArg(alpha,runblock);
  *
  *     RooAbsPdf* pdf_run1 = cust.build("run1") ;
@@ -64,9 +64,9 @@
  *
  *     RooSimultaneous simpdf("simpdf","simpdf",RooArgSet(*pdf_run1,*pdf_run2))
  * ```
- * If the master category state is a super category, leafs may be split
+ * If the master category state is a super category, leaves may be split
  * by any subset of that master category. E.g. if the master category
- * is 'A x B', leafs may be split by A, B or AxB.
+ * is 'A x B', leaves may be split by A, B or AxB.
  *
  * In addition to replacing leaf nodes, RooCustomizer clones all branch
  * nodes that depend directly or indirectly on modified leaf nodes, so
@@ -77,7 +77,7 @@
  * composites are needed.
  *
  * Any leaf nodes that are created by the customizer will be put into
- * the leaf list that is passed into the customizers constructor (splitLeafs in
+ * the leaf list that is passed into the customizers constructor (splitLeaves in
  * the above example. The list owner is responsible for deleting these leaf
  * nodes after the customizer is deleted.
  *
@@ -86,23 +86,23 @@
  *
  * ### Reuse nodes to customise a different PDF
  * By default, the customizer clones the prototype leaf node when splitting a leaf,
- * but the user can feed pre-defined split leafs in leaf list. These leafs
+ * but the user can feed pre-defined split leaves in leaf list. These leaves
  * must have the name `<split_leaf>_<splitcat_label>` to be picked up. The list
- * of pre-supplied leafs may be partial, any missing split leafs will be auto
+ * of pre-supplied leaves may be partial, any missing split leaves will be auto
  * generated.
  *
  * Another common construction is to have two prototype PDFs, each to be customized
  * by a separate customizer instance, that share parameters. To ensure that
- * the customized clones also share their respective split leafs, i.e.
+ * the customized clones also share their respective split leaves, i.e.
  * ```
  *   PDF1(x,y, A) and PDF2(z, A) ---> PDF1_run1(x,y, A_run1) and PDF2_run1(x,y, A_run1)
  *                                    PDF1_run2(x,y, A_run2) and PDF2_run2(x,y, A_run2)
  * ```
  * feed the same split leaf list into both customizers. In that case, the second customizer
- * will pick up the split leafs instantiated by the first customizer and the link between
+ * will pick up the split leaves instantiated by the first customizer and the link between
  * the two PDFs is retained.
  *
- * ### Customising with pre-defined leafs
+ * ### Customising with pre-defined leaves
  * If leaf nodes are provided in the sets, the customiser will use them. This is a complete
  * example that customises the `yield` parameter, and splits (automatically clones) the
  * mean of the Gaussian. This is a short version of the tutorial rf514_RooCustomizer.C.
@@ -127,17 +127,17 @@
  *  sample.defineType("BBG2m2T");
  *
  *
- *  RooArgSet customisedLeafs;
- *  RooArgSet allLeafs;
+ *  RooArgSet customisedLeaves;
+ *  RooArgSet allLeaves;
  *
  *  RooRealVar mass("M", "M", 1, 0, 12000);
  *  RooFormulaVar yield1("yieldSig_BBG1m2T","sigy1","M/3.360779",mass);
  *  RooFormulaVar yield2("yieldSig_BBG2m2T","sigy2","M/2",mass);
- *  allLeafs.add(yield1);
- *  allLeafs.add(yield2);
+ *  allLeaves.add(yield1);
+ *  allLeaves.add(yield2);
  *
  *
- *  RooCustomizer cust(model, sample, customisedLeafs, &allLeafs);
+ *  RooCustomizer cust(model, sample, customisedLeaves, &allLeaves);
  *  cust.splitArg(yieldSig, sample);
  *  cust.splitArg(meanG, sample);
  *
@@ -154,8 +154,6 @@
 #include "RooAbsCategory.h"
 #include "RooAbsArg.h"
 #include "RooAbsPdf.h"
-#include "RooArgSet.h"
-#include "RooArgList.h"
 #include "RooMsgService.h"
 #include "RooHelpers.h"
 
@@ -174,7 +172,7 @@
 #endif
 
 
-using namespace std;
+using std::endl, std::ostream, std::string;
 
 
 namespace {
@@ -206,35 +204,35 @@ Int_t init()
 /// replaceArg() and splitArg() functionality.
 /// \param[in] pdf Proto PDF to be customised.
 /// \param[in] masterCat Category to be used for splitting.
-/// \param[in,out] splitLeafs All nodes created in
+/// \param[in,out] splitLeaves All nodes created in
 /// the customisation process are added to this set.
 /// The user can provide nodes that are *taken*
 /// from the set if they have a name that matches `<parameterNameToBeReplaced>_<category>`.
 /// \note The set needs to own its contents if they are user-provided.
 /// Use *e.g.*
 /// ```
-///  RooArgSet customisedLeafs;
+///  RooArgSet customisedLeaves;
 ///  auto yield1 = new RooFormulaVar("yieldSig_BBG1m2T","sigy1","M/3.360779",mass);
-///  customisedLeafs.addOwned(*yield1);
+///  customisedLeaves.addOwned(*yield1);
 /// ```
-/// \param[in,out] splitLeafsAll All leafs that are used when customising are collected here.
+/// \param[in,out] splitLeavesAll All leaves that are used when customising are collected here.
 /// If this set already contains leaves, they will be used for customising if the names match
 /// as above.
 ///
 
-RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const RooAbsCategoryLValue& masterCat,
-    RooArgSet& splitLeafs, RooArgSet* splitLeafsAll) :
-  _sterile(false),
-  _owning(true),
-  _masterPdf((RooAbsArg*)&pdf),
-  _masterCat((RooAbsCategoryLValue*)&masterCat),
-  _masterBranchList("masterBranchList"),
-  _masterLeafList("masterLeafList"),
-  _internalCloneBranchList("cloneBranchList"),
-  _cloneNodeListAll(splitLeafsAll),
-  _cloneNodeListOwned(&splitLeafs)
+RooCustomizer::RooCustomizer(const RooAbsArg &pdf, const RooAbsCategoryLValue &masterCat, RooArgSet &splitLeaves,
+                             RooArgSet *splitLeavesAll)
+   : _sterile(false),
+     _owning(true),
+     _masterPdf(const_cast<RooAbsArg *>(&pdf)),
+     _masterCat(const_cast<RooAbsCategoryLValue *>(&masterCat)),
+     _masterBranchList("masterBranchList"),
+     _masterLeafList("masterLeafList"),
+     _internalCloneBranchList("cloneBranchList"),
+     _cloneBranchList(&_internalCloneBranchList),
+     _cloneNodeListAll(splitLeavesAll),
+     _cloneNodeListOwned(&splitLeaves)
 {
-  _cloneBranchList = &_internalCloneBranchList ;
 
   initialize() ;
 }
@@ -246,16 +244,16 @@ RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const RooAbsCategoryLValue& m
 /// offer only the replace() method. The supplied 'name' is used as
 /// suffix for any cloned branch nodes
 
-RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const char* name) :
-  _sterile(true),
-  _owning(false),
-  _name(name),
-  _masterPdf((RooAbsArg*)&pdf),
-  _masterBranchList("masterBranchList"),
-  _masterLeafList("masterLeafList"),
-  _internalCloneBranchList("cloneBranchList")
+RooCustomizer::RooCustomizer(const RooAbsArg &pdf, const char *name)
+   : _sterile(true),
+     _owning(false),
+     _name(name),
+     _masterPdf(const_cast<RooAbsArg *>(&pdf)),
+     _masterBranchList("masterBranchList"),
+     _masterLeafList("masterLeafList"),
+     _internalCloneBranchList("cloneBranchList"),
+     _cloneBranchList(&_internalCloneBranchList)
 {
-  _cloneBranchList = &_internalCloneBranchList ;
 
   initialize() ;
 }
@@ -305,7 +303,7 @@ void RooCustomizer::splitArgs(const RooArgSet& set, const RooAbsCategory& splitC
 
 void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCat)
 {
-  if (_splitArgList.FindObject(arg.GetName())) {
+  if (_splitArgList.find(arg.GetName())) {
     oocoutE(nullptr, InputArguments) << "RooCustomizer(" << _masterPdf->GetName() << ") ERROR: multiple splitting rules defined for "
            << arg.GetName() << " only using first rule" << endl ;
     return ;
@@ -317,8 +315,8 @@ void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCa
     return ;
   }
 
-  _splitArgList.Add((RooAbsArg*)&arg) ;
-  _splitCatList.Add((RooAbsCategory*)&splitCat) ;
+  _splitArgList.add(arg);
+  _splitCatList.add(splitCat);
 }
 
 
@@ -328,14 +326,14 @@ void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCa
 
 void RooCustomizer::replaceArg(const RooAbsArg& orig, const RooAbsArg& subst)
 {
-  if (_replaceArgList.FindObject(orig.GetName())) {
+  if (_replaceArgList.find(orig.GetName())) {
     oocoutE(nullptr, InputArguments) << "RooCustomizer(" << _masterPdf->GetName() << ") ERROR: multiple replacement rules defined for "
     << orig.GetName() << " only using first rule" << endl ;
     return ;
   }
 
-  _replaceArgList.Add((RooAbsArg*)&orig) ;
-  _replaceSubList.Add((RooAbsArg*)&subst) ;
+  _replaceArgList.add(orig);
+  _replaceSubList.add(subst);
 }
 
 
@@ -365,7 +363,7 @@ RooAbsArg* RooCustomizer::build(bool verbose)
 
   // If list with owned objects is not empty, assign
   // head node as owner
-  if (allOwned.getSize()>0) {
+  if (!allOwned.empty()) {
     ret->addOwnedComponents(allOwned) ;
   }
 
@@ -417,11 +415,11 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, bool verbose)
   RooArgSet nodeList(_masterLeafList) ;
   nodeList.add(_masterBranchList) ;
 
-  //   cout << "loop over " << nodeList.getSize() << " nodes" << endl ;
+  //   cout << "loop over " << nodeList.size() << " nodes" << endl ;
   for (auto node : nodeList) {
-    RooAbsArg* theSplitArg = !_sterile?(RooAbsArg*) _splitArgList.FindObject(node->GetName()):nullptr ;
+    RooAbsArg* theSplitArg = !_sterile?static_cast<RooAbsArg*>(_splitArgList.find(node->GetName())):nullptr ;
     if (theSplitArg) {
-      RooAbsCategory* splitCat = (RooAbsCategory*) _splitCatList.At(_splitArgList.IndexOf(theSplitArg)) ;
+      RooAbsCategory* splitCat = static_cast<RooAbsCategory*>(_splitCatList.at(_splitArgList.index(theSplitArg))) ;
       if (verbose) {
    oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
                << "): tree node " << node->GetName() << " is split by category " << splitCat->GetName() << endl ;
@@ -468,7 +466,7 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, bool verbose)
    newTitle.Append(")") ;
 
    // Create a new clone
-   RooAbsArg* clone = (RooAbsArg*) node->Clone(newName.Data()) ;
+   RooAbsArg* clone = static_cast<RooAbsArg*>(node->Clone(newName.Data())) ;
    clone->removeStringAttribute("factory_tag") ;
    clone->SetTitle(newTitle) ;
 
@@ -495,9 +493,9 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, bool verbose)
       masterNodesToBeSplit.add(*node) ;
     }
 
-    RooAbsArg* ReplaceArg = (RooAbsArg*) _replaceArgList.FindObject(node->GetName()) ;
+    RooAbsArg* ReplaceArg = static_cast<RooAbsArg*>(_replaceArgList.find(node->GetName())) ;
     if (ReplaceArg) {
-      RooAbsArg* substArg = (RooAbsArg*) _replaceSubList.At(_replaceArgList.IndexOf(ReplaceArg)) ;
+      RooAbsArg* substArg = static_cast<RooAbsArg*>(_replaceSubList.at(_replaceArgList.index(ReplaceArg))) ;
       if (verbose) {
    oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
                << "): tree node " << node->GetName() << " will be replaced by " << substArg->GetName() << endl ;
@@ -559,7 +557,7 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, bool verbose)
     }
 
     // Affix attribute with old name to clone to support name changing server redirect
-    RooAbsArg* clone = (RooAbsArg*) branch->Clone(newName.Data()) ;
+    RooAbsArg* clone = static_cast<RooAbsArg*>(branch->Clone(newName.Data())) ;
     clone->removeStringAttribute("factory_tag") ;
     TString nameAttrib("ORIGNAME:") ;
     nameAttrib.Append(branch->GetName()) ;
@@ -613,19 +611,20 @@ void RooCustomizer::printMultiline(ostream& os, Int_t /*content*/, bool /*verbos
 {
   os << indent << "RooCustomizer for " << _masterPdf->GetName() << (_sterile?" (sterile)":"") << endl ;
 
-  Int_t i, nsplit = _splitArgList.GetSize() ;
+  Int_t i;
+  Int_t nsplit = _splitArgList.size();
   if (nsplit>0) {
     os << indent << "  Splitting rules:" << endl ;
     for (i=0 ; i<nsplit ; i++) {
-      os << indent << "   " << _splitArgList.At(i)->GetName() << " is split by " << _splitCatList.At(i)->GetName() << endl ;
+      os << indent << "   " << _splitArgList.at(i)->GetName() << " is split by " << _splitCatList.at(i)->GetName() << endl ;
     }
   }
 
-  Int_t nrepl = _replaceArgList.GetSize() ;
+  Int_t nrepl = _replaceArgList.size() ;
   if (nrepl>0) {
     os << indent << "  Replacement rules:" << endl ;
     for (i=0 ; i<nrepl ; i++) {
-      os << indent << "   " << _replaceSubList.At(i)->GetName() << " replaces " << _replaceArgList.At(i)->GetName() << endl ;
+      os << indent << "   " << _replaceSubList.at(i)->GetName() << " replaces " << _replaceArgList.at(i)->GetName() << endl ;
     }
   }
 
