@@ -19,7 +19,7 @@
 \class RooDerivative
 \ingroup Roofitcore
 
-RooDerivative represents the first, second, or third order derivative
+Represents the first, second, or third order derivative
 of any RooAbsReal as calculated (numerically) by the MathCore Richardson
 derivator class.
 **/
@@ -39,8 +39,6 @@ derivator class.
 #include "Math/WrappedFunction.h"
 #include "Math/RichardsonDerivator.h"
 
-using namespace std;
-
 ClassImp(RooDerivative);
 
 
@@ -61,28 +59,19 @@ RooDerivative::RooDerivative(const char* name, const char* title, RooAbsReal& fu
   _x("x","x",this,x)
 {
   if (_order<0 || _order>3 ) {
-    throw std::string(Form("RooDerivative::ctor(%s) ERROR, derivation order must be 1,2 or 3",name)) ;
+    throw std::runtime_error(Form("RooDerivative::ctor(%s) ERROR, derivation order must be 1,2 or 3",name)) ;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-RooDerivative::RooDerivative(const char* name, const char* title, RooAbsReal& func, RooRealVar& x, const RooArgSet& nset, Int_t orderIn, double epsIn) :
-  RooAbsReal(name, title),
-  _order(orderIn),
-  _eps(epsIn),
-  _nset("nset","nset",this,false,false),
-  _func("function","function",this,func),
-  _x("x","x",this,x)
+RooDerivative::RooDerivative(const char *name, const char *title, RooAbsReal &func, RooRealVar &x,
+                             const RooArgSet &nset, Int_t orderIn, double epsIn)
+   : RooDerivative(name, title, func, x, orderIn, epsIn)
 {
-  if (_order<0 || _order>3) {
-    throw std::string(Form("RooDerivative::ctor(%s) ERROR, derivation order must be 1,2 or 3",name)) ;
-  }
-  _nset.add(nset) ;
+   _nset.add(nset);
 }
 
-
-
 ////////////////////////////////////////////////////////////////////////////////
 
 RooDerivative::RooDerivative(const RooDerivative& other, const char* name) :
@@ -95,18 +84,7 @@ RooDerivative::RooDerivative(const RooDerivative& other, const char* name) :
 {
 }
 
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor
-
-RooDerivative::~RooDerivative()
-{
-  if (_rd) delete _rd ;
-  if (_ftor) delete _ftor ;
-}
-
-
+RooDerivative::~RooDerivative() = default;
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Calculate value
@@ -114,9 +92,9 @@ RooDerivative::~RooDerivative()
 double RooDerivative::evaluate() const
 {
   if (!_ftor) {
-    _ftor = _func.arg().functor(_x.arg(),RooArgSet(),_nset)  ;
+    _ftor = std::unique_ptr<RooFunctor>{_func.arg().functor(_x.arg(),RooArgSet(),_nset)};
     ROOT::Math::WrappedFunction<RooFunctor&> wf(*_ftor);
-    _rd = new ROOT::Math::RichardsonDerivator(wf,_eps*(_x.max()-_x.min()),true) ;
+    _rd = std::make_unique<ROOT::Math::RichardsonDerivator>(wf,_eps,true);
   }
 
   switch (_order) {
@@ -134,8 +112,6 @@ double RooDerivative::evaluate() const
 
 bool RooDerivative::redirectServersHook(const RooAbsCollection& newServerList, bool mustReplaceAll, bool nameChange, bool isRecursive)
 {
-  delete _ftor ;
-  delete _rd ;
   _ftor = nullptr ;
   _rd = nullptr ;
   return RooAbsReal::redirectServersHook(newServerList, mustReplaceAll, nameChange, isRecursive);
