@@ -316,5 +316,7 @@ double RooFormulaVar::defaultErrorLevel() const
 void RooFormulaVar::translate(RooFit::Detail::CodeSquashContext &ctx) const
 {
    getVal(); // to trigger the creation of the TFormula
-   ctx.addResult(this, ctx.buildCall(_formula->getTFormula()->GetUniqueFuncName().Data(), _actualVars));
+   std::string funcName = _formula->getTFormula()->GetUniqueFuncName().Data();
+   ctx.collectFunction(funcName);
+   ctx.addResult(this, ctx.buildCall(funcName, _actualVars));
 }
