@@ -30,12 +30,12 @@
 #include <RooRealConstant.h>
 #include <RooRealVar.h>
 
+#include <TColor.h>
 #include <TH1.h>
+#include <TInterpreter.h>
 
 #include <algorithm>
 
-using std::ostream;
-
 namespace RooFit {
 
 // anonymous namespace for helper functions for the implementation of the global functions
@@ -80,6 +80,36 @@ RooCmdArg processFlatMap(const char *name, Func_t func, Detail::FlatMap<Key_t, V
    return container;
 }
 
+int interpretString(std::string const &s)
+{
+   return gInterpreter->ProcessLine(s.c_str());
+}
+
+Color_t interpretColorString(std::string const &color)
+{
+   using Map = std::unordered_map<std::string, Color_t>;
+   // Color dictionary to define matplotlib conventions
+   static Map colorMap{{"r", kRed},   {"b", kBlue},  {"g", kGreen},   {"y", kYellow},
+                       {"w", kWhite}, {"k", kBlack}, {"m", kMagenta}, {"c", kCyan}};
+   auto found = colorMap.find(color);
+   if (found != colorMap.end())
+      return found->second;
+   // Lookup color from static color map otherwise
+   return TColor::GetColorByName(color.c_str());
+}
+
+Style_t interpretLineStyleString(std::string const &style)
+{
+   using Map = std::unordered_map<std::string, Style_t>;
+   // Style dictionary to define matplotlib conventions
+   static Map styleMap{{"-", kSolid}, {"--", kDashed}, {":", kDotted}, {"-.", kDashDotted}};
+   auto found = styleMap.find(style);
+   if (found != styleMap.end())
+      return found->second;
+   // Use interpreter if style was not matched in the style map
+   return gInterpreter->ProcessLine(style.c_str());
+}
+
 } // namespace
 
 namespace Experimental {
@@ -186,10 +216,35 @@ RooCmdArg LineColor(Color_t color)
 {
    return RooCmdArg("LineColor", color);
 }
+
+/// The `color` string argument argument will be evaluated with the ROOT
+/// interpreter to get the actual ROOT color enum value, like `ROOT.kRed`.
+/// Here is what you can do with it:
+///
+///   1. Pass a string with the enum value name instead, e.g.:
+/// ~~~ {.cxx}
+/// pdf.plotOn(frame, LineColor("kRed"))
+/// ~~~
+///   2. Pass a string with the corresponding single-character color code following the matplotlib convention:
+/// ~~~ {.cxx}
+/// pdf.plotOn(frame, LineColor("r"))
+/// ~~~
+///   3. Pass a string with the enum value name instead followed by some manipulation of the enum value:
+/// ~~~ {.cxx}
+/// pdf.plotOn(frame, LineColor("kRed+1"))
+/// ~~~
+RooCmdArg LineColor(std::string const &color)
+{
+   return LineColor(interpretColorString(color));
+}
 RooCmdArg LineStyle(Style_t style)
 {
    return RooCmdArg("LineStyle", style);
 }
+RooCmdArg LineStyle(std::string const &style)
+{
+   return LineStyle(interpretLineStyleString(style));
+}
 RooCmdArg LineWidth(Width_t width)
 {
    return RooCmdArg("LineWidth", width);
@@ -198,10 +253,18 @@ RooCmdArg FillColor(Color_t color)
 {
    return RooCmdArg("FillColor", color);
 }
+RooCmdArg FillColor(std::string const &color)
+{
+   return RooCmdArg("FillColor", interpretColorString(color));
+}
 RooCmdArg FillStyle(Style_t style)
 {
    return RooCmdArg("FillStyle", style);
 }
+RooCmdArg FillStyle(std::string const &style)
+{
+   return FillStyle(interpretString(style));
+}
 RooCmdArg ProjectionRange(const char *rangeName)
 {
    return RooCmdArg("ProjectionRange", 0, 0, 0, 0, rangeName, nullptr, nullptr, nullptr);
@@ -278,6 +341,10 @@ RooCmdArg MarkerStyle(Style_t style)
 {
    return RooCmdArg("MarkerStyle", style, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
 }
+RooCmdArg MarkerStyle(std::string const &color)
+{
+   return MarkerStyle(interpretString(color));
+}
 RooCmdArg MarkerSize(Size_t size)
 {
    return RooCmdArg("MarkerSize", 0, 0, size, 0, nullptr, nullptr, nullptr, nullptr);
@@ -286,6 +353,10 @@ RooCmdArg MarkerColor(Color_t color)
 {
    return RooCmdArg("MarkerColor", color, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
 }
+RooCmdArg MarkerColor(std::string const &color)
+{
+   return MarkerColor(interpretColorString(color));
+}
 RooCmdArg CutRange(const char *rangeName)
 {
    return RooCmdArg("CutRange", 0, 0, 0, 0, rangeName, nullptr, nullptr, nullptr);
@@ -942,9 +1013,9 @@ RooCmdArg TagName(const char *name)
 {
    return RooCmdArg("LabelName", 0, 0, 0, 0, name, nullptr, nullptr, nullptr);
 }
-RooCmdArg OutputStream(ostream &os)
+RooCmdArg OutputStream(std::ostream &os)
 {
-   return RooCmdArg("OutputStream", 0, 0, 0, 0, nullptr, nullptr, new RooHelpers::WrapIntoTObject<ostream>(os),
+   return RooCmdArg("OutputStream", 0, 0, 0, 0, nullptr, nullptr, new RooHelpers::WrapIntoTObject<std::ostream>(os),
                     nullptr);
 }
 RooCmdArg Prefix(bool flag)
@@ -1085,7 +1156,7 @@ RooCmdArg LinkFlatMap(FlatMap<std::string, RooAbsData *> const &args)
    return processFlatMap("LinkDataSliceMany", processLinkItem<RooAbsData>, args);
 }
 
-}
+} // namespace Detail
 
 } // namespace RooFit
 
