@@ -19,7 +19,7 @@
 \class RooHistError
 \ingroup Roofitcore
 
-RooHistError is a singleton class used to calculate the error bars
+Singleton class used to calculate the error bars
 for each bin of a RooHist object. Errors are calculated by integrating
 a specified area of a Poisson or Binomail error distribution.
 **/
@@ -32,7 +32,7 @@ a specified area of a Poisson or Binomail error distribution.
 #include "Riostream.h"
 #include <cassert>
 
-using namespace std;
+using std::endl;
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -261,12 +261,13 @@ bool RooHistError::getInterval(const RooAbsFunc *Qu, const RooAbsFunc *Ql, doubl
   assert(nullptr != Qu || nullptr != Ql);
 
   // convert number of sigma into a confidence level
-  double beta= TMath::Erf(nSigma/sqrt(2.));
+  double beta= std::erf(nSigma/sqrt(2.));
   double alpha= 0.5*(1-beta);
 
   // Does the central interval contain the point estimate?
   bool ok(true);
-  double loProb(1),hiProb(0);
+  double loProb(1);
+  double hiProb(0);
   if(nullptr != Ql) loProb= (*Ql)(&pointEstimate);
   if(nullptr != Qu) hiProb= (*Qu)(&pointEstimate);
 
@@ -290,7 +291,8 @@ bool RooHistError::getInterval(const RooAbsFunc *Qu, const RooAbsFunc *Ql, doubl
     // use a central interval
     lo= seek(*Ql,pointEstimate,-stepSize,alpha+beta);
     hi= seek(*Qu,pointEstimate,+stepSize,alpha);
-    RooBrentRootFinder lFinder(*Ql),uFinder(*Qu);
+    RooBrentRootFinder lFinder(*Ql);
+    RooBrentRootFinder uFinder(*Qu);
     ok= lFinder.findRoot(lo,lo,lo+stepSize,alpha+beta);
     ok|= uFinder.findRoot(hi,hi-stepSize,hi,alpha);
   }
@@ -307,8 +309,10 @@ bool RooHistError::getInterval(const RooAbsFunc *Qu, const RooAbsFunc *Ql, doubl
 double RooHistError::seek(const RooAbsFunc &f, double startAt, double step, double value) const
 {
   Int_t steps(1000);
-  double min(f.getMinLimit(1)),max(f.getMaxLimit(1));
-  double x(startAt), f0= f(&startAt) - value;
+  double min(f.getMinLimit(1));
+  double max(f.getMaxLimit(1));
+  double x(startAt);
+  double f0 = f(&startAt) - value;
   do {
     x+= step;
   }
