@@ -19,10 +19,10 @@
 \class RooProduct
 \ingroup Roofitcore
 
-A RooProduct represents the product of a given set of RooAbsReal objects.
-
+Represents the product of a given set of RooAbsReal objects.
 **/
 
+#include "RooConstVar.h"
 #include "RooProduct.h"
 
 #include "RooNameReg.h"
@@ -34,10 +34,7 @@ A RooProduct represents the product of a given set of RooAbsReal objects.
 #include <cmath>
 #include <memory>
 
-using namespace std ;
-
 ClassImp(RooProduct);
-;
 
 class RooProduct::ProdMap : public  std::vector<std::pair<RooArgSet*,RooArgList*> > {} ;
 
@@ -45,27 +42,25 @@ class RooProduct::ProdMap : public  std::vector<std::pair<RooArgSet*,RooArgList*
 namespace {
   typedef RooProduct::ProdMap::iterator RPPMIter ;
   std::pair<RPPMIter,RPPMIter> findOverlap2nd(RPPMIter i, RPPMIter end)  ;
-  void dump_map(ostream& os, RPPMIter i, RPPMIter end) ;
+  void dump_map(std::ostream& os, RPPMIter i, RPPMIter end) ;
 }
 
 
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
 
 RooProduct::RooProduct() : _cacheMgr(this,10)
 {
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Destructor
 
 RooProduct::~RooProduct()
 {
-  TRACE_DESTROY
+  TRACE_DESTROY;
 }
 
 
@@ -82,7 +77,7 @@ RooProduct::RooProduct(const char* name, const char* title, const RooArgList& pr
   for (auto comp : prodSet) {
     addTerm(comp);
   }
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -99,7 +94,7 @@ RooProduct::RooProduct(const RooProduct& other, const char* name) :
   _compCSet("!compCSet",this,other._compCSet),
   _cacheMgr(other._cacheMgr,this)
 {
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -112,7 +107,7 @@ void RooProduct::addTerm(RooAbsArg* term) {
     _compCSet.add(*term) ;
   } else {
     coutE(InputArguments) << "RooProduct::addTerm(" << GetName() << ") ERROR: component " << term->GetName()
-        << " is not of type RooAbsReal or RooAbsCategory" << endl ;
+        << " is not of type RooAbsReal or RooAbsCategory" << std::endl ;
     throw std::invalid_argument("RooProduct can only handle terms deriving from RooAbsReal or RooAbsCategory.");
   }
 }
@@ -150,7 +145,7 @@ RooProduct::ProdMap* RooProduct::groupProductTerms(const RooArgSet& allVars) con
   for (auto const* rcomp : static_range_cast<RooAbsReal*>(_compRSet)) {
     if( !rcomp->dependsOn(allVars) ) indep->add(*rcomp);
   }
-  if (indep->getSize()!=0) {
+  if (!indep->empty()) {
     map->push_back( std::make_pair(new RooArgSet(),indep) );
   } else {
      delete indep;
@@ -192,13 +187,14 @@ RooProduct::ProdMap* RooProduct::groupProductTerms(const RooArgSet& allVars) con
 #ifndef NDEBUG
   // check that we have all variables to be integrated over on the LHS
   // of the map, and all terms in the product do appear on the RHS
-  int nVar=0; int nFunc=0;
+  std::size_t nVar=0;
+  std::size_t nFunc=0;
   for (ProdMap::iterator i = map->begin();i!=map->end();++i) {
-    nVar+=i->first->getSize();
-    nFunc+=i->second->getSize();
+    nVar+=i->first->size();
+    nFunc+=i->second->size();
   }
-  assert(nVar==allVars.getSize());
-  assert(nFunc==_compRSet.getSize());
+  assert(nVar==allVars.size());
+  assert(nFunc==_compRSet.size());
 #endif
   return map;
 }
@@ -215,8 +211,8 @@ Int_t RooProduct::getPartIntList(const RooArgSet* iset, const char *isetRange) c
 
   // check if we already have integrals for this combination of factors
   Int_t sterileIndex(-1);
-  CacheElem* cache = (CacheElem*) _cacheMgr.getObj(iset,iset,&sterileIndex,RooNameReg::ptr(isetRange));
-  if (cache!=0) {
+  CacheElem* cache = static_cast<CacheElem*>(_cacheMgr.getObj(iset,iset,&sterileIndex,RooNameReg::ptr(isetRange)));
+  if (cache!=nullptr) {
     Int_t code = _cacheMgr.lastIndex();
     return code;
   }
@@ -226,7 +222,7 @@ Int_t RooProduct::getPartIntList(const RooArgSet* iset, const char *isetRange) c
   cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") groupProductTerms returned map" ;
   if (dologD(Integration)) {
     dump_map(ccoutD(Integration),map->begin(),map->end());
-    ccoutD(Integration) << endl;
+    ccoutD(Integration) << std::endl;
   }
 
   // did we find any factorizable terms?
@@ -242,33 +238,33 @@ Int_t RooProduct::getPartIntList(const RooArgSet* iset, const char *isetRange) c
   cache = new CacheElem();
 
   for (ProdMap::const_iterator i = map->begin();i!=map->end();++i) {
-    RooAbsReal *term(0);
-    if (i->second->getSize()>1) { // create a RooProd for this subexpression
+    RooAbsReal *term(nullptr);
+    if (i->second->size()>1) { // create a RooProd for this subexpression
       const char *name = makeFPName("SUBPROD_",*i->second);
       auto ownedTerm = std::make_unique<RooProduct>(name,name,*i->second);
       term = ownedTerm.get();
       cache->_ownedList.addOwned(std::move(ownedTerm));
-      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") created subexpression " << term->GetName() << endl;
+      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") created subexpression " << term->GetName() << std::endl;
     } else {
-      assert(i->second->getSize()==1);
+      assert(i->second->size()==1);
       term = static_cast<RooAbsReal*>(i->second->at(0));
     }
-    assert(term!=0);
+    assert(term!=nullptr);
     if (i->first->empty()) { // check whether we need to integrate over this term or not...
       cache->_prodList.add(*term);
-      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") adding simple factor " << term->GetName() << endl;
+      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") adding simple factor " << term->GetName() << std::endl;
     } else {
       std::unique_ptr<RooAbsReal> integral{term->createIntegral(*i->first,isetRange)};
       cache->_prodList.add(*integral);
-      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") adding integral for " << term->GetName() << " : " << integral->GetName() << endl;
+      cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") adding integral for " << term->GetName() << " : " << integral->GetName() << std::endl;
       cache->_ownedList.addOwned(std::move(integral));
     }
   }
   // add current set-up to cache, and return index..
   Int_t code = _cacheMgr.setObj(iset,iset,(RooAbsCacheElement*)cache,RooNameReg::ptr(isetRange));
 
-  cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") created list " << cache->_prodList << " with code " << code+1 << endl
-             << " for iset=" << *iset << " @" << iset << " range: " << (isetRange?isetRange:"<none>") << endl ;
+  cxcoutD(Integration) << "RooProduct::getPartIntList(" << GetName() << ") created list " << cache->_prodList << " with code " << code+1 << std::endl
+             << " for iset=" << *iset << " @" << iset << " range: " << (isetRange?isetRange:"<none>") << std::endl ;
 
   for (ProdMap::iterator iter = map->begin() ; iter != map->end() ; ++iter) {
     delete iter->first ;
@@ -303,8 +299,8 @@ Int_t RooProduct::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVar
 double RooProduct::analyticalIntegral(Int_t code, const char* rangeName) const
 {
   // note: rangeName implicit encoded in code: see _cacheMgr.setObj in getPartIntList...
-  CacheElem *cache = (CacheElem*) _cacheMgr.getObjByIndex(code-1);
-  if (cache==0) {
+  CacheElem *cache = static_cast<CacheElem*>(_cacheMgr.getObjByIndex(code-1));
+  if (cache==nullptr) {
     // cache got sterilized, trigger repopulation of this slot, then try again...
     std::unique_ptr<RooArgSet> vars( getParameters(RooArgSet()) );
     RooArgSet iset = _cacheMgr.selectFromSet2(*vars, code-1);
@@ -312,7 +308,7 @@ double RooProduct::analyticalIntegral(Int_t code, const char* rangeName) const
     assert(code==code2); // must have revived the right (sterilized) slot...
     return analyticalIntegral(code2,rangeName);
   }
-  assert(cache!=0);
+  assert(cache!=nullptr);
 
   return calculate(cache->_prodList);
 }
@@ -375,15 +371,18 @@ double RooProduct::evaluate() const
 }
 
 
-void RooProduct::computeBatch(cudaStream_t* /*stream*/, double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
+void RooProduct::doEval(RooFit::EvalContext & ctx) const
 {
+  std::span<double> output = ctx.output();
+  std::size_t nEvents = output.size();
+
   for (unsigned int i = 0; i < nEvents; ++i) {
     output[i] = 1.;
   }
 
   for (const auto item : _compRSet) {
     auto rcomp = static_cast<const RooAbsReal*>(item);
-    auto componentValues = dataMap.at(rcomp);
+    auto componentValues = ctx.at(rcomp);
 
     for (unsigned int i = 0; i < nEvents; ++i) {
       output[i] *= componentValues.size() == 1 ? componentValues[0] : componentValues[i];
@@ -409,13 +408,12 @@ std::list<double>* RooProduct::binBoundaries(RooAbsRealLValue& obs, double xlo,
   for (const auto item : _compRSet) {
     auto func = static_cast<const RooAbsReal*>(item);
 
-    list<double>* binb = func->binBoundaries(obs,xlo,xhi) ;
-    if (binb) {
+    if (std::list<double>* binb = func->binBoundaries(obs,xlo,xhi)) {
       return binb ;
     }
   }
 
-  return 0 ;
+  return nullptr ;
 }
 
 
@@ -445,13 +443,12 @@ std::list<double>* RooProduct::plotSamplingHint(RooAbsRealLValue& obs, double xl
   for (const auto item : _compRSet) {
     auto func = static_cast<const RooAbsReal*>(item);
 
-    list<double>* hint = func->plotSamplingHint(obs,xlo,xhi) ;
-    if (hint) {
+    if (std::list<double>* hint = func->plotSamplingHint(obs,xlo,xhi)) {
       return hint ;
     }
   }
 
-  return 0 ;
+  return nullptr ;
 }
 
 
@@ -486,21 +483,23 @@ void RooProduct::setCacheAndTrackHints(RooArgSet& trackNodes)
     if (parg->isDerived()) {
       if (parg->canNodeBeCached()==Always) {
         trackNodes.add(*parg) ;
-   //cout << "tracking node RooProduct component " << parg->ClassName() << "::" << parg->GetName() << endl ;
       }
     }
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
 
-
-
+void RooProduct::translate(RooFit::Detail::CodeSquashContext &ctx) const
+{
+   ctx.addResult(this, ctx.buildCall("RooFit::Detail::MathFuncs::product", _compRSet, _compRSet.size()));
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Customized printing of arguments of a RooProduct to more intuitively reflect the contents of the
 /// product operator construction
 
-void RooProduct::printMetaArgs(ostream& os) const
+void RooProduct::printMetaArgs(std::ostream& os) const
 {
   bool first(true) ;
 
@@ -524,7 +523,7 @@ void RooProduct::ioStreamerPass2() {
   RooAbsReal::ioStreamerPass2(); // call the baseclass method
 
   if(numProxies() < 2) {
-    throw std::runtime_error("RooProduct::ioStreamerPass2(): the number of proxies in the proxy list should be at leat 2!");
+    throw std::runtime_error("RooProduct::ioStreamerPass2(): the number of proxies in the proxy list should be at least 2!");
   }
 
   // If the proxy data members are evolved by schema evolution, the proxy list
@@ -576,16 +575,18 @@ namespace {
 std::pair<RPPMIter,RPPMIter> findOverlap2nd(RPPMIter i, RPPMIter end)
 {
   // Utility function finding pairs of overlapping input functions
-  for (; i!=end; ++i) for ( RPPMIter j(i+1); j!=end; ++j) {
-    if (i->second->overlaps(*j->second)) {
-      return std::make_pair(i,j);
+  for (; i != end; ++i) {
+    for (RPPMIter j(i + 1); j != end; ++j) {
+      if (i->second->overlaps(*j->second)) {
+        return std::make_pair(i, j);
+      }
     }
   }
   return std::make_pair(end,end);
 }
 
 
-void dump_map(ostream& os, RPPMIter i, RPPMIter end)
+void dump_map(std::ostream& os, RPPMIter i, RPPMIter end)
 {
   // Utility dump function for debugging
   bool first(true);
