@@ -44,9 +44,7 @@ using std::endl, std::ostream;
 
 ClassImp(RooSuperCategory);
 
-RooSuperCategory::RooSuperCategory() : _multiCat("MultiCatProxy", "Stores a RooMultiCategory", this, true, true, true)
-{
-}
+RooSuperCategory::RooSuperCategory() = default;
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Construct a super category from other categories.
@@ -57,7 +55,7 @@ RooSuperCategory::RooSuperCategory() : _multiCat("MultiCatProxy", "Stores a RooM
 RooSuperCategory::RooSuperCategory(const char *name, const char *title, const RooArgSet& inputCategories) :
   RooAbsCategoryLValue(name, title),
   _multiCat("MultiCatProxy", "Stores a RooMultiCategory", this,
-      *new RooMultiCategory((std::string(name) + "_internalMultiCat").c_str(), title, inputCategories), true, true, true)
+      std::make_unique<RooMultiCategory>((std::string(name) + "_internalMultiCat").c_str(), title, inputCategories), true, true)
 {
   // Check category list
   for (const auto arg : inputCategories) {
