@@ -19,7 +19,7 @@
 \class RooVectorDataStore
 \ingroup Roofitcore
 
-RooVectorDataStore uses std::vectors to store data columns. Each of these vectors
+Uses `std::vector` to store data columns. Each of these vectors
 is associated to an instance of a RooAbsReal, whose values it represents. Those
 RooAbsReal are the observables of the dataset.
 In addition to the observables, a data column can be bound to a different instance
@@ -47,7 +47,7 @@ which returns spans pointing directly to the data.
 #include "TBuffer.h"
 
 #include <iomanip>
-using namespace std;
+using std::string, std::vector, std::cout, std::endl, std::list;
 
 ClassImp(RooVectorDataStore);
 ClassImp(RooVectorDataStore::RealVector);
@@ -58,7 +58,7 @@ ClassImp(RooVectorDataStore::RealFullVector);
 
 RooVectorDataStore::RooVectorDataStore()
 {
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -75,7 +75,7 @@ RooVectorDataStore::RooVectorDataStore(RooStringView name, RooStringView title,
   }
 
   setAllBuffersNative() ;
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -135,7 +135,7 @@ RooRealVar* RooVectorDataStore::weightVar(const RooArgSet& allVars, const char*
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Regular copy ctor
+/// Regular copy constructor.
 
 RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const char* newname) :
   RooAbsDataStore(other,newname),
@@ -150,20 +150,20 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const ch
   _currentWeightIndex(other._currentWeightIndex)
 {
   for (const auto realVec : other._realStoreList) {
-    _realStoreList.push_back(new RealVector(*realVec, (RooAbsReal*)_varsww.find(realVec->_nativeReal->GetName()))) ;
+    _realStoreList.push_back(new RealVector(*realVec, static_cast<RooAbsReal*>(_varsww.find(realVec->_nativeReal->GetName())))) ;
   }
 
   for (const auto realFullVec : other._realfStoreList) {
-    _realfStoreList.push_back(new RealFullVector(*realFullVec, (RooAbsReal*)_varsww.find(realFullVec->_nativeReal->GetName()))) ;
+    _realfStoreList.push_back(new RealFullVector(*realFullVec, static_cast<RooAbsReal*>(_varsww.find(realFullVec->_nativeReal->GetName())))) ;
   }
 
   for (const auto catVec : other._catStoreList) {
-    _catStoreList.push_back(new CatVector(*catVec, (RooAbsCategory*)_varsww.find(catVec->_cat->GetName()))) ;
+    _catStoreList.push_back(new CatVector(*catVec, static_cast<RooAbsCategory*>(_varsww.find(catVec->_cat->GetName())))) ;
  }
 
   setAllBuffersNative() ;
 
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -187,13 +187,13 @@ RooVectorDataStore::RooVectorDataStore(const RooTreeDataStore& other, const RooA
     _varsww.assign(other._varsww) ;
     fill() ;
   }
-  TRACE_CREATE
+  TRACE_CREATE;
 
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Clone ctor, must connect internal storage to given new external set of vars
+/// Clone constructor, must connect internal storage to given new external set of variables.
 
 RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const RooArgSet& vars, const char* newname) :
   RooAbsDataStore(other,varsNoWeight(vars,other._wgtVar?other._wgtVar->GetName():nullptr),newname),
@@ -219,7 +219,7 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const Ro
 
   auto forwardIter = other._realfStoreList.begin() ;
   for (; forwardIter!=other._realfStoreList.end() ; ++forwardIter) {
-    RooAbsReal* real = (RooAbsReal*) vars.find((*forwardIter)->bufArg()->GetName()) ;
+    RooAbsReal* real = static_cast<RooAbsReal*>(vars.find((*forwardIter)->bufArg()->GetName())) ;
     if (real) {
       // Clone vector
       _realfStoreList.push_back(new RealFullVector(**forwardIter,real)) ;
@@ -230,7 +230,7 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const Ro
 
   vector<CatVector*>::const_iterator citer = other._catStoreList.begin() ;
   for (; citer!=other._catStoreList.end() ; ++citer) {
-    RooAbsCategory* cat = (RooAbsCategory*) vars.find((*citer)->bufArg()->GetName()) ;
+    RooAbsCategory* cat = static_cast<RooAbsCategory*>(vars.find((*citer)->bufArg()->GetName())) ;
     if (cat) {
       // Clone vector
       _catStoreList.push_back(new CatVector(**citer,cat)) ;
@@ -241,7 +241,7 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const Ro
 
   setAllBuffersNative() ;
 
-  TRACE_CREATE
+  TRACE_CREATE;
 
 }
 
@@ -289,7 +289,7 @@ RooVectorDataStore::RooVectorDataStore(RooStringView name, RooStringView title,
 
   loadValues(&tds,cloneVar.get(),cutRange,nStart,nStop);
 
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -315,7 +315,7 @@ RooVectorDataStore::~RooVectorDataStore()
   }
 
   delete _cache ;
-  TRACE_DESTROY
+  TRACE_DESTROY;
 }
 
 
@@ -394,7 +394,8 @@ double RooVectorDataStore::weightError(RooAbsData::ErrorType etype) const
     // We have a weight array, use that info
 
     // Return symmetric error on current bin calculated either from Poisson statistics or from SumOfWeights
-    double lo = 0, hi = 0 ;
+    double lo = 0;
+    double hi = 0;
     weightError(lo,hi,etype) ;
     return (lo+hi)/2 ;
 
@@ -447,7 +448,8 @@ void RooVectorDataStore::weightError(double& lo, double& hi, RooAbsData::ErrorTy
 
       // Otherwise Calculate poisson errors
       wgt = weight();
-      double ym,yp ;
+      double ym;
+      double yp;
       RooHistError::instance().getPoissonInterval(Int_t(wgt+0.5),ym,yp,1);
       lo = wgt-ym;
       hi = yp-wgt;
@@ -646,10 +648,10 @@ RooAbsArg* RooVectorDataStore::addColumn(RooAbsArg& newVar, bool /*adjustRange*/
   CatVector* cv(nullptr) ;
   assert(numEvt != 0);
   if (dynamic_cast<RooAbsReal*>(valHolder)) {
-    rv = addReal((RooAbsReal*)valHolder);
+    rv = addReal(static_cast<RooAbsReal*>(valHolder));
     rv->resize(numEvt) ;
-  } else if (dynamic_cast<RooAbsCategory*>((RooAbsCategory*)valHolder)) {
-    cv = addCategory((RooAbsCategory*)valHolder) ;
+  } else if (dynamic_cast<RooAbsCategory*>(static_cast<RooAbsCategory*>(valHolder))) {
+    cv = addCategory(static_cast<RooAbsCategory*>(valHolder)) ;
     cv->resize(numEvt) ;
   }
 
@@ -822,7 +824,7 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
     cloneSet.add(*newVarClone) ;
   }
 
-  _cacheOwner = (RooAbsArg*) owner ;
+  _cacheOwner = const_cast<RooAbsArg *>(owner);
   RooVectorDataStore* newCache = new RooVectorDataStore("cache","cache",orderedArgs) ;
 
 
@@ -845,7 +847,7 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
 //       cout << "RooVectorDataStore::cacheArgs() cached node " << arg->GetName() << " has a normalization set specification CATNormSet = " << catNset << endl ;
 
       RooArgSet anset = RooHelpers::selectFromArgSet(nset ? *nset : RooArgSet{}, catNset);
-      normSet = (RooArgSet*) anset.selectCommon(*argObs) ;
+      normSet = anset.selectCommon(*argObs);
 
     }
     const char* catCset = arg->getStringAttribute("CATCondSet") ;
@@ -890,7 +892,7 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
 
     // Activate change tracking mode, if requested
     if (!arg->getAttribute("ConstantExpression") && dynamic_cast<RooAbsReal*>(arg)) {
-      RealVector* rv = newCache->addReal((RooAbsReal*)arg) ;
+      RealVector* rv = newCache->addReal(static_cast<RooAbsReal*>(arg)) ;
       RooArgSet deps;
       arg->getParameters(&_vars, deps);
       rv->setDependents(deps) ;
@@ -982,7 +984,7 @@ void RooVectorDataStore::attachCache(const RooAbsArg* newOwner, const RooArgSet&
   // Only applicable if a cache exists
   if (!_cache) return ;
 
-  // Clone ctor, must connect internal storage to given new external set of vars
+  // Clone constructor, must connect internal storage to given new external set of variables
   std::vector<RealVector*> cacheElements(_cache->realStoreList());
   cacheElements.insert(cacheElements.end(), _cache->_realfStoreList.begin(), _cache->_realfStoreList.end());
 
@@ -1002,7 +1004,7 @@ void RooVectorDataStore::attachCache(const RooAbsArg* newOwner, const RooArgSet&
     }
   }
 
-  _cacheOwner = (RooAbsArg*) newOwner ;
+  _cacheOwner = const_cast<RooAbsArg*>(newOwner);
 }
 
 
@@ -1147,7 +1149,7 @@ RooAbsData::RealSpans RooVectorDataStore::getBatches(std::size_t first, std::siz
     if (realVec->_real) {
       // If a buffer is attached, i.e. we are ready to load into a RooAbsReal outside of our dataset,
       // we can directly map our spans to this real.
-      evalData.emplace(realVec->_real, std::move(span));
+      evalData.emplace(realVec->_real, span);
     }
   };
 
