@@ -15,14 +15,14 @@
 #include "RooFit/TestStatistics/LikelihoodGradientWrapper.h"
 #include "RooFit/TestStatistics/SharedOffset.h"
 
-#include "RooRandom.h"
-#include "RooWorkspace.h"
-#include "RooDataHist.h" // complete type in Binned test
 #include "RooCategory.h" // complete type in MultiBinnedConstraint test
-#include "RooHelpers.h"
-#include "RooMinimizer.h"
+#include "RooDataHist.h" // complete type in Binned test
 #include "RooFitResult.h"
 #include "RooGenericPdf.h"
+#include "RooHelpers.h"
+#include "RooMinimizer.h"
+#include "RooRandom.h"
+#include "RooWorkspace.h"
 #include "RooFit/TestStatistics/LikelihoodWrapper.h"
 #include "RooFit/TestStatistics/RooUnbinnedL.h"
 #include "RooFit/TestStatistics/RooRealL.h"
@@ -63,9 +63,21 @@ ValAndError getValAndError(RooArgSet const &parsFinal, const char *name)
    return {var.getVal(), var.getError()};
 };
 
+std::vector<double> getParamVals(RooAbsMinimizerFcn &fcn)
+{
+   std::vector<double> values(fcn.getNDim());
+
+   for (std::size_t i = 0; i < values.size(); ++i) {
+      values[i] = fcn.floatableParam(i).getVal();
+   }
+
+   return values;
+}
+
 } // namespace
 
-using RooFit::TestStatistics::LikelihoodWrapper;
+namespace RFMP = RooFit::MultiProcess;
+namespace RFTS = RooFit::TestStatistics;
 
 class Environment : public testing::Environment {
 public:
@@ -101,8 +113,6 @@ class LikelihoodGradientJobTest : public ::testing::TestWithParam<std::tuple<std
       RooRandom::randomGenerator()->SetSeed(seed);
    }
 
-   void TearDown() override { RooMinimizer::cleanup(); }
-
 protected:
    std::size_t NWorkers = 0;
    std::size_t seed = 0;
@@ -151,8 +161,7 @@ TEST_P(LikelihoodGradientJobTest, Gaussian1D)
 
    values->assign(savedValues);
 
-   RooFit::TestStatistics::RooRealL likelihood("likelihood", "likelihood",
-                                               std::make_unique<RooFit::TestStatistics::RooUnbinnedL>(pdf, data.get()));
+   RFTS::RooRealL likelihood("likelihood", "likelihood", std::make_unique<RFTS::RooUnbinnedL>(pdf, data.get()));
 
    // Convert to RooRealL to enter into minimizer
    RooMinimizer::Config cfg1;
@@ -201,8 +210,7 @@ TEST(LikelihoodGradientJob, RepeatMigrad)
 
    // --------
 
-   RooFit::TestStatistics::RooRealL likelihood("likelihood", "likelihood",
-                                               std::make_unique<RooFit::TestStatistics::RooUnbinnedL>(pdf, data.get()));
+   RFTS::RooRealL likelihood("likelihood", "likelihood", std::make_unique<RFTS::RooUnbinnedL>(pdf, data.get()));
    RooMinimizer::Config cfg;
    cfg.parallelize = NWorkers;
    RooMinimizer m1(likelihood, cfg);
@@ -272,8 +280,7 @@ TEST_P(LikelihoodGradientJobTest, GaussianND)
 
    // --------
 
-   RooFit::TestStatistics::RooRealL likelihood("likelihood", "likelihood",
-                                               std::make_unique<RooFit::TestStatistics::RooUnbinnedL>(pdf, data.get()));
+   RFTS::RooRealL likelihood("likelihood", "likelihood", std::make_unique<RFTS::RooUnbinnedL>(pdf, data.get()));
    RooMinimizer::Config cfg1;
    cfg1.parallelize = NWorkers;
    RooMinimizer m1(likelihood, cfg1);
@@ -392,12 +399,10 @@ TEST(SimBinnedConstrainedTestBasic, BasicParameters)
 
    // dummy offsets (normally they are shared with other objects):
    SharedOffset shared_offset;
-   auto nll_ts = LikelihoodWrapper::create(RooFit::TestStatistics::LikelihoodMode::serial,
-                                           RooFit::TestStatistics::NLLFactory{*pdf, *data}
-                                              .GlobalObservables({*w.var("alpha_bkg_obs_A"), *w.var("alpha_bkg_obs_B")})
-                                              .build(),
-                                           std::make_unique<RooFit::TestStatistics::WrapperCalculationCleanFlags>(),
-                                           shared_offset);
+   auto nll_ts = RFTS::LikelihoodWrapper::create(
+      RFTS::LikelihoodMode::serial,
+      RFTS::NLLFactory{*pdf, *data}.GlobalObservables({*w.var("alpha_bkg_obs_A"), *w.var("alpha_bkg_obs_B")}).build(),
+      std::make_unique<RFTS::WrapperCalculationCleanFlags>(), shared_offset);
 
    nll_ts->evaluate();
    auto nll1 = nll_ts->getResult();
@@ -416,9 +421,8 @@ TEST_P(SimBinnedConstrainedTest, ConstrainedAndOffset)
    // This is a simultaneous fit, so its likelihood has multiple components. In that case, splitting over
    // components is always preferable, since it is more precise, due to component offsets matching
    // the (-log) function values better.
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks = 1;
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
-      99999; // just a high number, so every component is a task
+   RFMP::Config::LikelihoodJob::defaultNEventTasks = 1;
+   RFMP::Config::LikelihoodJob::defaultNComponentTasks = 99999; // just a high number, so every component is a task
 
    std::unique_ptr<RooWorkspace> wPtr = makeSimBinnedConstrainedWorkspace();
    auto &w = *wPtr;
@@ -490,10 +494,8 @@ TEST_P(SimBinnedConstrainedTest, ConstrainedAndOffset)
    EXPECT_FLOAT_EQ(mu_sig_nominal.error, mu_sig_GradientJob.error);
 
    // reset static variables to automatic
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-      RooFit::MultiProcess::Config::LikelihoodJob::automaticNEventTasks;
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
-      RooFit::MultiProcess::Config::LikelihoodJob::automaticNComponentTasks;
+   RFMP::Config::LikelihoodJob::defaultNEventTasks = RFMP::Config::LikelihoodJob::automaticNEventTasks;
+   RFMP::Config::LikelihoodJob::defaultNComponentTasks = RFMP::Config::LikelihoodJob::automaticNComponentTasks;
 }
 
 INSTANTIATE_TEST_SUITE_P(LikelihoodGradientJob, SimBinnedConstrainedTest, testing::Values(false, true),
@@ -546,8 +548,7 @@ TEST_P(LikelihoodGradientJobTest, Gaussian1DAlsoWithLikelihoodJob)
 
    values->assign(savedValues);
 
-   RooFit::TestStatistics::RooRealL likelihood("likelihood", "likelihood",
-                                               std::make_unique<RooFit::TestStatistics::RooUnbinnedL>(pdf, data.get()));
+   RFTS::RooRealL likelihood("likelihood", "likelihood", std::make_unique<RFTS::RooUnbinnedL>(pdf, data.get()));
    RooMinimizer::Config cfg;
    cfg.parallelize = NWorkers;
    cfg.enableParallelDescent = true;
@@ -602,22 +603,16 @@ class LikelihoodGradientJobErrorTest
 
       // we want to split only over components so we can test component-offsets precisely
       // (event-offsets give more variation)
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-         1; // just one events task (i.e. don't split over events)
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
+      RFMP::Config::LikelihoodJob::defaultNEventTasks = 1; // just one events task (i.e. don't split over events)
+      RFMP::Config::LikelihoodJob::defaultNComponentTasks =
          1000000; // assuming components < 1000000: each component = 1 separate task
    }
 
    void TearDown() override
    {
       // reset static variables to automatic
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-         RooFit::MultiProcess::Config::LikelihoodJob::automaticNEventTasks;
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
-         RooFit::MultiProcess::Config::LikelihoodJob::automaticNComponentTasks;
-
-      // cleanup to make sure JobManager is shut down after any test
-      RooMinimizer::cleanup();
+      RFMP::Config::LikelihoodJob::defaultNEventTasks = RFMP::Config::LikelihoodJob::automaticNEventTasks;
+      RFMP::Config::LikelihoodJob::defaultNComponentTasks = RFMP::Config::LikelihoodJob::automaticNComponentTasks;
    }
 
 protected:
@@ -784,22 +779,16 @@ class LikelihoodGradientJobBinnedErrorTest : public ::testing::TestWithParam<std
 
       // we want to split only over components so we can test component-offsets precisely
       // (event-offsets give more variation)
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-         1; // just one events task (i.e. don't split over events)
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
+      RFMP::Config::LikelihoodJob::defaultNEventTasks = 1; // just one events task (i.e. don't split over events)
+      RFMP::Config::LikelihoodJob::defaultNComponentTasks =
          1000000; // assuming components < 1000000: each component = 1 separate task
    }
 
    void TearDown() override
    {
       // reset static variables to automatic
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-         RooFit::MultiProcess::Config::LikelihoodJob::automaticNEventTasks;
-      RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
-         RooFit::MultiProcess::Config::LikelihoodJob::automaticNComponentTasks;
-
-      // cleanup to make sure JobManager is shut down after any test
-      RooMinimizer::cleanup();
+      RFMP::Config::LikelihoodJob::defaultNEventTasks = RFMP::Config::LikelihoodJob::automaticNEventTasks;
+      RFMP::Config::LikelihoodJob::defaultNComponentTasks = RFMP::Config::LikelihoodJob::automaticNComponentTasks;
    }
 
 protected:
@@ -980,9 +969,8 @@ TEST(MinuitFcnGrad, DISABLED_CompareToRooMinimizerFcn)
    // set up minimizers
    RooMinimizer m_vanilla(*nll_vanilla);
    // we want to split only over components so we can test component-offsets
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-      1; // just one events task (i.e. don't split over events)
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
+   RFMP::Config::LikelihoodJob::defaultNEventTasks = 1; // just one events task (i.e. don't split over events)
+   RFMP::Config::LikelihoodJob::defaultNComponentTasks =
       1000000; // assuming components < 1000000: each component = 1 separate task
    RooMinimizer::Config cfg;
    cfg.parallelize = 1;
@@ -991,25 +979,19 @@ TEST(MinuitFcnGrad, DISABLED_CompareToRooMinimizerFcn)
    RooMinimizer m_modularL(*nll_modularL, cfg);
 
    // now use these minimizers to build the corresponding external RooAbsMinimizerFcns
-   auto nll_real = dynamic_cast<RooFit::TestStatistics::RooRealL *>(nll_modularL.get());
-   RooFit::TestStatistics::MinuitFcnGrad modularL_fcn(
-      nll_real->getRooAbsL(), &m_modularL, m_modularL.fitter()->Config().ParamsSettings(),
-      cfg.enableParallelDescent ? RooFit::TestStatistics::LikelihoodMode::multiprocess
-                                : RooFit::TestStatistics::LikelihoodMode::serial,
-      RooFit::TestStatistics::LikelihoodGradientMode::multiprocess);
+   auto nll_real = dynamic_cast<RFTS::RooRealL *>(nll_modularL.get());
+   RFTS::MinuitFcnGrad modularL_fcn(nll_real->getRooAbsL(), &m_modularL, m_modularL.fitter()->Config().ParamsSettings(),
+                                    cfg.enableParallelDescent ? RFTS::LikelihoodMode::multiprocess
+                                                              : RFTS::LikelihoodMode::serial,
+                                    RFTS::LikelihoodGradientMode::multiprocess);
    RooMinimizerFcn vanilla_fcn(nll_vanilla.get(), &m_vanilla);
 
-   EXPECT_EQ(vanilla_fcn(vanilla_fcn.getParameterValues().data()),
-             modularL_fcn(modularL_fcn.getParameterValues().data()));
+   EXPECT_EQ(vanilla_fcn(getParamVals(vanilla_fcn).data()), modularL_fcn(getParamVals(modularL_fcn).data()));
    // let's also check with absolutely certain same parameter values, both of them
-   EXPECT_EQ(vanilla_fcn(vanilla_fcn.getParameterValues().data()),
-             modularL_fcn(vanilla_fcn.getParameterValues().data()));
-   EXPECT_EQ(vanilla_fcn(modularL_fcn.getParameterValues().data()),
-             modularL_fcn(modularL_fcn.getParameterValues().data()));
+   EXPECT_EQ(vanilla_fcn(getParamVals(vanilla_fcn).data()), modularL_fcn(getParamVals(vanilla_fcn).data()));
+   EXPECT_EQ(vanilla_fcn(getParamVals(modularL_fcn).data()), modularL_fcn(getParamVals(modularL_fcn).data()));
 
    // reset static variables to automatic
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNEventTasks =
-      RooFit::MultiProcess::Config::LikelihoodJob::automaticNEventTasks;
-   RooFit::MultiProcess::Config::LikelihoodJob::defaultNComponentTasks =
-      RooFit::MultiProcess::Config::LikelihoodJob::automaticNComponentTasks;
+   RFMP::Config::LikelihoodJob::defaultNEventTasks = RFMP::Config::LikelihoodJob::automaticNEventTasks;
+   RFMP::Config::LikelihoodJob::defaultNComponentTasks = RFMP::Config::LikelihoodJob::automaticNComponentTasks;
 }
