@@ -4,17 +4,17 @@
 
 #include <RooBinning.h>
 #include <RooCategory.h>
-#include <RooCategory.h>
 #include <RooConstVar.h>
 #include <RooDataHist.h>
 #include <RooDataSet.h>
 #include <RooExtendPdf.h>
 #include <RooFitResult.h>
+#include <RooGlobalFunc.h>
 #include <RooHelpers.h>
 #include <RooHistFunc.h>
 #include <RooHistPdf.h>
 #ifdef ROOFIT_LEGACY_EVAL_BACKEND
-#include <RooNLLVar.h>
+#include "../src/RooNLLVar.h"
 #endif
 #include <RooPlot.h>
 #include <RooPolyVar.h>
@@ -25,6 +25,8 @@
 #include <RooSimultaneous.h>
 #include <RooWorkspace.h>
 
+#include <TMath.h>
+
 #include "gtest_wrapper.h"
 
 #include <cmath>
@@ -81,7 +83,7 @@ private:
 TEST_P(TestStatisticTest, IntegrateBins)
 {
    RooWorkspace ws;
-   ws.factory("EXPR::pow('std::pow(x, a)', {x[0.1, 5.1], a[-0.3, -5., 5.]})");
+   ws.factory("PowerSum::pow(x[0.1, 5.1], {1.0}, {a[-0.3, -5., 5.]})");
 
    RooRealVar &x = *ws.var("x");
    RooRealVar &a = *ws.var("a");
@@ -127,7 +129,7 @@ TEST_P(TestStatisticTest, IntegrateBins)
 TEST_P(TestStatisticTest, IntegrateBins_SubRange)
 {
    RooWorkspace ws;
-   ws.factory("EXPR::pow('std::pow(x, a)', {x[0.1, 5.1], a[-0.3, -5., 5.]})");
+   ws.factory("PowerSum::pow(x[0.1, 5.1], {1.0}, {a[-0.3, -5., 5.]})");
 
    RooRealVar &x = *ws.var("x");
    RooRealVar &a = *ws.var("a");
@@ -176,7 +178,7 @@ TEST_P(TestStatisticTest, IntegrateBins_SubRange)
 TEST_P(TestStatisticTest, IntegrateBins_CustomBinning)
 {
    RooWorkspace ws;
-   ws.factory("EXPR::pow('std::pow(x, a)', {x[1.0, 5.], a[-0.3, -5., 5.]})");
+   ws.factory("PowerSum::pow(x[1.0, 5.], {1.0}, {a[-0.3, -5., 5.]})");
 
    RooRealVar &x = *ws.var("x");
    RooRealVar &a = *ws.var("a");
@@ -229,7 +231,7 @@ TEST_P(TestStatisticTest, IntegrateBins_CustomBinning)
 TEST_P(TestStatisticTest, IntegrateBins_RooDataHist)
 {
    RooWorkspace ws;
-   ws.factory("EXPR::pow('std::pow(x, a)', {x[.1, 5.], a[-0.3, -5., 5.]})");
+   ws.factory("PowerSum::pow(x[0.1, 5.0], {1.0}, {a[-0.3, -5., 5.]})");
 
    RooRealVar &x = *ws.var("x");
    RooRealVar &a = *ws.var("a");
@@ -270,6 +272,43 @@ TEST_P(TestStatisticTest, IntegrateBins_RooDataHist)
       << "Expect chi2/ndf at least 10% better.";
 }
 
+// Verify that fitting an empty RooDataSet or a RooDataHist with only empty
+// bins does not do anything to the parameters. The point of this test is to
+// validate that the new CPU backend behaves the same as the legacy evaluation
+// backend for empty data objects.
+TEST_P(TestStatisticTest, EmptyData)
+{
+   RooWorkspace ws;
+   ws.factory("Gaussian::model(x[0, 10], mean[6, 0, 10], sigma[2.0, 0.01, 10.0])");
+
+   RooRealVar &x = *ws.var("x");
+   RooRealVar &mean = *ws.var("mean");
+   RooRealVar &sigma = *ws.var("sigma");
+   RooAbsPdf &model = *ws.pdf("model");
+
+   const double meanOrigVal = mean.getVal();
+   const double sigmaOrigVal = sigma.getVal();
+
+   std::unique_ptr<RooDataSet> data{model.generate(x, 0)};
+   std::unique_ptr<RooDataHist> dataHist{data->binnedClone()};
+
+   model.fitTo(*data, _evalBackend, RooFit::PrintLevel(-1));
+
+   EXPECT_EQ(mean.getVal(), meanOrigVal) << "Fitting an empty RooDataSet changed \"mean\" value!";
+   EXPECT_EQ(sigma.getVal(), sigmaOrigVal) << "Fitting an empty RooDataSet changed \"sigma\" value!";
+
+   // Reset the parameters for the check with the RooDataHist
+   mean.setVal(meanOrigVal);
+   sigma.setVal(sigmaOrigVal);
+   mean.setError(0.0);
+   sigma.setError(0.0);
+
+   model.fitTo(*dataHist, _evalBackend, RooFit::PrintLevel(-1));
+
+   EXPECT_EQ(mean.getVal(), meanOrigVal) << "Fitting an empty RooDataSet changed \"mean\" value!";
+   EXPECT_EQ(sigma.getVal(), sigmaOrigVal) << "Fitting an empty RooDataSet changed \"sigma\" value!";
+}
+
 TEST(RooChi2Var, IntegrateBins)
 {
    RooHelpers::LocalChangeMsgLevel changeMsgLvl(RooFit::WARNING);
@@ -277,7 +316,7 @@ TEST(RooChi2Var, IntegrateBins)
    RooRandom::randomGenerator()->SetSeed(1337ul);
 
    RooWorkspace ws;
-   ws.factory("EXPR::pow('std::pow(x, a)', {x[.1, 5.1], a[-0.3, -5., 5.]})");
+   ws.factory("PowerSum::pow(x[0.1, 5.1], {1.0}, {a[-0.3, -5., 5.]})");
 
    RooRealVar &x = *ws.var("x");
    RooRealVar &a = *ws.var("a");
@@ -536,9 +575,10 @@ TEST_P(TestStatisticTest, BinnedLikelihood)
    x.setBins(numBins);
 
    {
-      // Uniform RooDataHist
+      // Uniform RooDataHist, but with some empty bins to validate that empty
+      // bins are treated correctly.
       RooDataHist dataHist{"data_hist", "data_hist", x};
-      for (int iBin = 0; iBin < numBins; ++iBin) {
+      for (int iBin = 2; iBin < numBins; ++iBin) {
          dataHist.set(iBin, nEvents / numBins, -1);
       }
 
@@ -578,6 +618,31 @@ TEST_P(TestStatisticTest, BinnedLikelihood)
    EXPECT_DOUBLE_EQ(prodNllVal, simNllVal);
 }
 
+// Make sure that the offset is correctly hidden for the likelihoods, even if
+// we evaluated the same likelihood without hiding before. This is tested
+// because it was fragile before: a change in offset hiding was not considered
+// in the dirty state propagation in the new CPU backend.
+TEST_P(TestStatisticTest, HideOffset)
+{
+   RooWorkspace ws;
+   ws.factory("Gaussian::model(x[0, 10], mean[6, 0, 10], sigma[2.0, 0.01, 10.0])");
+
+   RooRealVar &x = *ws.var("x");
+   RooAbsPdf &model = *ws.pdf("model");
+
+   std::unique_ptr<RooDataSet> data{model.generate(x, 1000)};
+   std::unique_ptr<RooAbsReal> nllNoOffset{model.createNLL(*data, _evalBackend)};
+   std::unique_ptr<RooAbsReal> nll{model.createNLL(*data, RooFit::Offset("initial"), _evalBackend)};
+
+   bool hideOffsetOrig = RooAbsReal::hideOffset();
+   RooAbsReal::setHideOffset(false);
+   EXPECT_FLOAT_EQ(nll->getVal(), 0.);
+   RooAbsReal::setHideOffset(true);
+   EXPECT_FLOAT_EQ(nll->getVal(), nllNoOffset->getVal());
+
+   RooAbsReal::setHideOffset(hideOffsetOrig);
+}
+
 INSTANTIATE_TEST_SUITE_P(RooNLLVar, TestStatisticTest, testing::Values(ROOFIT_EVAL_BACKENDS),
                          [](testing::TestParamInfo<TestStatisticTest::ParamType> const &paramInfo) {
                             std::stringstream ss;
@@ -587,11 +652,11 @@ INSTANTIATE_TEST_SUITE_P(RooNLLVar, TestStatisticTest, testing::Values(ROOFIT_EV
 
 INSTANTIATE_TEST_SUITE_P(RooNLLVar, OffsetBinTest,
                          testing::Combine(testing::Values(ROOFIT_EVAL_BACKENDS), // EvalBackend
-                                          testing::Values(false, true),   // unbinned or binned
-                                          testing::Values(false, true),   // extended fit
-                                          testing::Values(false, true),   // use sumW2
-                                          testing::Values(false, true),   // wrap in a RooSimultaneous
-                                          testing::Values(false)          // binned likelihood code path
+                                          testing::Values(false, true),          // unbinned or binned
+                                          testing::Values(false, true),          // extended fit
+                                          testing::Values(false, true),          // use sumW2
+                                          testing::Values(false, true),          // wrap in a RooSimultaneous
+                                          testing::Values(false)                 // binned likelihood code path
                                           ),
                          [](testing::TestParamInfo<OffsetBinTest::ParamType> const &paramInfo) {
                             std::stringstream ss;
@@ -606,11 +671,11 @@ INSTANTIATE_TEST_SUITE_P(RooNLLVar, OffsetBinTest,
 
 INSTANTIATE_TEST_SUITE_P(RooNLLVarBinnedL, OffsetBinTest,
                          testing::Combine(testing::Values(ROOFIT_EVAL_BACKENDS), // EvalBackend
-                                          testing::Values(true),          // unbinned or binned
-                                          testing::Values(false),         // extended fit
-                                          testing::Values(false),         // use sumW2
-                                          testing::Values(false, true),   // wrap in a RooSimultaneous
-                                          testing::Values(true)           // binned likelihood code path
+                                          testing::Values(true),                 // unbinned or binned
+                                          testing::Values(false),                // extended fit
+                                          testing::Values(false),                // use sumW2
+                                          testing::Values(false, true),          // wrap in a RooSimultaneous
+                                          testing::Values(true)                  // binned likelihood code path
                                           ),
                          [](testing::TestParamInfo<OffsetBinTest::ParamType> const &paramInfo) {
                             std::stringstream ss;
@@ -622,3 +687,110 @@ INSTANTIATE_TEST_SUITE_P(RooNLLVarBinnedL, OffsetBinTest,
                             ss << (std::get<5>(paramInfo.param) ? "BinnedL" : "");
                             return ss.str();
                          });
+
+// Test if the data can be correctly reset for both individual and simultaneous
+// pdfs.
+TEST(NLL, SetData)
+{
+   RooHelpers::LocalChangeMsgLevel changeMsgLvl(RooFit::WARNING);
+
+   RooWorkspace workspace;
+   workspace.factory("Gaussian::pdf_1(x[-10, 10], mu[0, -10, 10], sigma[1, 0.1, 10])");
+   workspace.factory("SIMUL::pdf(index_cat[A=0], A=pdf_1)");
+
+   RooAbsPdf &pdf1 = *workspace.pdf("pdf_1");
+   RooAbsPdf &pdf = *workspace.pdf("pdf");
+   RooRealVar &x = *workspace.var("x");
+   RooAbsCategory &cat = *workspace.cat("index_cat");
+
+   {
+      // Build simple single-entry datasets so that it's easy to know the
+      // reference result.
+      const double xa = 0.0;
+      const double xb = 1.0;
+
+      RooDataSet data1a{"data_1_a", "data_1_a", {x, cat}};
+      x.setVal(xa);
+      data1a.add({x, cat});
+      workspace.import(data1a);
+
+      RooDataSet data1b{"data_1_b", "data_1_b", {x, cat}};
+      x.setVal(xb);
+      data1b.add({x, cat});
+      workspace.import(data1b);
+   }
+   RooAbsData &data1a = *workspace.data("data_1_a");
+   RooAbsData &data1b = *workspace.data("data_1_b");
+
+   std::unique_ptr<RooAbsReal> nll1{pdf1.createNLL(data1a)};
+   std::unique_ptr<RooAbsReal> nllSim{pdf.createNLL(data1a)};
+
+   double val1A = nll1->getVal();
+   double valSimA = nllSim->getVal();
+
+   nll1->setData(data1b);
+   nllSim->setData(data1b);
+
+   double val1B = nll1->getVal();
+   double valSimB = nllSim->getVal();
+
+   // Make sure the combined and single channel pdfs give consistent results.
+   EXPECT_DOUBLE_EQ(val1A, valSimA);
+   EXPECT_DOUBLE_EQ(val1B, valSimB);
+
+   // The dataset was built in such a way that the updated NLL is shifted by
+   // 0.5, so this is what we analytically expect.
+   EXPECT_DOUBLE_EQ(valSimB, valSimA + 0.5);
+}
+
+// In some frameworks like CMS combine, all constraints are contained in a
+// single RooProdPdf, even if the constrained parameters are not used in this
+// pdf. The RooFit logic to figure out constrained parameters should however
+// now be confused by this, and not strip away these parameters from the list
+// of constrained parameters.
+TEST(CreateNLL, CombineStyleConstraints)
+{
+   RooHelpers::LocalChangeMsgLevel changeMsgLvl(RooFit::WARNING);
+
+   RooWorkspace ws;
+   ws.factory("Gaussian::g_main_1(x_1[0., -10, 10], mu_1[0., -10, 10], 1.0)");
+   ws.factory("Gaussian::g_main_2(x_2[0., -10, 10], mu_2[0., -10, 10], 1.0)");
+   ws.factory("Gaussian::g_constr_1(mu_1, 0.0, 1.0)");
+   ws.factory("Gaussian::g_constr_2(mu_2, 0.0, 1.0)");
+
+   // The RooProdPdf model_1 will contain all the constraints, also the one
+   // that applies to g_main_1. This is the corner case that this test is
+   // covering.
+   ws.factory("PROD::model_1(g_main_1, g_constr_1, g_constr_2)");
+   ws.factory("PROD::model_2(g_main_2)");
+   ws.factory("SIMUL::model(cat[A=0, B=1], A=model_1, B=model_2)");
+
+   auto &x1 = *ws.var("x_1");
+   auto &x2 = *ws.var("x_2");
+   auto &cat = *ws.cat("cat");
+   auto &model = static_cast<RooSimultaneous &>(*ws.pdf("model"));
+
+   RooArgSet vars{x1, x2, cat};
+   RooDataSet data{"data", "data", vars};
+   cat.setIndex(0);
+   data.add(vars);
+   cat.setIndex(1);
+   data.add(vars);
+
+   RooArgSet parameters;
+   model.getParameters(data.get(), parameters);
+
+   model.getAllConstraints(*data.get(), parameters, true);
+
+   EXPECT_EQ(parameters.size(), 2);
+   EXPECT_EQ(parameters.find("mu_1"), ws.var("mu_1"));
+   EXPECT_EQ(parameters.find("mu_2"), ws.var("mu_2"));
+
+   // Now, try it out in the context of createNLL
+   std::unique_ptr<RooAbsReal> nll{model.createNLL(data)};
+   const double proba = TMath::Gaus(0, 0, 1, true);
+   const double nChannels = model.indexCat().size();
+   //                                     main Gaussians                one constraint per channel
+   const double refNllVal = -nChannels * (std::log(proba / nChannels) + std::log(proba));
+   EXPECT_FLOAT_EQ(nll->getVal(), refNllVal);
+}
