@@ -30,7 +30,7 @@ The use of ToyMCSampler as the TestStatSampler is assumed.
 ClassImp(RooStats::FrequentistCalculator);
 
 using namespace RooStats;
-using namespace std;
+using std::endl;
 
 ////////////////////////////////////////////////////////////////////////////////
 
