@@ -34,9 +34,7 @@ Moreover it provides tools for the combination of models and datasets.
 
 */
 
-using namespace std;
-
-ClassImp(RooStats::HLFactory); ;
+ClassImp(RooStats::HLFactory);
 
 
 using namespace RooStats;
@@ -47,71 +45,40 @@ using namespace RooFit;
 /// verbosity flag. The extension for the config files is assumed to
 /// be ".rs".
 
-HLFactory::HLFactory(const char *name,
-                     const char *fileName,
-                     bool isVerbose):
-    TNamed(name,name),
-    fComboCat(0),
-    fComboBkgPdf(0),
-    fComboSigBkgPdf(0),
-    fComboDataset(0),
-    fCombinationDone(false),
-    fVerbose(isVerbose),
-    fInclusionLevel(0),
-    fOwnWs(true){
-    TString wsName(name);
-    wsName+="_ws";
-    fWs = new RooWorkspace(wsName,true);
+HLFactory::HLFactory(const char *name, const char *fileName, bool isVerbose)
+   : TNamed(name, name), fVerbose(isVerbose), fOwnWs(true)
+{
+   TString wsName(name);
+   wsName += "_ws";
+   fWs = new RooWorkspace(wsName);
 
-    fSigBkgPdfNames.SetOwner();
-    fBkgPdfNames.SetOwner();
-    fDatasetsNames.SetOwner();
+   fSigBkgPdfNames.SetOwner();
+   fBkgPdfNames.SetOwner();
+   fDatasetsNames.SetOwner();
 
-    // Start the parsing
-    fReadFile(fileName);
+   // Start the parsing
+   fReadFile(fileName);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor without a card but with an external workspace.
 
-HLFactory::HLFactory(const char* name,
-                     RooWorkspace* externalWs,
-                     bool isVerbose):
-    TNamed(name,name),
-    fComboCat(0),
-    fComboBkgPdf(0),
-    fComboSigBkgPdf(0),
-    fComboDataset(0),
-    fCombinationDone(false),
-    fVerbose(isVerbose),
-    fInclusionLevel(0),
-    fOwnWs(false){
-    fWs=externalWs;
-    fSigBkgPdfNames.SetOwner();
-    fBkgPdfNames.SetOwner();
-    fDatasetsNames.SetOwner();
-
+HLFactory::HLFactory(const char *name, RooWorkspace *externalWs, bool isVerbose)
+   : TNamed(name, name), fVerbose(isVerbose), fWs(externalWs)
+{
+   fSigBkgPdfNames.SetOwner();
+   fBkgPdfNames.SetOwner();
+   fDatasetsNames.SetOwner();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-HLFactory::HLFactory():
-    TNamed("hlfactory","hlfactory"),
-    fComboCat(0),
-    fComboBkgPdf(0),
-    fComboSigBkgPdf(0),
-    fComboDataset(0),
-    fCombinationDone(false),
-    fVerbose(false),
-    fInclusionLevel(0),
-    fOwnWs(true){
-    fWs = new RooWorkspace("hlfactory_ws",true);
-
-    fSigBkgPdfNames.SetOwner();
-    fBkgPdfNames.SetOwner();
-    fDatasetsNames.SetOwner();
-
-    }
+HLFactory::HLFactory() : TNamed("hlfactory", "hlfactory"), fWs(new RooWorkspace("hlfactory_ws")), fOwnWs(true)
+{
+   fSigBkgPdfNames.SetOwner();
+   fBkgPdfNames.SetOwner();
+   fDatasetsNames.SetOwner();
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// destructor
@@ -148,7 +115,7 @@ int HLFactory::AddChannel(const char* label,
         return -1;
         }
 
-    if (SigBkgPdfName!=0){
+    if (SigBkgPdfName!=nullptr){
         if (fWs->pdf(SigBkgPdfName)==nullptr){
             std::cerr << "Pdf " << SigBkgPdfName << " not found in workspace!\n";
             return -1;
@@ -157,7 +124,7 @@ int HLFactory::AddChannel(const char* label,
         fSigBkgPdfNames.Add(name);
         }
 
-    if (BkgPdfName!=0){
+    if (BkgPdfName!=nullptr){
         if (fWs->pdf(BkgPdfName)==nullptr){
             std::cerr << "Pdf " << BkgPdfName << " not found in workspace!\n";
             return -1;
@@ -166,7 +133,7 @@ int HLFactory::AddChannel(const char* label,
         fBkgPdfNames.Add(name);
         }
 
-    if (DatasetName!=0){
+    if (DatasetName!=nullptr){
         if (fWs->data(DatasetName)==nullptr){
             std::cerr << "Dataset " << DatasetName << " not found in workspace!\n";
             return -1;
@@ -175,7 +142,7 @@ int HLFactory::AddChannel(const char* label,
         fDatasetsNames.Add(name);
         }
 
-    if (label!=0){
+    if (label!=nullptr){
         TObjString* name = new TObjString(label);
         fLabelsNames.Add(name);
         }
@@ -189,7 +156,7 @@ int HLFactory::AddChannel(const char* label,
 
 RooAbsPdf* HLFactory::GetTotSigBkgPdf(){
     if (fSigBkgPdfNames.GetSize()==0)
-        return 0;
+        return nullptr;
 
     if (fComboSigBkgPdf!=nullptr)
         return fComboSigBkgPdf;
@@ -198,8 +165,7 @@ RooAbsPdf* HLFactory::GetTotSigBkgPdf(){
         return nullptr;
 
     if (fSigBkgPdfNames.GetSize()==1){
-        TString name(((TObjString*)fSigBkgPdfNames.At(0))->String());
-        fComboSigBkgPdf=fWs->pdf(name);
+        fComboSigBkgPdf=fWs->pdf(static_cast<TObjString*>(fSigBkgPdfNames.At(0))->String().Data());
         return fComboSigBkgPdf;
         }
 
@@ -209,18 +175,18 @@ RooAbsPdf* HLFactory::GetTotSigBkgPdf(){
     RooArgList pdfs("pdfs");
 
     for(auto * ostring : static_range_cast<TObjString*>(fSigBkgPdfNames)) {
-        pdfs.add( *(fWs->pdf(ostring->String())) );
+        pdfs.add( *(fWs->pdf(ostring->String().Data())) );
     }
 
-    TString name(GetName());
+    std::string name(GetName());
     name+="_sigbkg";
 
-    TString title(GetName());
+    std::string title(GetName());
     title+="_sigbkg";
 
     fComboSigBkgPdf=
-      new RooSimultaneous(name,
-                          title,
+      new RooSimultaneous(name.c_str(),
+                          title.c_str(),
                           pdfs,
                           *fComboCat);
 
@@ -235,7 +201,7 @@ RooAbsPdf* HLFactory::GetTotSigBkgPdf(){
 
 RooAbsPdf* HLFactory::GetTotBkgPdf(){
     if (fBkgPdfNames.GetSize()==0)
-        return 0;
+        return nullptr;
 
     if (fComboBkgPdf!=nullptr)
         return fComboBkgPdf;
@@ -244,7 +210,7 @@ RooAbsPdf* HLFactory::GetTotBkgPdf(){
         return nullptr;
 
     if (fBkgPdfNames.GetSize()==1){
-        fComboBkgPdf=fWs->pdf(((TObjString*)fBkgPdfNames.First())->String());
+        fComboBkgPdf=fWs->pdf(static_cast<TObjString*>(fBkgPdfNames.First())->String().Data());
         return fComboBkgPdf;
         }
 
@@ -254,7 +220,7 @@ RooAbsPdf* HLFactory::GetTotBkgPdf(){
     RooArgList pdfs("pdfs");
 
     for(auto * ostring : static_range_cast<TObjString*>(fBkgPdfNames)) {
-        pdfs.add( *(fWs->pdf(ostring->String())) );
+        pdfs.add( *fWs->pdf(ostring->String().Data()) );
     }
 
     TString name(GetName());
@@ -280,7 +246,7 @@ RooAbsPdf* HLFactory::GetTotBkgPdf(){
 
 RooDataSet* HLFactory::GetTotDataSet(){
     if (fDatasetsNames.GetSize()==0)
-        return 0;
+        return nullptr;
 
     if (fComboDataset!=nullptr)
         return fComboDataset;
@@ -289,7 +255,7 @@ RooDataSet* HLFactory::GetTotDataSet(){
         return nullptr;
 
     if (fDatasetsNames.GetSize()==1){
-        fComboDataset=(RooDataSet*)fWs->data(((TObjString*)fDatasetsNames.First())->String());
+        fComboDataset=static_cast<RooDataSet*>(fWs->data(static_cast<TObjString*>(fDatasetsNames.First())->String().Data()));
         return fComboDataset;
         }
 
@@ -301,7 +267,7 @@ RooDataSet* HLFactory::GetTotDataSet(){
     TObjString* ostring;
     ostring = static_cast<TObjString*>(*it);
     ++it;
-    fComboDataset = (RooDataSet*) fWs->data(ostring->String()) ;
+    fComboDataset = static_cast<RooDataSet*>(fWs->data(ostring->String().Data())) ;
     if (!fComboDataset) return nullptr;
     fComboDataset->Print();
     TString dataname(GetName());
@@ -312,7 +278,7 @@ RooDataSet* HLFactory::GetTotDataSet(){
     for(; it != fDatasetsNames.end() ; ++it) {
         ostring = static_cast<TObjString*>(*it);
         catindex++;
-        RooDataSet * data = (RooDataSet*)fWs->data(ostring->String());
+        RooDataSet * data = static_cast<RooDataSet*>(fWs->data(ostring->String().Data()));
         if (!data) return nullptr;
         RooDataSet* dummy = new RooDataSet(*data,"");
         fComboCat->setIndex(catindex);
@@ -348,7 +314,7 @@ RooCategory* HLFactory::GetTotCategory(){
 /// Process an additional configuration file
 
 int HLFactory::ProcessCard(const char* filename){
-    return fReadFile(filename,0);
+    return fReadFile(filename,false);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -380,10 +346,11 @@ int HLFactory::ProcessCard(const char* filename){
 
 int HLFactory::fReadFile(const char*fileName, bool is_included){
     // Check the deepness of the inclusion
-    if (is_included)
+    if (is_included) {
         fInclusionLevel+=1;
-    else
-        fInclusionLevel=0;
+    } else {
+        fInclusionLevel = 0;
+    }
 
     const int maxDeepness=50;
     if (fInclusionLevel>maxDeepness){
@@ -422,13 +389,14 @@ int HLFactory::fReadFile(const char*fileName, bool is_included){
         TString line = (static_cast<TObjString*>(line_o))->GetString();
 
         // Are we in a multiline comment?
-        if (in_comment)
+        if (in_comment) {
             if (line.EndsWith("*/")){
                 in_comment=false;
                 if (fVerbose) Info("fReadFile","Out of multiline comment ...");
 
                 continue;
-                }
+            }
+        }
 
         // Was line a single line comment?
 
@@ -534,11 +502,11 @@ void HLFactory::fCreateCategory(){
 /// is not empty prompt an error.
 
 bool HLFactory::fNamesListsConsistent(){
-    if ((fSigBkgPdfNames.GetEntries()==fBkgPdfNames.GetEntries() || fBkgPdfNames.GetEntries()==0) &&
-        (fSigBkgPdfNames.GetEntries()==fDatasetsNames.GetEntries() || fDatasetsNames.GetEntries()==0) &&
-        (fSigBkgPdfNames.GetEntries()==fLabelsNames.GetEntries() || fLabelsNames.GetEntries()==0))
+        if ((fSigBkgPdfNames.GetEntries() == fBkgPdfNames.GetEntries() || fBkgPdfNames.GetEntries() == 0) &&
+            (fSigBkgPdfNames.GetEntries() == fDatasetsNames.GetEntries() || fDatasetsNames.GetEntries() == 0) &&
+            (fSigBkgPdfNames.GetEntries() == fLabelsNames.GetEntries() || fLabelsNames.GetEntries() == 0)) {
         return true;
-    else{
+        } else {
         std::cerr << "The number of datasets and models added as channels "
                   << " is not the same!\n";
         return false;
@@ -564,7 +532,7 @@ int HLFactory::fParseLine(TString& line){
          nequals>0 &&    // It is a cat like "tag[B0=1,B0bar=-1]"
          ! line.Contains("(") &&
          ! line.Contains(")"))) {
-      fWs->factory(line);
+      fWs->factory(line.Data());
       return 0;
       }
 
@@ -601,7 +569,7 @@ int HLFactory::fParseLine(TString& line){
         TString rootfile_name (static_cast<TObjString*>(descr_array->At(0))->GetString());
 
         std::unique_ptr<TFile> ifile{TFile::Open(rootfile_name)};
-        if (ifile==0)
+        if (ifile==nullptr)
             return 1;
 
         if (n_descr_parts==3){// in presence of a Ws
@@ -609,9 +577,10 @@ int HLFactory::fParseLine(TString& line){
           fWs->import(o_descr);
           }
         else if(n_descr_parts==2){ // in presence of an object in rootfile
-          if (fVerbose)
-            Info("fParseLine","Importing %s from %s under the name of %s",
-                 obj_name.Data(), rootfile_name.Data(), o_name.Data());
+          if (fVerbose) {
+             Info("fParseLine", "Importing %s from %s under the name of %s", obj_name.Data(), rootfile_name.Data(),
+                  o_name.Data());
+          }
           TObject* the_obj=ifile->Get(obj_name);
           fWs->import(*the_obj,o_name);
           }
@@ -625,13 +594,13 @@ int HLFactory::fParseLine(TString& line){
             std::cout << "DEBUG: new_line: " << new_line.Data() << std::endl;
             }
 
-        fWs->factory(new_line);
+        fWs->factory(new_line.Data());
 
         return 0;
         }
 
     else { // In case we do not know what to do we pipe it..
-        fWs->factory(line);
+        fWs->factory(line.Data());
         }
 
     return 0;
