@@ -45,8 +45,6 @@ ClassImp(RooStats::HypoTestInverterResult);
 
 using namespace RooStats;
 using namespace RooFit;
-using namespace std;
-
 
 // initialize static value
 double HypoTestInverterResult::fgAsymptoticMaxSigma      = 5;
@@ -78,24 +76,24 @@ HypoTestInverterResult::HypoTestInverterResult(const char * name ) :
 ////////////////////////////////////////////////////////////////////////////////
 /// copy constructor
 
-HypoTestInverterResult::HypoTestInverterResult( const HypoTestInverterResult& other, const char * name ) :
-   SimpleInterval(other,name),
-   fUseCLs(other.fUseCLs),
-   fIsTwoSided(other.fIsTwoSided),
-   fInterpolateLowerLimit(other.fInterpolateLowerLimit),
-   fInterpolateUpperLimit(other.fInterpolateUpperLimit),
-   fFittedLowerLimit(other.fFittedLowerLimit),
-   fFittedUpperLimit(other.fFittedUpperLimit),
-   fInterpolOption(other.fInterpolOption),
-   fLowerLimitError(other.fLowerLimitError),
-   fUpperLimitError(other.fUpperLimitError),
-   fCLsCleanupThreshold(other.fCLsCleanupThreshold)
+HypoTestInverterResult::HypoTestInverterResult(const HypoTestInverterResult &other, const char *name)
+   : SimpleInterval(other, name),
+     fUseCLs(other.fUseCLs),
+     fIsTwoSided(other.fIsTwoSided),
+     fInterpolateLowerLimit(other.fInterpolateLowerLimit),
+     fInterpolateUpperLimit(other.fInterpolateUpperLimit),
+     fFittedLowerLimit(other.fFittedLowerLimit),
+     fFittedUpperLimit(other.fFittedUpperLimit),
+     fInterpolOption(other.fInterpolOption),
+     fLowerLimitError(other.fLowerLimitError),
+     fUpperLimitError(other.fUpperLimitError),
+     fCLsCleanupThreshold(other.fCLsCleanupThreshold),
+     fXValues(other.fXValues)
 {
    fLowerLimit = TMath::QuietNaN();
    fUpperLimit = TMath::QuietNaN();
    int nOther = other.ArraySize();
 
-   fXValues = other.fXValues;
    for (int i = 0; i < nOther; ++i)
      fYObjects.Add( other.fYObjects.At(i)->Clone() );
    for (int i = 0; i <  fExpPValues.GetSize() ; ++i)
@@ -258,7 +256,7 @@ int HypoTestInverterResult::ExclusionCleanup()
       q[3] = values[i3];
       q[4] = values[i4];
     } else {
-      double * z = const_cast<double *>( &values[0] ); // need to change TMath::Quantiles
+      double * z = const_cast<double *>(&values[0] ); // need to change TMath::Quantiles
       TMath::Quantiles(values.size(), 5, z, q, p, false);
     }
 
@@ -352,7 +350,7 @@ bool HypoTestInverterResult::Add( const HypoTestInverterResult& otherResult   )
    else {
       for (int i = 0; i < nOther; ++i) {
          double otherVal = otherResult.fXValues[i];
-         HypoTestResult * otherHTR = (HypoTestResult*) otherResult.fYObjects.At(i);
+         HypoTestResult * otherHTR = static_cast<HypoTestResult*>(otherResult.fYObjects.At(i));
          if (otherHTR == nullptr) continue;
          bool sameXFound = false;
          for (int j = 0; j < nThis; ++j) {
@@ -361,11 +359,11 @@ bool HypoTestInverterResult::Add( const HypoTestInverterResult& otherResult   )
             // if same value merge the result
             if ( (std::abs(otherVal) < 1  && TMath::AreEqualAbs(otherVal, thisVal,1.E-12) ) ||
                  (std::abs(otherVal) >= 1 && TMath::AreEqualRel(otherVal, thisVal,1.E-12) ) ) {
-               HypoTestResult * thisHTR = (HypoTestResult*) fYObjects.At(j);
+               HypoTestResult * thisHTR = static_cast<HypoTestResult*>(fYObjects.At(j));
                thisHTR->Append(otherHTR);
                sameXFound = true;
                if (mergeExpPValues) {
-                  ((SamplingDistribution*) fExpPValues.At(j))->Add( (SamplingDistribution*)otherResult.fExpPValues.At(i) );
+                  (static_cast<SamplingDistribution*>(fExpPValues.At(j)))->Add( static_cast<SamplingDistribution*>(otherResult.fExpPValues.At(i)) );
                   // check if same toys have been used for the test statistic distribution
                   int thisNToys = (thisHTR->GetNullDistribution() ) ? thisHTR->GetNullDistribution()->GetSize() : 0;
                   int otherNToys = (otherHTR->GetNullDistribution() ) ? otherHTR->GetNullDistribution()->GetSize() : 0;
@@ -388,13 +386,13 @@ bool HypoTestInverterResult::Add( const HypoTestInverterResult& otherResult   )
       }
    }
 
-   if (ArraySize() > nThis)
+   if (ArraySize() > nThis) {
       coutI(Eval) << "HypoTestInverterResult::Add  - new number of points is " << fXValues.size()
                          << std::endl;
-   else
+   } else {
       coutI(Eval) << "HypoTestInverterResult::Add  - new toys/point is "
-                         <<  ((HypoTestResult*) fYObjects.At(0))->GetNullDistribution()->GetSize()
-                         << std::endl;
+                  << (static_cast<HypoTestResult *>(fYObjects.At(0)))->GetNullDistribution()->GetSize() << std::endl;
+   }
 
    // reset cached limit values
    fLowerLimit = TMath::QuietNaN();
@@ -554,7 +552,7 @@ HypoTestResult* HypoTestInverterResult::GetResult( int index ) const
       return nullptr;
    }
 
-   return ((HypoTestResult*) fYObjects.At(index));
+   return (static_cast<HypoTestResult*>(fYObjects.At(index)));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -597,7 +595,7 @@ double HypoTestInverterResult::GetGraphX(const TGraph & graph, double y0, bool l
 
    double varmin = - TMath::Infinity();
    double varmax = TMath::Infinity();
-   const RooRealVar* var = dynamic_cast<RooRealVar*>( fParameters.first() );
+   const RooRealVar* var = dynamic_cast<RooRealVar*>(fParameters.first() );
    if (var) {
        varmin = var->getMin();
        varmax = var->getMax();
@@ -651,7 +649,7 @@ double HypoTestInverterResult::GetGraphX(const TGraph & graph, double y0, bool l
 
    ROOT::Math::BrentRootFinder brf;
    brf.SetFunction(f1d,xmin,xmax);
-   brf.SetNpx(TMath::Max(graph.GetN()*2,100) );
+   brf.SetNpx(std::max(graph.GetN()*2,100) );
 #ifdef DO_DEBUG
    std::cout << "findind root for " << xmin << " ,  "<< xmax << "f(x) : " << graph.Eval(xmin) << " , " << graph.Eval(0.5*(xmax+xmin))
              << " , " << graph.Eval(xmax) << " target " << y0 << std::endl;
@@ -715,7 +713,7 @@ double HypoTestInverterResult::FindInterpolatedLimit(double target, bool lowSear
    // variable minimum and maximum
    double varmin = - TMath::Infinity();
    double varmax = TMath::Infinity();
-   const RooRealVar* var = dynamic_cast<RooRealVar*>( fParameters.first() );
+   const RooRealVar* var = dynamic_cast<RooRealVar*>(fParameters.first() );
    if (var) {
        varmin = var->getMin();
        varmax = var->getMax();
@@ -1001,7 +999,8 @@ double HypoTestInverterResult::CalculateEstimatedError(double target, bool lower
   TMath::SortItr(fXValues.begin(), fXValues.end(), indx.begin(), false);
   // make a graph with the sorted point
   TGraphErrors graph;
-  int ip = 0, np = 0;
+  int ip = 0;
+  int np = 0;
   for (int i = 0; i < ArraySize(); ++i) {
      if ( (xmin < xmax) && ( GetXValue(indx[i]) >= xmin && GetXValue(indx[i]) <= xmax) ) {
         np++;
@@ -1069,10 +1068,11 @@ double HypoTestInverterResult::CalculateEstimatedError(double target, bool lower
      coutW(Eval) << "HypoTestInverterResult::CalculateEstimatedError - cannot estimate  the " << type << " limit error " << std::endl;
      theError = 0;
   }
-  if (lower)
+  if (lower) {
      fLowerLimitError = theError;
-  else
+  } else {
      fUpperLimitError = theError;
+  }
 
 #ifdef DO_DEBUG
   std::cout << "closes point to the limit is " << index << "  " << GetXValue(index) << " and has error " << GetYError(index) << std::endl;
@@ -1107,7 +1107,7 @@ double HypoTestInverterResult::UpperLimitEstimatedError()
 
 SamplingDistribution *  HypoTestInverterResult::GetBackgroundTestStatDist(int index ) const {
 
-   HypoTestResult * firstResult = (HypoTestResult*) fYObjects.At(index);
+   HypoTestResult * firstResult = static_cast<HypoTestResult*>(fYObjects.At(index));
    if (!firstResult) return nullptr;
    return firstResult->GetBackGroundIsAlt() ? firstResult->GetAltDistribution() : firstResult->GetNullDistribution();
 }
@@ -1116,7 +1116,7 @@ SamplingDistribution *  HypoTestInverterResult::GetBackgroundTestStatDist(int in
 /// get the signal and background test statistic distribution
 
 SamplingDistribution *  HypoTestInverterResult::GetSignalAndBackgroundTestStatDist(int index) const {
-   HypoTestResult * result = (HypoTestResult*) fYObjects.At(index);
+   HypoTestResult * result = static_cast<HypoTestResult*>(fYObjects.At(index));
    if (!result) return nullptr;
    return !result->GetBackGroundIsAlt() ? result->GetAltDistribution() : result->GetNullDistribution();
 }
@@ -1129,7 +1129,7 @@ SamplingDistribution *  HypoTestInverterResult::GetExpectedPValueDist(int index)
    if (index < 0 || index >=  ArraySize() ) return nullptr;
 
    if (fExpPValues.GetSize() == ArraySize()  ) {
-      return (SamplingDistribution*)  fExpPValues.At(index)->Clone();
+      return static_cast<SamplingDistribution*>( fExpPValues.At(index)->Clone());
    }
 
    static bool useFirstB = false;
@@ -1139,7 +1139,7 @@ SamplingDistribution *  HypoTestInverterResult::GetExpectedPValueDist(int index)
    SamplingDistribution * bDistribution = GetBackgroundTestStatDist(bIndex);
    SamplingDistribution * sbDistribution = GetSignalAndBackgroundTestStatDist(index);
 
-   HypoTestResult * result = (HypoTestResult*) fYObjects.At(index);
+   HypoTestResult * result = static_cast<HypoTestResult*>(fYObjects.At(index));
 
    if (bDistribution && sbDistribution) {
 
@@ -1260,12 +1260,11 @@ SamplingDistribution *  HypoTestInverterResult::GetLimitDistribution(bool lower
 
   }
 
-
-  if (lower)
+  if (lower) {
      return new SamplingDistribution("Expected lower Limit","Expected lower limits",limits);
-  else
-     return new SamplingDistribution("Expected upper Limit","Expected upper limits",limits);
-
+  } else {
+     return new SamplingDistribution("Expected upper Limit", "Expected upper limits", limits);
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
