@@ -52,7 +52,7 @@ ToyMCStudy with the RooStudyManager.
 
 
 using namespace RooFit;
-using namespace std;
+using std::endl;
 
 
 ClassImp(RooStats::ToyMCSampler);
@@ -146,70 +146,30 @@ void ToyMCSampler::SetAlwaysUseMultiGen(bool flag) { fgAlwaysUseMultiGen = flag
 ////////////////////////////////////////////////////////////////////////////////
 /// Proof constructor. Do not use.
 
-ToyMCSampler::ToyMCSampler() : fSamplingDistName("SD"), fNToys(1)
+ToyMCSampler::ToyMCSampler()
+   : fSamplingDistName("SD"),
+     fNToys(1),
+     fMaxToys(RooNumber::infinity()),
+     fAdaptiveLowLimit(-RooNumber::infinity()),
+     fAdaptiveHighLimit(RooNumber::infinity())
 {
-
-   fPdf = nullptr;
-   fPriorNuisance = nullptr;
-   fNuisancePars = nullptr;
-   fObservables = nullptr;
-   fGlobalObservables = nullptr;
-
-   fSize = 0.05;
-   fNEvents = 0;
-   fGenerateBinned = false;
-   fGenerateBinnedTag = "";
-   fGenerateAutoBinned = true;
-   fExpectedNuisancePar = false;
-
-   fToysInTails = 0.0;
-   fMaxToys = RooNumber::infinity();
-   fAdaptiveLowLimit = -RooNumber::infinity();
-   fAdaptiveHighLimit = RooNumber::infinity();
-
-   fProtoData = nullptr;
-
-   fProofConfig = nullptr;
-   fNuisanceParametersSampler = nullptr;
-
    //suppress messages for num integration of Roofit
    RooMsgService::instance().getStream(1).removeTopic(RooFit::NumIntegration);
-
-   fUseMultiGen = false ;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ToyMCSampler::ToyMCSampler(TestStatistic &ts, Int_t ntoys) : fSamplingDistName(ts.GetVarName().Data()), fNToys(ntoys)
+ToyMCSampler::ToyMCSampler(TestStatistic &ts, Int_t ntoys)
+   : fSamplingDistName(ts.GetVarName().Data()),
+     fNToys(ntoys),
+     fMaxToys(RooNumber::infinity()),
+     fAdaptiveLowLimit(-RooNumber::infinity()),
+     fAdaptiveHighLimit(RooNumber::infinity())
 {
-   fPdf = nullptr;
-   fPriorNuisance = nullptr;
-   fNuisancePars = nullptr;
-   fObservables = nullptr;
-   fGlobalObservables = nullptr;
-
-   fSize = 0.05;
-   fNEvents = 0;
-   fGenerateBinned = false;
-   fGenerateBinnedTag = "";
-   fGenerateAutoBinned = true;
-   fExpectedNuisancePar = false;
-
-   fToysInTails = 0.0;
-   fMaxToys = RooNumber::infinity();
-   fAdaptiveLowLimit = -RooNumber::infinity();
-   fAdaptiveHighLimit = RooNumber::infinity();
-
-   fProtoData = nullptr;
-
-   fProofConfig = nullptr;
-   fNuisanceParametersSampler = nullptr;
 
    //suppress messages for num integration of Roofit
    RooMsgService::instance().getStream(1).removeTopic(RooFit::NumIntegration);
 
-   fUseMultiGen = false ;
-
    AddTestStatistic(&ts);
 }
 
@@ -244,7 +204,7 @@ bool ToyMCSampler::CheckConfig(void) {
 RooArgList* ToyMCSampler::EvaluateAllTestStatistics(RooAbsData& data, const RooArgSet& poi) {
    DetailedOutputAggregator detOutAgg;
    const RooArgList* allTS = EvaluateAllTestStatistics(data, poi, detOutAgg);
-   if (!allTS) return 0;
+   if (!allTS) return nullptr;
    // no need to delete allTS, it is deleted in destructor of detOutAgg
    return  dynamic_cast<RooArgList*>(allTS->snapshot());
 }
@@ -264,8 +224,9 @@ const RooArgList* ToyMCSampler::EvaluateAllTestStatistics(RooAbsData& data, cons
    for( unsigned int i = 0; i < fTestStatistics.size(); i++ ) {
       if( fTestStatistics[i] == nullptr ) continue;
       TString name( TString::Format("%s_TS%u", fSamplingDistName.c_str(), i) );
-      std::unique_ptr<RooArgSet> parForTS(poi.snapshot());
-      RooRealVar ts( name, fTestStatistics[i]->GetVarName(), fTestStatistics[i]->Evaluate( data, *parForTS ) );
+      RooArgSet parForTS;
+      poi.snapshot(parForTS);
+      RooRealVar ts( name, fTestStatistics[i]->GetVarName(), fTestStatistics[i]->Evaluate( data, parForTS ) );
       RooArgList tset(ts);
       detOutAgg.AppendArgSet(&tset);
       if (const RooArgSet* detOut = fTestStatistics[i]->GetDetailedOutput()) {
@@ -330,22 +291,21 @@ RooDataSet* ToyMCSampler::GetSamplingDistributions(RooArgSet& paramPointIn)
    fNToys = (int)ceil((double)fNToys / (double)fProofConfig->GetNExperiments()); // round up
 
    // create the study instance for parallel processing
-   ToyMCStudy* toymcstudy = new ToyMCStudy ;
-   toymcstudy->SetToyMCSampler(*this);
-   toymcstudy->SetParamPoint(paramPointIn);
-   toymcstudy->SetRandomSeed(RooRandom::randomGenerator()->Integer(TMath::Limits<unsigned int>::Max() ) );
+   ToyMCStudy toymcstudy{};
+   toymcstudy.SetToyMCSampler(*this);
+   toymcstudy.SetParamPoint(paramPointIn);
+   toymcstudy.SetRandomSeed(RooRandom::randomGenerator()->Integer(TMath::Limits<unsigned int>::Max() ) );
 
    // temporary workspace for proof to avoid messing with TRef
    RooWorkspace w(fProofConfig->GetWorkspace());
-   RooStudyManager studymanager(w, *toymcstudy);
+   RooStudyManager studymanager(w, toymcstudy);
    studymanager.runProof(fProofConfig->GetNExperiments(), fProofConfig->GetHost(), fProofConfig->GetShowGui());
 
-   RooDataSet* output = toymcstudy->merge();
+   RooDataSet* output = toymcstudy.merge();
 
    // reset the number of toys
    fNToys = totToys;
 
-   delete toymcstudy;
    return output;
 }
 
@@ -374,9 +334,9 @@ RooDataSet* ToyMCSampler::GetSamplingDistributionsSingleWorker(RooArgSet& paramP
 
    // important to cache the paramPoint b/c test statistic might
    // modify it from event to event
-   RooArgSet *paramPoint = (RooArgSet*) paramPointIn.snapshot();
+   std::unique_ptr<RooArgSet> paramPoint{paramPointIn.snapshot()};
    std::unique_ptr<RooArgSet> allVars{fPdf->getVariables()};
-   RooArgSet *saveAll = (RooArgSet*) allVars->snapshot();
+   std::unique_ptr<RooArgSet> saveAll{allVars->snapshot()};
 
 
    DetailedOutputAggregator detOutAgg;
@@ -399,32 +359,33 @@ RooDataSet* ToyMCSampler::GetSamplingDistributionsSingleWorker(RooArgSet& paramP
       // TODO: change this treatment to keep track of all values so that the threshold
       // for adaptive sampling is counted for all distributions and not just the
       // first one.
-      double valueFirst = -999.0, weight = 1.0;
+      double valueFirst = -999.0;
+      double weight = 1.0;
 
       // set variables to requested parameter point
       allVars->assign(*saveAll); // important for example for SimpleLikelihoodRatioTestStat
 
-      RooAbsData* toydata = GenerateToyData(*paramPoint, weight);
+      std::unique_ptr<RooAbsData> toydata{GenerateToyData(*paramPoint, weight)};
       if (i == 0 && !fPdf->canBeExtended() && dynamic_cast<RooSimultaneous*>(fPdf)) {
         const RooArgSet* toySet = toydata->get();
-        if (std::none_of(toySet->begin(), toySet->end(), [](const RooAbsArg* arg){
-          return dynamic_cast<const RooAbsCategory*>(arg) != nullptr;
-        }))
-          oocoutE(nullptr, Generation) << "ToyMCSampler: Generated toy data didn't contain a category variable, although"
-            " a simultaneous PDF is in use. To generate events for a simultaneous PDF, all components need to be"
-            " extended. Otherwise, the number of events to generate per component cannot be determined." << std::endl;
+        if (std::none_of(toySet->begin(), toySet->end(),
+                         [](const RooAbsArg *arg) { return dynamic_cast<const RooAbsCategory *>(arg) != nullptr; })) {
+           oocoutE(nullptr, Generation)
+              << "ToyMCSampler: Generated toy data didn't contain a category variable, although"
+                 " a simultaneous PDF is in use. To generate events for a simultaneous PDF, all components need to be"
+                 " extended. Otherwise, the number of events to generate per component cannot be determined."
+              << std::endl;
+        }
       }
 
       allVars->assign(*fParametersForTestStat);
 
       const RooArgList* allTS = EvaluateAllTestStatistics(*toydata, *fParametersForTestStat, detOutAgg);
-      if (allTS->getSize() > Int_t(fTestStatistics.size()))
+      if (allTS->size() > fTestStatistics.size())
         detOutAgg.AppendArgSet( fGlobalObservables, "globObs_" );
       if (RooRealVar* firstTS = dynamic_cast<RooRealVar*>(allTS->first()))
          valueFirst = firstTS->getVal();
 
-      delete toydata;
-
       // check for nan
       if(valueFirst != valueFirst) {
          oocoutW(nullptr, Generation) << "skip: " << valueFirst << ", " << weight << endl;
@@ -442,8 +403,6 @@ RooDataSet* ToyMCSampler::GetSamplingDistributionsSingleWorker(RooArgSet& paramP
 
    // clean up
    allVars->assign(*saveAll);
-   delete saveAll;
-   delete paramPoint;
 
    return detOutAgg.GetAsDataSet(fSamplingDistName, fSamplingDistName);
 }
@@ -463,7 +422,7 @@ void ToyMCSampler::GenerateGlobalObservables(RooAbsPdf& pdf) const {
 
       // generate one set of global observables and assign it
       // has problem for sim pdfs
-      RooSimultaneous* simPdf = dynamic_cast<RooSimultaneous*>( &pdf );
+      RooSimultaneous* simPdf = dynamic_cast<RooSimultaneous*>(&pdf );
       if (!simPdf) {
          std::unique_ptr<RooDataSet> one{pdf.generate(*fGlobalObservables, 1)};
 
@@ -476,7 +435,7 @@ void ToyMCSampler::GenerateGlobalObservables(RooAbsPdf& pdf) const {
       } else {
 
          if (_pdfList.empty()) {
-            RooCategory& channelCat = (RooCategory&)simPdf->indexCat();
+            auto& channelCat = const_cast<RooCategory&>(static_cast<RooCategory const&>(simPdf->indexCat()));
             int nCat = channelCat.numTypes();
             for (int i=0; i < nCat; ++i){
                channelCat.setIndex(i);
@@ -537,14 +496,14 @@ RooAbsData* ToyMCSampler::GenerateToyData(RooArgSet& paramPoint, double& weight,
 
    // generate global observables
    RooArgSet observables(*fObservables);
-   if(fGlobalObservables  &&  fGlobalObservables->getSize()) {
+   if(fGlobalObservables  &&  !fGlobalObservables->empty()) {
       observables.remove(*fGlobalObservables);
       GenerateGlobalObservables(pdf);
    }
 
    // save values to restore later.
    // but this must remain after(!) generating global observables
-   const RooArgSet* saveVars = (const RooArgSet*)allVars->snapshot();
+   std::unique_ptr<RooArgSet> saveVars{allVars->snapshot()};
 
    if(fNuisanceParametersSampler) { // use nuisance parameters?
       // Construct a set of nuisance parameters that has the parameters
@@ -571,7 +530,6 @@ RooAbsData* ToyMCSampler::GenerateToyData(RooArgSet& paramPoint, double& weight,
    // We generated the data with the randomized nuisance parameter (if hybrid)
    // but now we are setting the nuisance parameters back to where they were.
    allVars->assign(*saveVars);
-   delete saveVars;
 
    return data;
 }
@@ -685,7 +643,7 @@ void ToyMCSampler::ClearCache() {
   _allVars = nullptr;
 
   // no need to delete the _pdfList since it is managed by the RooSimultaneous object
-  if (_pdfList.size() > 0) {
+  if (!_pdfList.empty()) {
     _pdfList.clear();
     _obsList.clear();
     _gsList.clear();
