@@ -1,3 +1,4 @@
+#include "Byteswap.h"
 #include "TMVA/RModelParser_ONNX.hxx"
 #include "onnx_proto3.pb.h"
 
@@ -20,6 +21,7 @@ extern ParserFuncSignature ParseSqrt;
 extern ParserFuncSignature ParseReciprocal;
 extern ParserFuncSignature ParseNeg;
 extern ParserFuncSignature ParseExp;
+extern ParserFuncSignature ParseLog;
 // Binary operators
 extern ParserFuncSignature ParseAdd;
 extern ParserFuncSignature ParseSub;
@@ -31,12 +33,20 @@ extern ParserFuncSignature ParseMax;
 extern ParserFuncSignature ParseMin;
 extern ParserFuncSignature ParseMean;
 extern ParserFuncSignature ParseSum;
+//Comparision Operators
+extern ParserFuncSignature ParseEq;
+extern ParserFuncSignature ParseLess;
+extern ParserFuncSignature ParseLessEq;
+extern ParserFuncSignature ParseGreater;
+extern ParserFuncSignature ParseGreaterEq;
 // Reduce operators
 extern ParserFuncSignature ParseReduceMean;
-extern ParserFuncSignature ParseReduceSumsquare;
+extern ParserFuncSignature ParseReduceSum;
+extern ParserFuncSignature ParseReduceSumSquare;
 extern ParserFuncSignature ParseReduceProd;
 // Others
 extern ParserFuncSignature ParseBatchNormalization;
+extern ParserFuncSignature ParseConstant;
 extern ParserFuncSignature ParseTranspose;
 extern ParserFuncSignature ParseRelu;
 extern ParserFuncSignature ParseTanh;
@@ -58,8 +68,17 @@ extern ParserFuncSignature ParseConcat;
 extern ParserFuncSignature ParseCast;
 extern ParserFuncSignature ParseExpand;
 extern ParserFuncSignature ParseShape;
+extern ParserFuncSignature ParseMatMul;
 extern ParserFuncSignature ParseLayerNormalization;
 extern ParserFuncSignature ParseGather;
+extern ParserFuncSignature ParseErf;
+extern ParserFuncSignature ParseElu;
+extern ParserFuncSignature ParseEyeLike;
+extern ParserFuncSignature ParseRange;
+extern ParserFuncSignature ParseTopK;
+extern ParserFuncSignature ParseTile;
+extern ParserFuncSignature ParseSplit;
+extern ParserFuncSignature ParseIf;
 // Decalaration of fused operators
 extern ParserFuseFuncSignature ParseFuseConvAdd;
 extern ParserFuseFuncSignature ParseFuseConvTransposeAdd;
@@ -71,6 +90,57 @@ struct RModelParser_ONNX::OperatorsMapImpl {
    std::unordered_map<std::string, ParserFuncSignature> fOperatorsMap;
 };
 
+// helper function to get initialized tensor data
+template<typename T>
+struct ExtractDataFromTP {
+};
+// trait function to extract data from TensorProto
+template<>
+struct ExtractDataFromTP<float> {
+   static void Copy(onnx::TensorProto * tensor, void * data) {
+      tensor->mutable_float_data()->ExtractSubrange(0, tensor->float_data_size(),
+                                                            static_cast<float *>(data));
+   }
+};
+template<>
+struct ExtractDataFromTP<double> {
+   static void Copy(onnx::TensorProto * tensor, void * data) {
+      tensor->mutable_double_data()->ExtractSubrange(0, tensor->double_data_size(),
+                                                            static_cast<double *>(data));
+   }
+};
+template<>
+struct ExtractDataFromTP<int32_t> {
+   static void Copy(onnx::TensorProto * tensor, void * data) {
+      tensor->mutable_int32_data()->ExtractSubrange(0, tensor->int32_data_size(),
+                                                            static_cast<int32_t *>(data));
+   }
+};
+template<>
+struct ExtractDataFromTP<int64_t> {
+   static void Copy(onnx::TensorProto * tensor, void * data) {
+      tensor->mutable_int64_data()->ExtractSubrange(0, tensor->int64_data_size(),
+                                                            static_cast<int64_t *>(data));
+   }
+};
+template<typename T>
+std::shared_ptr<void> GetInitializedTensorData(onnx::TensorProto * tensorproto, size_t length) {
+   std::shared_ptr<void> data(malloc(length * sizeof(T)), free);
+
+   if (!tensorproto->raw_data().empty()) {
+#ifdef R__BYTESWAP
+      std::memcpy(data.get(), tensorproto->raw_data().c_str(), length * sizeof(T));
+#else
+      for (std::size_t k = 0; k < length; ++k)
+         (reinterpret_cast<typename RByteSwap<sizeof(T)>::value_type *>(data.get()))[k] =
+            RByteSwap<sizeof(T)>::bswap((reinterpret_cast<const typename RByteSwap<sizeof(T)>::value_type *>(tensorproto->raw_data().c_str()))[k]);
+#endif
+   } else {
+      ExtractDataFromTP<T>::Copy(tensorproto, data.get());
+   }
+   return data;
+}
+
 // Constructor of the parser
 RModelParser_ONNX::RModelParser_ONNX() noexcept : fOperatorsMapImpl(std::make_unique<OperatorsMapImpl>()) {
    // Register operators
@@ -79,6 +149,7 @@ RModelParser_ONNX::RModelParser_ONNX() noexcept : fOperatorsMapImpl(std::make_un
    RegisterOperator("Reciprocal", ParseReciprocal);
    RegisterOperator("Neg", ParseNeg);
    RegisterOperator("Exp", ParseExp);
+   RegisterOperator("Log", ParseLog);
    // Binary operators
    RegisterOperator("Add", ParseAdd);
    RegisterOperator("Sub", ParseSub);
@@ -90,12 +161,21 @@ RModelParser_ONNX::RModelParser_ONNX() noexcept : fOperatorsMapImpl(std::make_un
    RegisterOperator("Min", ParseMin);
    RegisterOperator("Mean", ParseMean);
    RegisterOperator("Sum", ParseSum);
+   //Comparision Operators
+   RegisterOperator("Equal", ParseEq);
+   RegisterOperator("Less", ParseLess);
+   RegisterOperator("LessOrEqual", ParseLessEq);
+   RegisterOperator("Greater", ParseGreater);
+   RegisterOperator("GreaterOrEqual", ParseGreaterEq);
    // Reduce operators
    RegisterOperator("ReduceMean", ParseReduceMean);
-   RegisterOperator("ReduceSumsquare", ParseReduceSumsquare);
+   RegisterOperator("ReduceSum", ParseReduceSum);
+   RegisterOperator("ReduceSumSquare", ParseReduceSumSquare);
    RegisterOperator("ReduceProd", ParseReduceProd);
    // Others
    RegisterOperator("BatchNormalization", ParseBatchNormalization);
+   RegisterOperator("Constant", ParseConstant);
+   RegisterOperator("ConstantOfShape", ParseConstant);
    RegisterOperator("Cast", ParseCast);
    RegisterOperator("Concat", ParseConcat);
    RegisterOperator("Conv", ParseConv);
@@ -119,13 +199,20 @@ RModelParser_ONNX::RModelParser_ONNX() noexcept : fOperatorsMapImpl(std::make_un
    RegisterOperator("Sigmoid", ParseSigmoid);
    RegisterOperator("Slice", ParseSlice);
    RegisterOperator("Softmax", ParseSoftmax);
-   RegisterOperator("Slice", ParseSlice);
-   RegisterOperator("Softmax", ParseSoftmax);
    RegisterOperator("Tanh", ParseTanh);
    RegisterOperator("Transpose", ParseTranspose);
+   RegisterOperator("MatMul", ParseMatMul);
    RegisterOperator("LayerNormalization", ParseLayerNormalization);
    RegisterOperator("Expand", ParseExpand);
    RegisterOperator("Gather", ParseGather);
+   RegisterOperator("Erf", ParseErf);
+   RegisterOperator("Elu", ParseElu);
+   RegisterOperator("EyeLike", ParseEyeLike);
+   RegisterOperator("Range", ParseRange);
+   RegisterOperator("TopK", ParseTopK);
+   RegisterOperator("Tile", ParseTile);
+   RegisterOperator("Split", ParseSplit);
+   RegisterOperator("If", ParseIf);
 }
 
 // Destructor of the parser
@@ -170,34 +257,40 @@ ETensorType RModelParser_ONNX::GetTensorType(const std::string &name)
 std::unique_ptr<ROperator>
 RModelParser_ONNX::ParseOperator(const size_t i, const onnx::GraphProto &graphproto, const std::vector<size_t> &nodes)
 {
-   int idx = (nodes.size() > i) ? nodes[i] : (int)i;
+   if (i >= nodes.size())
+      throw std::runtime_error("TMVA::SOFIE - Error in parsing ordered operators " + std::to_string(i) + " is >=  " + std::to_string(nodes.size()));
+   int idx = nodes[i];
    const auto &nodeproto = graphproto.node(idx);
    const std::string op_type = nodeproto.op_type();
    if (fVerbose)
-      std::cout << "Parsing an operator " << op_type << std::endl;
+      std::cout << "Parsing operator " << op_type << std::endl;
 
-
-   if (op_type == "MatMul") {
-      // Fuse MatMul and Add
-      int idx2 = (nodes.size() > i + 1) ? nodes[i + 1] : (int)i + 1;
-      if (idx2 < graphproto.node_size() && graphproto.node(idx2).op_type() == "Add") {
-         return ParseFuseMatMulAdd(*this, graphproto.node(idx), graphproto.node(idx2));
-      }
-   } else if (nodeproto.op_type() == "Conv" || nodeproto.op_type() == "ConvTranspose") {
+   // try to fuse with following operator in case it is not last one
+   if (i < nodes.size() - 1) {
+      int idx2 = nodes[i+1];
+      if (op_type == "MatMul") {
+        // Fuse MatMul and Add
+         if (idx2 < graphproto.node_size() && graphproto.node(idx2).op_type() == "Add") {
+            return ParseFuseMatMulAdd(*this, graphproto.node(idx), graphproto.node(idx2));
+         }
+         else {
+            return ParseMatMul(*this, graphproto.node(idx));
+         }
+      } else if (nodeproto.op_type() == "Conv" || nodeproto.op_type() == "ConvTranspose") {
       // Fuse Conv or ConvTranspose without bias and Add
-      int j = (nodes.size() > i + 1) ? nodes[i + 1] : (int)i + 1;
-      if (j < graphproto.node_size() && graphproto.node(j).op_type() == "Add") {
-         if (nodeproto.op_type() == "Conv") {
-            return ParseFuseConvAdd(*this, graphproto.node(idx), graphproto.node(j));
-         } else {
-            return ParseFuseConvTransposeAdd(*this, graphproto.node(idx), graphproto.node(j));
+         if (idx2 < graphproto.node_size() && graphproto.node(idx2).op_type() == "Add") {
+            if (nodeproto.op_type() == "Conv") {
+               return ParseFuseConvAdd(*this, graphproto.node(idx), graphproto.node(idx2));
+            } else {
+               return ParseFuseConvTransposeAdd(*this, graphproto.node(idx), graphproto.node(idx2));
+            }
          }
       }
    }
 
-   // skip then the following Add
+   // skip then the following Add if it was fused before
    if (idx > 0 && op_type == "Add") {
-      int idx0 = (nodes.size() > i) ? nodes[i - 1] : (int)i - 1;
+      int idx0 = nodes[i - 1];
       if (graphproto.node(idx0).op_type() == "MatMul")
          return nullptr;
       else if (graphproto.node(idx0).op_type() == "ConvTranspose")
@@ -206,6 +299,7 @@ RModelParser_ONNX::ParseOperator(const size_t i, const onnx::GraphProto &graphpr
 
    auto it = fOperatorsMapImpl->fOperatorsMap.find(op_type);
    if (it == fOperatorsMapImpl->fOperatorsMap.end()) {
+      std::cout << "operator " << op_type << " is not supported" << std::endl;
       throw std::runtime_error("TMVA::SOFIE Operator type " + op_type + " is not yet supported");
    }
    if (fVerbose) {
@@ -228,14 +322,11 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
       filename_nodir = (filename.substr(isep + 1, filename.length() - isep));
    }
 
-   std::time_t ttime = std::time(0);
-   std::tm *gmt_time = std::gmtime(&ttime);
-   std::string parsetime(std::asctime(gmt_time));
 
    GOOGLE_PROTOBUF_VERIFY_VERSION;
    // model I/O
    onnx::ModelProto model;
-   RModel rmodel(filename_nodir, parsetime);
+
 
    fTensorTypeMap.clear();
 
@@ -252,6 +343,25 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
       std::cout << "ONNX Version " << model.ir_version() << std::endl;
    }
 
+   std::time_t ttime = std::time(0);
+   std::tm *gmt_time = std::gmtime(&ttime);
+   std::string parsetime(std::asctime(gmt_time));
+
+   RModel rmodel(filename_nodir, parsetime);
+   ParseONNXGraph(rmodel, graph, filename_nodir);
+   return rmodel;
+}
+
+void RModelParser_ONNX::ParseONNXGraph(RModel & rmodel, const onnx::GraphProto & graph, std::string  graphName)
+{
+   bool verbose = fVerbose;
+
+   if (graphName.empty())
+      graphName = graph.name();
+
+   if (verbose)
+      std::cout << "\nParsing Graph - " << graphName << std::endl;
+
    std::unordered_set<std::string> initializer_names;
    for (int i = 0; i < graph.initializer_size(); i++) {
       initializer_names.insert(graph.initializer(i).name());
@@ -283,12 +393,19 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
       std::vector<Dim> fShape;
       bool existParam = false;
       if (!valueinfoproto.type().tensor_type().has_shape())
-         throw std::runtime_error("TMVA::SOFIE datanode with no shape restrictions is not supported yet");
+         throw std::runtime_error("TMVA::SOFIE data node with no shape restrictions is not supported yet");
       for (int j = 0; j < valueinfoproto.type().tensor_type().shape().dim_size(); j++) {
          Dim dim;
          if (valueinfoproto.type().tensor_type().shape().dim(j).value_case() ==
              onnx::TensorShapeProto_Dimension::ValueCase::kDimValue) {
-            dim.dim = valueinfoproto.type().tensor_type().shape().dim(j).dim_value();
+             int dim_value = valueinfoproto.type().tensor_type().shape().dim(j).dim_value();
+             dim.dim = dim_value;
+             // case input dim is -1 - set a parametric shape
+             if (dim_value < 0) {
+               dim.isParam = true;
+               existParam = true;
+               dim.param = UTILITY::Clean_name(input_name) + "_size";
+             }
          } else if (valueinfoproto.type().tensor_type().shape().dim(j).value_case() ==
                     onnx::TensorShapeProto_Dimension::ValueCase::kDimParam) {
             dim.isParam = true;
@@ -340,34 +457,34 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
          std::cout << "\t initializer " << i << " name " << input_name << " type " << graph.initializer(i).data_type()
                    << std::endl;
 
-      switch (static_cast<ETensorType>(graph.initializer(i).data_type())) {
+      // register also the initialized tensors
+      auto tensor_type = static_cast<ETensorType>(graph.initializer(i).data_type());
+      RegisterTensorType(input_name, tensor_type);
+
+      switch (tensor_type) {
       case ETensorType::FLOAT: {
-         std::shared_ptr<void> data(malloc(fLength * sizeof(float)), free);
-
-         if (tensorproto->raw_data().empty() == false) {
-            auto raw_data_ptr = reinterpret_cast<float *>(const_cast<char *>(tensorproto->raw_data().c_str()));
-            std::memcpy(data.get(), raw_data_ptr, fLength * sizeof(float));
-         } else {
-            tensorproto->mutable_float_data()->ExtractSubrange(0, tensorproto->float_data_size(),
-                                                               static_cast<float *>(data.get()));
-         }
-
+         std::shared_ptr<void> data = GetInitializedTensorData<float>(tensorproto, fLength);
          if (verbose) std::cout << "add FLOAT initialized tensor " << input_name << " shape " << ConvertShapeToString(shape) << std::endl;
          rmodel.AddInitializedTensor(input_name, ETensorType::FLOAT, shape, data);
          allInitializedTensors[input_name] = i;
          break;
       }
+      case ETensorType::DOUBLE: {
+         std::shared_ptr<void> data = GetInitializedTensorData<double>(tensorproto, fLength);
+         if (verbose) std::cout << "add DOUBLE initialized tensor " << input_name << " shape " << ConvertShapeToString(shape) << std::endl;
+         rmodel.AddInitializedTensor(input_name, ETensorType::DOUBLE, shape, data);
+         allInitializedTensors[input_name] = i;
+         break;
+      }
+      case ETensorType::INT32: {
+         std::shared_ptr<void> data = GetInitializedTensorData<int32_t>(tensorproto, fLength);
+         if (verbose) std::cout << "add INT32 initialized tensor " << input_name << " shape " << ConvertShapeToString(shape) << std::endl;
+         rmodel.AddInitializedTensor(input_name, ETensorType::INT32, shape, data);
+         allInitializedTensors[input_name] = i;
+         break;
+      }
       case ETensorType::INT64: {
-         std::shared_ptr<void> data(malloc(fLength * sizeof(int64_t)), free);
-
-         if (tensorproto->raw_data().empty() == false) {
-            auto raw_data_ptr = reinterpret_cast<int64_t *>(const_cast<char *>(tensorproto->raw_data().c_str()));
-            std::memcpy(data.get(), raw_data_ptr, fLength * sizeof(int64_t));
-         } else {
-            tensorproto->mutable_int64_data()->ExtractSubrange(0, tensorproto->int64_data_size(),
-                                                               static_cast<int64_t *>(data.get()));
-         }
-
+         std::shared_ptr<void> data = GetInitializedTensorData<int64_t>(tensorproto, fLength);
          if (verbose) std::cout << "add INT64 initialized tensor " << input_name << " shape " << ConvertShapeToString(shape) << std::endl;
          rmodel.AddInitializedTensor(input_name, ETensorType::INT64, shape, data);
          allInitializedTensors[input_name] = i;
@@ -400,7 +517,7 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
    nodesOrder.reserve(graph.node_size());
    std::vector<bool> foundNodes(graph.node_size());
    // loop at graph inputs
-   std::map<std::string, int> allInputs;
+   //std::map<std::string, int> allInputs;
    for (int i = 0; i < graph.input_size(); i++) {
       allInputs[graph.input(i).name()] = -1;
    }
@@ -413,14 +530,16 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
          bool existInputs = true;
          int input_size = graph.node(i).input_size();
          // special case for Reshape where shape is input and not a weight tensor
+         if (fVerbose )
+            std::cout << "Checking input of  Node " << i << " : " << graph.node(i).name() << std::endl;
          for (int j = 0; j < input_size; j++) {
             std::string name = graph.node(i).input(j);
             // skip empty names
             if (!name.empty()) {
                existInputs &= (allInputs.find(name) != allInputs.end() ||
                                allInitializedTensors.find(name) != allInitializedTensors.end());
-               if (fVerbose) {
-                  std::cout << graph.node(i).op_type() << " input " << name << " "
+               if (fVerbose ) {
+                  std::cout << "\t\t input " << name << " "
                      << bool(allInputs.find(name) != allInputs.end()) << "  " <<
                      bool(allInitializedTensors.find(name) != allInitializedTensors.end()) <<
                      existInputs << std::endl;
@@ -429,7 +548,7 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
          }
          if (!existInputs) {
             if (fVerbose) {
-               std::cout << "skip op " << graph.node(i).op_type() << " inputs are ";
+               std::cout << "skip node " << graph.node(i).op_type() << "  " << graph.node(i).name() << " inputs are not existing ";
                for (int j = 0; j < input_size; j++) {
                   std::cout << graph.node(i).input(j) << " ";
                }
@@ -438,17 +557,20 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
             continue;
          }
          if (verbose)
-            std::cout << "\tadd node " << graph.node(i).op_type() << " order " << i << std::endl;
+            std::cout << "===> New node " << graph.node(i).op_type() << "  " << graph.node(i).name() << " order " << i << std::endl;
 
          nodesOrder.push_back(i);
          foundNodes[i] = true;
          // register the outputs
          for (int j = 0; j < graph.node(i).output_size(); j++) {
+            if (fVerbose) std::cout << "\toutput : " << graph.node(i).output(j) << std::endl;
             allInputs[graph.node(i).output(j)] = i;
          }
       }
       // no increment in nodes - something wrong
       if (nodesOrder.size() == psize) {
+         int ilast = nodesOrder.back();
+         std::cout << "cannot find a new node after " << graph.node(ilast).op_type() << " " << graph.node(ilast).name() << std::endl;
          throw std::runtime_error("TMVA::SOFIE - cannot find a new node ");
       }
    } while ((int)nodesOrder.size() < graph.node_size());
@@ -501,7 +623,7 @@ RModel RModelParser_ONNX::Parse(std::string filename, bool verbose)
    }
    rmodel.AddOutputTensorNameList(outputnames);
 
-   return rmodel;
+   return;
 }
 
 } // namespace SOFIE
