@@ -18,6 +18,7 @@
 #include <TFile.h>
 #include <TSystem.h>
 #include <TString.h>
+#include <TROOT.h>
 
 using namespace TMVA;
 
@@ -75,12 +76,9 @@ public:
 
    void CreateDataLoader()
    {
-      TString fname = "./tmva_class_example.root";
+      TString fname = gROOT->GetTutorialDir() + "/tmva/data/tmva_class_example.root";
       if (!gSystem->AccessPathName(fname)) {
          inputfile = TFile::Open(fname); // check if file in local directory exists
-      } else {
-         TFile::SetCacheFileDir(".");
-         inputfile = TFile::Open("http://root.cern/files/tmva_class_example.root", "CACHEREAD");
       }
       if (!inputfile) {
          std::cout << "ERROR: could not open data file" << std::endl;
