@@ -8,21 +8,30 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
+#include <ROOT/RDataSource.hxx>
 #include <ROOT/RDF/InterfaceUtils.hxx>
-#include <ROOT/RDataFrame.hxx>
-#include <ROOT/RStringView.hxx>
-#include <ROOT/TSeq.hxx>
-#include <RtypesCore.h>
-#include <TDirectory.h>
-#include <TChain.h>
+#include <ROOT/RDF/RColumnRegister.hxx>
+#include <ROOT/RDF/RDisplay.hxx>
+#include <ROOT/RDF/RInterface.hxx>
+#include <ROOT/RDF/RJittedDefine.hxx>
+#include <ROOT/RDF/RJittedFilter.hxx>
+#include <ROOT/RDF/RJittedVariation.hxx>
+#include <ROOT/RDF/RLoopManager.hxx>
+#include <ROOT/RDF/RNodeBase.hxx>
+#include <ROOT/RDF/Utils.hxx>
+#include <string_view>
+#include <TBranch.h>
 #include <TClass.h>
 #include <TClassEdit.h>
-#include <TFriendElement.h>
-#include <TInterpreter.h>
-#include <TObject.h>
+#include <TDataType.h>
+#include <TError.h>
+#include <TLeaf.h>
+#include <TObjArray.h>
 #include <TPRegexp.h>
+#include <TROOT.h>
 #include <TString.h>
 #include <TTree.h>
+#include <TVirtualMutex.h>
 
 // pragma to disable warnings on Rcpp which have
 // so many noise compiling
@@ -38,25 +47,32 @@
 
 #include <algorithm>
 #include <cassert>
-#include <unordered_set>
+#include <cstdlib>  // for size_t
+#include <iterator> // for back_insert_iterator
+#include <map>
+#include <memory>
+#include <set>
+#include <sstream>
 #include <stdexcept>
 #include <string>
-#include <sstream>
+#include <type_traits> // for remove_reference<>::type
 #include <typeinfo>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility> // for pair
+#include <vector>
 
 namespace ROOT {
 namespace Detail {
 namespace RDF {
 class RDefineBase;
-class RFilterBase;
-class RLoopManager;
-class RRangeBase;
 } // namespace RDF
-} // namespace Detail
-
+namespace Internal {
 namespace RDF {
-class RDataSource;
-} // namespace RDF
+class RJittedAction;
+}
+} // namespace Internal
+} // namespace Detail
 
 } // namespace ROOT
 
@@ -76,10 +92,9 @@ struct ParsedExpression {
 };
 
 /// Look at expression `expr` and return a pair of (column names used, aliases used)
-static std::pair<ColumnNames_t, ColumnNames_t>
+std::pair<ColumnNames_t, ColumnNames_t>
 FindUsedColsAndAliases(const std::string &expr, const ColumnNames_t &treeBranchNames,
-                       const ROOT::Internal::RDF::RColumnRegister &customColumns,
-                       const ColumnNames_t &dataSourceColNames)
+                       const ROOT::Internal::RDF::RColumnRegister &colRegister, const ColumnNames_t &dataSourceColNames)
 {
    lexertk::generator tokens;
    const auto tokensOk = tokens.process(expr);
@@ -116,7 +131,7 @@ FindUsedColsAndAliases(const std::string &expr, const ColumnNames_t &treeBranchN
       // in an expression such as `a.b`, if `a` is a column alias add it to `usedAliases` and
       // replace the alias with the real column name in `potentialColNames`.
       const auto maybeAnAlias = potentialColNames[0]; // intentionally a copy as we'll modify potentialColNames later
-      const auto &resolvedAlias = customColumns.ResolveAlias(maybeAnAlias);
+      const auto &resolvedAlias = colRegister.ResolveAlias(maybeAnAlias);
       if (resolvedAlias != maybeAnAlias) { // this is an alias
          usedAliases.insert(maybeAnAlias);
          for (auto &s : potentialColNames)
@@ -126,7 +141,8 @@ FindUsedColsAndAliases(const std::string &expr, const ColumnNames_t &treeBranchN
       // find the longest potential column name that is an actual column name
       // (potential columns are sorted by length, so we search from the end to find the longest)
       auto isRDFColumn = [&](const std::string &col) {
-         if (customColumns.HasName(col) || IsStrInVec(col, treeBranchNames) || IsStrInVec(col, dataSourceColNames))
+         if (colRegister.IsDefineOrAlias(col) || IsStrInVec(col, treeBranchNames) ||
+             IsStrInVec(col, dataSourceColNames))
             return true;
          return false;
       };
@@ -139,7 +155,7 @@ FindUsedColsAndAliases(const std::string &expr, const ColumnNames_t &treeBranchN
 }
 
 /// Substitute each '.' in a string with '\.'
-static std::string EscapeDots(const std::string &s)
+std::string EscapeDots(const std::string &s)
 {
    TString out(s);
    TPRegexp dot("\\.");
@@ -147,7 +163,7 @@ static std::string EscapeDots(const std::string &s)
    return std::string(std::move(out));
 }
 
-static TString ResolveAliases(const TString &expr, const ColumnNames_t &usedAliases,
+TString ResolveAliases(const TString &expr, const ColumnNames_t &usedAliases,
                               const ROOT::Internal::RDF::RColumnRegister &colRegister)
 {
    TString out(expr);
@@ -155,13 +171,13 @@ static TString ResolveAliases(const TString &expr, const ColumnNames_t &usedAlia
    for (const auto &alias : usedAliases) {
       const auto &col = colRegister.ResolveAlias(alias);
       TPRegexp replacer("\\b" + EscapeDots(alias) + "\\b");
-      replacer.Substitute(out, col, "g");
+      replacer.Substitute(out, col.data(), "g");
    }
 
    return out;
 }
 
-static ParsedExpression ParseRDFExpression(std::string_view expr, const ColumnNames_t &treeBranchNames,
+ParsedExpression ParseRDFExpression(std::string_view expr, const ColumnNames_t &treeBranchNames,
                                            const ROOT::Internal::RDF::RColumnRegister &colRegister,
                                            const ColumnNames_t &dataSourceColNames)
 {
@@ -203,20 +219,20 @@ static ParsedExpression ParseRDFExpression(std::string_view expr, const ColumnNa
    return ParsedExpression{std::string(std::move(exprWithVars)), std::move(usedCols), std::move(varNames)};
 }
 
-/// Return the static global map of Filter/Define lambda expressions that have been jitted.
+/// Return the static global map of Filter/Define functions that have been jitted.
 /// It's used to check whether a given expression has already been jitted, and
 /// to look up its associated variable name if it is.
 /// Keys in the map are the body of the expression, values are the name of the
 /// jitted variable that corresponds to that expression. For example, for:
-///     auto lambda1 = [] { return 42; };
-/// key would be "[] { return 42; }" and value would be "lambda1".
-static std::unordered_map<std::string, std::string> &GetJittedExprs() {
+///     auto f1(){ return 42; }
+/// key would be "(){ return 42; }" and value would be "f1".
+std::unordered_map<std::string, std::string> &GetJittedExprs() {
    static std::unordered_map<std::string, std::string> jittedExpressions;
    return jittedExpressions;
 }
 
-static std::string
-BuildLambdaString(const std::string &expr, const ColumnNames_t &vars, const ColumnNames_t &varTypes)
+std::string
+BuildFunctionString(const std::string &expr, const ColumnNames_t &vars, const ColumnNames_t &varTypes)
 {
    assert(vars.size() == varTypes.size());
 
@@ -257,7 +273,7 @@ BuildLambdaString(const std::string &expr, const ColumnNames_t &vars, const Colu
    };
 
    std::stringstream ss;
-   ss << "[](";
+   ss << "(";
    for (auto i = 0u; i < vars.size(); ++i) {
       std::string fullType;
       const auto &type = varTypes[i];
@@ -283,81 +299,66 @@ BuildLambdaString(const std::string &expr, const ColumnNames_t &vars, const Colu
    return ss.str();
 }
 
-/// Declare a lambda expression to the interpreter in namespace R_rdf, return the name of the jitted lambda.
-/// If the lambda expression is already in GetJittedExprs, return the name for the lambda that has already been jitted.
-static std::string DeclareLambda(const std::string &expr, const ColumnNames_t &vars, const ColumnNames_t &varTypes)
+/// Declare a function to the interpreter in namespace R_rdf, return the name of the jitted function.
+/// If the function is already in GetJittedExprs, return the name for the function that has already been jitted.
+std::string DeclareFunction(const std::string &expr, const ColumnNames_t &vars, const ColumnNames_t &varTypes)
 {
    R__LOCKGUARD(gROOTMutex);
 
-   const auto lambdaExpr = BuildLambdaString(expr, vars, varTypes);
+   const auto funcCode = BuildFunctionString(expr, vars, varTypes);
    auto &exprMap = GetJittedExprs();
-   const auto exprIt = exprMap.find(lambdaExpr);
+   const auto exprIt = exprMap.find(funcCode);
    if (exprIt != exprMap.end()) {
       // expression already there
-      const auto lambdaName = exprIt->second;
-      return lambdaName;
+      const auto funcName = exprIt->second;
+      return funcName;
    }
 
    // new expression
-   const auto lambdaBaseName = "lambda" + std::to_string(exprMap.size());
-   const auto lambdaFullName = "R_rdf::" + lambdaBaseName;
+   const auto funcBaseName = "func" + std::to_string(exprMap.size());
+   const auto funcFullName = "R_rdf::" + funcBaseName;
 
-   const auto toDeclare = "namespace R_rdf {\nauto " + lambdaBaseName + " = " + lambdaExpr + ";\nusing " +
-                          lambdaBaseName + "_ret_t = typename ROOT::TypeTraits::CallableTraits<decltype(" +
-                          lambdaBaseName + ")>::ret_type;\n}";
-   ROOT::Internal::RDF::InterpreterDeclare(toDeclare.c_str());
+   const auto toDeclare = "namespace R_rdf {\nauto " + funcBaseName + funcCode + "\nusing " + funcBaseName +
+                          "_ret_t = typename ROOT::TypeTraits::CallableTraits<decltype(" + funcBaseName +
+                          ")>::ret_type;\n}";
+   ROOT::Internal::RDF::InterpreterDeclare(toDeclare);
 
-   // InterpreterDeclare could throw. If it doesn't, mark the lambda as already jitted
-   exprMap.insert({lambdaExpr, lambdaFullName});
+   // InterpreterDeclare could throw. If it doesn't, mark the function as already jitted
+   exprMap.insert({funcCode, funcFullName});
 
-   return lambdaFullName;
+   return funcFullName;
 }
 
-/// Each jitted lambda comes with a lambda_ret_t type alias for its return type.
+/// Each jitted function comes with a func_ret_t type alias for its return type.
 /// Resolve that alias and return the true type as string.
-static std::string RetTypeOfLambda(const std::string &lambdaName)
+std::string RetTypeOfFunc(const std::string &funcName)
 {
-   const auto dt = gROOT->GetType((lambdaName + "_ret_t").c_str());
+   const auto dt = gROOT->GetType((funcName + "_ret_t").c_str());
    R__ASSERT(dt != nullptr);
    const auto type = dt->GetFullTypeName();
    return type;
 }
 
-static void GetTopLevelBranchNamesImpl(TTree &t, std::set<std::string> &bNamesReg, ColumnNames_t &bNames,
-                                       std::set<TTree *> &analysedTrees, const std::string friendName = "")
+[[noreturn]] void
+ThrowJitBuildActionHelperTypeError(const std::string &actionTypeNameBase, const std::type_info &helperArgType)
 {
-   if (!analysedTrees.insert(&t).second) {
-      return;
-   }
+   int err = 0;
+   const char *cname = TClassEdit::DemangleTypeIdName(helperArgType, err);
+   std::string actionHelperTypeName = cname;
+   delete[] cname;
+   if (err != 0)
+      actionHelperTypeName = helperArgType.name();
 
-   auto branches = t.GetListOfBranches();
-   if (branches) {
-      for (auto branchObj : *branches) {
-         const auto name = branchObj->GetName();
-         if (bNamesReg.insert(name).second) {
-            bNames.emplace_back(name);
-         } else if (!friendName.empty()) {
-            // If this is a friend and the branch name has already been inserted, it might be because the friend
-            // has a branch with the same name as a branch in the main tree. Let's add it as <friendname>.<branchname>.
-            // If used for a Snapshot, this name will become <friendname>_<branchname> (with an underscore).
-            const auto longName = friendName + "." + name;
-            if (bNamesReg.insert(longName).second)
-               bNames.emplace_back(longName);
-         }
-      }
-   }
+   std::string exceptionText =
+      "RDataFrame::Jit: cannot just-in-time compile a \"" + actionTypeNameBase + "\" action using helper type \"" +
+      actionHelperTypeName +
+      "\". This typically happens in a custom `Fill` or `Book` invocation where the types of the input columns have "
+      "not been specified as template parameters and the ROOT interpreter has no knowledge of this type of action "
+      "helper. Please add template parameters for the types of the input columns to avoid jitting this action (i.e. "
+      "`df.Fill<float>(..., {\"x\"})`, where `float` is the type of `x`) or declare the action helper type to the "
+      "interpreter, e.g. via gInterpreter->Declare.";
 
-   auto friendTrees = t.GetListOfFriends();
-
-   if (!friendTrees)
-      return;
-
-   for (auto friendTreeObj : *friendTrees) {
-      auto friendElement = static_cast<TFriendElement *>(friendTreeObj);
-      auto friendTree = friendElement->GetTree();
-      const std::string frName(friendElement->GetName()); // this gets us the TTree name or the friend alias if any
-      GetTopLevelBranchNamesImpl(*friendTree, bNamesReg, bNames, analysedTrees, frName);
-   }
+   throw std::runtime_error(exceptionText);
 }
 
 } // anonymous namespace
@@ -429,23 +430,13 @@ void CheckValidCppVarName(std::string_view var, const std::string &where)
          isValid = false;
 
    if (!isValid) {
-      const auto error =
-         "RDataFrame::" + where + ": cannot define column \"" + std::string(var) + "\". Not a valid C++ variable name.";
+      const auto objName = where == "Define" ? "column" : "variation";
+      const auto error = "RDataFrame::" + where + ": cannot define " + objName + " \"" + std::string(var) +
+                         "\". Not a valid C++ variable name.";
       throw std::runtime_error(error);
    }
 }
 
-///////////////////////////////////////////////////////////////////////////////
-/// Get all the top-level branches names, including the ones of the friend trees
-ColumnNames_t GetTopLevelBranchNames(TTree &t)
-{
-   std::set<std::string> bNamesSet;
-   ColumnNames_t bNames;
-   std::set<TTree *> analysedTrees;
-   GetTopLevelBranchNamesImpl(t, bNamesSet, bNames, analysedTrees);
-   return bNames;
-}
-
 std::string DemangleTypeIdName(const std::type_info &typeInfo)
 {
    int dummy(0);
@@ -492,74 +483,89 @@ ConvertRegexToColumns(const ColumnNames_t &colNames, std::string_view columnName
 }
 
 /// Throw if column `definedColView` is already there.
-void CheckForRedefinition(const std::string &where, std::string_view definedColView, const RColumnRegister &customCols,
+void CheckForRedefinition(const std::string &where, std::string_view definedColView, const RColumnRegister &colRegister,
                           const ColumnNames_t &treeColumns, const ColumnNames_t &dataSourceColumns)
 {
-   const std::string definedCol(definedColView); // convert to std::string
 
-   std::string error;
-   if (customCols.IsAlias(definedCol))
-      error = "An alias with that name, pointing to column \"" + customCols.ResolveAlias(definedCol) +
+   std::string error{};
+   if (colRegister.IsAlias(definedColView))
+      error = "An alias with that name, pointing to column \"" + std::string(colRegister.ResolveAlias(definedColView)) +
               "\", already exists in this branch of the computation graph.";
-   else if (customCols.HasName(definedCol))
+   else if (colRegister.IsDefineOrAlias(definedColView))
       error = "A column with that name has already been Define'd. Use Redefine to force redefinition.";
-   // else, check if definedCol is in the list of tree branches. This is a bit better than interrogating the TTree
+   // else, check if definedColView is in the list of tree branches. This is a bit better than interrogating the TTree
    // directly because correct usage of GetBranch, FindBranch, GetLeaf and FindLeaf can be tricky; so let's assume we
    // got it right when we collected the list of available branches.
-   else if (std::find(treeColumns.begin(), treeColumns.end(), definedCol) != treeColumns.end())
+   else if (std::find(treeColumns.begin(), treeColumns.end(), definedColView) != treeColumns.end())
       error =
          "A branch with that name is already present in the input TTree/TChain. Use Redefine to force redefinition.";
-   else if (std::find(dataSourceColumns.begin(), dataSourceColumns.end(), definedCol) != dataSourceColumns.end())
+   else if (std::find(dataSourceColumns.begin(), dataSourceColumns.end(), definedColView) != dataSourceColumns.end())
       error =
          "A column with that name is already present in the input data source. Use Redefine to force redefinition.";
 
    if (!error.empty()) {
-      error = "RDataFrame::" + where + ": cannot define column \"" + definedCol + "\". " + error;
+      error = "RDataFrame::" + where + ": cannot define column \"" + std::string(definedColView) + "\". " + error;
       throw std::runtime_error(error);
    }
 }
 
 /// Throw if column `definedColView` is _not_ already there.
-void CheckForDefinition(const std::string &where, std::string_view definedColView, const RColumnRegister &customCols,
+void CheckForDefinition(const std::string &where, std::string_view definedColView, const RColumnRegister &colRegister,
                         const ColumnNames_t &treeColumns, const ColumnNames_t &dataSourceColumns)
 {
-   const std::string definedCol(definedColView); // convert to std::string
-   std::string error;
+   std::string error{};
 
-   if (customCols.IsAlias(definedCol)) {
-      error = "An alias with that name, pointing to column \"" + customCols.ResolveAlias(definedCol) +
+   if (colRegister.IsAlias(definedColView)) {
+      error = "An alias with that name, pointing to column \"" + std::string(colRegister.ResolveAlias(definedColView)) +
               "\", already exists. Aliases cannot be Redefined or Varied.";
    }
 
    if (error.empty()) {
-      const bool isAlreadyDefined = customCols.HasName(definedCol);
+      const bool isAlreadyDefined = colRegister.IsDefineOrAlias(definedColView);
       // check if definedCol is in the list of tree branches. This is a bit better than interrogating the TTree
       // directly because correct usage of GetBranch, FindBranch, GetLeaf and FindLeaf can be tricky; so let's assume we
       // got it right when we collected the list of available branches.
-      const bool isABranch = std::find(treeColumns.begin(), treeColumns.end(), definedCol) != treeColumns.end();
+      const bool isABranch = std::find(treeColumns.begin(), treeColumns.end(), definedColView) != treeColumns.end();
       const bool isADSColumn =
-         std::find(dataSourceColumns.begin(), dataSourceColumns.end(), definedCol) != dataSourceColumns.end();
+         std::find(dataSourceColumns.begin(), dataSourceColumns.end(), definedColView) != dataSourceColumns.end();
 
       if (!isAlreadyDefined && !isABranch && !isADSColumn)
          error = "No column with that name was found in the dataset. Use Define to create a new column.";
    }
 
    if (!error.empty()) {
-      error = "RDataFrame::" + where + ": cannot redefine or vary column \"" + definedCol + "\". " + error;
+      if (where == "DefaultValueFor")
+         error = "RDataFrame::" + where + ": cannot provide default values for column \"" +
+                 std::string(definedColView) + "\". " + error;
+      else
+         error = "RDataFrame::" + where + ": cannot redefine or vary column \"" + std::string(definedColView) + "\". " +
+                 error;
       throw std::runtime_error(error);
    }
 }
 
 /// Throw if the column has systematic variations attached.
-void CheckForNoVariations(const std::string &where, std::string_view definedColView, const RColumnRegister &customCols)
+void CheckForNoVariations(const std::string &where, std::string_view definedColView, const RColumnRegister &colRegister)
 {
    const std::string definedCol(definedColView);
-   const auto &variationDeps = customCols.GetVariationDeps(definedCol);
+   const auto &variationDeps = colRegister.GetVariationDeps(definedCol);
    if (!variationDeps.empty()) {
-      const std::string error =
-         "RDataFrame::" + where + ": cannot redefine column \"" + definedCol +
-         "\". The column depends on one or more systematic variations and re-defining varied columns is not supported.";
-      throw std::runtime_error(error);
+      if (where == "Redefine") {
+         const std::string error = "RDataFrame::" + where + ": cannot redefine column \"" + definedCol +
+                                   "\". The column depends on one or more systematic variations and re-defining varied "
+                                   "columns is not supported.";
+         throw std::runtime_error(error);
+      } else if (where == "DefaultValueFor") {
+         const std::string error = "RDataFrame::" + where + ": cannot provide a default value for column \"" +
+                                   definedCol +
+                                   "\". The column depends on one or more systematic variations and it should not be "
+                                   "possible to have missing values in varied columns.";
+         throw std::runtime_error(error);
+      } else {
+         const std::string error =
+            "RDataFrame::" + where + ": this operation cannot work with columns that depend on systematic variations.";
+         throw std::runtime_error(error);
+      }
    }
 }
 
@@ -610,7 +616,7 @@ ColumnNames_t FindUnknownColumns(const ColumnNames_t &requiredCols, const Column
       const auto isBranch = std::find(datasetColumns.begin(), datasetColumns.end(), column) != datasetColumns.end();
       if (isBranch)
          continue;
-      if (definedCols.HasName(column))
+      if (definedCols.IsDefineOrAlias(column))
          continue;
       const auto isDataSourceColumn =
          std::find(dataSourceColumns.begin(), dataSourceColumns.end(), column) != dataSourceColumns.end();
@@ -650,31 +656,31 @@ std::string PrettyPrintAddr(const void *const addr)
 /// Book the jitting of a Filter call
 std::shared_ptr<RDFDetail::RJittedFilter>
 BookFilterJit(std::shared_ptr<RDFDetail::RNodeBase> *prevNodeOnHeap, std::string_view name, std::string_view expression,
-              const ColumnNames_t &branches, const RColumnRegister &customCols, TTree *tree, RDataSource *ds)
+              const ColumnNames_t &branches, const RColumnRegister &colRegister, TTree *tree, RDataSource *ds)
 {
    const auto &dsColumns = ds ? ds->GetColumnNames() : ColumnNames_t{};
 
-   const auto parsedExpr = ParseRDFExpression(expression, branches, customCols, dsColumns);
+   const auto parsedExpr = ParseRDFExpression(expression, branches, colRegister, dsColumns);
    const auto exprVarTypes =
-      GetValidatedArgTypes(parsedExpr.fUsedCols, customCols, tree, ds, "Filter", /*vector2rvec=*/true);
-   const auto lambdaName = DeclareLambda(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
-   const auto type = RetTypeOfLambda(lambdaName);
+      GetValidatedArgTypes(parsedExpr.fUsedCols, colRegister, tree, ds, "Filter", /*vector2RVec=*/true);
+   const auto funcName = DeclareFunction(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
+   const auto type = RetTypeOfFunc(funcName);
    if (type != "bool")
       std::runtime_error("Filter: the following expression does not evaluate to bool:\n" + std::string(expression));
 
    // definesOnHeap is deleted by the jitted call to JitFilterHelper
-   ROOT::Internal::RDF::RColumnRegister *definesOnHeap = new ROOT::Internal::RDF::RColumnRegister(customCols);
+   ROOT::Internal::RDF::RColumnRegister *definesOnHeap = new ROOT::Internal::RDF::RColumnRegister(colRegister);
    const auto definesOnHeapAddr = PrettyPrintAddr(definesOnHeap);
    const auto prevNodeAddr = PrettyPrintAddr(prevNodeOnHeap);
 
    const auto jittedFilter = std::make_shared<RDFDetail::RJittedFilter>(
       (*prevNodeOnHeap)->GetLoopManagerUnchecked(), name,
-      Union(customCols.GetVariationDeps(parsedExpr.fUsedCols), (*prevNodeOnHeap)->GetVariations()));
+      Union(colRegister.GetVariationDeps(parsedExpr.fUsedCols), (*prevNodeOnHeap)->GetVariations()));
 
    // Produce code snippet that creates the filter and registers it with the corresponding RJittedFilter
    // Windows requires std::hex << std::showbase << (size_t)pointer to produce notation "0x1234"
    std::stringstream filterInvocation;
-   filterInvocation << "ROOT::Internal::RDF::JitFilterHelper(" << lambdaName << ", new const char*["
+   filterInvocation << "ROOT::Internal::RDF::JitFilterHelper(" << funcName << ", new const char*["
                     << parsedExpr.fUsedCols.size() << "]{";
    for (const auto &col : parsedExpr.fUsedCols)
       filterInvocation << "\"" << col << "\", ";
@@ -699,26 +705,26 @@ BookFilterJit(std::shared_ptr<RDFDetail::RNodeBase> *prevNodeOnHeap, std::string
 
 /// Book the jitting of a Define call
 std::shared_ptr<RJittedDefine> BookDefineJit(std::string_view name, std::string_view expression, RLoopManager &lm,
-                                             RDataSource *ds, const RColumnRegister &customCols,
+                                             RDataSource *ds, const RColumnRegister &colRegister,
                                              const ColumnNames_t &branches,
                                              std::shared_ptr<RNodeBase> *upcastNodeOnHeap)
 {
    auto *const tree = lm.GetTree();
    const auto &dsColumns = ds ? ds->GetColumnNames() : ColumnNames_t{};
 
-   const auto parsedExpr = ParseRDFExpression(expression, branches, customCols, dsColumns);
+   const auto parsedExpr = ParseRDFExpression(expression, branches, colRegister, dsColumns);
    const auto exprVarTypes =
-      GetValidatedArgTypes(parsedExpr.fUsedCols, customCols, tree, ds, "Define", /*vector2rvec=*/true);
-   const auto lambdaName = DeclareLambda(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
-   const auto type = RetTypeOfLambda(lambdaName);
+      GetValidatedArgTypes(parsedExpr.fUsedCols, colRegister, tree, ds, "Define", /*vector2RVec=*/true);
+   const auto funcName = DeclareFunction(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
+   const auto type = RetTypeOfFunc(funcName);
 
-   auto definesCopy = new RColumnRegister(customCols);
+   auto definesCopy = new RColumnRegister(colRegister);
    auto definesAddr = PrettyPrintAddr(definesCopy);
-   auto jittedDefine = std::make_shared<RDFDetail::RJittedDefine>(name, type, lm, customCols, parsedExpr.fUsedCols);
+   auto jittedDefine = std::make_shared<RDFDetail::RJittedDefine>(name, type, lm, colRegister, parsedExpr.fUsedCols);
 
    std::stringstream defineInvocation;
-   defineInvocation << "ROOT::Internal::RDF::JitDefineHelper<ROOT::Internal::RDF::DefineTypes::RDefineTag>("
-                    << lambdaName << ", new const char*[" << parsedExpr.fUsedCols.size() << "]{";
+   defineInvocation << "ROOT::Internal::RDF::JitDefineHelper<ROOT::Internal::RDF::DefineTypes::RDefineTag>(" << funcName
+                    << ", new const char*[" << parsedExpr.fUsedCols.size() << "]{";
    for (const auto &col : parsedExpr.fUsedCols) {
       defineInvocation << "\"" << col << "\", ";
    }
@@ -742,20 +748,20 @@ std::shared_ptr<RJittedDefine> BookDefineJit(std::string_view name, std::string_
 
 /// Book the jitting of a DefinePerSample call
 std::shared_ptr<RJittedDefine> BookDefinePerSampleJit(std::string_view name, std::string_view expression,
-                                                      RLoopManager &lm, const RColumnRegister &customCols,
+                                                      RLoopManager &lm, const RColumnRegister &colRegister,
                                                       std::shared_ptr<RNodeBase> *upcastNodeOnHeap)
 {
-   const auto lambdaName = DeclareLambda(std::string(expression), {"rdfslot_", "rdfsampleinfo_"},
+   const auto funcName = DeclareFunction(std::string(expression), {"rdfslot_", "rdfsampleinfo_"},
                                          {"unsigned int", "const ROOT::RDF::RSampleInfo"});
-   const auto retType = RetTypeOfLambda(lambdaName);
+   const auto retType = RetTypeOfFunc(funcName);
 
-   auto definesCopy = new RColumnRegister(customCols);
+   auto definesCopy = new RColumnRegister(colRegister);
    auto definesAddr = PrettyPrintAddr(definesCopy);
-   auto jittedDefine = std::make_shared<RDFDetail::RJittedDefine>(name, retType, lm, customCols, ColumnNames_t{});
+   auto jittedDefine = std::make_shared<RDFDetail::RJittedDefine>(name, retType, lm, colRegister, ColumnNames_t{});
 
    std::stringstream defineInvocation;
    defineInvocation << "ROOT::Internal::RDF::JitDefineHelper<ROOT::Internal::RDF::DefineTypes::RDefinePerSampleTag>("
-                    << lambdaName << ", nullptr, 0, ";
+                    << funcName << ", nullptr, 0, ";
    // lifetime of pointees:
    // - lm is the loop manager, and if that goes out of scope jitting does not happen at all (i.e. will always be valid)
    // - jittedDefine: heap-allocated weak_ptr that will be deleted by JitDefineHelper after usage
@@ -776,21 +782,25 @@ std::shared_ptr<RJittedVariation>
 BookVariationJit(const std::vector<std::string> &colNames, std::string_view variationName,
                  const std::vector<std::string> &variationTags, std::string_view expression, RLoopManager &lm,
                  RDataSource *ds, const RColumnRegister &colRegister, const ColumnNames_t &branches,
-                 std::shared_ptr<RNodeBase> *upcastNodeOnHeap)
+                 std::shared_ptr<RNodeBase> *upcastNodeOnHeap, bool isSingleColumn)
 {
    auto *const tree = lm.GetTree();
    const auto &dsColumns = ds ? ds->GetColumnNames() : ColumnNames_t{};
 
    const auto parsedExpr = ParseRDFExpression(expression, branches, colRegister, dsColumns);
    const auto exprVarTypes =
-      GetValidatedArgTypes(parsedExpr.fUsedCols, colRegister, tree, ds, "Vary", /*vector2rvec=*/true);
-   const auto lambdaName = DeclareLambda(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
-   const auto type = RetTypeOfLambda(lambdaName);
+      GetValidatedArgTypes(parsedExpr.fUsedCols, colRegister, tree, ds, "Vary", /*vector2RVec=*/true);
+   const auto funcName = DeclareFunction(parsedExpr.fExpr, parsedExpr.fVarNames, exprVarTypes);
+   const auto type = RetTypeOfFunc(funcName);
 
-   if (type.rfind("ROOT::VecOps::RVec", 0) != 0)
+   if (type.rfind("ROOT::VecOps::RVec", 0) != 0) {
+      // Avoid leak
+      delete upcastNodeOnHeap;
+      upcastNodeOnHeap = nullptr;
       throw std::runtime_error(
          "Jitted Vary expressions must return an RVec object. The following expression returns a " + type +
          " instead:\n" + parsedExpr.fExpr);
+   }
 
    auto colRegisterCopy = new RColumnRegister(colRegister);
    const auto colRegisterAddr = PrettyPrintAddr(colRegisterCopy);
@@ -804,8 +814,8 @@ BookVariationJit(const std::vector<std::string> &colNames, std::string_view vari
    // - jittedVariation: heap-allocated weak_ptr that will be deleted by JitDefineHelper after usage
    // - definesAddr: heap-allocated, will be deleted by JitDefineHelper after usage
    std::stringstream varyInvocation;
-   varyInvocation << "ROOT::Internal::RDF::JitVariationHelper(" << lambdaName << ", new const char*["
-                  << parsedExpr.fUsedCols.size() << "]{";
+   varyInvocation << "ROOT::Internal::RDF::JitVariationHelper<" << (isSingleColumn ? "true" : "false") << ">("
+                  << funcName << ", new const char*[" << parsedExpr.fUsedCols.size() << "]{";
    for (const auto &col : parsedExpr.fUsedCols) {
       varyInvocation << "\"" << col << "\", ";
    }
@@ -838,35 +848,32 @@ BookVariationJit(const std::vector<std::string> &colNames, std::string_view vari
 // (see comments in the body for actual jitted code)
 std::string JitBuildAction(const ColumnNames_t &cols, std::shared_ptr<RDFDetail::RNodeBase> *prevNode,
                            const std::type_info &helperArgType, const std::type_info &at, void *helperArgOnHeap,
-                           TTree *tree, const unsigned int nSlots, const RColumnRegister &customCols, RDataSource *ds,
-                           std::weak_ptr<RJittedAction> *jittedActionOnHeap)
+                           TTree *tree, const unsigned int nSlots, const RColumnRegister &colRegister, RDataSource *ds,
+                           std::weak_ptr<RJittedAction> *jittedActionOnHeap, const bool vector2RVec)
 {
-   // retrieve type of result of the action as a string
-   auto helperArgClass = TClass::GetClass(helperArgType);
-   if (!helperArgClass) {
-      std::string exceptionText = "An error occurred while inferring the result type of an operation.";
-      throw std::runtime_error(exceptionText.c_str());
-   }
-   const auto helperArgClassName = helperArgClass->GetName();
-
    // retrieve type of action as a string
    auto actionTypeClass = TClass::GetClass(at);
    if (!actionTypeClass) {
       std::string exceptionText = "An error occurred while inferring the action type of the operation.";
-      throw std::runtime_error(exceptionText.c_str());
+      throw std::runtime_error(exceptionText);
    }
    const std::string actionTypeName = actionTypeClass->GetName();
    const std::string actionTypeNameBase = actionTypeName.substr(actionTypeName.rfind(':') + 1);
 
-   auto definesCopy = new RColumnRegister(customCols); // deleted in jitted CallBuildAction
+   // retrieve type of result of the action as a string
+   const auto helperArgTypeName = TypeID2TypeName(helperArgType);
+   if (helperArgTypeName.empty()) {
+      ThrowJitBuildActionHelperTypeError(actionTypeNameBase, helperArgType);
+   }
+
+   auto definesCopy = new RColumnRegister(colRegister); // deleted in jitted CallBuildAction
    auto definesAddr = PrettyPrintAddr(definesCopy);
 
    // Build a call to CallBuildAction with the appropriate argument. When run through the interpreter, this code will
    // just-in-time create an RAction object and it will assign it to its corresponding RJittedAction.
    std::stringstream createAction_str;
    createAction_str << "ROOT::Internal::RDF::CallBuildAction<" << actionTypeName;
-   const auto columnTypeNames =
-      GetValidatedArgTypes(cols, customCols, tree, ds, actionTypeNameBase, /*vector2rvec=*/true);
+   const auto columnTypeNames = GetValidatedArgTypes(cols, colRegister, tree, ds, actionTypeNameBase, vector2RVec);
    for (auto &colType : columnTypeNames)
       createAction_str << ", " << colType;
    // on Windows, to prefix the hexadecimal value of a pointer with '0x',
@@ -878,8 +885,8 @@ std::string JitBuildAction(const ColumnNames_t &cols, std::shared_ptr<RDFDetail:
          createAction_str << ", ";
       createAction_str << '"' << cols[i] << '"';
    }
-   createAction_str << "}, " << cols.size() << ", " << nSlots << ", reinterpret_cast<" << helperArgClassName << "*>("
-                    << PrettyPrintAddr(helperArgOnHeap)
+   createAction_str << "}, " << cols.size() << ", " << nSlots << ", reinterpret_cast<shared_ptr<" << helperArgTypeName
+                    << ">*>(" << PrettyPrintAddr(helperArgOnHeap)
                     << "), reinterpret_cast<std::weak_ptr<ROOT::Internal::RDF::RJittedAction>*>("
                     << PrettyPrintAddr(jittedActionOnHeap)
                     << "), reinterpret_cast<ROOT::Internal::RDF::RColumnRegister*>(" << definesAddr << "));";
@@ -906,26 +913,36 @@ std::shared_ptr<RNodeBase> UpcastNode(std::shared_ptr<RNodeBase> ptr)
 /// * replace column names from aliases by the actual column name
 /// Return the list of selected column names.
 ColumnNames_t GetValidatedColumnNames(RLoopManager &lm, const unsigned int nColumns, const ColumnNames_t &columns,
-                                      const RColumnRegister &customColumns, RDataSource *ds)
+                                      const RColumnRegister &colRegister, RDataSource *ds)
 {
    auto selectedColumns = SelectColumns(nColumns, columns, lm.GetDefaultColumnNames());
 
    for (auto &col : selectedColumns) {
-      col = customColumns.ResolveAlias(col);
+      col = colRegister.ResolveAlias(col);
    }
 
    // Complain if there are still unknown columns at this point
-   const auto unknownColumns = FindUnknownColumns(selectedColumns, lm.GetBranchNames(), customColumns,
-                                                  ds ? ds->GetColumnNames() : ColumnNames_t{});
+   auto unknownColumns = FindUnknownColumns(selectedColumns, lm.GetBranchNames(), colRegister,
+                                            ds ? ds->GetColumnNames() : ColumnNames_t{});
 
    if (!unknownColumns.empty()) {
-      std::stringstream unknowns;
-      std::string delim = unknownColumns.size() > 1 ? "s: " : ": "; // singular/plural
-      for (auto &unknownColumn : unknownColumns) {
-         unknowns << delim << unknownColumn;
-         delim = ',';
+      // Some columns are still unknown, we need to understand if the error
+      // should be printed or if the user requested to explicitly disable it.
+      // Look for a possible overlap between the unknown columns and the
+      // columns we should ignore for the purpose of the following exception
+      std::set<std::string> intersection;
+      auto colsToIgnore = lm.GetSuppressErrorsForMissingBranches();
+      std::sort(unknownColumns.begin(), unknownColumns.end());
+      std::sort(colsToIgnore.begin(), colsToIgnore.end());
+      std::set_intersection(unknownColumns.begin(), unknownColumns.end(), colsToIgnore.begin(), colsToIgnore.end(),
+                            std::inserter(intersection, intersection.begin()));
+      if (intersection.empty()) {
+         std::string errMsg = std::string("Unknown column") + (unknownColumns.size() > 1 ? "s: " : ": ");
+         for (auto &unknownColumn : unknownColumns)
+            errMsg += '"' + unknownColumn + "\", ";
+         errMsg.resize(errMsg.size() - 2); // remove last ", "
+         throw std::runtime_error(errMsg);
       }
-      throw std::runtime_error("Unknown column" + unknowns.str());
    }
 
    return selectedColumns;
@@ -933,11 +950,11 @@ ColumnNames_t GetValidatedColumnNames(RLoopManager &lm, const unsigned int nColu
 
 std::vector<std::string> GetValidatedArgTypes(const ColumnNames_t &colNames, const RColumnRegister &colRegister,
                                               TTree *tree, RDataSource *ds, const std::string &context,
-                                              bool vector2rvec)
+                                              bool vector2RVec)
 {
    auto toCheckedArgType = [&](const std::string &c) {
-      RDFDetail::RDefineBase *define = colRegister.HasName(c) ? colRegister.GetColumns().at(c).get() : nullptr;
-      const auto colType = ColumnName2ColumnTypeName(c, tree, ds, define, vector2rvec);
+      RDFDetail::RDefineBase *define = colRegister.GetDefine(c);
+      const auto colType = ColumnName2ColumnTypeName(c, tree, ds, define, vector2RVec);
       if (colType.rfind("CLING_UNKNOWN_TYPE", 0) == 0) { // the interpreter does not know this type
          const auto msg =
             "The type of custom column \"" + c + "\" (" + colType.substr(19) +
@@ -978,16 +995,6 @@ void CheckForDuplicateSnapshotColumns(const ColumnNames_t &cols)
    }
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// \brief Trigger the execution of an RDataFrame computation graph.
-/// \param[in] node A node of the computation graph (not a result).
-///
-/// This function calls the RLoopManager::Run method on the \p fLoopManager data
-/// member of the input argument. It is intended for internal use only.
-void TriggerRun(ROOT::RDF::RNode &node){
-   node.fLoopManager->Run();
-}
-
 /// Return copies of colsWithoutAliases and colsWithAliases with size branches for variable-sized array branches added
 /// in the right positions (i.e. before the array branches that need them).
 std::pair<std::vector<std::string>, std::vector<std::string>>
