@@ -9,56 +9,54 @@ using namespace ROOT;
 using namespace ROOT::RDF;
 using namespace ROOT::VecOps;
 
-static const std::string DisplayPrintDefaultRows("+-----+----+----+-----------+\n"
-                                                 "| Row | b1 | b2 | b3        | \n"
-                                                 "+-----+----+----+-----------+\n"
-                                                 "| 0   | 0  | 1  | 2.0000000 | \n"
-                                                 "|     |    | 2  |           | \n"
-                                                 "|     |    | 3  |           | \n"
-                                                 "+-----+----+----+-----------+\n"
-                                                 "| 1   | 0  | 1  | 2.0000000 | \n"
-                                                 "|     |    | 2  |           | \n"
-                                                 "|     |    | 3  |           | \n"
-                                                 "+-----+----+----+-----------+\n"
-                                                 "| 2   | 0  | 1  | 2.0000000 | \n"
-                                                 "|     |    | 2  |           | \n"
-                                                 "|     |    | 3  |           | \n"
-                                                 "+-----+----+----+-----------+\n"
-                                                 "| 3   | 0  | 1  | 2.0000000 | \n"
-                                                 "|     |    | 2  |           | \n"
-                                                 "|     |    | 3  |           | \n"
-                                                 "+-----+----+----+-----------+\n"
-                                                 "| 4   | 0  | 1  | 2.0000000 | \n"
-                                                 "|     |    | 2  |           | \n"
-                                                 "|     |    | 3  |           | \n"
-                                                 "+-----+----+----+-----------+\n"
-);
+static const std::string DisplayPrintDefaultRows("+-----+----+----+----------+\n"
+                                                 "| Row | b1 | b2 | b3       | \n"
+                                                 "+-----+----+----+----------+\n"
+                                                 "| 0   | 0  | 1  | 2.000000 | \n"
+                                                 "|     |    | 2  |          | \n"
+                                                 "|     |    | 3  |          | \n"
+                                                 "+-----+----+----+----------+\n"
+                                                 "| 1   | 0  | 1  | 2.000000 | \n"
+                                                 "|     |    | 2  |          | \n"
+                                                 "|     |    | 3  |          | \n"
+                                                 "+-----+----+----+----------+\n"
+                                                 "| 2   | 0  | 1  | 2.000000 | \n"
+                                                 "|     |    | 2  |          | \n"
+                                                 "|     |    | 3  |          | \n"
+                                                 "+-----+----+----+----------+\n"
+                                                 "| 3   | 0  | 1  | 2.000000 | \n"
+                                                 "|     |    | 2  |          | \n"
+                                                 "|     |    | 3  |          | \n"
+                                                 "+-----+----+----+----------+\n"
+                                                 "| 4   | 0  | 1  | 2.000000 | \n"
+                                                 "|     |    | 2  |          | \n"
+                                                 "|     |    | 3  |          | \n"
+                                                 "+-----+----+----+----------+\n");
 
-static const std::string DisplayAsStringDefaultRows("+-----+----+----+-----------+\n"
-                                                    "| Row | b1 | b2 | b3        | \n"
-                                                    "+-----+----+----+-----------+\n"
-                                                    "| 0   | 0  | 1  | 2.0000000 | \n"
-                                                    "|     |    | 2  |           | \n"
-                                                    "|     |    | 3  |           | \n"
-                                                    "+-----+----+----+-----------+\n"
-                                                    "| 1   | 0  | 1  | 2.0000000 | \n"
-                                                    "|     |    | 2  |           | \n"
-                                                    "|     |    | 3  |           | \n"
-                                                    "+-----+----+----+-----------+\n"
-                                                    "| 2   | 0  | 1  | 2.0000000 | \n"
-                                                    "|     |    | 2  |           | \n"
-                                                    "|     |    | 3  |           | \n"
-                                                    "+-----+----+----+-----------+\n"
-                                                    "| 3   | 0  | 1  | 2.0000000 | \n"
-                                                    "|     |    | 2  |           | \n"
-                                                    "|     |    | 3  |           | \n"
-                                                    "+-----+----+----+-----------+\n"
-                                                    "| 4   | 0  | 1  | 2.0000000 | \n"
-                                                    "|     |    | 2  |           | \n"
-                                                    "|     |    | 3  |           | \n"
-                                                    "|     |    |    |           | \n"
-                                                    "+-----+----+----+-----------+\n"
-);
+static const std::string DisplayAsStringDefaultRows("+-----+----+----+----------+\n"
+                                                    "| Row | b1 | b2 | b3       | \n"
+                                                    "+-----+----+----+----------+\n"
+                                                    "| 0   | 0  | 1  | 2.000000 | \n"
+                                                    "|     |    | 2  |          | \n"
+                                                    "|     |    | 3  |          | \n"
+                                                    "+-----+----+----+----------+\n"
+                                                    "| 1   | 0  | 1  | 2.000000 | \n"
+                                                    "|     |    | 2  |          | \n"
+                                                    "|     |    | 3  |          | \n"
+                                                    "+-----+----+----+----------+\n"
+                                                    "| 2   | 0  | 1  | 2.000000 | \n"
+                                                    "|     |    | 2  |          | \n"
+                                                    "|     |    | 3  |          | \n"
+                                                    "+-----+----+----+----------+\n"
+                                                    "| 3   | 0  | 1  | 2.000000 | \n"
+                                                    "|     |    | 2  |          | \n"
+                                                    "|     |    | 3  |          | \n"
+                                                    "+-----+----+----+----------+\n"
+                                                    "| 4   | 0  | 1  | 2.000000 | \n"
+                                                    "|     |    | 2  |          | \n"
+                                                    "|     |    | 3  |          | \n"
+                                                    "|     |    |    |          | \n"
+                                                    "+-----+----+----+----------+\n");
 
 TEST(RDFDisplayTests, DisplayNoJitDefaultRows)
 {
@@ -141,30 +139,30 @@ TEST(RDFDisplayTests, DisplayRegexDefaultRows)
    EXPECT_EQ(dd->AsString(), DisplayAsStringDefaultRows);
 }
 
-static const std::string DisplayPrintTwoRows("+-----+----+----+-----------+\n"
-                                             "| Row | b1 | b2 | b3        | \n"
-                                             "+-----+----+----+-----------+\n"
-                                             "| 0   | 0  | 1  | 2.0000000 | \n"
-                                             "|     |    | 2  |           | \n"
-                                             "|     |    | 3  |           | \n"
-                                             "+-----+----+----+-----------+\n"
-                                             "| 1   | 0  | 1  | 2.0000000 | \n"
-                                             "|     |    | 2  |           | \n"
-                                             "|     |    | 3  |           | \n"
-                                             "+-----+----+----+-----------+\n");
+static const std::string DisplayPrintTwoRows("+-----+----+----+----------+\n"
+                                             "| Row | b1 | b2 | b3       | \n"
+                                             "+-----+----+----+----------+\n"
+                                             "| 0   | 0  | 1  | 2.000000 | \n"
+                                             "|     |    | 2  |          | \n"
+                                             "|     |    | 3  |          | \n"
+                                             "+-----+----+----+----------+\n"
+                                             "| 1   | 0  | 1  | 2.000000 | \n"
+                                             "|     |    | 2  |          | \n"
+                                             "|     |    | 3  |          | \n"
+                                             "+-----+----+----+----------+\n");
 
-static const std::string DisplayAsStringTwoRows("+-----+----+----+-----------+\n"
-                                                "| Row | b1 | b2 | b3        | \n"
-                                                "+-----+----+----+-----------+\n"
-                                                "| 0   | 0  | 1  | 2.0000000 | \n"
-                                                "|     |    | 2  |           | \n"
-                                                "|     |    | 3  |           | \n"
-                                                "+-----+----+----+-----------+\n"
-                                                "| 1   | 0  | 1  | 2.0000000 | \n"
-                                                "|     |    | 2  |           | \n"
-                                                "|     |    | 3  |           | \n"
-                                                "|     |    |    |           | \n"
-                                                "+-----+----+----+-----------+\n");
+static const std::string DisplayAsStringTwoRows("+-----+----+----+----------+\n"
+                                                "| Row | b1 | b2 | b3       | \n"
+                                                "+-----+----+----+----------+\n"
+                                                "| 0   | 0  | 1  | 2.000000 | \n"
+                                                "|     |    | 2  |          | \n"
+                                                "|     |    | 3  |          | \n"
+                                                "+-----+----+----+----------+\n"
+                                                "| 1   | 0  | 1  | 2.000000 | \n"
+                                                "|     |    | 2  |          | \n"
+                                                "|     |    | 3  |          | \n"
+                                                "|     |    |    |          | \n"
+                                                "+-----+----+----+----------+\n");
 
 TEST(RDFDisplayTests, DisplayJitTwoRows)
 {
