@@ -12,11 +12,6 @@
 
 #include <gtest/gtest.h>
 
-// Backward compatibility for gtest version < 1.10.0
-#ifndef INSTANTIATE_TEST_SUITE_P
-#define INSTANTIATE_TEST_SUITE_P INSTANTIATE_TEST_CASE_P
-#endif
-
 using ROOT::RDF::Experimental::VariationsFor;
 
 class RDFVary : public ::testing::TestWithParam<bool> {
@@ -26,7 +21,7 @@ protected:
       if (GetParam())
          ROOT::EnableImplicitMT(NSLOTS);
    }
-   ~RDFVary()
+   ~RDFVary() override
    {
       if (GetParam())
          ROOT::DisableImplicitMT();
@@ -1629,7 +1624,7 @@ struct HelperWithCallback : ROOT::Detail::RDF::RActionImpl<HelperWithCallback> {
    void Initialize() {}
    void Finalize() { *fResult = fCallbackCallCount; }
    std::shared_ptr<Result_t> GetResultPtr() const { return fResult; }
-   ROOT::RDF::SampleCallback_t GetSampleCallback() // increments fCallbackCallCount
+   ROOT::RDF::SampleCallback_t GetSampleCallback() override // increments fCallbackCallCount
    {
       auto callback = [](unsigned int, const ROOT::RDF::RSampleInfo &) { ++fCallbackCallCount; };
       return callback;
