@@ -13,8 +13,7 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
-#include <ROOT/RColumnElement.hxx>
-#include <ROOT/RColumnModel.hxx>
+#include <ROOT/RColumnElementBase.hxx>
 #include <ROOT/RNTupleDescriptor.hxx>
 #include <ROOT/RNTupleUtil.hxx>
 
@@ -33,37 +32,38 @@ struct ClusterInfo {
    std::uint32_t fBytesOnStorage = 0;
    std::uint32_t fBytesInMemory = 0;
 
-   bool operator ==(const ClusterInfo &other) const {
-      return fFirstEntry == other.fFirstEntry;
-   }
+   bool operator==(const ClusterInfo &other) const { return fFirstEntry == other.fFirstEntry; }
 
-   bool operator <(const ClusterInfo &other) const {
-      return fFirstEntry < other.fFirstEntry;
-   }
+   bool operator<(const ClusterInfo &other) const { return fFirstEntry < other.fFirstEntry; }
 };
 
 struct ColumnInfo {
    ROOT::Experimental::DescriptorId_t fPhysicalColumnId = 0;
    ROOT::Experimental::DescriptorId_t fLogicalColumnId = 0;
    ROOT::Experimental::DescriptorId_t fFieldId = 0;
-   std::uint64_t fLocalOrder = 0;
    std::uint64_t fNElements = 0;
    std::uint64_t fNPages = 0;
    std::uint64_t fBytesOnStorage = 0;
    std::uint32_t fElementSize = 0;
+   std::uint32_t fColumnIndex = 0;
+   std::uint16_t fRepresentationIndex = 0;
    ROOT::Experimental::EColumnType fType;
    std::string fFieldName;
    std::string fFieldDescription;
 
-   bool operator <(const ColumnInfo &other) const {
-      if (fFieldName == other.fFieldName)
-         return fLocalOrder < other.fLocalOrder;
+   bool operator<(const ColumnInfo &other) const
+   {
+      if (fFieldName == other.fFieldName) {
+         if (fRepresentationIndex == other.fRepresentationIndex)
+            return fColumnIndex < other.fColumnIndex;
+         return fRepresentationIndex < other.fRepresentationIndex;
+      }
       return fFieldName < other.fFieldName;
    }
 };
 
-std::string GetFieldName(ROOT::Experimental::DescriptorId_t fieldId,
-   const ROOT::Experimental::RNTupleDescriptor &ntupleDesc)
+std::string
+GetFieldName(ROOT::Experimental::DescriptorId_t fieldId, const ROOT::Experimental::RNTupleDescriptor &ntupleDesc)
 {
    const auto &fieldDesc = ntupleDesc.GetFieldDescriptor(fieldId);
    if (fieldDesc.GetParentId() == ROOT::Experimental::kInvalidDescriptorId)
@@ -72,7 +72,7 @@ std::string GetFieldName(ROOT::Experimental::DescriptorId_t fieldId,
 }
 
 std::string GetFieldDescription(ROOT::Experimental::DescriptorId_t fFieldId,
-   const ROOT::Experimental::RNTupleDescriptor &ntupleDesc)
+                                const ROOT::Experimental::RNTupleDescriptor &ntupleDesc)
 {
    const auto &fieldDesc = ntupleDesc.GetFieldDescriptor(fFieldId);
    return fieldDesc.GetFieldDescription();
@@ -85,6 +85,7 @@ void ROOT::Experimental::RNTupleDescriptor::PrintInfo(std::ostream &output) cons
    std::vector<ColumnInfo> columns;
    std::vector<ClusterInfo> clusters;
    std::unordered_map<DescriptorId_t, unsigned int> cluster2Idx;
+   clusters.reserve(fClusterDescriptors.size());
    for (const auto &cluster : fClusterDescriptors) {
       ClusterInfo info;
       info.fFirstEntry = cluster.second.GetFirstEntryIndex();
@@ -105,18 +106,22 @@ void ROOT::Experimental::RNTupleDescriptor::PrintInfo(std::ostream &output) cons
 
       // We generate the default memory representation for the given column type in order
       // to report the size _in memory_ of column elements
-      auto elementSize = Internal::RColumnElementBase::Generate(column.second.GetModel().GetType())->GetSize();
+      auto elementSize = Internal::RColumnElementBase::Generate(column.second.GetType())->GetSize();
 
       ColumnInfo info;
       info.fPhysicalColumnId = column.second.GetPhysicalId();
       info.fLogicalColumnId = column.second.GetLogicalId();
       info.fFieldId = column.second.GetFieldId();
-      info.fLocalOrder = column.second.GetIndex();
+      info.fColumnIndex = column.second.GetIndex();
       info.fElementSize = elementSize;
-      info.fType = column.second.GetModel().GetType();
+      info.fType = column.second.GetType();
+      info.fRepresentationIndex = column.second.GetRepresentationIndex();
 
       for (const auto &cluster : fClusterDescriptors) {
          auto columnRange = cluster.second.GetColumnRange(column.second.GetPhysicalId());
+         if (columnRange.fIsSuppressed)
+            continue;
+
          info.fNElements += columnRange.fNElements;
          if (compression == -1) {
             compression = columnRange.fCompressionSettings;
@@ -138,44 +143,46 @@ void ROOT::Experimental::RNTupleDescriptor::PrintInfo(std::ostream &output) cons
    }
    auto headerSize = GetOnDiskHeaderSize();
    auto footerSize = GetOnDiskFooterSize();
-   output << "============================================================" << std::endl;
-   output << "NTUPLE:      " << GetName() << std::endl;
-   output << "Compression: " << compression << std::endl;
-   output << "------------------------------------------------------------" << std::endl;
-   output << "  # Entries:        " << GetNEntries() << std::endl;
-   output << "  # Fields:         " << GetNFields() << std::endl;
-   output << "  # Columns:        " << GetNPhysicalColumns() << std::endl;
-   output << "  # Alias Columns:  " << GetNLogicalColumns() - GetNPhysicalColumns() << std::endl;
-   output << "  # Pages:          " << nPages << std::endl;
-   output << "  # Clusters:       " << GetNClusters() << std::endl;
-   output << "  Size on storage:  " << bytesOnStorage << " B" << std::endl;
+   output << "============================================================\n";
+   output << "NTUPLE:      " << GetName() << "\n";
+   output << "Compression: " << compression << "\n";
+   output << "------------------------------------------------------------\n";
+   output << "  # Entries:        " << GetNEntries() << "\n";
+   output << "  # Fields:         " << GetNFields() << "\n";
+   output << "  # Columns:        " << GetNPhysicalColumns() << "\n";
+   output << "  # Alias Columns:  " << GetNLogicalColumns() - GetNPhysicalColumns() << "\n";
+   output << "  # Pages:          " << nPages << "\n";
+   output << "  # Clusters:       " << GetNClusters() << "\n";
+   output << "  Size on storage:  " << bytesOnStorage << " B"
+          << "\n";
    output << "  Compression rate: " << std::fixed << std::setprecision(2)
-                                    << float(bytesInMemory) / float(bytesOnStorage) << std::endl;
-   output << "  Header size:      " << headerSize << " B" << std::endl;
-   output << "  Footer size:      " << footerSize << " B" << std::endl;
+          << float(bytesInMemory) / float(bytesOnStorage) << "\n";
+   output << "  Header size:      " << headerSize << " B"
+          << "\n";
+   output << "  Footer size:      " << footerSize << " B"
+          << "\n";
    output << "  Meta-data / data: " << std::fixed << std::setprecision(3)
-                                    << float(headerSize + footerSize) / float(bytesOnStorage) << std::endl;
-   output << "------------------------------------------------------------" << std::endl;
-   output << "CLUSTER DETAILS" << std::endl;
+          << float(headerSize + footerSize) / float(bytesOnStorage) << "\n";
+   output << "------------------------------------------------------------\n";
+   output << "CLUSTER DETAILS\n";
    output << "------------------------------------------------------------" << std::endl;
 
    std::sort(clusters.begin(), clusters.end());
    for (unsigned int i = 0; i < clusters.size(); ++i) {
-      output << "  # " << std::setw(5) << i
-             << "   Entry range:     [" << clusters[i].fFirstEntry << ".."
-             << clusters[i].fFirstEntry + clusters[i].fNEntries - 1 << "]  --  " << clusters[i].fNEntries << std::endl;
+      output << "  # " << std::setw(5) << i << "   Entry range:     [" << clusters[i].fFirstEntry << ".."
+             << clusters[i].fFirstEntry + clusters[i].fNEntries - 1 << "]  --  " << clusters[i].fNEntries << "\n";
       output << "         "
-             << "   # Pages:         " << clusters[i].fNPages << std::endl;
+             << "   # Pages:         " << clusters[i].fNPages << "\n";
       output << "         "
-             << "   Size on storage: " << clusters[i].fBytesOnStorage << " B" << std::endl;
+             << "   Size on storage: " << clusters[i].fBytesOnStorage << " B\n";
       output << "         "
              << "   Compression:     " << std::fixed << std::setprecision(2)
              << float(clusters[i].fBytesInMemory) / float(float(clusters[i].fBytesOnStorage)) << std::endl;
    }
 
-   output << "------------------------------------------------------------" << std::endl;
-   output << "COLUMN DETAILS" << std::endl;
-   output << "------------------------------------------------------------" << std::endl;
+   output << "------------------------------------------------------------\n";
+   output << "COLUMN DETAILS\n";
+   output << "------------------------------------------------------------\n";
    for (auto &col : columns) {
       col.fFieldName = GetFieldName(col.fFieldId, *this).substr(1);
       col.fFieldDescription = GetFieldDescription(col.fFieldId, *this);
@@ -184,21 +191,23 @@ void ROOT::Experimental::RNTupleDescriptor::PrintInfo(std::ostream &output) cons
    for (const auto &col : columns) {
       auto avgPageSize = (col.fNPages == 0) ? 0 : (col.fBytesOnStorage / col.fNPages);
       auto avgElementsPerPage = (col.fNPages == 0) ? 0 : (col.fNElements / col.fNPages);
-      std::string nameAndType = std::string("  ") + col.fFieldName + " [#" + std::to_string(col.fLocalOrder) + "]" +
-                                "  --  " + Internal::RColumnElementBase::GetTypeName(col.fType);
+      std::string nameAndType = std::string("  ") + col.fFieldName + " [#" + std::to_string(col.fColumnIndex);
+      if (col.fRepresentationIndex > 0)
+         nameAndType += " / R." + std::to_string(col.fRepresentationIndex);
+      nameAndType += "]  --  " + std::string{Internal::RColumnElementBase::GetColumnTypeName(col.fType)};
       std::string id = std::string("{id:") + std::to_string(col.fLogicalColumnId) + "}";
       if (col.fLogicalColumnId != col.fPhysicalColumnId)
          id += " --alias--> " + std::to_string(col.fPhysicalColumnId);
-      output << nameAndType << std::setw(60 - nameAndType.length()) << id << std::endl;
+      output << nameAndType << std::setw(60 - nameAndType.length()) << id << "\n";
       if (!col.fFieldDescription.empty())
-         output << "    Description:         " << col.fFieldDescription << std::endl;
-      output << "    # Elements:          " << col.fNElements << std::endl;
-      output << "    # Pages:             " << col.fNPages << std::endl;
-      output << "    Avg elements / page: " << avgElementsPerPage << std::endl;
-      output << "    Avg page size:       " << avgPageSize << " B" << std::endl;
-      output << "    Size on storage:     " << col.fBytesOnStorage << " B" << std::endl;
+         output << "    Description:         " << col.fFieldDescription << "\n";
+      output << "    # Elements:          " << col.fNElements << "\n";
+      output << "    # Pages:             " << col.fNPages << "\n";
+      output << "    Avg elements / page: " << avgElementsPerPage << "\n";
+      output << "    Avg page size:       " << avgPageSize << " B\n";
+      output << "    Size on storage:     " << col.fBytesOnStorage << " B\n";
       output << "    Compression:         " << std::fixed << std::setprecision(2)
-             << float(col.fElementSize * col.fNElements) / float(col.fBytesOnStorage) << std::endl;
+             << float(col.fElementSize * col.fNElements) / float(col.fBytesOnStorage) << "\n";
       output << "............................................................" << std::endl;
    }
 }
