@@ -19,24 +19,27 @@
 #include <TError.h>
 
 #include <cstdlib>
+#include <utility>
 
-void ROOT::Experimental::Internal::RPagePool::RegisterPage(const RPage &page, const RPageDeleter &deleter)
+ROOT::Experimental::Internal::RPageRef
+ROOT::Experimental::Internal::RPagePool::RegisterPage(RPage page, std::type_index inMemoryType)
 {
    std::lock_guard<std::mutex> lockGuard(fLock);
-   fPages.emplace_back(page);
+   fPages.emplace_back(std::move(page));
+   fPageInfos.emplace_back(RPageInfo{inMemoryType});
    fReferences.emplace_back(1);
-   fDeleters.emplace_back(deleter);
+   return RPageRef(page, this);
 }
 
-void ROOT::Experimental::Internal::RPagePool::PreloadPage(const RPage &page, const RPageDeleter &deleter)
+void ROOT::Experimental::Internal::RPagePool::PreloadPage(RPage page, std::type_index inMemoryType)
 {
    std::lock_guard<std::mutex> lockGuard(fLock);
-   fPages.emplace_back(page);
+   fPages.emplace_back(std::move(page));
+   fPageInfos.emplace_back(RPageInfo{inMemoryType});
    fReferences.emplace_back(0);
-   fDeleters.emplace_back(deleter);
 }
 
-void ROOT::Experimental::Internal::RPagePool::ReturnPage(const RPage &page)
+void ROOT::Experimental::Internal::RPagePool::ReleasePage(const RPage &page)
 {
    if (page.IsNull()) return;
    std::lock_guard<std::mutex> lockGuard(fLock);
@@ -46,45 +49,50 @@ void ROOT::Experimental::Internal::RPagePool::ReturnPage(const RPage &page)
       if (fPages[i] != page) continue;
 
       if (--fReferences[i] == 0) {
-         fDeleters[i](fPages[i]);
-         fPages[i] = fPages[N-1];
-         fReferences[i] = fReferences[N-1];
-         fDeleters[i] = fDeleters[N-1];
-         fPages.resize(N-1);
-         fReferences.resize(N-1);
-         fDeleters.resize(N-1);
+         fPages[i] = std::move(fPages[N - 1]);
+         fPageInfos[i] = fPageInfos[N - 1];
+         fReferences[i] = fReferences[N - 1];
+         fPages.resize(N - 1);
+         fPageInfos.resize(N - 1);
+         fReferences.resize(N - 1);
       }
       return;
    }
    R__ASSERT(false);
 }
 
-ROOT::Experimental::Internal::RPage
-ROOT::Experimental::Internal::RPagePool::GetPage(ColumnId_t columnId, NTupleSize_t globalIndex)
+ROOT::Experimental::Internal::RPageRef ROOT::Experimental::Internal::RPagePool::GetPage(ColumnId_t columnId,
+                                                                                        std::type_index inMemoryType,
+                                                                                        NTupleSize_t globalIndex)
 {
    std::lock_guard<std::mutex> lockGuard(fLock);
    unsigned int N = fPages.size();
    for (unsigned int i = 0; i < N; ++i) {
       if (fReferences[i] < 0) continue;
       if (fPages[i].GetColumnId() != columnId) continue;
+      if (fPageInfos[i].fInMemoryType != inMemoryType)
+         continue;
       if (!fPages[i].Contains(globalIndex)) continue;
       fReferences[i]++;
-      return fPages[i];
+      return RPageRef(fPages[i], this);
    }
-   return RPage();
+   return RPageRef();
 }
 
-ROOT::Experimental::Internal::RPage
-ROOT::Experimental::Internal::RPagePool::GetPage(ColumnId_t columnId, RClusterIndex clusterIndex)
+ROOT::Experimental::Internal::RPageRef ROOT::Experimental::Internal::RPagePool::GetPage(ColumnId_t columnId,
+                                                                                        std::type_index inMemoryType,
+                                                                                        RClusterIndex clusterIndex)
 {
    std::lock_guard<std::mutex> lockGuard(fLock);
    unsigned int N = fPages.size();
    for (unsigned int i = 0; i < N; ++i) {
       if (fReferences[i] < 0) continue;
       if (fPages[i].GetColumnId() != columnId) continue;
+      if (fPageInfos[i].fInMemoryType != inMemoryType)
+         continue;
       if (!fPages[i].Contains(clusterIndex)) continue;
       fReferences[i]++;
-      return fPages[i];
+      return RPageRef(fPages[i], this);
    }
-   return RPage();
+   return RPageRef();
 }
