@@ -45,6 +45,16 @@ TEST(RNTuple, ReconstructModel)
    auto variant =
       modelReconstructed->GetDefaultEntry().GetPtr<std::variant<double, std::variant<std::string, double>>>("variant");
    EXPECT_TRUE(variant != nullptr);
+
+   auto createOpts = RNTupleDescriptor::RCreateModelOptions();
+   createOpts.fCreateBare = true;
+   auto modelReconstructedBare = source.GetSharedDescriptorGuard()->CreateModel(createOpts);
+   try {
+      modelReconstructedBare->GetDefaultEntry();
+      FAIL() << "getting the default entry of a bare model should throw";
+   } catch (const RException &err) {
+      EXPECT_THAT(err.what(), testing::HasSubstr("bare model"));
+   }
 }
 
 TEST(RNTuple, MultipleInFile)
@@ -85,7 +95,6 @@ TEST(RNTuple, MultipleInFile)
    EXPECT_EQ(1, n);
 }
 
-
 TEST(RNTuple, WriteRead)
 {
    FileRaii fileGuard("test_ntuple_writeread.root");
@@ -189,6 +198,25 @@ TEST(RNTuple, WriteReadInlinedModel)
    EXPECT_FLOAT_EQ(3.2, (*readvpz)[2]);
 }
 
+TEST(RNTuple, WriteReadSubdir)
+{
+   FileRaii fileGuard("test_ntuple_writeread_subdir.root");
+
+   auto model = RNTupleModel::Create();
+   *model->MakeField<float>("pt") = 137.0;
+   {
+      auto file = std::unique_ptr<TFile>(TFile::Open(fileGuard.GetPath().c_str(), "RECREATE"));
+      auto dir = file->mkdir("foo");
+      auto writer = RNTupleWriter::Append(std::move(model), "ntpl", *dir);
+      writer->Fill();
+   }
+
+   auto reader = RNTupleReader::Open("/foo/ntpl", fileGuard.GetPath());
+   EXPECT_EQ(1U, reader->GetNEntries());
+   reader->LoadEntry(0);
+   EXPECT_FLOAT_EQ(137.0, *reader->GetModel().GetDefaultEntry().GetPtr<float>("pt"));
+}
+
 TEST(RNTuple, FileAnchor)
 {
    FileRaii fileGuard("test_ntuple_file_anchor.root");
@@ -214,7 +242,8 @@ TEST(RNTuple, FileAnchor)
    auto readerB = RNTupleReader::Open("B", fileGuard.GetPath());
 
    auto f = std::unique_ptr<TFile>(TFile::Open(fileGuard.GetPath().c_str()));
-   auto readerA = RNTupleReader::Open(f->Get<RNTuple>("A"));
+   auto ntuple = std::unique_ptr<ROOT::RNTuple>(f->Get<ROOT::RNTuple>("A"));
+   auto readerA = RNTupleReader::Open(*ntuple);
 
    EXPECT_EQ(1U, readerA->GetNEntries());
    EXPECT_EQ(1U, readerB->GetNEntries());
@@ -314,7 +343,7 @@ TEST(RNTuple, ClusterEntries)
       auto ntuple = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath(), opt);
       for (int i = 0; i < 100; i++) {
          ntuple->Fill();
-         if (i && ((i % 5) == 0))
+         if (((i + 1) % 5) == 0)
             ntuple->CommitCluster();
       }
    }
@@ -324,6 +353,61 @@ TEST(RNTuple, ClusterEntries)
    EXPECT_EQ(20, ntuple->GetDescriptor().GetNClusters());
 }
 
+TEST(RNTuple, ClusterEntriesAuto)
+{
+   FileRaii fileGuard("test_ntuple_cluster_entries_auto.root");
+   auto model = RNTupleModel::Create();
+   auto field = model->MakeField<float>({"pt", "transverse momentum"}, 42.0);
+
+   {
+      RNTupleWriteOptions options;
+      options.SetCompression(0);
+      options.SetEnablePageChecksums(false);
+      options.SetApproxZippedClusterSize(5 * sizeof(float));
+      options.SetPageBufferBudget(1000 * 1000);
+      auto ntuple = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath(), options);
+      for (int i = 0; i < 100; i++) {
+         ntuple->Fill();
+      }
+   }
+
+   auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
+   // 100 entries / 5 entries per cluster
+   EXPECT_EQ(20, ntuple->GetDescriptor().GetNClusters());
+}
+
+TEST(RNTuple, ClusterEntriesAutoStatus)
+{
+   FileRaii fileGuard("test_ntuple_cluster_entries_auto_status.root");
+   {
+      auto model = RNTupleModel::CreateBare();
+      auto field = model->MakeField<float>({"pt", "transverse momentum"}, 42.0);
+
+      int FlushClusterCalled = 0;
+      RNTupleFillStatus status;
+
+      RNTupleWriteOptions options;
+      options.SetCompression(0);
+      options.SetEnablePageChecksums(false);
+      options.SetApproxZippedClusterSize(5 * sizeof(float));
+      options.SetPageBufferBudget(1000 * 1000);
+      auto ntuple = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath(), options);
+      auto entry = ntuple->CreateEntry();
+      for (int i = 0; i < 100; i++) {
+         ntuple->FillNoFlush(*entry, status);
+         if (status.ShouldFlushCluster()) {
+            ntuple->FlushCluster();
+            FlushClusterCalled++;
+         }
+      }
+      EXPECT_EQ(20, FlushClusterCalled);
+   }
+
+   auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
+   // 100 entries / 5 entries per cluster
+   EXPECT_EQ(20, ntuple->GetDescriptor().GetNClusters());
+}
+
 TEST(RNTuple, PageSize)
 {
    FileRaii fileGuard("test_ntuple_elements_per_page.root");
@@ -332,12 +416,11 @@ TEST(RNTuple, PageSize)
 
    {
       RNTupleWriteOptions opt;
-      opt.SetApproxUnzippedPageSize(200);
-      auto ntuple = RNTupleWriter::Recreate(
-         std::move(model), "ntuple", fileGuard.GetPath(), opt
-      );
+      opt.SetInitialNElementsPerPage(10);
+      opt.SetMaxUnzippedPageSize(200);
+      auto writer = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath(), opt);
       for (int i = 0; i < 1000; i++) {
-         ntuple->Fill();
+         writer->Fill();
       }
    }
 
@@ -347,127 +430,6 @@ TEST(RNTuple, PageSize)
    EXPECT_EQ(20, col0_pages.fPageInfos.size());
 }
 
-TEST(RNTupleModel, EnforceValidFieldNames)
-{
-   auto model = RNTupleModel::Create();
-
-   // MakeField
-   try {
-      auto field3 = model->MakeField<float>("", 42.0);
-      FAIL() << "empty string as field name should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("name cannot be empty string"));
-   }
-   try {
-      auto field3 = model->MakeField<float>("pt.pt", 42.0);
-      FAIL() << "field name with periods should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("name 'pt.pt' cannot contain dot characters '.'"));
-   }
-
-   // Previous failures to create 'pt' should not block the name
-   auto field = model->MakeField<float>("pt", 42.0);
-
-   try {
-      auto field2 = model->MakeField<float>("pt", 42.0);
-      FAIL() << "repeated field names should throw";
-   } catch (const RException &err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
-   }
-
-   // AddField
-   try {
-      model->AddField(std::make_unique<RField<float>>("pt"));
-      FAIL() << "repeated field names should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
-   }
-
-   // MakeCollection
-   try {
-      auto otherModel = RNTupleModel::Create();
-      auto collection = model->MakeCollection("pt", std::move(otherModel));
-      FAIL() << "repeated field names should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
-   }
-
-   // subfield names don't throw because full name differs (otherModel.pt)
-   auto otherModel = RNTupleModel::Create();
-   auto otherField = otherModel->MakeField<float>("pt", 42.0);
-   auto collection = model->MakeCollection("otherModel", std::move(otherModel));
-}
-
-TEST(RNTupleModel, FieldDescriptions)
-{
-   FileRaii fileGuard("test_ntuple_field_descriptions.root");
-   auto model = RNTupleModel::Create();
-
-   auto pt = model->MakeField<float>({"pt", "transverse momentum"}, 42.0);
-
-   auto charge = std::make_unique<RField<float>>(RField<float>("charge"));
-   charge->SetDescription("electric charge");
-   model->AddField(std::move(charge));
-
-   {
-      RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath());
-   }
-
-   auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
-   std::vector<std::string> fieldDescriptions;
-   for (auto &f : ntuple->GetDescriptor().GetTopLevelFields()) {
-      fieldDescriptions.push_back(f.GetFieldDescription());
-   }
-   ASSERT_EQ(2u, fieldDescriptions.size());
-   EXPECT_EQ(std::string("transverse momentum"), fieldDescriptions[0]);
-   EXPECT_EQ(std::string("electric charge"), fieldDescriptions[1]);
-}
-
-TEST(RNTupleModel, CollectionFieldDescriptions)
-{
-   FileRaii fileGuard("test_ntuple_collection_field_descriptions.root");
-   {
-      auto muon = RNTupleModel::Create();
-      muon->SetDescription("muons after basic selection");
-
-      auto model = RNTupleModel::Create();
-      model->MakeCollection("Muon", std::move(muon));
-      RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath());
-   }
-
-   auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
-   const auto &muon_desc = *ntuple->GetDescriptor().GetTopLevelFields().begin();
-   EXPECT_EQ(std::string("muons after basic selection"), muon_desc.GetFieldDescription());
-}
-
-TEST(RNTupleModel, GetField)
-{
-   auto m = RNTupleModel::Create();
-   m->MakeField<int>("x");
-   m->MakeField<CustomStruct>("cs");
-   m->Freeze();
-   EXPECT_EQ(m->GetField("x").GetFieldName(), "x");
-   EXPECT_EQ(m->GetField("x").GetTypeName(), "std::int32_t");
-   EXPECT_EQ(m->GetField("cs.v1").GetFieldName(), "v1");
-   EXPECT_EQ(m->GetField("cs.v1").GetTypeName(), "std::vector<float>");
-   try {
-      m->GetField("nonexistent");
-      FAIL() << "invalid field name should throw";
-   } catch (const RException &err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("invalid field"));
-   }
-   try {
-      m->GetField("");
-      FAIL() << "empty field name should throw";
-   } catch (const RException &err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("invalid field"));
-   }
-   EXPECT_EQ("", m->GetFieldZero().GetQualifiedFieldName());
-   EXPECT_EQ("x", m->GetField("x").GetQualifiedFieldName());
-   EXPECT_EQ("cs", m->GetField("cs").GetQualifiedFieldName());
-   EXPECT_EQ("cs.v1", m->GetField("cs.v1").GetQualifiedFieldName());
-}
-
 TEST(RNTuple, EmptyString)
 {
    // empty storage string
@@ -475,13 +437,13 @@ TEST(RNTuple, EmptyString)
       auto model = RNTupleModel::Create();
       auto ntuple = RNTupleWriter::Recreate(std::move(model), "myNTuple", "");
       FAIL() << "empty writer storage location should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("empty storage location"));
    }
    try {
       auto ntuple = RNTupleReader::Open("myNTuple", "");
       FAIL() << "empty reader storage location should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("empty storage location"));
    }
 
@@ -490,31 +452,24 @@ TEST(RNTuple, EmptyString)
       auto model = RNTupleModel::Create();
       auto ntuple = RNTupleWriter::Recreate(std::move(model), "", "file.root");
       FAIL() << "empty RNTuple name should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("empty RNTuple name"));
    }
    try {
       auto ntuple = RNTupleReader::Open("", "file.root");
       FAIL() << "empty RNTuple name should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("empty RNTuple name"));
    }
 }
 
 TEST(RNTuple, NullSafety)
 {
-   // RNTupleModel
    auto model = RNTupleModel::Create();
-   try {
-      auto bad = model->MakeCollection("bad", nullptr);
-      FAIL() << "null submodels should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null collectionModel"));
-   }
    try {
       model->AddField(nullptr);
       FAIL() << "null fields should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("null field"));
    }
 }
@@ -525,6 +480,7 @@ TEST(RNTuple, ModelId)
    auto m2 = RNTupleModel::Create();
    EXPECT_FALSE(m1->IsFrozen());
    EXPECT_NE(m1->GetModelId(), m2->GetModelId());
+   EXPECT_NE(m1->GetSchemaId(), m2->GetSchemaId());
 
    m1->Freeze();
    EXPECT_TRUE(m1->IsFrozen());
@@ -548,8 +504,23 @@ TEST(RNTuple, ModelId)
    EXPECT_TRUE(m1->IsFrozen());
    EXPECT_EQ(id, m1->GetModelId());
 
+   // At this point, m1 is frozen while m2 is not.
+   auto m1c = m1->Clone();
+   EXPECT_TRUE(m1c->IsFrozen());
+   EXPECT_NE(m1->GetModelId(), m1c->GetModelId());
+   EXPECT_EQ(m1->GetSchemaId(), m1c->GetSchemaId());
+
    auto m2c = m2->Clone();
+   EXPECT_FALSE(m2c->IsFrozen());
    EXPECT_NE(m2->GetModelId(), m2c->GetModelId());
+   EXPECT_NE(m2->GetSchemaId(), m2c->GetSchemaId());
+
+   // Now unfreeze the cloned model again and it should get new ids.
+   id = m1c->GetModelId();
+   m1c->Unfreeze();
+   EXPECT_FALSE(m1c->IsFrozen());
+   EXPECT_NE(m1c->GetModelId(), id);
+   EXPECT_NE(m1c->GetSchemaId(), m1->GetSchemaId());
 }
 
 TEST(RNTuple, Entry)
@@ -584,6 +555,7 @@ TEST(RNTuple, BareEntry)
 {
    auto m = RNTupleModel::CreateBare();
    auto f = m->MakeField<float>("pt");
+   EXPECT_TRUE(m->IsBare());
    EXPECT_FALSE(f);
 
    FileRaii fileGuard("test_ntuple_bare_entry.root");
@@ -685,72 +657,21 @@ TEST(RNTuple, FillBytesWritten)
 {
    FileRaii fileGuard("test_ntuple_fillbytes.ntuple");
 
-   auto checkFillReturnValue = [&](const RNTupleWriteOptions &options) {
-      const std::size_t indexColumnSz = options.GetHasSmallClusters() ? sizeof(std::uint32_t) : sizeof(std::uint64_t);
-
-      // TODO(jalopezg): improve test coverage by adding other field types
-      auto model = RNTupleModel::Create();
-      auto fieldI32 = model->MakeField<std::int32_t>("i32");
-      auto fieldStr = model->MakeField<std::string>("s");
-      auto fieldBoolVec = model->MakeField<std::vector<bool>>("bool_vec");
-      auto fieldFloatVec = model->MakeField<std::vector<float>>("float_vec");
-      auto ntuple = RNTupleWriter::Recreate(std::move(model), "f", fileGuard.GetPath(), options);
-      *fieldI32 = 42;
-      *fieldStr = "abc";
-      // A 32bit integer + "abc" literal + one (32|64)bit integer for each index column
-      EXPECT_EQ(7U + (3 * indexColumnSz), ntuple->Fill());
-      *fieldBoolVec = {true, false, true};
-      *fieldFloatVec = {42.0f, 1.1f};
-      // A 32bit integer + "abc" literal + one (32|64)bit integer for each index column + 3 bools + 2 floats
-      EXPECT_EQ(18U + (3 * indexColumnSz), ntuple->Fill());
-   };
-
-   checkFillReturnValue({});
-
-   RNTupleWriteOptions optsSmall;
-   optsSmall.SetHasSmallClusters(true);
-   checkFillReturnValue(optsSmall);
-}
-
-TEST(RNTuple, FillBytesWrittenCollections)
-{
-   FileRaii fileGuard("test_ntuple_fillbytes_collections.root");
-
-   auto eventModel = RNTupleModel::Create();
-   auto fldPt = eventModel->MakeField<float>("pt", 0.0);
-
-   auto hitModel = RNTupleModel::Create();
-   auto fldHitX = hitModel->MakeField<float>("x", 0.0);
-   auto fldHitY = hitModel->MakeField<float>("y", 0.0);
-
-   auto trackModel = RNTupleModel::Create();
-   auto fldTrackEnergy = trackModel->MakeField<float>("energy", 0.0);
-
-   auto fldHits = trackModel->MakeCollection("hits", std::move(hitModel));
-   auto fldTracks = eventModel->MakeCollection("tracks", std::move(trackModel));
-
-   {
-      RNTupleWriteOptions options;
-      options.SetApproxZippedClusterSize(1024 * 1024);
-      auto writer = RNTupleWriter::Recreate(std::move(eventModel), "myNTuple", fileGuard.GetPath(), options);
-
-      for (unsigned i = 0; i < 30000; ++i) {
-         for (unsigned t = 0; t < 3; ++t) {
-            for (unsigned h = 0; h < 2; ++h) {
-               *fldHitX = 4.0;
-               *fldHitY = 8.0;
-               EXPECT_EQ(4 + 4, fldHits->Fill());
-            }
-            *fldTrackEnergy = i * t;
-            EXPECT_EQ(2 * (4 + 4) + 8 + 4, fldTracks->Fill());
-         }
-         *fldPt = float(i);
-         EXPECT_EQ(3 * (2 * (4 + 4) + 8 + 4) + 8 + 4, writer->Fill());
-      }
-   }
-
-   auto reader = RNTupleReader::Open("myNTuple", fileGuard.GetPath());
-   EXPECT_EQ(reader->GetDescriptor().GetNClusters(), 2);
+   // TODO(jalopezg): improve test coverage by adding other field types
+   auto model = RNTupleModel::Create();
+   auto fieldI32 = model->MakeField<std::int32_t>("i32");
+   auto fieldStr = model->MakeField<std::string>("s");
+   auto fieldBoolVec = model->MakeField<std::vector<bool>>("bool_vec");
+   auto fieldFloatVec = model->MakeField<std::vector<float>>("float_vec");
+   auto ntuple = RNTupleWriter::Recreate(std::move(model), "f", fileGuard.GetPath());
+   *fieldI32 = 42;
+   *fieldStr = "abc";
+   // A 32bit integer + "abc" literal + one 64bit integer for each index column
+   EXPECT_EQ(7U + (3 * sizeof(std::uint64_t)), ntuple->Fill());
+   *fieldBoolVec = {true, false, true};
+   *fieldFloatVec = {42.0f, 1.1f};
+   // A 32bit integer + "abc" literal + one 64bit integer for each index column + 3 bools + 2 floats
+   EXPECT_EQ(18U + (3 * sizeof(std::uint64_t)), ntuple->Fill());
 }
 
 TEST(RNTuple, RValue)
@@ -777,6 +698,7 @@ TEST(RNTuple, RValue)
 TEST(REntry, Basics)
 {
    auto model = RNTupleModel::Create();
+   EXPECT_FALSE(model->IsBare());
    model->MakeField<float>("pt");
    model->Freeze();
 
@@ -790,15 +712,32 @@ TEST(REntry, Basics)
    EXPECT_THROW(e->GetToken("eta"), ROOT::Experimental::RException);
    EXPECT_THROW(model->GetToken("eta"), ROOT::Experimental::RException);
 
-   std::shared_ptr<float> ptrPt;
+   auto ptrPt = std::make_shared<float>();
    e->BindValue("pt", ptrPt);
    EXPECT_EQ(ptrPt.get(), e->GetPtr<float>("pt").get());
 
-   auto model2 = model->Clone();
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<void>(e->GetToken("pt")).get());
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<void>(model->GetToken("pt")).get());
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<void>(model->GetDefaultEntry().GetToken("pt")).get());
+
+   // Using tokens across frozen clones works
+   auto clone = model->Clone();
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<void>(clone->GetToken("pt")).get());
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<void>(clone->GetDefaultEntry().GetToken("pt")).get());
+
+   // Tokens from a new model are incompatible
+   auto model2 = RNTupleModel::Create();
+   model2->MakeField<float>("pt");
+   model2->Freeze();
+   EXPECT_THROW(e->GetPtr<void>(model2->GetToken("pt")), ROOT::Experimental::RException);
    EXPECT_THROW(e->GetPtr<void>(model2->GetDefaultEntry().GetToken("pt")), ROOT::Experimental::RException);
    std::shared_ptr<double> ptrDouble;
    EXPECT_THROW(e->BindValue("pt", ptrDouble), ROOT::Experimental::RException);
 
+   // Default constructed tokens cannot be used
+   ROOT::Experimental::REntry::RFieldToken token;
+   EXPECT_THROW(e->GetPtr<void>(token), ROOT::Experimental::RException);
+
    float pt;
    e->BindRawPtr("pt", &pt);
    EXPECT_EQ(&pt, e->GetPtr<void>("pt").get());
@@ -828,3 +767,66 @@ TEST(RFieldBase, CreateObject)
    auto ptrClass = RField<LowPrecisionFloats>("name").CreateObject<LowPrecisionFloats>();
    EXPECT_DOUBLE_EQ(1.0, ptrClass->b);
 }
+
+TEST(RNTupleWriter, ForbidModelWithSubfields)
+{
+   FileRaii fileGuard("test_ntuple_writer_forbid_model_with_subfields.root");
+
+   auto model = RNTupleModel::Create();
+   model->MakeField<CustomStruct>("struct");
+   model->RegisterSubfield("struct.a");
+
+   try {
+      auto writer = RNTupleWriter::Recreate(std::move(model), "f", fileGuard.GetPath());
+      FAIL() << "should not able to create a writer using a model with registered subfields";
+   } catch (const RException &err) {
+      EXPECT_THAT(err.what(),
+                  testing::HasSubstr("cannot create an RNTupleWriter from a model with registered subfields"));
+   }
+}
+
+TEST(RNTupleWriter, ForbiddenCharactersInRNTupleName)
+{
+   FileRaii fileGuard("test_ntuple_writer_forbidden_characters_in_rntuple_name.root");
+
+   std::array<const char *, 6> names{"ntu.ple", "nt/uple", "n tuple", "ntupl\\e", "n\ntuple", "ntup\tle"};
+
+   for (auto &&name : names) {
+      try {
+         auto writer = RNTupleWriter::Recreate(RNTupleModel::Create(), name, fileGuard.GetPath());
+         FAIL() << "Should not be able to create an RNTuple with name '" << name << "'.";
+      } catch (const RException &err) {
+         EXPECT_THAT(err.what(),
+                     testing::HasSubstr("RNTuple name '" + std::string(name) + "' cannot contain character"));
+      }
+   }
+}
+
+TEST(RNTuple, ForbiddenCharactersInField)
+{
+   std::array<const char *, 6> names{"fie.ld", "fi/eld", "f ield", "fiel\\d", "f\nield", "fi\teld"};
+
+   for (auto &&name : names) {
+      try {
+         auto field = RFieldBase::Create(name, "int").Unwrap();
+         FAIL() << "Should not be able to create an RNTuple field with name '" << name << "'.";
+      } catch (const RException &err) {
+         EXPECT_THAT(err.what(), testing::HasSubstr("Field name '" + std::string(name) + "' cannot contain character"));
+      }
+   }
+}
+
+TEST(RNTuple, ForbiddenCharactersInModelField)
+{
+   std::array<const char *, 6> names{"fie.ld", "fi/eld", "f ield", "fiel\\d", "f\nield", "fi\teld"};
+
+   auto model = RNTupleModel::Create();
+   for (auto &&name : names) {
+      try {
+         auto field = model->MakeField<int>(name);
+         FAIL() << "Should not be able to create an RNTuple field with name '" << name << "'.";
+      } catch (const RException &err) {
+         EXPECT_THAT(err.what(), testing::HasSubstr("Field name '" + std::string(name) + "' cannot contain character"));
+      }
+   }
+}
