@@ -333,47 +333,60 @@ TEST(RNTupleShow, Objects)
 
 TEST(RNTupleShow, Collections)
 {
+   using ROOT::Experimental::RRecordField;
+   using ROOT::Experimental::RVectorField;
+
    std::string rootFileName{"test_ntuple_show_collection.root"};
    std::string ntupleName{"Collections"};
    FileRaii fileGuard(rootFileName);
    {
+      struct MyStruct {
+         short myShort;
+         float myFloat;
+      };
+
       auto model = RNTupleModel::Create();
-      auto collection_model = RNTupleModel::Create();
-      auto int_field = collection_model->MakeField<int>("myInt");
-      auto float_field = collection_model->MakeField<float>("myFloat");
-      auto collection = model->MakeCollection("collection", std::move(collection_model));
-      auto ntuple = RNTupleWriter::Recreate(std::move(model), ntupleName, rootFileName);
-      *int_field = 0;
-      *float_field = 10.0;
-      collection->Fill();
-      *int_field = 1;
-      *float_field = 20.0;
-      collection->Fill();
-      ntuple->Fill();
+      std::vector<std::unique_ptr<RFieldBase>> leafFields;
+      leafFields.emplace_back(std::make_unique<RField<short>>("myShort"));
+      leafFields.emplace_back(std::make_unique<RField<float>>("myFloat"));
+      auto recordField = std::make_unique<RRecordField>("_0", std::move(leafFields));
+      EXPECT_EQ(offsetof(MyStruct, myShort), recordField->GetOffsets()[0]);
+      EXPECT_EQ(offsetof(MyStruct, myFloat), recordField->GetOffsets()[1]);
+
+      auto collectionField = RVectorField::CreateUntyped("myCollection", std::move(recordField));
+      model->AddField(std::move(collectionField));
+      model->Freeze();
+
+      auto v = std::static_pointer_cast<std::vector<MyStruct>>(model->GetDefaultEntry().GetPtr<void>("myCollection"));
+      auto writer = RNTupleWriter::Recreate(std::move(model), ntupleName, rootFileName);
+
+      v->emplace_back(MyStruct({1, 10.0}));
+      v->emplace_back(MyStruct({2, 20.0}));
+      writer->Fill();
    }
 
-   auto ntuple = RNTupleReader::Open(ntupleName, rootFileName);
+   auto reader = RNTupleReader::Open(ntupleName, rootFileName);
    std::ostringstream osData;
-   ntuple->Show(0, osData);
+   reader->Show(0, osData);
    // clang-format off
    std::string outputData{ std::string("")
       + "{\n"
-      + "  \"collection\": [{\"myInt\": 0, \"myFloat\": 10}, {\"myInt\": 1, \"myFloat\": 20}]\n"
+      + "  \"myCollection\": [{\"myShort\": 1, \"myFloat\": 10}, {\"myShort\": 2, \"myFloat\": 20}]\n"
       + "}\n" };
    // clang-format on
    EXPECT_EQ(outputData, osData.str());
 
    std::ostringstream osFields;
-   ntuple->PrintInfo(ROOT::Experimental::ENTupleInfo::kSummary, osFields);
+   reader->PrintInfo(ROOT::Experimental::ENTupleInfo::kSummary, osFields);
    // clang-format off
    std::string outputFields{ std::string("")
       + "************************************ NTUPLE ************************************\n"
       + "* N-Tuple : Collections                                                        *\n"
       + "* Entries : 1                                                                  *\n"
       + "********************************************************************************\n"
-      + "* Field 1           : collection (std::vector<>)                               *\n"
+      + "* Field 1           : myCollection                                             *\n"
       + "*   Field 1.1       : _0                                                       *\n"
-      + "*     Field 1.1.1   : myInt (std::int32_t)                                     *\n"
+      + "*     Field 1.1.1   : myShort (std::int16_t)                                   *\n"
       + "*     Field 1.1.2   : myFloat (float)                                          *\n"
       + "********************************************************************************\n" };
    // clang-format on
@@ -626,7 +639,7 @@ TEST(RNTupleShow, Friends)
       writer->Fill();
    }
 
-   std::vector<RNTupleReader::ROpenSpec> friends = {{"ntpl1", fileGuard1.GetPath()}, {"ntpl2", fileGuard2.GetPath()}};
+   std::vector<RNTupleOpenSpec> friends = {{"ntpl1", fileGuard1.GetPath()}, {"ntpl2", fileGuard2.GetPath()}};
    auto ntuple = RNTupleReader::OpenFriends(friends);
    std::ostringstream os;
    ntuple->Show(0, os);
