@@ -9,10 +9,10 @@
 #include "CustomStructUtil.hxx"
 #include "ntupleutil_test.hxx"
 
+using ROOT::RNTuple;
 using ROOT::Experimental::EColumnType;
 using ROOT::Experimental::RField;
 using ROOT::Experimental::RFieldBase;
-using ROOT::Experimental::RNTuple;
 using ROOT::Experimental::RNTupleInspector;
 using ROOT::Experimental::RNTupleModel;
 using ROOT::Experimental::RNTupleWriteOptions;
@@ -27,12 +27,9 @@ TEST(RNTupleInspector, CreateFromPointer)
    }
 
    std::unique_ptr<TFile> file(TFile::Open(fileGuard.GetPath().c_str()));
-   auto ntuple = file->Get<RNTuple>("ntuple");
-   auto inspector = RNTupleInspector::Create(ntuple);
+   auto ntuple = std::unique_ptr<RNTuple>(file->Get<RNTuple>("ntuple"));
+   auto inspector = RNTupleInspector::Create(*ntuple);
    EXPECT_EQ(inspector->GetDescriptor()->GetName(), "ntuple");
-
-   auto nullNTuple = file->Get<RNTuple>("null");
-   EXPECT_THROW(RNTupleInspector::Create(nullNTuple), ROOT::Experimental::RException);
 }
 
 TEST(RNTupleInspector, CreateFromString)
@@ -241,7 +238,7 @@ TEST(RNTupleInspector, SizeProjectedFields)
       muonPt->emplace_back(1.0);
       muonPt->emplace_back(2.0);
 
-      auto nMuons = RFieldBase::Create("nMuons", "ROOT::Experimental::RNTupleCardinality<std::uint64_t>").Unwrap();
+      auto nMuons = RFieldBase::Create("nMuons", "ROOT::RNTupleCardinality<std::uint64_t>").Unwrap();
       model->AddProjectedField(std::move(nMuons), [](const std::string &) { return "muonPt"; });
 
       auto writer = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath());
@@ -300,11 +297,11 @@ TEST(RNTupleInspector, ColumnInfoUncompressed)
       auto model = RNTupleModel::Create();
 
       auto int32fld = std::make_unique<RField<std::int32_t>>("int32");
-      int32fld->SetColumnRepresentative({EColumnType::kInt32});
+      int32fld->SetColumnRepresentatives({{EColumnType::kInt32}});
       model->AddField(std::move(int32fld));
 
       auto splitReal64fld = std::make_unique<RField<double>>("splitReal64");
-      splitReal64fld->SetColumnRepresentative({EColumnType::kSplitReal64});
+      splitReal64fld->SetColumnRepresentatives({{EColumnType::kSplitReal64}});
       model->AddField(std::move(splitReal64fld));
 
       auto writeOptions = RNTupleWriteOptions();
@@ -537,7 +534,8 @@ TEST(RNTupleInspector, PageSizeDistribution)
 
       auto writeOptions = RNTupleWriteOptions();
       writeOptions.SetCompression(505);
-      writeOptions.SetApproxUnzippedPageSize(64);
+      writeOptions.SetInitialNElementsPerPage(1);
+      writeOptions.SetMaxUnzippedPageSize(64);
       auto ntuple = RNTupleWriter::Recreate(std::move(model), "ntuple", fileGuard.GetPath(), writeOptions);
 
       for (unsigned i = 0; i < 100; ++i) {
@@ -763,3 +761,28 @@ TEST(RNTupleInspector, FieldsByName)
       EXPECT_EQ("std::int32_t", inspector->GetFieldTreeInspector(fieldId).GetDescriptor().GetTypeName());
    }
 }
+
+TEST(RNTupleInspector, MultiColumnRepresentations)
+{
+   FileRaii fileGuard("test_ntuple_inspector_multi_column_representations.root");
+   {
+      auto model = RNTupleModel::Create();
+      auto fldPx = RFieldBase::Create("px", "float").Unwrap();
+      fldPx->SetColumnRepresentatives({{EColumnType::kReal32}, {EColumnType::kReal16}});
+      model->AddField(std::move(fldPx));
+      auto writer = RNTupleWriter::Recreate(std::move(model), "ntpl", fileGuard.GetPath());
+      writer->Fill();
+      writer->CommitCluster();
+      ROOT::Experimental::Internal::RFieldRepresentationModifier::SetPrimaryColumnRepresentation(
+         const_cast<RFieldBase &>(writer->GetModel().GetConstField("px")), 1);
+      writer->Fill();
+   }
+
+   auto inspector = RNTupleInspector::Create("ntpl", fileGuard.GetPath());
+   auto px0Inspector = inspector->GetColumnInspector(0);
+   auto px1Inspector = inspector->GetColumnInspector(1);
+   EXPECT_EQ(EColumnType::kReal32, px0Inspector.GetType());
+   EXPECT_EQ(1u, px0Inspector.GetNElements());
+   EXPECT_EQ(EColumnType::kReal16, px1Inspector.GetType());
+   EXPECT_EQ(1u, px1Inspector.GetNElements());
+}
