@@ -93,5 +93,7 @@
 #pragma read sourceClass="TTree" targetClass="TTree" version="[-18]" source="" target="fNClusterRange" code="{ fNClusterRange = 0; }"
 
 #pragma link C++ namespace ROOT::Internal::TreeUtils;
+#pragma link C++ class ROOT::Internal::TreeUtils::RNoCleanupNotifier;
+#pragma link C++ class TNotifyLink<ROOT::Internal::TreeUtils::RNoCleanupNotifierHelper>;
 
 #endif
